/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.affichage.contentstreams;

import fr.exemole.bdfserver.response.*;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.servlets.affichage.*;
import fr.exemole.bdfserver.managers.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.basedefiches.selection.*;
import net.mapeadores.basedefiches.extraction.*;
import fr.exemole.bdfserver.balayages.engine.*;
import fr.exemole.bdfserver.balayages.*;
import java.io.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.io.*;
import net.mapeadores.util.langinteger.*;


/**
 *
 * @author Vincent Calame
 */
public class BalayageUnitStream extends AffichageContentStream {

    private Parameters parameters;
    private Extraction extraction;
    private Object extractionSource;
    private boolean xmlVersion;
    private String errorKey = null;
    private TransformManager transformManager;
    private String lang;
    private Balayage balayage;

    public BalayageUnitStream(BdfResponseParameters responseParameters) {
        super(responseParameters);
        parameters = (Parameters) responseParameters.getParameter(OBJECT_OBJ);
        init();
    }

    private void init() {
        BalayageManager balayageManager = bdfServer.getBalayageManager();
        balayage = parameters.getBalayage();
        BalayageParameters balayageParameters = new BalayageParameters(balayage, bdfServer);
        LocalisationLists localisationLists = balayageParameters.getLocalisationLists();
        FicheEligibility ficheEligibility = balayageParameters.getFicheEligibility();
        int langInteger = parameters.getLangInteger();
        if (langInteger == 0) {
            langInteger = localisationLists.getWorkingLangInteger(0);
        }
        lang = LangInteger.toString(langInteger);
        BalayageUnit balayageUnit = parameters.getBalayageUnit();
        short balayageType = balayageUnit.getType();
        String extractionPath = balayageUnit.getExtractionPath();
        if (extractionPath != null) {
            extraction = balayageManager.getExtraction(balayage, extractionPath, localisationLists, ficheEligibility);
        } else {
            extraction = balayageManager.getDefaultExtraction(balayageType, localisationLists, ficheEligibility);
        }
        extractionSource = parameters.getSubsetItem();
        if (extractionSource == null) {
            extractionSource = parameters.getSubset();
        }
        xmlVersion = parameters.isXmlVersion();
        if (!xmlVersion) {
            if (!balayage.isWithTransformation()) {
                xmlVersion = true;
            } else {
                Output output = getOutput();
                if (output == null) {
                    xmlVersion = true;
                } else {
                    initTransformer(output);
                }
            }
        }
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        if (xmlVersion) {
            printXml(response, extraction, extractionSource, null);
        } else {
            if (errorKey != null) {
                throw new HttpAccessException(HttpAccessException.INTERNAL_ERROR, errorKey);
            }
            printHtml(response, extraction, extractionSource, transformManager.getTransformer(extractionSource));
        }
    }

    private void initTransformer(Output output) {
        InternalTransformerBuilderErrorHandler transformerBuilderErrorHandler = new InternalTransformerBuilderErrorHandler();
        TransformManagerBuilder transformerBuilder = new TransformManagerBuilder(bdfServer, balayage, transformerBuilderErrorHandler);
        transformManager = transformerBuilder.build(output);
        if (transformManager != null) {
            BalayageUnit balayageUnit = parameters.getBalayageUnit();
            ValueResolver patternValueSet = EngineUtils.getValueResolver(extractionSource);
            String path = EngineUtils.getPath(output, patternValueSet);
            String fileName = EngineUtils.getFileName(output, patternValueSet, balayageUnit.getType(), true);
            transformManager.setCurrentFileName(fileName);
            transformManager.setCurrentPath(path);
            transformManager.setCurrentLang(lang);
        }
    }

    private Output getOutput() {
        Output output = parameters.getOutput();
        if (output != null) {
            return output;
        }
        BalayageUnit balayageUnit = parameters.getBalayageUnit();
        int count = balayageUnit.getOutputCount();
        if (count == 0) {
            return null;
        }
        return balayageUnit.getOutput(0);
    }


    private class InternalTransformerBuilderErrorHandler implements TransformManagerBuilder.ErrorHandler {

        public void transformerError(String xsltPath, TransformerErrorListener errorListener) {
            errorKey = "XSLT error : " + errorListener.getErrorMessage();
        }

        public void wrongXslPath(String xsltPath) {
            errorKey = "wrong xslt path : " + xsltPath;
        }

        public void undefinedXslPath() {
            errorKey = "xslt attribute of output element is not defined";
        }

        public void wrongBalayageUnitType() {
        }

    }


    public final static class Parameters {

        private Balayage balayage;
        private BalayageUnit balayageUnit;
        private Subset subset;
        private SubsetItem subsetItem;
        private int langInteger;
        private Output output;
        private boolean xmlVersion;

        public Parameters() {
        }

        public Balayage getBalayage() {
            return balayage;
        }

        public void setBalayage(Balayage balayage) {
            this.balayage = balayage;
        }

        public BalayageUnit getBalayageUnit() {
            return balayageUnit;
        }

        public void setBalayageUnit(BalayageUnit balayageUnit) {
            this.balayageUnit = balayageUnit;
        }

        public Subset getSubset() {
            return subset;
        }

        public void setSubset(Subset subset) {
            this.subset = subset;
        }

        public SubsetItem getSubsetItem() {
            return subsetItem;
        }

        public void setSubsetItem(SubsetItem subsetItem) {
            this.subsetItem = subsetItem;
        }

        public int getLangInteger() {
            return langInteger;
        }

        public void setLangInteger(int langInteger) {
            this.langInteger = langInteger;
        }

        public Output getOutput() {
            return output;
        }

        public void setOutput(Output output) {
            this.output = output;
        }

        public boolean isXmlVersion() {
            return xmlVersion;
        }

        public void setXmlVersion(boolean xmlVersion) {
            this.xmlVersion = xmlVersion;
        }

    }

}
