/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.request.FileValue;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.util.servlets.*;
import java.util.*;
import net.mapeadores.basedefiches.corpus.*;
import java.io.*;
import org.w3c.dom.*;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.text.tableparser.CsvParameters;
import net.mapeadores.util.text.tableparser.TableParser;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractCSVCommand extends AbstractBdfCommand {

    protected Corpus corpus;
    protected ImportationResponseParameters responseParameters;
    protected Thesaurus thesaurus;
    protected File file;
    protected String[] fieldLine;

    public AbstractCSVCommand() {
    }

    public BdfResponseParameters doCommand() throws ParameterException {
        checkParameters();
        if (responseParameters.mayBeDone()) {
            String csvString = getCSVString();
            CsvParameters csvParameters = new CsvParameters();
            String[][] valeurs = TableParser.parse(csvString, csvParameters);
            int length = valeurs.length;
            if (length < 2) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "empty file");
            }
            fieldLine = valeurs[0];
            int maxLength = fieldLine.length;
            if (maxLength == 0) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "empty field line");
            }
            for (int i = 2; i < length; i++) {
                if (valeurs[i].length > maxLength) {
                    int line = i + 1;
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "too long line :" + line);
                }
            }
            InternalRequestMap irq = new InternalRequestMap(fieldLine);
            start();
            for (int i = 1; i < length; i++) {
                irq.update(valeurs[i]);
                resolveRequest(irq);
            }
            irq.clear();
            end();
        }
        return responseParameters;
    }

    private String getCSVString() throws ParameterException {
        try {
                Reader reader = new InputStreamReader(new FileInputStream(file), "UTF-8");
                String s = IOUtils.toString(reader);
                reader.close();
                return s;
            } catch (IOException ioe) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "error with file = " + ioe.getLocalizedMessage());
            }
    }

    protected CommandTest checkParameters() throws ParameterException {
        responseParameters = new ImportationResponseParameters(bdfServer, bdfUser);
        if (needThesaurus()) {
            thesaurus = RequestUtils.getThesaurus(requestMap, bdfServer.getBasedefiches());
            responseParameters.setThesaurus(thesaurus);
        }
        if (needCorpus()) {
            corpus = RequestUtils.getCorpus(requestMap, bdfServer.getBasedefiches());
            responseParameters.setCorpus(corpus);
        }
        String fileString = getMandatoryParameter("file");
        file = new File(fileString);
        if (!file.exists()) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "file do not exist = " + fileString);
        }
        if (file.isDirectory()) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, "file is a directory = " + fileString);
        }
        return responseParameters;
    }

    public abstract void start();

    public abstract void resolveRequest(RequestMap requestMap) throws ParameterException;

    public abstract void end();

    public boolean needCorpus() {
        return false;
    }

    public boolean needThesaurus() {
        return false;
    }

    public boolean needSynchronisation() {
        return true;
    }

    public ResponsePrinterFactory getResponsePrinterFactory() {
        return new ImportationResponsePrinterFactory();
    }

    public short getCommandType() {
        return ADMIN_ONLY_COMMAND;
    }


    private class InternalRequestMap implements RequestMap {

        ParameterBundle[] parameterArray;
        HashMap<String, ParameterBundle> parameterMap = new HashMap<String, ParameterBundle>();

        private InternalRequestMap(String[] fieldLine) {
            int length = fieldLine.length;
            parameterArray = new ParameterBundle[length];
            for (int i = 0; i < length; i++) {
                String name = fieldLine[i];
                ParameterBundle parameter = (ParameterBundle) parameterMap.get(name);
                if (parameter == null) {
                    parameter = new ParameterBundle(name);
                    parameterMap.put(name, parameter);
                }
                parameterArray[i] = parameter;
            }
        }

        public Document getDocument() {
            return null;
        }

        public FileValue getFileValue(String name) {
            return null;
        }

        public FileValue[] getFileValues(String name) {
            return null;
        }

        public String getParameter(String name) {
            ParameterBundle parameter = (ParameterBundle) parameterMap.get(name);
            if (parameter == null) {
                return null;
            }
            return parameter.getValue();
        }

        public String[] getParameterValues(String name) {
            ParameterBundle parameter = (ParameterBundle) parameterMap.get(name);
            if (parameter == null) {
                return null;
            }
            return parameter.getValueArray();
        }

        public Set<String> getParameterNameSet() {
            return parameterMap.keySet();
        }

        public short getResponseDataType() {
            return 0;
        }

        void update(String[] valuesLine) {
            clear();
            int length = valuesLine.length;
            for (int i = 0; i < length; i++) {
                parameterArray[i].add(valuesLine[i]);
            }
        }

        void clear() {
            for (Iterator it = parameterMap.values().iterator(); it.hasNext();) {
                ParameterBundle parameter = (ParameterBundle) it.next();
                parameter.clear();
            }
        }


        private class ParameterBundle {

            String name;
            List values = new ArrayList();

            ParameterBundle(String name) {
                this.name = name;
            }

            public void clear() {
                values.clear();
            }

            public void add(String value) {
                values.add(value);
            }

            String getValue() {
                if (values.size() == 0) {
                    return "";
                } else {
                    return (String) values.get(0);
                }
            }

            String[] getValueArray() {
                int size = values.size();
                if (size == 0) {
                    String[] result = new String[1];
                    result[0] = "";
                    return result;
                } else {
                    String[] result = new String[size];
                    for (int i = 0; i < size; i++) {
                        result[i] = (String) values.get(i);
                    }
                    return result;
                }
            }

        }

    }

}
