/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.servlets.instructions.actions;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.commands2.BdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Factory;
import fr.exemole.bdfserver.commands2.BdfCommand2Result;
import fr.exemole.bdfserver.commands2.BdfCommandPageParameters;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import fr.exemole.bdfserver.commands2.PermissionBdfCommand2Exception;
import fr.exemole.bdfserver.extension.BdfExtension;
import fr.exemole.bdfserver.extension.BdfExtensionManager;
import fr.exemole.bdfserver.htmlproducers.HtmlProducerFactory;
import fr.exemole.bdfserver.instruction.BdfRequestMap;
import fr.exemole.bdfserver.managers.configuration.PageSubstitution;
import fr.exemole.bdfserver.request.ParameterException;
import fr.exemole.bdfserver.response.BdfConstants;
import fr.exemole.bdfserver.servlets.ServletCommons;
import fr.exemole.bdfserver.servlets.instructions.ActionBdfInstruction;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.CommandMessageJsonSource;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.CommandMessageResponseHandler;
import net.mapeadores.util.servlets.handlers.HtmlResponseHandler;
import net.mapeadores.util.servlets.handlers.JsonResponseHandler;
import net.mapeadores.util.servlets.handlers.RedirectResponseHandler;
import net.mapeadores.util.servlets.handlers.SimpleResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class CommandInstruction extends ActionBdfInstruction {

    private BdfCommand2 bdfCommand2;
    private String actionName;
    private boolean noCommand = false;
    private CommandMessage initErrorMessage = null;

    public CommandInstruction(BdfContext bdfContext, BdfRequestMap requestMap, String actionName) {
        super(bdfContext, requestMap);
        this.actionName = actionName;
        String cmd = requestMap.getCommandParameter();
        if (cmd != null) {
            BdfCommand2Factory factory = DefaultCommandCatalog.getBdfCommandFactory(actionName);
            if (factory != null) {
                bdfCommand2 = factory.getBdfCommand(bdfContext, requestMap, cmd);
                if (bdfCommand2 == null) {
                    initErrorMessage = CommandMessage.error("global_err_bdfcommand", cmd);
                }
            } else {
                initErrorMessage = CommandMessage.error("global_err_bdfcommandfactory", actionName);
            }
        } else {
            noCommand = true;
        }
    }

    public short getBdfUserNeed() {
        if (noCommand) {
            return DefaultCommandCatalog.getNoCommandUserNeed(actionName, requestMap);
        } else {
            if (bdfCommand2 == null) {
                return NONE_BDFUSER;
            } else {
                return bdfCommand2.getBdfUserNeed();
            }
        }
    }

    public ResponseHandler run() throws ParameterException {
        if (noCommand) {
            return getResponseHandler(null);
        }
        if (initErrorMessage != null) {
            return getCommandMessageResponseHandler(initErrorMessage);
        }
        if (requestMap.isTest()) {
            try {
                CommandMessage commandMessage = bdfCommand2.testCommand(bdfUser);
                return getCommandMessageResponseHandler(commandMessage);
            } catch (BdfCommand2Exception bce) {
                return getExceptionResponseHandler(bce);
            }
        }
        try {
            BdfCommand2Result bdfCommandResult = null;
            if (bdfCommand2.needSynchronisation()) {
                synchronized (bdfServer) {
                    bdfCommandResult = bdfCommand2.doCommand(bdfUser);
                }
            } else {
                bdfCommandResult = bdfCommand2.doCommand(bdfUser);
            }
            return getResponseHandler(bdfCommandResult);
        } catch (BdfCommand2Exception bce) {
            return getExceptionResponseHandler(bce);
        }
    }

    private ResponseHandler getResponseHandler(BdfCommand2Result bdfCommandResult) throws ParameterException {
        short responseDataType = requestMap.getResponseDataType();
        switch (responseDataType) {
            case JSON_DATATYPE:
            case JSON_IN_HTML_DATATYPE:
                return getJsonCommandMessageResponseHandler(bdfCommandResult.getCommandMessage(), responseDataType);
        }
        if (requestMap.isRedirectDefined()) {
            return new RedirectResponseHandler(requestMap.getRedirectParameter());
        }
        if (!requestMap.isPageDefined()) {
            return getHtmlCommandMessageResponseHandler(CommandMessage.error("global_err_nopage", null));
        }
        String page = requestMap.getPageParameter();
        CommandMessage commandeMessage = null;
        if (bdfCommandResult != null) {
            commandeMessage = bdfCommandResult.getCommandMessage();
            if ((commandeMessage != null) && (commandeMessage.isErrorMessage())) {
                String errorPage = requestMap.getParameter("page-err");
                if (errorPage != null) {
                    page = errorPage;
                }
            }
        }
        BdfCommandPageParameters bdfCommandPageParameters = new BdfCommandPageParameters();
        bdfCommandPageParameters.setBdfUser(bdfUser);
        bdfCommandPageParameters.setRequestMap(requestMap);
        bdfCommandPageParameters.setActionName(actionName);
        bdfCommandPageParameters.setBdfCommandResult(bdfCommandResult);
        if (!page.startsWith("ext:")) {
            PageSubstitution pageSubstitution = bdfServer.getConfigurationManager().getPageSubstitution(actionName + "/" + page);
            if (pageSubstitution != null) {
                page = pageSubstitution.getSubstitutionName();
            }
        }
        if (page.startsWith("ext:")) {
            int idx = page.indexOf('/');
            if (idx == -1) {
                return getHtmlCommandMessageResponseHandler(CommandMessage.error("global_err_extpage_slash", page));
            }
            String extensionName = page.substring(4, idx);
            String objectKey = page.substring(idx + 1);
            BdfExtensionManager bdfExtensionManager = (BdfExtensionManager) bdfContext.getContextObject(BdfContextConstants.BDFEXTENSIONMANAGER_NAME);
            BdfExtension bdfExtension = bdfExtensionManager.getBdfExtension(bdfContext, extensionName);
            if (bdfExtension == null) {
                return getHtmlCommandMessageResponseHandler(CommandMessage.error("global_err_extpage_noext", extensionName));
            }
            ResponseHandler responseHandler = (ResponseHandler) bdfExtension.newInstance("page:" + objectKey, bdfCommandPageParameters);
            if (responseHandler == null) {
                return getHtmlCommandMessageResponseHandler(CommandMessage.error("global_err_extpage_nopage", objectKey));
            }
            return responseHandler;
        } else if (page.equals(BdfConstants.MESSAGE_PAGE)) {
            if (commandeMessage != null) {
                return getHtmlCommandMessageResponseHandler(commandeMessage);
            } else {
                SimpleResponseHandler srh = new SimpleResponseHandler("");
                srh.setHtml(true);
                return srh;
            }
        } else {
            HtmlProducer htmlProducer = null;
            try {
                htmlProducer = HtmlProducerFactory.getHtmlProducer(page, bdfContext, bdfCommandPageParameters);
            } catch (BdfCommand2Exception bce) {
                return getExceptionResponseHandler(bce);
            }
            if (htmlProducer == null) {
                return getHtmlCommandMessageResponseHandler(CommandMessage.error("global_err_unknownpage", page));
            }
            return new HtmlResponseHandler(htmlProducer);
        }

    }

    private ResponseHandler getCommandMessageResponseHandler(CommandMessage commandMessage) {
        short responseDataType = requestMap.getResponseDataType();
        switch (responseDataType) {
            case JSON_DATATYPE:
            case JSON_IN_HTML_DATATYPE:
                return getJsonCommandMessageResponseHandler(commandMessage, responseDataType);
        }
        return getHtmlCommandMessageResponseHandler(commandMessage);
    }

    private ResponseHandler getJsonCommandMessageResponseHandler(CommandMessage commandMessage, short dataType) {
        boolean jsonResponseEmbedded = (dataType == JSON_IN_HTML_DATATYPE);
        JsonProducer jsonProducer = null;
        if (commandMessage != null) {
            jsonProducer = new CommandMessageJsonSource(commandMessage, getRessourceLocalisation());
        }
        return new JsonResponseHandler(jsonProducer, jsonResponseEmbedded);
    }

    private ResponseHandler getHtmlCommandMessageResponseHandler(CommandMessage commandMessage) {
        return new CommandMessageResponseHandler(commandMessage, getRessourceLocalisation());
    }

    private ResponseHandler getExceptionResponseHandler(BdfCommand2Exception bce) {
        String msg = bce.getMessage();
        CommandMessage errorMessage = null;
        if (bce instanceof ParameterBdfCommand2Exception) {
            errorMessage = CommandMessage.error("global_err_parameterexception", msg);
        } else if (bce instanceof PermissionBdfCommand2Exception) {
            errorMessage = ((PermissionBdfCommand2Exception) bce).getCommandMessage();
        } else {
            errorMessage = CommandMessage.error("global_err_commandexception", msg);
        }
        return getCommandMessageResponseHandler(errorMessage);
    }

    private ResourceLocalisation getRessourceLocalisation() {
        return ServletCommons.getResourceLocalisation(localisationInfo.getFormatLocale());
    }

    public static boolean isValidActionName(String actionName) {
        return (DefaultCommandCatalog.getBdfCommandFactory(actionName) != null);
    }

}
