/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.pioche.data;

import java.io.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.selection.*;
import fr.exemole.bdfserver.servlets.pioche.*;
import java.util.*;
import net.mapeadores.basedefiches.selection.util.*;
import net.mapeadores.util.conditions.*;
import net.mapeadores.util.json.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class FichePiocheData implements JsonProducer {

    private Basedefiches basedefiches;
    private int workingLangInteger;
    private CorpusList corpusList;
    private ConditionGroup conditionGroup;
    private Collection<AbregeFiche> result;
    private FicheEligibility ficheEligibility;
    private MotcleSelectOptions[] motcleSelectOptionsArray;

    public FichePiocheData(Basedefiches basedefiches, int workingLangInteger) {
        this.basedefiches = basedefiches;
        this.workingLangInteger = workingLangInteger;
        corpusList = basedefiches;
    }

    public void setFicheEligibility(FicheEligibility ficheEligibility) {
        this.ficheEligibility = ficheEligibility;
    }

    public void setSubsetList(List<Subset> subsetList) {
        if (subsetList == null) {
            corpusList = basedefiches;
        } else {
            MutableCorpusList mutableCorpusList = new MutableCorpusList();
            for (Subset subset : subsetList) {
                mutableCorpusList.add((Corpus) subset);
            }
            corpusList = mutableCorpusList;
        }
    }

    public void setConditionGroup(ConditionGroup conditionGroup) {
        this.conditionGroup = conditionGroup;
    }

    public void setMotcleSelectOptionsArray(MotcleSelectOptions[] motcleSelectOptionsArray) {
        this.motcleSelectOptionsArray = motcleSelectOptionsArray;
    }

    public void init() {
        if ((conditionGroup != null) || (motcleSelectOptionsArray != null)) {
            int corpusCount = corpusList.getCorpusCount();
            SubsetKey[] corpusKeyArray = new SubsetKey[corpusCount];
            for (int i = 0; i < corpusCount; i++) {
                corpusKeyArray[i] = corpusList.getCorpus(i).getSubsetKey();
            }
            FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
            ficheSelectOptions.setCorpusKeyArray(corpusKeyArray);
            if (conditionGroup != null) {
                ficheSelectOptions.setTitreConditionGroup(conditionGroup);
            }
            if (motcleSelectOptionsArray != null) {
                ficheSelectOptions.setMotcleSelectOptionsArray(motcleSelectOptionsArray);
            }
            FicheSelector ficheSelector = new FicheSelector(basedefiches, workingLangInteger);
            ficheSelector.addFicheSelectOptions(ficheSelectOptions);
            ficheSelector.setFicheEligibility(ficheEligibility);
            result = selection(ficheSelector);
        } else {
            result = new ArrayList<AbregeFiche>();
            int corpusCount = corpusList.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = corpusList.getCorpus(i);
                int length = corpus.getTicketLength();
                for (int j = 0; j < length; j++) {
                    AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(j);
                    if (abregeFiche != null) {
                        if ((ficheEligibility == null) || (ficheEligibility.accept(abregeFiche))) {
                            result.add(abregeFiche);
                        }
                    }
                }
            }
        }
    }

    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            int length = result.size();
            jsonWriter.key("length");
            jsonWriter.value(length);
            if (length > 0) {
                jsonWriter.key("array");
                jsonWriter.array();
                for (AbregeFiche abregeFiche : result) {
                    int idcorpus = abregeFiche.getIdcorpus();
                    String corpusName = abregeFiche.getCorpus().getSubsetKey().getSubsetName();
                    jsonWriter.object();
                    jsonWriter.key("intkey");
                    jsonWriter.value(idcorpus);
                    jsonWriter.key("idalphakey");
                    jsonWriter.value(corpusName + '/' + idcorpus);
                    String titre = abregeFiche.getTitre();
                    if (titre.length() > 0) {
                        jsonWriter.key("text");
                        jsonWriter.value(titre);
                    }
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

    private Collection selection(FicheSelector ficheSelector) {
        TreeSet treeset = new TreeSet(new FicheComparator());
        int corpusCount = corpusList.getCorpusCount();
        for (int j = 0; j < corpusCount; j++) {
            Corpus corpus = corpusList.getCorpus(j);
            int length = corpus.getTicketLength();
            for (int i = 0; i < length; i++) {
                AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(i);
                if (abregeFiche == null) {
                    continue;
                }
                if (ficheSelector.isSelected(abregeFiche)) {
                    treeset.add(abregeFiche);
                }
            }
        }
        return treeset;
    }


    private static class FicheComparator implements Comparator {

        public int compare(Object o1, Object o2) {
            AbregeFiche af1 = (AbregeFiche) o1;
            AbregeFiche af2 = (AbregeFiche) o2;
            int result = af1.getCorpus().getSubsetKey().compareTo(af2.getCorpus().getSubsetKey());
            if (result != 0) {
                return result;
            }
            int id1 = af1.getIdcorpus();
            int id2 = af2.getIdcorpus();
            if (id1 < id2) {
                return -1;
            }
            if (id1 > id2) {
                return 1;
            }
            return 0;
        }

    }

}
