/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.pioche.data;

import java.io.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.selection.*;
import fr.exemole.bdfserver.servlets.pioche.*;
import java.util.*;
import net.mapeadores.basedefiches.extraction.*;
import net.mapeadores.basedefiches.selection.util.*;
import net.mapeadores.util.conditions.*;
import net.mapeadores.util.json.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class MotclePiocheData implements JsonProducer {

    private Basedefiches basedefiches;
    private int workingLangInteger;
    private ThesaurusList thesaurusList;
    private ConditionGroup conditionGroup;
    private Collection<Motcle> result;

    public MotclePiocheData(Basedefiches basedefiches, int workingLangInteger) {
        this.basedefiches = basedefiches;
        this.workingLangInteger = workingLangInteger;
        thesaurusList = basedefiches;
    }

    public void setSubsetList(List<Subset> subsetList) {
        if (subsetList == null) {
            thesaurusList = basedefiches;
        } else {
            MutableThesaurusList mutableThesaurusList = new MutableThesaurusList();
            for (Subset subset : subsetList) {
                mutableThesaurusList.add((Thesaurus) subset);
            }
            thesaurusList = mutableThesaurusList;
        }
    }

    public void setConditionGroup(ConditionGroup conditionGroup) {
        this.conditionGroup = conditionGroup;
    }

    public void init() {
        if (conditionGroup != null) {
            int thesaurusCount = thesaurusList.getThesaurusCount();
            SubsetKey[] thesaurusKeyArray = new SubsetKey[thesaurusCount];
            for (int i = 0; i < thesaurusCount; i++) {
                thesaurusKeyArray[i] = thesaurusList.getThesaurus(i).getSubsetKey();
            }
            MotcleSelectOptions motcleSelectOptions = new MotcleSelectOptions();
            motcleSelectOptions.setThesaurusKeyArray(thesaurusKeyArray);
            motcleSelectOptions.initConditionGroup(conditionGroup);
            motcleSelectOptions.setIdalphaOnly(false);
            MotcleSelector motcleSelector = new MotcleSelector(basedefiches, workingLangInteger);
            motcleSelector.addMotcleSelectOptions(motcleSelectOptions);
            result = selection(motcleSelector);
        } else {
            result = new ArrayList<Motcle>();
            int thesaurusCount = thesaurusList.getThesaurusCount();
            for (int i = 0; i < thesaurusCount; i++) {
                Thesaurus thesaurus = thesaurusList.getThesaurus(i);
                appendMotcle(thesaurus.getFirstLevelList());
            }
        }
    }

    private void appendMotcle(MotcleList list) {
        for (int i = 0; i < list.getMotcleCount(); i++) {
            Motcle motcle = list.getMotcle(i);
            result.add(motcle);
            appendMotcle(motcle.getChildList());
        }
    }

    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            int length = result.size();
            jsonWriter.key("length");
            jsonWriter.value(length);
            if (length > 0) {
                jsonWriter.key("array");
                jsonWriter.array();
                for (Motcle motcle : result) {
                    jsonWriter.object();
                    jsonWriter.key("intkey");
                    jsonWriter.value(motcle.getIdths());
                    String idalpha = motcle.getIdalpha();
                    if (idalpha != null) {
                        jsonWriter.key("idalphakey");
                        jsonWriter.value(idalpha);
                    }
                    String lib = motcle.getLibelleString(workingLangInteger, false);
                    if (lib != null) {
                        jsonWriter.key("text");
                        jsonWriter.value(lib);
                    }
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

    private Collection<Motcle> selection(MotcleSelector motcleSelector) {
        TreeSet<Motcle> treeSet = new TreeSet<Motcle>(new MotcleComparator(workingLangInteger));
        int thesaurusCount = thesaurusList.getThesaurusCount();
        for (int j = 0; j < thesaurusCount; j++) {
            Thesaurus thesaurus = thesaurusList.getThesaurus(j);
            int length = thesaurus.getTicketLength();
            for (int i = 0; i < length; i++) {
                Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(i);
                if (motcle == null) {
                    continue;
                }
                MotcleFilter filter = motcleSelector.getMotcleFilter(motcle);
                if (filter != null) {
                    treeSet.add(motcle);
                }
            }
        }
        return treeSet;
    }


    private static class MotcleComparator implements Comparator {

        private int langInteger;

        public MotcleComparator(int langInteger) {
            this.langInteger = langInteger;
        }

        public int compare(Object o1, Object o2) {
            Motcle mc1 = (Motcle) o1;
            Motcle mc2 = (Motcle) o2;
            String a1 = mc1.getIdalpha();
            String a2 = mc2.getIdalpha();
            String lib1 = mc1.getLibelleString(langInteger, true);
            String lib2 = mc2.getLibelleString(langInteger, true);
            if (a1 != null) {
                lib1 = a1 + " – " + lib1;
            }
            if (a2 != null) {
                lib2 = a2 + " – " + lib2;
            }
            int result = 0;
            result = lib1.compareTo(lib2);
            if (result != 0) {
                return result;
            }
            result = mc1.getThesaurus().getSubsetKey().compareTo(mc2.getThesaurus().getSubsetKey());
            if (result != 0) {
                return result;
            }
            int id1 = mc1.getIdths();
            int id2 = mc2.getIdths();
            if (id1 < id2) {
                return -1;
            }
            if (id1 > id2) {
                return 1;
            }
            return 0;
        }

    }

}
