/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.pioche.data;

import java.io.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.beans.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.basedefiches.selection.*;
import fr.exemole.bdfserver.servlets.pioche.*;
import java.util.*;
import net.mapeadores.basedefiches.selection.util.*;
import net.mapeadores.util.conditions.*;
import net.mapeadores.util.json.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurPiocheData implements JsonProducer {

    private Basedefiches basedefiches;
    private int workingLangInteger;
    private SphereList sphereList;
    private Collection<Redacteur> result;
    private ConditionGroup conditionGroup;
    private short style;
    private SubsetKey defaultSphereKey;

    public RedacteurPiocheData(Basedefiches basedefiches, int workingLangInteger) {
        this.basedefiches = basedefiches;
        this.workingLangInteger = workingLangInteger;
        sphereList = basedefiches;
    }

    public void setSubsetList(List<Subset> subsetList) {
        if (subsetList == null) {
            sphereList = basedefiches;
        } else {
            MutableSphereList mutableSphereList = new MutableSphereList();
            for (Subset subset : subsetList) {
                mutableSphereList.add((Sphere) subset);
            }
            sphereList = mutableSphereList;
        }
    }

    public void setConditionGroup(ConditionGroup conditionGroup) {
        this.conditionGroup = conditionGroup;
    }

    public void setDefaultSphereKey(SubsetKey defaultSphereKey) {
        this.defaultSphereKey = defaultSphereKey;
    }

    public void init() {
        style = PersonneBean.BIBLIO_STYLE;
        if (conditionGroup != null) {
            RedacteurSelectOptions redacteurSelectOptions = new RedacteurSelectOptions();
            redacteurSelectOptions.setNomcompletConditionGroup(conditionGroup);
            RedacteurSelector redacteurSelector = new RedacteurSelector(basedefiches, workingLangInteger);
            redacteurSelector.addRedacteurSelectOptions(redacteurSelectOptions);
            result = selection(redacteurSelector);
        } else {
            result = new ArrayList<Redacteur>();
            int sphereCount = sphereList.getSphereCount();
            for (int i = 0; i < sphereCount; i++) {
                Sphere corpus = sphereList.getSphere(i);
                int length = corpus.getTicketLength();
                for (int j = 0; j < length; j++) {
                    Redacteur redacteur = (Redacteur) corpus.getSubsetItemByTicket(j);
                    if (redacteur != null) {
                        result.add(redacteur);
                    }
                }
            }
        }
    }

    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            int length = result.size();
            jsonWriter.key("length");
            jsonWriter.value(length);
            if (length > 0) {
                jsonWriter.key("array");
                jsonWriter.array();
                for (Redacteur redacteur : result) {
                    RedacteurKey redacteurKey = redacteur.getRedacteurKey();
                    int intkey = redacteur.getIdsphere();
                    jsonWriter.object();
                    jsonWriter.key("intkey");
                    jsonWriter.value(intkey);
                    jsonWriter.key("idalphakey");
                    if ((defaultSphereKey != null) && (redacteurKey.getSphereKey().equals(defaultSphereKey))) {
                        jsonWriter.value(redacteurKey.getIdsphereString());
                    } else {
                        jsonWriter.value(redacteurKey.toString(RedacteurKey.BRACKET_STYLE));
                    }
                    PersonneBean personneBean = redacteur.getPersonneBean();
                    jsonWriter.key("text");
                    jsonWriter.value(personneBean.getNomComplet(style));
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

    private Collection selection(RedacteurSelector redacteurSelector) {
        TreeSet redacSet = new TreeSet(new RedacteurComparator());
        int sphereCount = sphereList.getSphereCount();
        for (int j = 0; j < sphereCount; j++) {
            Sphere sphere = sphereList.getSphere(j);
            int length = sphere.getTicketLength();
            for (int i = 0; i < length; i++) {
                Redacteur redacteur = (Redacteur) sphere.getSubsetItemByTicket(i);
                if (redacteur == null) {
                    continue;
                }
                if (!redacteur.isActif()) {
                    continue;
                }
                if (redacteurSelector.isSelected(redacteur)) {
                    redacSet.add(redacteur);
                }
            }
        }
        return redacSet;
    }


    private static class RedacteurComparator implements Comparator {

        public int compare(Object o1, Object o2) {
            Redacteur r1 = (Redacteur) o1;
            Redacteur r2 = (Redacteur) o2;
            String nc1 = r1.getPersonneBean().getNomComplet(PersonneBean.ANNUAIRE_UP_STYLE);
            String nc2 = r2.getPersonneBean().getNomComplet(PersonneBean.ANNUAIRE_UP_STYLE);
            int result = nc1.compareTo(nc2);
            if (result != 0) {
                return result;
            }
            String id1 = r1.getRedacteurKey().getIdsphereString();
            String id2 = r2.getRedacteurKey().getIdsphereString();
            result = id1.compareTo(id2);
            if (result != 0) {
                return result;
            }
            return r1.getRedacteurKey().getSphereKey().compareTo(r2.getRedacteurKey().getSphereKey());
        }

    }

}
