/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.session.htmlpages;

import fr.exemole.bdfserver.instruction.BdfRequestMap;
import fr.exemole.bdfserver.response.SessionResponseParameters;
import fr.exemole.bdfserver.session.LoginException;
import fr.exemole.bdfserver.storage.BdfResourceConstants;
import fr.exemole.bdfserver.storage.BdfResourceStorage;
import java.io.IOException;
import java.util.Enumeration;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadataUtils;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.text.UTF8Bridge;



/**
 *
 * @author Vincent Calame
 */
public class LoginPage extends SessionHtmlPage {

    private final static String FIELDSET_CLASS = "fs_LOGIN";
    private final static String TABLE_CLASS = "tb_LOGIN";
    private final static String COL_LIB_CLASS = "col_LOGIN_LIB";
    private final static String COL_COLON_CLASS = "col_LOGIN_COLON";
    private final static String COL_INPUT_CLASS = "col_LOGIN_INPUT";
    private final static String P_TEXTE_CLASS = "p_LOGIN_TEXTE";
    private final static String P_WARNING_CLASS = "p_LOGIN_WARNING";
    private final static String P_SOUSTITRE_CLASS = "p_LOGIN_SOUSTITRE";
    private final static String ID_LOGIN_FIELDSET = "ID_LOGIN_FIELDSET";
    private final static String ID_PARAM_FIELDSET = "ID_PARAM_FIELDSET";
    private HttpServletRequest httpServletRequest;
    private LoginException loginException;
    private String loginSubtitleKey;
    private String profondeurString;
    private String action;

    public LoginPage(SessionResponseParameters sessionResponseParameters) {
        super(sessionResponseParameters);
        this.profondeurString = sessionResponseParameters.getReversePath();
        this.httpServletRequest = (HttpServletRequest) sessionResponseParameters.getParameter(SessionResponseParameters.HTTPSERVLETREQUEST_PARAMNAME);
        this.loginException = (LoginException) sessionResponseParameters.getParameter(SessionResponseParameters.LOGINEXCEPTION_PARAMNAME);
        this.loginSubtitleKey = getLoginSubtitleKey();
        this.action = getAction();
    }

    private String getAction() {
        if (httpServletRequest == null) {
            return "Session";
        }
        return httpServletRequest.getRequestURL().toString();
    }

    private String getLoginSubtitleKey() {
        if (httpServletRequest == null) {
            return "session_subtitle_new";
        }
        if ((loginException != null) && (loginException.getLoginErrorKey().equals(LoginException.EXIT_ERROR))) {
            return "session_subtitle_new";
        }
        String servletPath = httpServletRequest.getServletPath();
        if ((servletPath.length() < 2) || (servletPath.equals("/Session"))) {
            return "session_subtitle_new";
        }
        String pathInfo = httpServletRequest.getPathInfo();
        if ((pathInfo != null) && (pathInfo.endsWith("/index.html"))) {
            return "session_subtitle_new";
        }
        if (Character.isUpperCase(servletPath.charAt(1))) {
            return "session_subtitle_expired";
        }
        return "session_subtitle_protected";
    }

    public String getIconPng() {
        return profondeurString + "rsc/icon.png";
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        if (bdfServer.getBdfResourceStorage().containsBdfResource(BdfResourceConstants.OTHER_RESOURCE, BdfResourceStorage.LOGO)) {
            HTMLAttributes imgAttributes = new HTMLAttributes();
            imgAttributes.setAlt("logo");
            imgAttributes.setId("LOGO");
            imgAttributes.setSrc(profondeurString + "rsc/logo.png");
            appendImg(imgAttributes);
        }
        startH1();
        escape(BasedefichesMetadataUtils.getLongIntituleString(bdfServer.getBasedefiches(), workingLangInteger));
        endH1();

        HTMLAttributes soustitreAttributes = new HTMLAttributes();
        soustitreAttributes.addClass(P_TEXTE_CLASS);
        soustitreAttributes.addClass(P_SOUSTITRE_CLASS);

        startP(soustitreAttributes);
        append(getLocalization(loginSubtitleKey));
        endP();

        String defaultSphere = "";
        String defaultIdsphere = "";
        if (loginException != null) {
            defaultSphere = loginException.getDefaultSphere();
            defaultIdsphere = loginException.getDefaultIdsphere();
        }

        HTMLAttributes formAttributes = new HTMLAttributes();
        formAttributes.setAction(action);
        formAttributes.setMethod("POST");
        startForm(formAttributes);

        HTMLAttributes fieldSetAttributes = new HTMLAttributes();
        fieldSetAttributes.setClasses(FIELDSET_CLASS);

        fieldSetAttributes.setId(ID_LOGIN_FIELDSET);
        startFieldset(fieldSetAttributes);
        startLegend(null);
        localize("login_lib_fieldset_identification");
        endLegend();
        startLignes();
        startLigne();
        printLibSpans("session_sphere");
        startInputSpan();
        appendTextInput(BdfRequestMap.BDF_SPHERE_PARAMETER, defaultSphere, -1);
        endInputSpan();
        endLigne();
        startLigne();
        printLibSpans("session_idsphere");
        startInputSpan();
        appendTextInput(BdfRequestMap.BDF_IDSPHERE_PARAMETER, defaultIdsphere, -1);
        endInputSpan();
        endLigne();
        startLigne();
        printLibSpans("session_motdepasse");
        startInputSpan();
        appendPasswordInput(BdfRequestMap.BDF_PASSWORD_PARAMETER, -1);
        endInputSpan();
        endLigne();
        endLignes();
        endFieldset();

        fieldSetAttributes.setId(ID_PARAM_FIELDSET);
        startFieldset(fieldSetAttributes);
        startLegend(null);
        localize("login_lib_fieldset_parametrage");
        endLegend();


        startLignes();

        startLigne();
        printLibSpans("login_lib_jquery");
        startInputSpan();
        append("&nbsp;");
        printScript();
        printNoScript();
        endInputSpan();
        endLigne();

        startLigne();
        printLibSpans("login_lib_screenwidth");
        startInputSpan();
        printScreenWidthSelect();
        endInputSpan();
        endLigne();

        endLignes();
        endFieldset();

        HTMLAttributes psubmitAttributes = new HTMLAttributes();
        psubmitAttributes.addClass(P_TEXTE_CLASS);
        startP(psubmitAttributes);
        addSubmit("session_valider", false);
        endP();

        HTMLAttributes hiddenAttributes = new HTMLAttributes();
        hiddenAttributes.setId("HIDDEN");
        startDiv(hiddenAttributes);
        if (httpServletRequest != null) {
            UTF8Bridge utf8Bridge = new UTF8Bridge(httpServletRequest.getCharacterEncoding());
            for (Enumeration enumeration = httpServletRequest.getParameterNames(); enumeration.hasMoreElements();) {
                String paramName = (String) enumeration.nextElement();
                if (!paramName.startsWith("bdf-")) {
                    String[] paramValues = httpServletRequest.getParameterValues(paramName);
                    if (paramValues != null) {
                        if (paramValues.length == 1) {
                            appendHiddenInput(paramName, utf8Bridge.toUTF8(paramValues[0]), true);
                        } else {
                            for (int i = 0; i < paramValues.length; i++) {
                                appendHiddenInput(paramName, utf8Bridge.toUTF8(paramValues[i]));
                            }
                        }
                    }
                }
            }
        }
        endDiv();

        endForm();

        if (loginException != null) {
            String errorKey = loginException.getLoginErrorKey();
            if ((!errorKey.equals(LoginException.UNDEFINED_ERROR)) && (!errorKey.equals(LoginException.EXIT_ERROR))) {
                HTMLAttributes warningAttributes = new HTMLAttributes();
                warningAttributes.addClass(P_TEXTE_CLASS);
                warningAttributes.addClass(P_WARNING_CLASS);
                startP(warningAttributes);
                localize("login_err_" + errorKey);
                endP();
            }
        }
        end();
    }

    public String getTitle() {
        return BasedefichesMetadataUtils.getLongIntituleString(bdfServer.getBasedefiches(), workingLangInteger);
    }

    public String getFormAction() {
        return null;
    }

    public String getCssThemeFile() {
        return "login.css";
    }

    private void printScreenWidthSelect() {
        HTMLAttributes selectAttributes = new HTMLAttributes();
        selectAttributes.setName(BdfRequestMap.BDF_SCREENWIDTH_PARAMETER);
        selectAttributes.setId("SCREENWIDTH");
        startSelect(selectAttributes);
        printScreenOption("800", false);
        printScreenOption("1024", true);
        printScreenOption("1280", false);
        endSelect();
    }

    private void printScreenOption(String value, boolean selected) {
        HTMLAttributes optionAttributes = new HTMLAttributes();
        optionAttributes.setSelected(selected);
        optionAttributes.setValue(value);
        startOption(optionAttributes);
        append(value);
        append("px");
        endOption();
    }

    private void printNoScript() {
        startNoscript();
        HTMLAttributes imgAttributes = new HTMLAttributes();
        imgAttributes.setSrc(profondeurString + "theme/icons/16x16/login_jquery_no.png");
        imgAttributes.setAlt(getLocalization("login_lib_jquery_no"));
        imgAttributes.setWidth("16");
        imgAttributes.setHeight("16");
        imgAttributes.setId("JQUERYTEST");
        imgAttributes.setTitle(getLocalization("login_lib_jquery_no"));
        appendImg(imgAttributes);
        endNoscript();
    }

    private void printScript() {
        HTMLAttributes scriptAttributes = new HTMLAttributes();
        startScript(scriptAttributes);
        append("\n //<!--\n document.write(\"<img id=\\\"JQUERYTEST\\\" src=\\\"");
        append(profondeurString);
        append("theme/icons/16x16/login_jquery_no.png\\\" title=\\\"");
        localize("login_lib_jquery_err");
        append("\\\" alt=\\\"");
        localize("login_lib_jquery_err");
        append("\\\">\");\n");
        append(" var jquery_ok = \"");
        localize("login_lib_jquery_ok");
        append("\";\n");
        append(" //-->\n");
        endScript();
        scriptAttributes.setSrc(profondeurString + "jquery/jquery.js");
        startScript(scriptAttributes);
        endScript();
        scriptAttributes.setSrc(profondeurString + "js/_testjquery.js");
        startScript(scriptAttributes);
        endScript();
    }

    private void startLignes() {
        HTMLAttributes tableAttributes = new HTMLAttributes();
        tableAttributes.addClass(TABLE_CLASS);
        startTable(tableAttributes);
        HTMLAttributes colAttributes = new HTMLAttributes();
        colAttributes.setClasses(COL_LIB_CLASS);
        appendCol(colAttributes);
        colAttributes.setClasses(COL_COLON_CLASS);
        appendCol(colAttributes);
        colAttributes.setClasses(COL_INPUT_CLASS);
        appendCol(colAttributes);
    }

    private void endLignes() {
        endTable();
    }

    private void startLigne() {
        startRow();
    }

    private void endLigne() {
        endRow();
    }

    private void printLibSpans(String libKey) {
        startCell();
        localize(libKey);
        endCell();
        startCell();
        append("&nbsp;:&nbsp;");
        endCell();
    }

    private void startInputSpan() {
        startCell();
    }

    private void endInputSpan() {
        endCell();
    }

    /*private final static String DIV_LIGNES_CLASS="d_LOGIN_LIGNES";
    private final static String DIV_LIGNE_CLASS="d_LOGIN_LIGNE";
    private final static String SPAN_LIB_CLASS="s_LOGIN_LIB";
    private final static String SPAN_COLON_CLASS="s_LOGIN_COLON";
    private final static String SPAN_INPUT_CLASS="s_LOGIN_INPUT";
    
    private void startLignes() {
    HTMLAttributes divAttributes = new HTMLAttributes();
    divAttributes.addClass(DIV_LIGNES_CLASS);
    startDiv(divAttributes);
    }
    
    private void endLignes() {
    endDiv();
    }
    
    private void startLigne() {
    HTMLAttributes divAttributes = new HTMLAttributes();
    divAttributes.addClass(DIV_LIGNE_CLASS);
    startDiv(divAttributes);
    }
    
    private void endLigne() {
    endDiv();
    }
    
    private void printLibSpans(String libKey) {
    startSpan(SPAN_LIB_CLASS);
    localize(libKey);
    endSpan();
    startSpan(SPAN_COLON_CLASS);
    append("&nbsp;:&nbsp;");
    endSpan();
    }
    
    private void startInputSpan() {
    startSpan(SPAN_INPUT_CLASS);
    }
    
    private void endInputSpan() {
    endSpan();
    }*/
}
