/* BasedefichesLib_API - Copyright (c) 2006-2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches;


/**
 * Clé permettant d'identifier un couple de deux sous-ensembles.
 * Les instances de SubsetCouple sont utilisées en particulier pour les croisements.
 *
 * @author Vincent Calame
 */
public final class SubsetCouple {

    private SubsetKey subsetKey1;
    private SubsetKey subsetKey2;
    private boolean unique;

    public SubsetCouple(SubsetKey subsetKey1, SubsetKey subsetKey2) {
        if (subsetKey1.compareTo(subsetKey2) > 0) {
            this.subsetKey2 = subsetKey1;
            this.subsetKey1 = subsetKey2;
        } else {
            unique = (subsetKey1.equals(subsetKey2));
            this.subsetKey1 = subsetKey1;
            this.subsetKey2 = subsetKey2;
        }
    }

    /**
     * Indique que le couple désigne la relation d'un sous-ensemble avec lui-même. 
     */
    public boolean isUnique() {
        return unique;
    }

    public SubsetKey getSubsetKey1() {
        return subsetKey1;
    }

    public SubsetKey getSubsetKey2() {
        return subsetKey2;
    }

    /**
     *@throws IllegalArgumentException si subsetKey n'appartient pas au couple
     */
    public SubsetKey getOtherSubsetKey(SubsetKey subsetKey) {
        if (subsetKey.equals(subsetKey1)) {
            return subsetKey2;
        } else if (subsetKey.equals(subsetKey2)) {
            return subsetKey1;
        }
        throw new IllegalArgumentException("SubsetKey argument is not a part of the subsetCouple : " + subsetKey.toString());
    }

    public int getSubsetNumber(SubsetKey subsetKey) {
        if (subsetKey.equals(subsetKey1)) {
            return 1;
        } else if (subsetKey.equals(subsetKey2)) {
            return 2;
        } else {
            return 0;
        }
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof SubsetCouple)) {
            return false;
        }
        SubsetCouple other = (SubsetCouple) obj;
        return ((other.subsetKey1.equals(this.subsetKey1)) && (other.subsetKey2.equals(this.subsetKey2)));
    }

    @Override
    public int hashCode() {
        return subsetKey1.hashCode() + subsetKey2.hashCode();
    }

    /**
     * Indique si le couple est entre deux corpus;
     */
    public boolean isCorpusCorpusCouple() {
        if (!subsetKey1.isCorpusSubset()) {
            return false;
        }
        if (!subsetKey2.isCorpusSubset()) {
            return false;
        }
        return true;
    }

    /**
     * Indique si le couple est entre un corpus et un thésaurus;
     */
    public boolean isCorpusThesaurusCouple() {
        if (!subsetKey1.isCorpusSubset()) {
            return false;
        }
        if (!subsetKey2.isThesaurusSubset()) {
            return false;
        }
        return true;
    }

    /**
     * Indique si le couple est entre un corpus et une sphère;
     */
    public boolean isCorpusSphereCouple() {
        if (!subsetKey1.isCorpusSubset()) {
            return false;
        }
        if (!subsetKey2.isSphereSubset()) {
            return false;
        }
        return true;
    }

    /**
     * Indique si le couple est entre un corpus et les documents;
     */
    public boolean isCorpusDocumentCouple() {
        if (!subsetKey1.isCorpusSubset()) {
            return false;
        }
        if (!subsetKey2.equals(SubsetKey.DOCUMENT_SUBSET)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return subsetKey1.toString() + " / " + subsetKey2.toString();
    }

}
