/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.metadata;

import net.mapeadores.util.text.StringUtils;
import java.util.Map;
import java.util.HashMap;
import java.text.ParseException;
import java.io.Serializable;


/**
 *<p>Identifie un champ d'un corpus. Il y a quatre types de champs :
 * <ul>
 *<li>Les champs spéciaux au nombre de six : IDCORPUS, LANG, REDACTEURS, TITRE, SOUTITRE et TEXTE.
 * Les quatre premiers sont obligatoires et doivent toujours se retrouver dans un corpus, SOUSTITRE et TEXTE sont facultatifs.
 *<li>Les chams Propriété, situés dans l'en-tête de la fiche ils contiennent au maximum une valeur de type FicheItem.
 *<li>Les champs Annexe situés dans le corps de fiche, qui représentent des blocs de texte complémentaires
 *<li>Les champs Informations, situés dans la source et qui contiennent zéro, une ou plusieurs valeurs de type FicheItem
 *</ul>
 *
 * @author Vincent Calame
 */
public final class FieldKey implements Serializable {

    private static final Map<String, FieldKey> internMap = new HashMap<String, FieldKey>();
    public static final short SPECIAL_CATEGORY = 0;
    public static final short PROPRIETE_CATEGORY = 1;
    public static final short ANNEXE_CATEGORY = 2;
    public static final short INFORMATION_CATEGORY = 3;
    private String fieldKeyString;
    private short category = 0;
    private String typeString = "";
    public final static FieldKey TITRE = new FieldKey("titre");
    public final static FieldKey SOUSTITRE = new FieldKey("soustitre");
    public final static FieldKey REDACTEURS = new FieldKey("redacteurs");
    public final static FieldKey TEXTE = new FieldKey("texte");
    public final static FieldKey IDCORPUS = new FieldKey("idcorpus");
    public final static FieldKey LANG = new FieldKey("lang");

    private FieldKey(String fieldKeyString) {
        this.fieldKeyString = fieldKeyString.intern();
        this.category = SPECIAL_CATEGORY;
        intern(this);
    }

    private FieldKey(String fieldKeyString, String typeString, short category) {
        this.fieldKeyString = fieldKeyString.intern();
        this.typeString = typeString.intern();
        this.category = category;
        intern(this);
    }

    private static FieldKey initFieldKey(String fieldKeyString, String typeString, short category) {
        FieldKey current = internMap.get(fieldKeyString);
        if (current != null) {
            return current;
        }
        return new FieldKey(fieldKeyString, typeString, category);
    }

    private synchronized static void intern(FieldKey fieldKey) {
        internMap.put(fieldKey.fieldKeyString, fieldKey);
    }

    /**
     *
     *@throws ParseException si s est incorrect
     */
    public static FieldKey parse(String s) throws ParseException {
        FieldKey current = internMap.get(s);
        if (current != null) {
            return current;
        }
        if (s.startsWith("propriete_")) {
            return new FieldKey(s, getTypeString(s, 10), PROPRIETE_CATEGORY);
        }
        if (s.startsWith("annexe_")) {
            return new FieldKey(s, getTypeString(s, 7), ANNEXE_CATEGORY);
        }
        if (s.startsWith("information_")) {
            return new FieldKey(s, getTypeString(s, 12), INFORMATION_CATEGORY);
        }
        throw new ParseException("unkown fieldkey", 0);
    }

    public static FieldKey parse(short category, String typeString) throws ParseException {
        StringUtils.checkTechnicalName(typeString, false);
        return new FieldKey(categoryToString(category) + "_" + typeString, typeString, category);
    }

    /**
     * Équivalent de parse() mais en envoyant IllegalArgumentException plutôt que ParseException.
     * à utiliser quand on est sûr de la syntaxe et permet d'éviter un try {} catch {}
     */
    public static FieldKey build(String s) {
        try {
            return parse(s);
        } catch (ParseException pe) {
            throw new IllegalArgumentException(pe.getMessage());
        }
    }

    /**
     * Équivalent de parse() mais en envoyant IllegalArgumentException plutôt que ParseException.
     * à utiliser quand on est sûr de la syntaxe et permet d'éviter un try {} catch {}
     */
    public static FieldKey build(short category, String s) {
        try {
            return parse(category, s);
        } catch (ParseException pe) {
            throw new IllegalArgumentException(pe.getMessage());
        }
    }

    private static String getTypeString(String s, int index) throws ParseException {
        String typeString = s.substring(index);
        StringUtils.checkTechnicalName(typeString, false);
        return typeString;
    }

    @Override
    public String toString() {
        return fieldKeyString;
    }

    public boolean isSpecialFieldKey() {
        return (category == SPECIAL_CATEGORY);
    }

    public boolean isProprieteFieldKey() {
        return (category == PROPRIETE_CATEGORY);
    }

    public boolean isAnnexeFieldKey() {
        return (category == ANNEXE_CATEGORY);
    }

    public boolean isInformationFieldKey() {
        return (category == INFORMATION_CATEGORY);
    }

    public short getCategory() {
        return category;
    }

    /**
     * Retourne <em>true</em> si le champ est obligatoire dans une fiche.
     */
    public boolean isMandatoryFieldKey() {
        if (!isSpecialFieldKey()) {
            return false;
        }
        if (this.equals(SOUSTITRE)) {
            return false;
        }
        if (this.equals(TEXTE)) {
            return false;
        }
        return true;
    }

    public String getTypeString() {
        return typeString;
    }

    @Override
    public int hashCode() {
        return fieldKeyString.hashCode();
    }

    public boolean ownsToCorpsdefiche() {
        if (category == ANNEXE_CATEGORY) {
            return true;
        }
        if (category != SPECIAL_CATEGORY) {
            return false;
        }
        return fieldKeyString.equals("texte");
    }

    public boolean ownsToEntete() {
        if (category == PROPRIETE_CATEGORY) {
            return true;
        }
        if (category == INFORMATION_CATEGORY) {
            return true;
        }
        if (category != SPECIAL_CATEGORY) {
            return false;
        }
        return (fieldKeyString.equals("titre") || fieldKeyString.equals("soustitre") || (fieldKeyString.equals("redacteurs")));
    }

    public boolean isUniqueValueField() {
        if (category == PROPRIETE_CATEGORY) {
            return true;
        }
        if (category == ANNEXE_CATEGORY) {
            return false;
        }
        if (category == INFORMATION_CATEGORY) {
            return false;
        }
        return (!fieldKeyString.equals("redacteurs")) && (!fieldKeyString.equals("texte"));
    }

    /**
     * Fait le test sur propriété et information. Fait également le test sur
     * Rédacteur et Langue suivant la valeur de mask.
     */
    public boolean isFicheItemField(int mask) {
        if (category == PROPRIETE_CATEGORY) {
            return true;
        }
        if (category == INFORMATION_CATEGORY) {
            return true;
        }
        if (category != SPECIAL_CATEGORY) {
            return false;
        }
        if (fieldKeyString.equals("redacteurs")) {
            return (mask & 1) != 0;
        }
        if (fieldKeyString.equals("lang")) {
            return (mask & 2) != 0;
        }
        return false;
    }

    /**
     *Fait seulement des tests sur la nullité et la longueur, ne fait pas appel à StringUtils.checkTechnicalName().
     *
     *@throws IllegalArgumentException si type est nul ou vide
     */
    public static FieldKey getAnnexeFieldKey(String type) {
        if ((type == null) || (type.length() == 0)) {
            throw new IllegalArgumentException();
        }
        return initFieldKey("annexe_" + type, type, ANNEXE_CATEGORY);
    }

    /**
     *Fait seulement des tests sur la nullité et la longueur, ne fait pas appel à StringUtils.checkTechnicalName().
     *
     *@throws IllegalArgumentException si type est nul ou vide
     */
    public static FieldKey getProprieteFieldKey(String type) {
        if ((type == null) || (type.length() == 0)) {
            throw new IllegalArgumentException();
        }
        return initFieldKey("propriete_" + type, type, PROPRIETE_CATEGORY);
    }

    /**
     *Fait seulement des tests sur la nullité et la longueur, ne fait pas appel à StringUtils.checkTechnicalName().
     *
     *@throws IllegalArgumentException si type est nul ou vide
     */
    public static FieldKey getInformationFieldKey(String type) {
        if ((type == null) || (type.length() == 0)) {
            throw new IllegalArgumentException();
        }
        return initFieldKey("information_" + type, type, INFORMATION_CATEGORY);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof FieldKey)) {
            return false;
        }
        FieldKey fk = (FieldKey) obj;
        return fk.fieldKeyString.equals(this.fieldKeyString);
    }

    /**
     * Retourne une expression sous forme de chaîne de la catégorie correspondante.
     *
     *@throws IllegalArgumentException si la valeur de la catégorie est incorrecte
     */
    public static String categoryToString(short category) {
        switch (category) {
            case SPECIAL_CATEGORY:
                return "special";
            case PROPRIETE_CATEGORY:
                return "propriete";
            case ANNEXE_CATEGORY:
                return "annexe";
            case INFORMATION_CATEGORY:
                return "information";
            default:
                throw new IllegalArgumentException("wrong Field category value");
        }
    }

    /**
     * Retourne une expression sous forme de chaîne de la catégorie correspondante.
     *
     *@throws IllegalArgumentException si la valeur de la catégorie est incorrecte
     */
    public static short categoryToShort(String s) {
        if (s.equals("special")) {
            return SPECIAL_CATEGORY;
        }
        if (s.equals("propriete")) {
            return PROPRIETE_CATEGORY;
        }
        if (s.equals("annexe")) {
            return ANNEXE_CATEGORY;
        }
        if (s.equals("information")) {
            return INFORMATION_CATEGORY;
        }
        throw new IllegalArgumentException("Wrong category string");
    }

}
