/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.metadata;


/**
 * Cette classe décrit les options qu'applique obligatoirement une implémentation de BaseDeFiches
 * Cette classe ne contient que des méthodes statiques.
 *
 * @author Vincent Calame
 */
public class FieldOptions {

    /**
     * Option propre au champ Information (et obligatoire pour ce champ) indiquant si les éléments d'information sont
     * de « en ligne « ou des « blocs ». L'objet doit être une instance d'Integer et avoir comme valeur INLINE_DISPLAY ou BLOCK_DISPLAY.
     */
    public final static String INFORMATIONDISPLAY_OPTION = "informationDisplay";
    /**
     * Option propre aux champs Propriété et Information qui sont de type Personne. Indique la sphère à utilisateur par défaut.
     * L'objet doit être une instance de SubsetKey ou null, indiquant ainsi que c'est la sphère de l'utilisateur qui doit être utilisée.
     */
    public final static String DEFAULTSPHEREKEY_OPTION = "defaultSphereKey";
    /**
     * Option propre aux champs Propriété qui indique que ce champ doit être décomposé en sous-champs
     * Les types acceptés sont « personne » et « geopoint »
     * La valeur renvoyée doit être Boolean.true ou null
     */
    public final static String SUBFIELDDISPLAY_OPTION = "subfieldDisplay";
    public final static String CURRENCYARRAY_OPTION = "currencyArray";
    /**
     * Option indiquant que le titre doit être généré à partir de la valeur d'autres champs.
     * La valeur est null si le titre ne doit pas être généré et sinon une instance de FicheFormatDef
     */
    public final static String TITREGENERATIONFORMAT_OPTION = "titreGenerationFormat";
    /**
     * Option générale au corpus (non liée à un champ particulier) indiquant l'existence d'un champ de géolocalisation. 
     */
    public final static String GEOLOCALISATIONFIELD_OPTION = "geolocalisationField";
    public final static String ADDRESSFIELDARRAY_OPTION = "addressFieldArray";
    public final static String BASEURL_OPTION = "baseUrl";
    public final static String IMAGESOURCE_OPTION = "imageSource";
    public final static short INLINE_DISPLAY = 1;
    public final static short BLOCK_DISPLAY = 2;
    /**
     * Option propre aux champs Propriété (pas encore Information) qui sont de type Image.
     */
    public final static String ALBUMKEY_OPTION = "albumKey";

    private FieldOptions() {
    }

    /**
     * Retourne la valeur de l'option INFORMATIONDISPLAY_OPTION qui doit être utilisée par défaut (il s'agit de INLINE_DISPLAY).
     */
    public static short getDefaultInformationDisplay() {
        return INLINE_DISPLAY;
    }

    /**
     *
     *@throws IllegalArgumentException
     */
    public static String informationDisplayToString(short informationDisplay) {
        switch (informationDisplay) {
            case INLINE_DISPLAY:
                return "inline";
            case BLOCK_DISPLAY:
                return "block";
            default:
                throw new IllegalArgumentException("wrong value");
        }
    }

    /**
     *
     *@throws IllegalArgumentException
     */
    public static short informationDisplayToShort(String informationDisplay) {
        if (informationDisplay.length() == 0) {
            return getDefaultInformationDisplay();
        }
        if (informationDisplay.equals("inline")) {
            return INLINE_DISPLAY;
        }
        if (informationDisplay.equals("block")) {
            return BLOCK_DISPLAY;
        }
        throw new IllegalArgumentException("wrong value");
    }

    public static boolean haveSubfield(int ficheItemType) {
        switch (ficheItemType) {
            case CorpusField.PERSONNE_FIELD:
            case CorpusField.GEOPOINT_FIELD:
            case CorpusField.IMAGE_FIELD:
                return true;
            default:
                return false;
        }
    }

    public static boolean testFieldOption(String optionName, CorpusField corpusField) {
        try {
            checkFieldOption(optionName, corpusField.getFieldKey(), corpusField.getFicheItemType());
            return true;
        } catch (FieldOptionException foe) {
            return false;
        }
    }

    public static void checkFieldOption(String optionName, FieldKey fieldKey, int ficheItemType) throws FieldOptionException {
        if (optionName.equals(FieldOptions.INFORMATIONDISPLAY_OPTION)) {
            if (!fieldKey.isInformationFieldKey()) {
                throw new FieldOptionException("informationDisplay Option applies only to Information Field");
            }
        } else if (optionName.equals(FieldOptions.DEFAULTSPHEREKEY_OPTION)) {
            if (ficheItemType != CorpusField.PERSONNE_FIELD) {
                throw new FieldOptionException("ficheItemType must be PERSONNE_FIELD");
            }
        } else if (optionName.equals(FieldOptions.SUBFIELDDISPLAY_OPTION)) {
            if (!fieldKey.isProprieteFieldKey()) {
                throw new FieldOptionException("mainPersonne Option applies only to Propriete Field");
            }
            if (!FieldOptions.haveSubfield(ficheItemType)) {
                throw new FieldOptionException("wrong ficheItemType = " + ficheItemType);
            }
        } else if (optionName.equals(FieldOptions.CURRENCYARRAY_OPTION)) {
            if (!fieldKey.isProprieteFieldKey()) {
                throw new FieldOptionException("currencyArray Option applies only to Propriete Field");
            }
            if (ficheItemType != CorpusField.MONTANT_FIELD) {
                throw new FieldOptionException("ficheItemType must be MONTANT_FIELD");
            }
        } else if (optionName.equals(FieldOptions.ADDRESSFIELDARRAY_OPTION)) {
            if (!fieldKey.isProprieteFieldKey()) {
                throw new FieldOptionException("addressFields Option applies only to Propriete Field");
            }
            if (ficheItemType != CorpusField.GEOPOINT_FIELD) {
                throw new FieldOptionException("ficheItemType must be GEOPOINT_FIELD");
            }
        } else if (optionName.equals(FieldOptions.BASEURL_OPTION)) {
            if (!fieldKey.isProprieteFieldKey()) {
                throw new FieldOptionException("baseUrl Option applies only to Propriete Field");
            }
            if (ficheItemType != CorpusField.IMAGE_FIELD) {
                throw new FieldOptionException("ficheItemType must be IMAGE_FIELD");
            }
        } else if (optionName.equals(FieldOptions.IMAGESOURCE_OPTION)) {
            if (!fieldKey.isProprieteFieldKey()) {
                throw new FieldOptionException("imageSource Option applies only to Propriete Field");
            }
            if (ficheItemType != CorpusField.IMAGE_FIELD) {
                throw new FieldOptionException("ficheItemType must be IMAGE_FIELD");
            }
        } else if (optionName.equals(FieldOptions.ALBUMKEY_OPTION)) {
            if (!fieldKey.isProprieteFieldKey()) {
                throw new FieldOptionException("imageSource Option applies only to Propriete Field");
            }
            if (ficheItemType != CorpusField.IMAGE_FIELD) {
                throw new FieldOptionException("ficheItemType must be IMAGE_FIELD");
            }
        }
    }

}
