/* BasedefichesLib_API - Copyright (c) 2006-2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.croisement;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import java.util.SortedSet;
import java.util.TreeSet;


/**
 *
 * @author Vincent Calame
 */
public class CroisementUtils {

    public static CroisementItem EMPTY_CROISEMENTITEM = new EmptyCroisementItem();

    private CroisementUtils() {
    }

    public static Croisement[] getCroisementWithCorpusArray(Thesaurus thesaurus) {
        Basedefiches basedefiches = thesaurus.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        Croisement[] croisements = new Croisement[corpusCount];
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            Croisement croisement = basedefiches.getCroisement(corpus.getSubsetKey(), thesaurus.getSubsetKey());
            croisements[i] = croisement;
        }
        return croisements;
    }

    public static CroisementItem getSingletonCroisementItem(int ticket, int poids) {
        return new SingletonCroisementItem(ticket, poids);
    }

    public static boolean hasAssociation(Document document) {
        Basedefiches basedefiches = document.getDocumentSubset().getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            SubsetKey corpusKey = corpus.getSubsetKey();
            Croisement croisement = basedefiches.getCroisement(corpusKey, SubsetKey.DOCUMENT_SUBSET);
            int count = croisement.getCroisementItem(SubsetKey.DOCUMENT_SUBSET, document.getTicket()).getCount();
            if (count != 0) {
                return true;
            }
        }
        return false;
    }

    public static boolean areCrossed(AbregeFiche abregeFiche, Document document) {
        Basedefiches basedefiches = abregeFiche.getCorpus().getBasedefiches();
        SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
        Croisement croisement = basedefiches.getCroisement(corpusKey, SubsetKey.DOCUMENT_SUBSET);
        int crossing = croisement.getPoids(corpusKey, abregeFiche.getTicket(), SubsetKey.DOCUMENT_SUBSET, document.getTicket());
        return (crossing != 0);
    }

    public static boolean isUniqueCroisement(AbregeFiche abregeFiche, Document document) {
        Basedefiches basedefiches = abregeFiche.getCorpus().getBasedefiches();
        SubsetKey documentKey = SubsetKey.DOCUMENT_SUBSET;
        SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
        Croisement croisement = basedefiches.getCroisement(corpusKey, documentKey);
        CroisementItem croisementItem = croisement.getCroisementItem(documentKey, document.getTicket());
        if (croisementItem.getCount() != 1) {
            return false;
        }
        int ticket = croisementItem.getTicket(0);
        return (ticket == abregeFiche.getTicket());
    }

    public static CroisementItem sortByPoids(CroisementItem croisementItem) {
        int count = croisementItem.getCount();
        if (count < 2) {
            return croisementItem;
        }
        SortedSet<SortBundle> treeSet = new TreeSet<SortBundle>();
        for (int i = 0; i < count; i++) {
            SortBundle sortBundle = new SortBundle(croisementItem.getTicket(i), croisementItem.getPoids(i), i);
            treeSet.add(sortBundle);
        }
        int p = 0;
        int[] poidsArray = new int[count];
        int[] ticketArray = new int[count];
        for (SortBundle sortBundle : treeSet) {
            poidsArray[p] = sortBundle.getPoids();
            ticketArray[p] = sortBundle.getTicket();
            p++;
        }
        return new ArrayCroisementItem(ticketArray, poidsArray);
    }

    public static CroisementList toCroisementList(SubsetKey subsetKey, CroisementItem croisementItem) {
        return new CroisementItemToCroisementList(subsetKey, croisementItem);
    }


    private static class EmptyCroisementItem implements CroisementItem {

        public int getCount() {
            return 0;
        }

        public int getPoids(int i) {
            throw new IndexOutOfBoundsException();
        }

        public int getTicket(int i) {
            throw new IndexOutOfBoundsException();
        }

        public int indexOfTicket(int ticket) {
            return -1;
        }

    }


    private static class SingletonCroisementItem implements CroisementItem {

        int ticket;
        int poids;

        SingletonCroisementItem(int ticket, int poids) {
            this.ticket = ticket;
            this.poids = poids;
        }

        public int getCount() {
            return 1;
        }

        public int getPoids(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException("i != 0");
            }
            return poids;
        }

        public int getTicket(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException("i != 0");
            }
            return ticket;
        }

        public int indexOfTicket(int ticket) {
            if (this.ticket == ticket) {
                return 0;
            }
            return -1;
        }

    }


    private static class ArrayCroisementItem implements CroisementItem {

        private int[] poidsArray;
        private int[] ticketArray;

        ArrayCroisementItem(int[] ticketArray, int[] poidsArray) {
            this.ticketArray = ticketArray;
            this.poidsArray = poidsArray;
        }

        public int getCount() {
            return ticketArray.length;
        }

        public int getPoids(int i) {
            return poidsArray[i];
        }

        public int getTicket(int i) {
            return ticketArray[i];
        }

        public int indexOfTicket(int ticket) {
            for (int i = 0; i < ticketArray[i]; i++) {
                if (ticketArray[i] == ticket) {
                    return i;
                }
            }
            return -1;
        }

    }


    private static class SortBundle implements Comparable {

        private int poids;
        private int position;
        private int ticket;

        SortBundle(int ticket, int poids, int position) {
            this.ticket = ticket;
            this.position = position;
            this.poids = poids;
        }

        @Override
        public int hashCode() {
            return ticket;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (!(obj instanceof SortBundle)) {
                return false;
            }
            SortBundle other = (SortBundle) obj;
            return (other.ticket == this.ticket);
        }

        public int compareTo(Object obj) {
            SortBundle other = (SortBundle) obj;
            if (other.poids > this.poids) {
                return -1;
            }
            if (other.poids < this.poids) {
                return 1;
            }
            if (other.position > this.position) {
                return -1;
            }
            if (other.position < this.position) {
                return 1;
            }
            if (other.ticket > this.ticket) {
                return -1;
            }
            if (other.ticket < this.ticket) {
                return 1;
            }
            return 0;
        }

        public int getPoids() {
            return poids;
        }

        public int getTicket() {
            return ticket;
        }

    }


    private static class CroisementItemToCroisementList implements CroisementList {

        private SubsetKey subsetKey;
        private CroisementItem croisementItem;

        private CroisementItemToCroisementList(SubsetKey subsetKey, CroisementItem croisementItem) {
            this.subsetKey = subsetKey;
            this.croisementItem = croisementItem;
        }

        public int getSubsetItemCount() {
            return croisementItem.getCount();
        }

        public int getTicket(int index) {
            return croisementItem.getTicket(index);
        }

        public int getPoids(int index) {
            return croisementItem.getPoids(index);
        }

        public SubsetKey getSubsetKey() {
            return subsetKey;
        }

    }

}
