/* BasedefichesLib_API - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.sphere;

import net.mapeadores.util.primitives.IntKey;
import java.text.ParseException;


/**
 * <p>Les identifiants de sphère ne sont pas désignés par des objets particuliers parce qu'ils sont exprimés sous forme d'entier.
 * Un identifiant de sphère peut être soit un entier positif soit une chaîne de 5 caractères au plus avec comme seuls caractères acceptés ceux de a à z, de A à Z et 0 à 9.
 * Cette limitation à cinq caractères permet d'exprimer cette chaîne sous forme d'un entier strictement négatif.
 * Les classes statiques de cette classe permettent de passer de la forme entier à la forme châine et vice-versa. </p>
 *<p>Les caractères sont codés sur 30 bits.
 * * La valeur de datation est entier, soit une FFFF FFFF
 * premier caractère :         0011 1111 0000 0000 0000 0000 0000 0000 (0x3F000000)
 * deuxième caractère :        0000 0000 1111 1100 0000 0000 0000 0000 (0xFC0000)
 * trosième caractère :        0000 0000 0000 0011 1111 0000 0000 0000 (0x3F000)
 * quatrième caractère :       0000 0000 0000 0000 0000 1111 1100 0000 (0xFC0)
 * cinquième caractère :       0000 0000 0000 0000 0000 0000 0011 1111 (0x3F)
 *
 *
 * @author Vincent Calame
 */
public class Idsphere extends IntKey {

    /* Aucune instance n'est possible */
    private Idsphere() {
    }

    /**
     * Le nombre de cas où l'entier est incorrect est très faible.
     */
    public static boolean checkIdsphere(int idsphere) {
        if (idsphere >= 0) {
            return true;
        }
        if (idsphere == 0x80000000) {
            return false;
        }
        if ((idsphere & 0x40000000) == 0x40000000) {
            return false;
        }
        boolean fin = false;
        for (int i = 0; i < 5; i++) {
            char carac = getChar(idsphere, i);
            if (carac == '\00') {
                return false;
            }
            if (carac == '\03') {
                fin = true;
            } else if (fin) {
                return false;
            }
        }
        return true;
    }

    /**
     * La méthode est laxiste puisqu'un résultat correct est renvoyé même si checkIdsphere retourne false.
     */
    public static String toString(int idsphere) {
        if (idsphere >= 0) {
            return String.valueOf(idsphere);
        }
        int p = 0;
        char[] resultat = new char[5];
        for (int i = 0; i < 5; i++) {
            char carac = getChar(idsphere, i);
            if (carac <= '\03') {
                if (i == 0) {
                    return "";
                } else {
                    return new String(resultat, 0, i);
                }
            }
            resultat[i] = carac;
        }
        return new String(resultat);
    }

    public static int parse(String strg_idsphere) throws ParseException {
        if (strg_idsphere == null) {
            throw new ParseException("strg_idsphere is null", 0);
        }
        int taille = strg_idsphere.length();
        if (taille == 0) {
            throw new ParseException("strg_idsphere length is null", 0);
        }
        if (taille > 5) {
            try {
                int idsphere = Integer.parseInt(strg_idsphere);
                if (idsphere < 0) {
                    throw new ParseException("begin by minus character", 0);
                }
                return idsphere;
            } catch (NumberFormatException nfe) {
                throw new ParseException("not integer", 0);
            }
        }
        char carac1 = strg_idsphere.charAt(0);
        if ((carac1 >= 48) && (carac1 <= 57)) {
            try {
                int idsphere = Integer.parseInt(strg_idsphere);
                if (idsphere < 0) {
                    throw new ParseException("begin by minus character", 0);
                }
                return idsphere;
            } catch (NumberFormatException nfe) {
                throw new ParseException("not integer", 0);
            }
        }
        if (carac1 == '-') {
            throw new ParseException("begin by minus character", 0);
        }
        int idsphere = 0x80000000;
        for (int i = 0; i < taille; i++) {
            int sixbits = toSixbits(strg_idsphere.charAt(i));
            if (sixbits == -1) {
                throw new ParseException("wrong character : " + strg_idsphere.charAt(i), i);
            }
            idsphere = putSixbits(idsphere, sixbits, i);
        }
        return idsphere;
    }

    private static char toChar(int sixbits) {
        if (sixbits == 0) {
            return '\03';
        }
        if (sixbits <= 10) {
            return (char) (sixbits + 47);
        }
        if (sixbits <= 36) {
            return (char) (sixbits + 54);
        }
        if (sixbits <= 62) {
            return (char) (sixbits + 60);
        }
        return '\00';
    }

    /*
     * Note : méthode utilisée également par RedacteurBaseKey
     */
    static int toSixbits(char c) {
        int sixbits = 0;
        if ((c >= 48) && (c <= 57)) {
            return (int) (c - 47);
        } else if ((c >= 65) && (c <= 90)) {
            return (int) (c - 54);
        } else if ((c >= 97) && (c <= 122)) {
            return (int) (c - 60);
        }
        return -1;
    }

    private static int putSixbits(int idsphere, int sixbits, int position) {
        switch (position) {
            case 0:
                return (idsphere | (sixbits << 24));
            case 1:
                return (idsphere | (sixbits << 18));
            case 2:
                return (idsphere | (sixbits << 12));
            case 3:
                return (idsphere | (sixbits << 6));
            case 4:
                return (idsphere | sixbits);
            default:
                return idsphere;
        }
    }

    private static char getChar(int idsphere, int position) {
        switch (position) {
            case 0:
                return toChar((idsphere & 0x3F000000) >>> 24);
            case 1:
                return toChar((idsphere & 0xFC0000) >>> 18);
            case 2:
                return toChar((idsphere & 0x3F000) >>> 12);
            case 3:
                return toChar((idsphere & 0xFC0) >>> 6);
            case 4:
                return toChar((idsphere & 0x3F));
            default:
                return '\00';
        }
    }

}
