/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.thesaurus;

import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesEditor;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.croisement.CroisementEditor;
import net.mapeadores.basedefiches.croisement.DefaultFicheCroisementList;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.IdalphaUtils;
import java.text.ParseException;
import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;
import java.io.IOException;


/**
 * Classe d'utilitaires pour les thésaurus. Cette classe ne comprend que des méthodes statiques.
 * <p>Outre leur caractère utilitaire, certaines méthodes font des vérifications
 * de la validité de l'implémentation. Elles renvoient alors une exception ImplementationException
 *
 * @author Vincent Calame
 */
public class ThesaurusUtils implements ThesaurusConstants {

    private ThesaurusUtils() {
    }

    /**
     *
     *@throws IllegalArgumentException si idalpha est nul ou de longueur nulle
     *@throws ParseException si idalpha possède des caractères incorrects.
     */
    public static void testIdalpha(String idalpha) throws ParseException {
        if (idalpha == null) {
            throw new IllegalArgumentException("idalpha is null");
        }
        if (idalpha.length() == 0) {
            throw new IllegalArgumentException("idalpha is empty");
        }
        int length = idalpha.length();
        for (int i = 0; i < length; i++) {
            char carac = idalpha.charAt(i);
            if (carac == ' ') {
                if (i == 0) {
                    throw new ParseException("starting with space : " + idalpha, i);
                }
                if (i == length - 1) {
                    throw new ParseException("idalpha ending with space : " + idalpha, i);
                }
                if (idalpha.charAt(i - 1) == ' ') {
                    throw new ParseException("two following spaces (idalpha : " + idalpha, i);
                }
            } else if (!IdalphaUtils.isValidNoSpaceChar(carac)) {
                throw new ParseException("Wrong character = " + carac + " (idalpha=" + idalpha + ")", i);
            }
        }
    }

    /**
     * Équivalent de testIdalpha mais retourne un booléen plutôt que des exceptions.
     */
    public static boolean isValidIdalpha(String idalpha) {
        try {
            testIdalpha(idalpha);
            return true;
        } catch (ParseException pe) {
            return false;
        } catch (IllegalArgumentException iae) {
            return false;
        }
    }

    /**
     * Test si la valeur <code>thesaurusType</code> en argument est une valeur correcte.
     */
    public static boolean testThesaurusType(short thesaurusType) {
        if ((thesaurusType < 1) || (thesaurusType > 3)) {
            return false;
        }
        return true;
    }

    public static boolean isIdalphaType(Thesaurus thesaurus) {
        return (thesaurus.getThesaurusMetadata().getThesaurusType() == ThesaurusMetadata.IDALPHA_TYPE);
    }

    public static boolean isBabelienType(Thesaurus thesaurus) {
        return (thesaurus.getThesaurusMetadata().getThesaurusType() == ThesaurusMetadata.BABELIEN_TYPE);
    }

    public static boolean isMultiType(Thesaurus thesaurus) {
        return (thesaurus.getThesaurusMetadata().getThesaurusType() == ThesaurusMetadata.MULTI_TYPE);
    }

    /**
     * Vérifie si la langue en argument peut être utilisée pour les libéllés de mots-clés
     * et renvoie la valeur à utiliser.
     *  Cette valeur est égale à la valeur en argument si la langue est autorisée sinon renvoie une langue par défaut.
     *
     */
    public static int checkLangIntegerDisponibility(Thesaurus thesaurus, int langInteger) {
        ThesaurusMetadata metadata = thesaurus.getThesaurusMetadata();
        if (metadata.getThesaurusType() == ThesaurusMetadata.BABELIEN_TYPE) {
            return langInteger;
        }
        int[] array = metadata.getAuthorizedLangIntegerArray();
        if ((array == null) || (array.length == 0)) {
            throw new ImplementationException("thesaurus.getMetadata.getAuthorizedLangIntegerList() is null or empty");
        }
        for (int i = 0; i < array.length; i++) {
            if (array[i] == langInteger) {
                return langInteger;
            }
        }
        return array[0];
    }

    /**
     * Retourne la langue par défaut pour les libellés de mots-clés. Retourne UNDETERMINED si le
     * thésaurus est babélien.
     *
     */
    public static int getDefaultLangInteger(Thesaurus thesaurus) {
        ThesaurusMetadata metadata = thesaurus.getThesaurusMetadata();
        if (metadata == null) {
            throw new ImplementationException("metadata is null");
        }
        if (metadata.getThesaurusType() == ThesaurusMetadata.BABELIEN_TYPE) {
            return LangInteger.UNDETERMINED;
        }
        int[] array = metadata.getAuthorizedLangIntegerArray();
        if (array == null) {
            throw new ImplementationException("metadata.getAuthorizedLangIntegerList() is null");
        }
        if (array.length == 0) {
            throw new ImplementationException("metadata.getAuthorizedLangIntegerList() is empty");
        }
        return array[0];
    }

    /**
     * Retourne la langue du mot-clé dans le cas d'un thésaurus babélien.
     * Rappelons que dans un thésaurus babélien, motcle.getLibelleList() doit contenir un
     * et un seul élément.
     *
     *@throws IllegalArgumentException si le thésaurus du mot-clé n'est pas babélien
     */
    public static int getDefaultPreferredLangInteger(Motcle motcle) {
        Libelle motcleLibelle = getBabelienLibelle(motcle);
        return motcleLibelle.getLangInteger();
    }

    /**
     *
     *@throws IllegalArgumentException si le thésaurus du mot-clé n'est pas babélien
     */
    public static Libelle getBabelienLibelle(Motcle motcle) {
        if (!isBabelienType(motcle.getThesaurus())) {
            throw new IllegalArgumentException("motcle.getThesaurus is not babelien");
        }
        int libelleCount = motcle.getLibelleCount();
        if (libelleCount == 0) {
            throw new ImplementationException("motcle.getLibelleList() is empty in a babélien thesaurus");
        }
        Libelle motcleLibelle = motcle.getLibelleByIndex(0);
        if (motcleLibelle == null) {
            throw new ImplementationException("motcle.getLibelleList().get(0) is null");
        }
        return motcleLibelle;
    }

    public static String getThesaurusIntitule(Thesaurus thesaurus, int langInteger) {
        Intitule thesaurusIntitule = thesaurus.getThesaurusMetadata().getIntituleByName(THESAURUS);
        if (thesaurusIntitule == null) {
            return thesaurus.getSubsetKey().toString();
        }
        return thesaurusIntitule.getLibelleString(langInteger, true);
    }

    public static String getIntituleThesaurusWithLangCheck(Thesaurus thesaurus, int langInteger) {
        ThesaurusMetadata metadata = thesaurus.getThesaurusMetadata();
        Intitule thesaurusIntitule = metadata.getIntituleByName(THESAURUS);
        String intitule = (thesaurusIntitule != null) ? thesaurusIntitule.getLibelleString(langInteger, true) : thesaurus.getSubsetKey().toString();
        if (metadata.getThesaurusType() == ThesaurusMetadata.MULTI_TYPE) {
            int[] array = metadata.getAuthorizedLangIntegerArray();
            int size = array.length;
            boolean missing = true;
            for (int i = 0; i < size; i++) {
                if (array[i] == langInteger) {
                    missing = false;
                    break;
                }
            }
            if (missing) {
                intitule = intitule + " (" + LangInteger.toString(array[0]) + ")";
            }
        }
        return intitule;
    }

    /**
     * Retourne le premier mot-clé du thésaurus ou null si le thésaurus est vide.
     * <p>Équivalent de thesaurus.getFirstLevel().get(0) en faisant la vérification de la présence d'au moins un mot-clé.
     */
    public static Motcle getFirst(Thesaurus thesaurus) {
        MotcleList firstLevel = thesaurus.getFirstLevelList();
        if (firstLevel.getMotcleCount() == 0) {
            return null;
        }
        return firstLevel.getMotcle(0);
    }

    /**
     * Retourne le mot-clé suivant le mot-clé en argument dans l'arbre de circulation du thésaurus.
     * La circulation dans un thésaurus se fait suivant le même principe que Xpath du langage XSLT :
     * le mot-clé suivant un mot-clé est son fils aîné, sinon son frère cadet, sinon le frère cadet de son père, etc.
     * La méthode retourne <em>null</em> si le mot-clé est le dernier.
     */
    public static Motcle getNext(Motcle motcle) {
        MotcleList childList = motcle.getChildList();
        if (childList.getMotcleCount() > 0) {
            return childList.getMotcle(0);
        }
        Motcle sibling = getNextSibling(motcle);
        if (sibling != null) {
            return sibling;
        }
        Motcle parent = motcle.getParent();
        while (parent != null) {
            sibling = getNextSibling(parent);
            if (sibling != null) {
                return sibling;
            }
            parent = parent.getParent();
        }
        return null;
    }

    /**
     * Retourne le mot-clé précédent le mot-clé en argument dans l'arbre de circulation du thésaurus.
     * Voir la méthode <em>getNext()</em>. Retourne <em>null</em> si le mot-clé est le premier.
     */
    public static Motcle getPrevious(Motcle motcle) {
        Motcle previous = getPreviousSibling(motcle);
        if (previous == null) {
            return motcle.getParent();
        }
        MotcleList childList = previous.getChildList();
        while (childList.getMotcleCount() != 0) {
            previous = childList.getMotcle(childList.getMotcleCount() - 1);
            childList = previous.getChildList();
        }
        return previous;
    }

    /**
     * Retourne le mot-clé suivant le mot-clé en argument dans la liste des mots-clés du même père.
     * Autrement dit, cette méthode retourne le frère cadet ou <em>null</em> si le mot-clé est le dernier de la fratrie.
     */
    public static Motcle getNextSibling(Motcle motcle) {
        int childIndex = motcle.getChildIndex();
        Motcle parent = motcle.getParent();
        MotcleList siblingList = (parent != null) ? parent.getChildList() : motcle.getThesaurus().getFirstLevelList();
        if (childIndex < siblingList.getMotcleCount() - 1) {
            return siblingList.getMotcle(childIndex + 1);
        } else {
            return null;
        }
    }

    /**
     * Retourne le mot-clé précédent le mot-clé en argument dans la liste des mots-clés du même père.
     * Autrement dit, cette méthode retourne le frère aîné ou <em>null</em> si le mot-clé est le premier de la fratrie.
     */
    public static Motcle getPreviousSibling(Motcle motcle) {
        int childIndex = motcle.getChildIndex();
        if (childIndex == 0) {
            return null;
        }
        Motcle parent = motcle.getParent();
        MotcleList siblingList = (parent != null) ? parent.getChildList() : motcle.getThesaurus().getFirstLevelList();
        return siblingList.getMotcle(childIndex - 1);
    }

    /**
     * Indique si un mot-clé peut être supprimé. Un mot-clé peut être supprimé si :
     *<ul><li>il ne possède pas de mots-clés fils, et,
     *<li>il n'indexe aucun fiche
     *</ul>
     * Une interface de ThesaurusEditor ne doit pouvoir supprimer un mot-clé que si cette méthode retourne <em>true</em>.
     *
     */
    public static boolean isRemoveable(Motcle motcle) {
        if (motcle.getChildList().getMotcleCount() > 0) {
            return false;
        }
        SubsetKey thesaurusKey = motcle.getThesaurus().getSubsetKey();
        Basedefiches basedefiches = motcle.getThesaurus().getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            Croisement croisement = basedefiches.getCroisement(corpus.getSubsetKey(), thesaurusKey);
            if (croisement.getCroisementItem(thesaurusKey, motcle.getTicket()).getCount() > 0) {
                return false;
            }
        }
        return true;
    }

    public static void sortByIdalpha(ThesaurusEditor thesaurusEditor, Motcle motcle, boolean recursive) {
        MotcleList childList = (motcle != null) ? motcle.getChildList() : thesaurusEditor.getThesaurus().getFirstLevelList();
        int size = childList.getMotcleCount();
        if (size == 0) {
            return;
        }
        if (size == 1) {
            if (recursive) {
                sortByIdalpha(thesaurusEditor, childList.getMotcle(0), true);
                return;
            }
        }
        Motcle[] array = toArray(childList);
        Arrays.sort(array, new IdalphaComparator());
        for (int i = 0; i < size; i++) {
            Motcle child = array[i];
            if (child.getChildIndex() != i) {
                thesaurusEditor.setChildIndex(child, i);
            }
            if (recursive) {
                sortByIdalpha(thesaurusEditor, child, true);
            }
        }
    }

    public static Motcle[] toArray(MotcleList motcleList) {
        int count = motcleList.getMotcleCount();
        Motcle[] result = new Motcle[count];
        for (int i = 0; i < count; i++) {
            result[i] = motcleList.getMotcle(i);
        }
        return result;
    }

    /**
     * Supprime tous les liens du mot-clé afin que celui-ci puisse être supprimé sans que cela affecte quoi que ce soit d'autre.
     */
    public static void prepareToRemove(BasedefichesEditor basedefichesEditor, Motcle motcle) {
        CroisementEditor croisementEditor = basedefichesEditor.getCroisementEditor();
        Thesaurus thesaurus = motcle.getThesaurus();
        SubsetKey thesaurusKey = thesaurus.getSubsetKey();
        Basedefiches basedefiches = thesaurus.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            Croisement croisement = basedefiches.getCroisement(corpus.getSubsetKey(), thesaurusKey);
            CroisementItem croisementItem = croisement.getCroisementItem(thesaurusKey, motcle.getTicket());
            int ciCount = croisementItem.getCount();
            for (int j = ciCount - 1; j >= 0; j--) {
                int ticket = croisementItem.getTicket(j);
                AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(ticket);
                croisementEditor.removeCroisement(abregeFiche, motcle);
            }
        }
        try {
            transfereChildren(basedefichesEditor, motcle, motcle.getParent());
        } catch (ParentRecursivityException pre) {
            throw new ImplementationException(pre);
        }
    }

    /**
     * Retourne
     */
    public static boolean isDescendant(Motcle motcle, Motcle ancestor) {
        Motcle parent = motcle.getParent();
        while (parent != null) {
            if (parent.equals(ancestor)) {
                return true;
            }
            parent = parent.getParent();
        }
        return false;
    }

    /**
     *
     *@throws IllegalArgumentException si le thésaurus de destination est celui du mot-clé
     */
    public static Motcle move(BasedefichesEditor basedefichesEditor, Motcle source, Thesaurus destination) {
        if (source.getThesaurus().equals(destination)) {
            throw new IllegalArgumentException("source.getThesaurus().equals(destination)");
        }
        Motcle copy = copy(basedefichesEditor, source, destination);
        try {
            basedefichesEditor.merge(source, copy);
        } catch (ParentRecursivityException pre) {
            throw new ImplementationException("recursivity has been checked before in the script", pre);
        }
        return copy;
    }

    /**
     * Transfère l'indexation d'un mot-clé vers un autre mot-clé.
     *
     */
    public static void transfereIndexation(BasedefichesEditor basedefichesEditor, Motcle source, Motcle destination) {
        CroisementEditor croisementEditor = basedefichesEditor.getCroisementEditor();
        int sourceTicket = source.getTicket();
        int destinationTicket = destination.getTicket();
        Thesaurus sourceThesaurus = source.getThesaurus();
        SubsetKey sourceThesaurusKey = sourceThesaurus.getSubsetKey();
        Thesaurus destinationThesaurus = destination.getThesaurus();
        SubsetKey destinationThesaurusKey = destinationThesaurus.getSubsetKey();
        Basedefiches basedefiches = sourceThesaurus.getBasedefiches();
        List<AbregeFiche> removeList = new ArrayList<AbregeFiche>();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            SubsetKey corpusKey = corpus.getSubsetKey();
            Croisement sourceCroisement = basedefiches.getCroisement(corpusKey, sourceThesaurusKey);
            CroisementItem croisementItem = sourceCroisement.getCroisementItem(sourceThesaurusKey, sourceTicket);
            int ciCount = croisementItem.getCount();
            Croisement destinationCroisement = basedefiches.getCroisement(corpusKey, destinationThesaurusKey);
            DefaultFicheCroisementList ficheCroisementList = new DefaultFicheCroisementList(corpusKey, ciCount);
            for (int j = 0; j < ciCount; j++) {
                int ticket = croisementItem.getTicket(j);
                int poids = croisementItem.getPoids(j);
                AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(ticket);
                if (destinationCroisement.getPoids(corpusKey, abregeFiche.getTicket(), destinationThesaurusKey, destinationTicket) == 0) {
                    ficheCroisementList.add(abregeFiche, poids);
                }
                removeList.add(abregeFiche);
            }
            for (AbregeFiche abregeFiche : removeList) {
                croisementEditor.removeCroisement(abregeFiche, source);
            }
            if (ficheCroisementList.getSubsetItemCount() > 0) {
                croisementEditor.updateCroisement(destination, ficheCroisementList, CroisementEditor.APPEND_AND_POIDS_REPLACE);
            }
        }
    }

    public static void transfereLibelles(BasedefichesEditor basedefichesEditor, Motcle source, Motcle destination) {
        Thesaurus thesaurus = destination.getThesaurus();
        if (isBabelienType(thesaurus)) {
            return;
        }
        int libelleCount = source.getLibelleCount();
        ThesaurusEditor thesaurusEditor = basedefichesEditor.getThesaurusEditor(thesaurus.getSubsetKey());
        ThesaurusMetadata metadata = thesaurus.getThesaurusMetadata();
        int[] array = metadata.getAuthorizedLangIntegerArray();
        for (int j = 0; j < libelleCount; j++) {
            Libelle libelle = source.getLibelleByIndex(j);
            int langInteger = libelle.getLangInteger();
            boolean accepte = false;
            for (int i = 0; i < array.length; i++) {
                if (array[i] == langInteger) {
                    accepte = true;
                    break;
                }
            }
            if (accepte) {
                Libelle current = destination.getLibelleByLang(langInteger);
                if (current == null) {
                    thesaurusEditor.setLibelle(destination, libelle.getLibelleString(), langInteger);
                }
            }
        }
    }

    /**
     * Transfère l'indexation d'un mot-clé vers un autre mot-clé.
     *
     *@throws ParentRecursivityException si la destination est un mot-clé fils de la source
     *@throws IllegalArgumentException si les mots-clés ne sont pas du même thésaurus
     */
    public static void transfereChildren(BasedefichesEditor basedefichesEditor, Motcle source, Motcle destination) throws ParentRecursivityException {
        MotcleList childList = source.getChildList();
        if (childList.getMotcleCount() == 0) {
            return;
        }
        Thesaurus thesaurus = source.getThesaurus();
        if (destination != null) {
            if (!destination.getThesaurus().equals(thesaurus)) {
                throw new IllegalArgumentException("source and destination are not coming from the same thesaurus");
            }
            if (isDescendant(destination, source)) {
                throw new ParentRecursivityException(destination, source);
            }
        }
        ThesaurusEditor thesaurusEditor = basedefichesEditor.getThesaurusEditor(thesaurus.getSubsetKey());
        Motcle[] array = toArray(childList);
        for (int i = 0; i < array.length; i++) {
            Motcle child = array[i];
            try {
                thesaurusEditor.setParent(child, destination);
            } catch (ParentRecursivityException pre) {
                throw new ImplementationException("recursivity has been checked before in the script", pre);
            }
        }
    }

    public static Motcle copy(BasedefichesEditor basedefichesEditor, Motcle source, Thesaurus destinationThesaurus) {
        return copy(basedefichesEditor.getThesaurusEditor(destinationThesaurus.getSubsetKey()), source);
    }

    public static Motcle copy(ThesaurusEditor thesaurusEditor, Motcle source) {
        try {
            Motcle destination = thesaurusEditor.createMotcle(-1, null);
            int libelleCount = source.getLibelleCount();
            for (int i = 0; i < libelleCount; i++) {
                Libelle libelle = source.getLibelleByIndex(i);
                thesaurusEditor.setLibelle(destination, libelle.getLibelleString(), libelle.getLangInteger());
            }
            return destination;
        } catch (ExistingIdException eie) {
            return null;
        } catch (ParseException pe) {
            return null;
        }
    }

    public static int removeRemoveableMotcle(ThesaurusEditor thesaurusEditor) {
        int count = 0;
        MotcleList firstLevel = thesaurusEditor.getThesaurus().getFirstLevelList();
        Motcle[] motcleArray = toArray(firstLevel);
        for (int i = 0; i < motcleArray.length; i++) {
            count = count + removeRemoveable(motcleArray[i], thesaurusEditor);
        }
        return count;
    }

    private static int removeRemoveable(Motcle motcle, ThesaurusEditor thesaurusEditor) {
        int count = 0;
        Motcle[] motcleArray = toArray(motcle.getChildList());
        for (int i = 0; i < motcleArray.length; i++) {
            count = count + removeRemoveable(motcleArray[i], thesaurusEditor);
        }
        if (isRemoveable(motcle)) {
            thesaurusEditor.removeMotcle(motcle);
            count++;
        }
        return count;
    }

    /**
     *
     *@throws IllegalArgumentException si le type est incorrect.
     */
    public static final String thesaurusTypeToString(short type) {
        switch (type) {
            case ThesaurusMetadata.BABELIEN_TYPE:
                return "babelien";
            case ThesaurusMetadata.IDALPHA_TYPE:
                return "idalpha";
            case ThesaurusMetadata.MULTI_TYPE:
                return "multi";
            default:
                throw new IllegalArgumentException("wrong type");
        }
    }

    /**
     *
     *@throws IllegalArgumentException si le type est incorrect.
     */
    public static final short thesaurusTypeToShort(String thesaurusType) {
        if (thesaurusType.equals("babelien")) {
            return ThesaurusMetadata.BABELIEN_TYPE;
        }
        if (thesaurusType.equals("idalpha")) {
            return ThesaurusMetadata.IDALPHA_TYPE;
        }
        if (thesaurusType.equals("multi")) {
            return ThesaurusMetadata.MULTI_TYPE;
        }
        throw new IllegalArgumentException("wrong type");
    }

    /**
     * Retourne la liste des clés des thésaurus babéliens. Cette liste est établie à un instant t. Elle n'est pas
     * synchronisée.
     *
     */
    public static List<SubsetKey> getBabelienThesaurusList(Basedefiches basedefiches) {
        List<SubsetKey> list = new ArrayList<SubsetKey>();
        int thesaurusCount = basedefiches.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = basedefiches.getThesaurus(i);
            if (ThesaurusUtils.isBabelienType(thesaurus)) {
                list.add(thesaurus.getSubsetKey());
            }
        }
        return list;
    }

    /**
     * Retourne la lslite des clés des thésaurus sans identifiant alphanumériques.
     */
    public static List<SubsetKey> getNoIdalphaThesaurusList(Basedefiches basedefiches) {
        List<SubsetKey> list = new ArrayList<SubsetKey>();
        int thesaurusCount = basedefiches.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = basedefiches.getThesaurus(i);
            if (!ThesaurusUtils.isIdalphaType(thesaurus)) {
                list.add(thesaurus.getSubsetKey());
            }
        }
        return list;
    }

    public static String getGlobalPosition(Motcle motcle) {
        StringBuilder buf = new StringBuilder();
        try  {
            appendGlobalPosition(buf, motcle);
        } catch(IOException ioe) {

        }
        return buf.toString();
    }

    public static void appendGlobalPosition(Appendable buf, Motcle motcle) throws IOException {
        Motcle parent = motcle.getParent();
        int position = motcle.getChildIndex() + 1;
        int size = 0;
        if (parent != null) {
            appendGlobalPosition(buf, parent);
            buf.append('.');
            size = parent.getChildList().getMotcleCount();
        } else {
            size = motcle.getThesaurus().getFirstLevelList().getMotcleCount();
        }
        if (size < 99) {
            size = 99;
        }
        appendZero(buf, size, position);
        buf.append(String.valueOf(position));
    }

    private static void appendZero(Appendable buf, int size, int position) throws IOException {
        int dizaine = 10;
        while ((size >= dizaine)) {
            if (position < dizaine) {
                buf.append('0');
            }
            dizaine = dizaine * 10;
        }
    }

}
