/* BasedefichesLib_API - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.thesaurus.metadata;

import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import java.util.Map;
import java.util.HashMap;
import java.text.ParseException;


/**
 *
 * @author Vincent Calame
 */
public final class ThesaurusFieldKey {

    private final static Map<String, ThesaurusFieldKey> internMap = new HashMap<String, ThesaurusFieldKey>();
    public final static ThesaurusFieldKey IDTHS = new ThesaurusFieldKey("idths");
    public final static ThesaurusFieldKey IDALPHA = new ThesaurusFieldKey("idalpha");
    public final static ThesaurusFieldKey PARENT_IDTHS = new ThesaurusFieldKey("parent_idths");
    public final static ThesaurusFieldKey PARENT_IDALPHA = new ThesaurusFieldKey("parent_idalpha");
    public final static ThesaurusFieldKey BABELIENLIB = new ThesaurusFieldKey("lib");
    public final static ThesaurusFieldKey BABELIENLANG = new ThesaurusFieldKey("lang");
    public final static ThesaurusFieldKey POSITION_LOCAL = new ThesaurusFieldKey("pos_loc");
    public final static ThesaurusFieldKey POSITION_GLOBAL = new ThesaurusFieldKey("pos_glob");
    public final static short SPECIAL_CATEGORY = 0;
    public final static short LIB_CATEGORY = 1;
    private String fieldKeyString;
    private short category = 0;
    private int langInteger;

    private ThesaurusFieldKey(String fieldKeyString) {
        this.fieldKeyString = fieldKeyString.intern();
        this.category = SPECIAL_CATEGORY;
        intern(this);
    }

    private ThesaurusFieldKey(short category, int langInteger) {
        this(toKeyString(category, langInteger), category, langInteger);
    }

    private ThesaurusFieldKey(String fieldKeyString, short category, int langInteger) {
        this.fieldKeyString = fieldKeyString.intern();
        this.category = category;
        this.langInteger = langInteger;
        intern(this);
    }

    private static ThesaurusFieldKey initThesaurusFieldKey(short category, int langInteger) {
        String fieldKeyString = toKeyString(category, langInteger);
        ThesaurusFieldKey current = internMap.get(fieldKeyString);
        if (current != null) {
            return current;
        }
        return new ThesaurusFieldKey(fieldKeyString, category, langInteger);
    }

    private static String toKeyString(short category, int langInteger) {
        StringBuilder buf = new StringBuilder();
        if (category == LIB_CATEGORY) {
            buf.append("lib_");
        }
        LangInteger.appendString(buf, langInteger);
        return buf.toString();
    }

    private synchronized static void intern(ThesaurusFieldKey thesaurusFieldKey) {
        internMap.put(thesaurusFieldKey.fieldKeyString, thesaurusFieldKey);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof ThesaurusFieldKey)) {
            return false;
        }
        ThesaurusFieldKey fk = (ThesaurusFieldKey) obj;
        return fk.fieldKeyString.equals(this.fieldKeyString);
    }

    @Override
    public int hashCode() {
        return fieldKeyString.hashCode();
    }

    @Override
    public String toString() {
        return fieldKeyString;
    }

    public boolean isSpecialThesaurusFieldKey() {
        return (category == SPECIAL_CATEGORY);
    }

    public boolean isLibThesaurusFieldKey() {
        return (category == LIB_CATEGORY);
    }

    public short getCategory() {
        return category;
    }

    public int getLangInteger() {
        return langInteger;
    }

    /**
     *
     *@throws ParseException si s est incorrect
     */
    public static ThesaurusFieldKey parse(String s) throws ParseException {
        ThesaurusFieldKey current = (ThesaurusFieldKey) internMap.get(s);
        if (current != null) {
            return current;
        }
        if (s.startsWith("lib_")) {
            String lg = s.substring(4);
            try {
                int langInteger = LangInteger.parse(lg);
                return new ThesaurusFieldKey(LIB_CATEGORY, langInteger);
            } catch (LangIntegerException iae) {
                throw new ParseException("wrong lang :" + lg, 4);
            }
        }
        throw new ParseException("unkown thesaurusfieldkey : " + s, 0);
    }

    public static ThesaurusFieldKey[] getDefaultFieldsArray(ThesaurusMetadata thesaurusMetadata) {
        short type = thesaurusMetadata.getThesaurusType();
        if (type == ThesaurusMetadata.BABELIEN_TYPE) {
            ThesaurusFieldKey[] result = {IDTHS, PARENT_IDTHS, BABELIENLANG, BABELIENLIB};
            return result;
        }
        int[] langArray = thesaurusMetadata.getAuthorizedLangIntegerArray();
        int length = langArray.length;
        ThesaurusFieldKey[] result = null;
        int decalage = 2;
        if (type == ThesaurusMetadata.IDALPHA_TYPE) {
            result = new ThesaurusFieldKey[4 + length];
            decalage = 4;
            result[2] = IDALPHA;
            result[3] = PARENT_IDALPHA;
        } else {
            result = new ThesaurusFieldKey[2 + length];
        }
        result[0] = IDTHS;
        result[1] = PARENT_IDTHS;

        for (int i = 0; i < langArray.length; i++) {
            int index = i + decalage;
            int langInteger = langArray[i];
            result[index] = initThesaurusFieldKey(LIB_CATEGORY, langInteger);
        }
        return result;
    }

}
