/* BasedefichesLib_Explore
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.extraction;

import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.extraction.groups.*;
import java.util.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.text.collation.group.*;
import net.mapeadores.util.text.alphabet.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.basedefiches.extraction.api.*;
import net.mapeadores.basedefiches.extraction.params.*;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;


/**
 *
 * @author Vincent Calame
 */
public class CorpusExtractor implements ExtractionConstants {

    private HashMap extraitMap = new HashMap();
    private List extraitList = new ArrayList();
    private Subset filterSubset;
    private ExtraitCorpusDef extraitCorpusDef;
    private AbstractExtrait namedExtrait = null;
    private boolean containsFiche = false;

    public CorpusExtractor(ExtraitCorpusDef extraitCorpusDef, Subset filterSubset) {
        this.filterSubset = filterSubset;
        this.extraitCorpusDef = extraitCorpusDef;
        String extraitName = extraitCorpusDef.getExtraitName();
        if (extraitName != null) {
            namedExtrait = newInstance(null);
            extraitList = Collections.singletonList(namedExtrait);
        } else {
            extraitList = new ArrayList();
        }
    }

    public void add(AbregeFiche abregeFiche, int poids) {
        if (poids < 0) {
            poids = 0;
        }
        containsFiche = true;
        FicheExtractBundle ficheExtractBundle = new FicheExtractBundle(abregeFiche, extraitCorpusDef.getFicheFilter(), poids);
        if (namedExtrait != null) {
            namedExtrait.add(ficheExtractBundle);
        } else {
            SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
            AbstractExtrait extrait = (AbstractExtrait) extraitMap.get(corpusKey);
            if (extrait == null) {
                extrait = newInstance(abregeFiche.getCorpus());
                extraitMap.put(corpusKey, extrait);
                extraitList.add(extrait);
            }
            extrait.add(ficheExtractBundle);
        }
    }

    public boolean containsFiche() {
        return containsFiche;
    }

    public int getExtraitCount() {
        return extraitList.size();
    }

    public Extrait getExtrait(int i) {
        return (Extrait) extraitList.get(i);
    }


    public static interface Extrait {

        public String getName();

        public String getTagName();

        public SubsetKey getCorpusKey();

        public boolean withCorpusIntitule();

        public FicheGroup[] getGroupArray();

        public String getRattachementType();

        public PackClause getPackClause();

    }

    private AbstractExtrait newInstance(Corpus corpus) {
        GroupClause groupClause = extraitCorpusDef.getGroupClause();
        if (groupClause == null) {
            return new NoGroupExtrait(extraitCorpusDef, filterSubset, corpus);
        }
        switch (groupClause.getGroupType()) {
            case POIDS_TYPE:
                return new PoidsExtrait(extraitCorpusDef, filterSubset, corpus);
            case TITRE_TYPE:
                return new TitreExtrait(extraitCorpusDef, filterSubset, corpus);
            case ANNEE_TYPE:
                return new FicheItemExtrait(extraitCorpusDef, filterSubset, corpus, CorpusField.DATATION_FIELD);
            case LANG_TYPE:
                return new FicheItemExtrait(extraitCorpusDef, filterSubset, corpus, CorpusField.LANGUE_FIELD);
            case PAYS_TYPE:
                return new FicheItemExtrait(extraitCorpusDef, filterSubset, corpus, CorpusField.PAYS_FIELD);
            case FIELDS_TYPE:
                return new FieldsExtrait(extraitCorpusDef, filterSubset, corpus);
            default:
                throw new SwitchException("groupClause.getGroupeType() =  " + groupClause.getGroupType());
        }
    }


    private static abstract class AbstractExtrait implements Extrait {

        protected ExtraitCorpusDef extraitCorpusDef;
        private SubsetKey corpusKey = null;
        private String rattachementType = null;
        protected GroupClause groupClause;
        protected boolean asc = true;

        AbstractExtrait(ExtraitCorpusDef extraitCorpusDef, Subset filterSubset, Corpus corpus) {
            this.extraitCorpusDef = extraitCorpusDef;
            this.groupClause = extraitCorpusDef.getGroupClause();
            if (groupClause != null) {
                asc = (groupClause.getSortOrder() == ExtractionConstants.ASCENDING_SORT);
            }
            if (corpus != null) {
                corpusKey = corpus.getSubsetKey();
                if (filterSubset != null) {
                    Subset masterSubset = corpus.getMasterSubset();
                    if (masterSubset != null) {
                        if (masterSubset.equals(filterSubset)) {
                            rattachementType = "slave";
                        }
                    } else {
                        masterSubset = filterSubset.getMasterSubset();
                        if (masterSubset != null) {
                            if (masterSubset.equals(corpus)) {
                                rattachementType = "master";
                            }
                        }
                    }
                }
            }
        }

        public String getName() {
            return extraitCorpusDef.getExtraitName();
        }

        public SubsetKey getCorpusKey() {
            return corpusKey;
        }

        public boolean withCorpusIntitule() {
            if (corpusKey == null) {
                return false;
            }
            IntituleClause intituleClause = extraitCorpusDef.getIntituleClause();
            if (intituleClause == null) {
                return false;
            }
            return intituleClause.withIntitule();
        }

        public String getRattachementType() {
            return rattachementType;
        }

        abstract void add(FicheExtractBundle ficheExtractBundle);

        public String getTagName() {
            return extraitCorpusDef.getTagName();
        }

        public PackClause getPackClause() {
            return extraitCorpusDef.getPackClause();
        }

    }


    private static class NoGroupExtrait extends AbstractExtrait {

        private MutableFicheGroup defaultGroup = new NullFicheGroup();
        private FicheGroup[] groupArray = {defaultGroup};

        NoGroupExtrait(ExtraitCorpusDef extraitCorpusDef, Subset filterSubset, Corpus corpus) {
            super(extraitCorpusDef, filterSubset, corpus);
        }

        public FicheGroup[] getGroupArray() {
            return groupArray;
        }

        void add(FicheExtractBundle ficheExtractBundle) {
            defaultGroup.add(ficheExtractBundle);
        }

    }


    private static abstract class SortExtrait extends AbstractExtrait {

        TreeMap treeMap = new TreeMap();

        SortExtrait(ExtraitCorpusDef extraitCorpusDef, Subset filterSubset, Corpus corpus) {
            super(extraitCorpusDef, filterSubset, corpus);
        }

        public FicheGroup[] getGroupArray() {
            int size = treeMap.size();
            int p = (asc) ? 0 : (size - 1);
            FicheGroup[] result = new FicheGroup[size];
            for (Iterator it = treeMap.values().iterator(); it.hasNext();) {
                FicheGroup group = (FicheGroup) it.next();
                result[p] = group;
                if (asc) {
                    p++;
                } else {
                    p--;
                }
            }
            return result;
        }

    }


    private static class PoidsExtrait extends SortExtrait {

        PoidsExtrait(ExtraitCorpusDef extraitCorpusDef, Subset filterSubset, Corpus corpus) {
            super(extraitCorpusDef, filterSubset, corpus);

        }

        void add(FicheExtractBundle ficheExtractBundle) {
            Integer poids = new Integer(ficheExtractBundle.getPoids());
            MutableFicheGroup group = (MutableFicheGroup) treeMap.get(poids);
            if (group == null) {
                group = new SimpleFicheGroup(poids.toString(), groupClause.getTagName());
                treeMap.put(poids, group);
            }
            group.add(ficheExtractBundle);
        }

    }


    private static class TitreExtrait extends SortExtrait {

        CollationGroupBuilder collationGroupBuilder;
        boolean alphabetMode = false;

        TitreExtrait(ExtraitCorpusDef extraitCorpusDef, Subset filterSubset, Corpus corpus) {
            super(extraitCorpusDef, filterSubset, corpus);
            FieldsGroupParams groupParams = (FieldsGroupParams) groupClause.getGroupParams();
            if (groupParams != null) {
                alphabetMode = (groupParams.getMode() == FieldsGroupParams.ALPHABET_MODE);
            }
            collationGroupBuilder = new CollationGroupBuilder();
        }

        void add(FicheExtractBundle ficheExtractBundle) {
            String titre = ficheExtractBundle.getAbregeFiche().getTitre();
            if (alphabetMode) {
                Alphabet alphabet = Alphabet.newInstance(titre, ficheExtractBundle.getAbregeFiche().getLangInteger());
                titre = alphabet.getAlphabetString();
                ficheExtractBundle.setGroupClauseObject(alphabet);
            }
            ObjectWrapper objectWrapper = ObjectWrapper.newCleanedInstance(titre, ficheExtractBundle);
            collationGroupBuilder.addObjectWrapper(objectWrapper);
        }

        public FicheGroup[] getGroupArray() {
            List list = collationGroupBuilder.getCollationGroupList();
            int size = list.size();
            FicheGroup[] result = new FicheGroup[size];
            for (int i = 0; i < size; i++) {
                CollationGroup collationGroup = (CollationGroup) list.get(i);
                CollationGroupBasedGroup collationGroupBasedGroup = new CollationGroupBasedGroup(collationGroup, asc, groupClause.getTagName());
                int p = (asc) ? i : (size - i - 1);
                result[p] = collationGroupBasedGroup;
            }
            return result;
        }

    }


    private static class FieldsExtrait extends SortExtrait {

        CollationGroupBuilder collationGroupBuilder;
        boolean alphabetMode = false;
        private FieldKey[] fieldKeyArray;

        FieldsExtrait(ExtraitCorpusDef extraitCorpusDef, Subset filterSubset, Corpus corpus) {
            super(extraitCorpusDef, filterSubset, corpus);
            FieldsGroupParams groupParams = (FieldsGroupParams) groupClause.getGroupParams();
            alphabetMode = (groupParams.getMode() == FieldsGroupParams.ALPHABET_MODE);
            collationGroupBuilder = new CollationGroupBuilder();
            this.fieldKeyArray = groupParams.getFieldKeyArray();
        }

        void add(FicheExtractBundle ficheExtractBundle) {
            String titre = getStringValue(ficheExtractBundle.getAbregeFiche());
            if (alphabetMode) {
                Alphabet alphabet = Alphabet.newInstance(titre, ficheExtractBundle.getAbregeFiche().getLangInteger());
                titre = alphabet.getAlphabetString();
                ficheExtractBundle.setGroupClauseObject(alphabet);
            }
            ObjectWrapper objectWrapper = ObjectWrapper.newCleanedInstance(titre, ficheExtractBundle);
            collationGroupBuilder.addObjectWrapper(objectWrapper);
        }

        private String getStringValue(AbregeFiche abregeFiche) {
            FichePointeur fichePointeur = new FichePointeur(abregeFiche);
            for (int i = 0; i < fieldKeyArray.length; i++) {
                Object obj = fichePointeur.getValue(fieldKeyArray[i]);
                if (obj == null) {
                    continue;
                }
                String value = "";
                if (obj instanceof String) {
                    value = (String) obj;
                } else if (obj instanceof FicheItem) {
                    value = getStringValue((FicheItem) obj);
                } else if (obj instanceof FicheItemList) {
                    value = getStringValue((FicheItemList) obj);
                }
                if (value.length() > 0) {
                    return value;
                } else {
                    continue;
                }
            }
            return "";
        }

        private String getStringValue(FicheItem ficheItem) {
            if (ficheItem == null) {
                return "";
            }
            if (ficheItem instanceof Item) {
                return ((Item) ficheItem).getValue();
            }
            if (ficheItem instanceof Para) {
                return ((Para) ficheItem).toRawString();
            }
            return "";
        }

        private String getStringValue(FicheItemList ficheItemList) {
            int size = ficheItemList.getFicheItemCount();
            for (int i = 0; i < size; i++) {
                String value = getStringValue(ficheItemList.getFicheItem(i));
                if (value.length() > 0) {
                    return value;
                } else {
                    continue;
                }
            }
            return "";
        }

        public FicheGroup[] getGroupArray() {
            List list = collationGroupBuilder.getCollationGroupList();
            int size = list.size();
            FicheGroup[] result = new FicheGroup[size];
            for (int i = 0; i < size; i++) {
                CollationGroup collationGroup = (CollationGroup) list.get(i);
                CollationGroupBasedGroup collationGroupBasedGroup = new CollationGroupBasedGroup(collationGroup, asc, groupClause.getTagName());
                int p = (asc) ? i : (size - i - 1);
                result[p] = collationGroupBasedGroup;
            }
            return result;
        }

    }


    private static class FicheItemExtrait extends SortExtrait {

        MutableFicheGroup emptyGroup;
        int ficheItemType;
        FieldKey fieldKey;

        FicheItemExtrait(ExtraitCorpusDef extraitCorpusDef, Subset filterSubset, Corpus corpus, int ficheItemType) {
            super(extraitCorpusDef, filterSubset, corpus);
            FicheItemGroupParams groupParams = (FicheItemGroupParams) groupClause.getGroupParams();
            if (groupParams != null) {
                fieldKey = groupParams.getFieldKey();
            }
            this.ficheItemType = ficheItemType;
        }

        void add(FicheExtractBundle ficheExtractBundle) {
            AbregeFiche abregeFiche = ficheExtractBundle.getAbregeFiche();
            int value = 0;
            if (fieldKey == null) {
                switch (ficheItemType) {
                    case CorpusField.LANGUE_FIELD:
                        value = abregeFiche.getLangInteger();
                        if (value == LangInteger.UNDETERMINED) {
                            value = 0;
                        }
                        break;
                    case CorpusField.DATATION_FIELD:
                        value = abregeFiche.getCreationDate();
                        if (value != 0) {
                            ficheExtractBundle.setInnerSortValue(value);
                            value = DateInteger.getAnnee(value);
                        }
                        break;
                }
            } else {
                Object obj = FichePointeur.getValue(abregeFiche, fieldKey);
                if (obj != null) {
                    if (obj instanceof FicheItem) {
                        value = getValue((FicheItem) obj, ficheExtractBundle);
                    } else if (obj instanceof FicheItemList) {
                        FicheItemList fil = (FicheItemList) obj;
                        if (fil.getFicheItemCount() > 0) {
                            value = getValue(fil.getFicheItem(0), ficheExtractBundle);
                        }
                    }
                }
            }
            MutableFicheGroup group = getGroup(value);
            group.add(ficheExtractBundle);
        }

        private int getValue(FicheItem ficheItem, FicheExtractBundle ficheExtractBundle) {
            switch (ficheItemType) {
                case CorpusField.LANGUE_FIELD:
                    if (ficheItem instanceof Langue) {
                        return ((Langue) ficheItem).getLangInteger();
                    } else {
                        return 0;
                    }
                case CorpusField.DATATION_FIELD:
                    if (ficheItem instanceof Datation) {
                        int value = ((Datation) ficheItem).getDateInteger();
                        ficheExtractBundle.setInnerSortValue(value);
                        return DateInteger.getAnnee(value);
                    } else {
                        return 0;
                    }
                case CorpusField.PAYS_FIELD:
                    if (ficheItem instanceof Pays) {
                        return ((Pays) ficheItem).getCountryInteger();
                    } else {
                        return 0;
                    }
                default:
                    return 0;
            }
        }

        private MutableFicheGroup getGroup(int value) {
            if (value == 0) {
                if (emptyGroup == null) {
                    emptyGroup = new SimpleFicheGroup("_", groupClause.getTagName());
                }
                return emptyGroup;
            }
            switch (ficheItemType) {
                case CorpusField.LANGUE_FIELD:
                    String langKey = LangInteger.toString(value);
                    MutableFicheGroup langGroup = (MutableFicheGroup) treeMap.get(langKey);
                    if (langGroup == null) {
                        langGroup = new SimpleFicheGroup(langKey, groupClause.getTagName());
                        treeMap.put(langKey, langGroup);
                        Langue langue = new Langue(value);
                        FicheItem[] matchingFicheItemArray = new FicheItem[1];
                        matchingFicheItemArray[0] = langue;
                        langGroup.setMatchingFicheItemArray(matchingFicheItemArray);
                    }
                    return langGroup;
                case CorpusField.DATATION_FIELD:
                    Integer anneeKey = new Integer(value);
                    MutableFicheGroup anneeGroup = (MutableFicheGroup) treeMap.get(anneeKey);
                    if (anneeGroup == null) {
                        anneeGroup = new InnerSortFicheGroup(anneeKey.toString(), groupClause.getTagName(), asc);
                        treeMap.put(anneeKey, anneeGroup);
                    }
                    return anneeGroup;
                case CorpusField.PAYS_FIELD:
                    String countryCode = LangInteger.getCountry(value);
                    MutableFicheGroup paysGroup = (MutableFicheGroup) treeMap.get(countryCode);
                    if (paysGroup == null) {
                        paysGroup = new SimpleFicheGroup(countryCode, groupClause.getTagName());
                        treeMap.put(countryCode, paysGroup);
                        Pays pays = new Pays(countryCode);
                        FicheItem[] matchingFicheItemArray = null;
                        CountryUtils.GeoPoint geoPoint = CountryUtils.getGeolocalisation(value);
                        if (geoPoint != null) {
                            matchingFicheItemArray = new FicheItem[2];
                            Geopoint geopoint = new Geopoint(geoPoint.getLatitude(), geoPoint.getLongitude());
                            matchingFicheItemArray[1] = geopoint;
                        } else {
                            matchingFicheItemArray = new FicheItem[1];
                        }
                        matchingFicheItemArray[0] = pays;
                        paysGroup.setMatchingFicheItemArray(matchingFicheItemArray);
                    }
                    return paysGroup;
                default:
                    throw new SwitchException("unknown FicheItemType : " + ficheItemType);
            }
        }

        public FicheGroup[] getGroupArray() {
            int size = treeMap.size();
            int max = size;
            if (emptyGroup != null) {
                size = size + 1;
            }
            FicheGroup[] result = new FicheGroup[size];
            int p = (asc) ? 0 : (max - 1);
            for (Iterator it = treeMap.values().iterator(); it.hasNext();) {
                FicheGroup group = (FicheGroup) it.next();
                result[p] = group;
                if (asc) {
                    p++;
                } else {
                    p--;
                }
            }
            if (emptyGroup != null) {
                result[max] = emptyGroup;
            }
            return result;
        }

    }

}
