/* BasedefichesLib_Explore
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.util.primitives.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.conditions.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class MotcleSelectOptions {

    public final static short IMPOSSIBLE = 0;
    public final static short INDEXATION_ONLY = 1;
    public final static short NOINDEXATION_ONLY = 2;
    public final static short NOINDEXATION_ALLOWED = 3;
    private RangeList idthsRangeList;
    private RangeList levelRangeList;
    private SubsetKey[] thesaurusKeyArray;
    private ConditionGroup contentConditionGroup;
    private boolean isImpossible = false;
    private boolean idalphaOnly;
    private ConditionGroup sourceConditionGroup;
    private boolean allowsNoIndexation = false;
    private boolean allowsIndexation = true;
    private short indexationType = INDEXATION_ONLY;

    public MotcleSelectOptions() {
    }

    public MotcleSelectOptions(MotcleSelectOptions motcleSelectOptions) {
        this.idthsRangeList = motcleSelectOptions.idthsRangeList;
        this.levelRangeList = motcleSelectOptions.levelRangeList;
        this.thesaurusKeyArray = motcleSelectOptions.thesaurusKeyArray;
    }

    /**
     * null ou contient au moins un élément
     */
    public RangeList getIdthsRangeList() {
        return idthsRangeList;
    }

    /**
     *
     *@throws IllegalArgumentException si rangeList n'est pas positif
     */
    public void setIdthsRangeList(RangeList rangeList) {
        if ((rangeList == null) || (rangeList.size() == 0)) {
            this.idthsRangeList = null;
            return;
        }
        if (!RangeUtils.isPositiveRangeList(rangeList)) {
            throw new IllegalArgumentException("rangeList is not positive");
        }
        this.idthsRangeList = rangeList;
    }

    /**
     * null ou contient au moins un élément
     */
    public RangeList getLevelRangeList() {
        return levelRangeList;
    }

    public void setLevelRangeList(RangeList levelRangeList) {
        if ((levelRangeList == null) || (levelRangeList.size() == 0)) {
            this.levelRangeList = null;
            return;
        }
        this.levelRangeList = levelRangeList;
    }

    public SubsetKey[] getThesaurusKeyArray() {
        return thesaurusKeyArray;
    }

    public void setThesaurusKeyArray(SubsetKey[] thesaurusKeyArray) {
        if ((thesaurusKeyArray != null) && (thesaurusKeyArray.length == 0)) {
            thesaurusKeyArray = null;
        } else {
            this.thesaurusKeyArray = thesaurusKeyArray;
        }
    }

    /**
     * Condition à appliquer aux libellés et idalpha. Son état à pour valeur Conditions.P
     */
    public ConditionGroup getContentConditionGroup() {
        return contentConditionGroup;
    }

    /**
     * Indique que la condition est impossible à tenir
     */
    public boolean isImpossible() {
        return isImpossible;
    }

    /**
     * Indique que la condition est vide.
     */
    public boolean isEmpty() {
        if (sourceConditionGroup != null) {
            return false;
        }
        if (idthsRangeList != null) {
            return false;
        }
        if (levelRangeList != null) {
            return false;
        }
        return true;
    }

    public boolean allowsIndexation() {
        return allowsIndexation;
    }

    /**
     * Indique que les fiches qui dépendent (éventuellement) de cette sélection
     * peuvent être sélectionnée si elles ne sont pas indexées par aucun mot-clé.
     */
    public boolean allowsNoIndexation() {
        return allowsNoIndexation;
    }

    public short getIndexationType() {
        return indexationType;
    }

    public void initConditionGroup(ConditionGroup conditionGroup) {
        this.sourceConditionGroup = conditionGroup;
        if (conditionGroup == null) {
            initToNull();
            return;
        }
        short state = conditionGroup.getState();
        switch (state) {
            case ConditionsConstants.A:
                contentConditionGroup = null;
                indexationType = NOINDEXATION_ONLY;
                isImpossible = false;
                break;
            case ConditionsConstants.T:
                contentConditionGroup = null;
                indexationType = INDEXATION_ONLY;
                isImpossible = false;
                break;
            case ConditionsConstants.P:
                contentConditionGroup = conditionGroup.toPartConditionGroup();
                indexationType = INDEXATION_ONLY;
                isImpossible = false;
                break;
            case ConditionsConstants.PA:
                contentConditionGroup = conditionGroup.toPartConditionGroup();
                indexationType = NOINDEXATION_ALLOWED;
                isImpossible = false;
                break;
            case ConditionsConstants.X:
                contentConditionGroup = null;
                indexationType = IMPOSSIBLE;
                isImpossible = true;
                break;
            case ConditionsConstants.U:
                initToNull();
                break;
            case ConditionsConstants.N:
                initToNull();
                break;
            default:
                throw new SwitchException("state = " + state);
        }


    }

    private void initToNull() {
        contentConditionGroup = null;
        isImpossible = false;
        sourceConditionGroup = null;
        indexationType = INDEXATION_ONLY;
    }

    public ConditionGroup getSourceConditionGroup() {
        return sourceConditionGroup;
    }

    public boolean isIdalphaOnly() {
        return idalphaOnly;
    }

    public void setIdalphaOnly(boolean idalphaOnly) {
        this.idalphaOnly = idalphaOnly;
    }

    /**
     * Retourne <em>true</em> s'il y a au moins une option.
     */
    public boolean containsOption() {
        if (idthsRangeList != null) {
            return true;
        }
        if (levelRangeList != null) {
            return true;
        }
        if (thesaurusKeyArray != null) {
            return true;
        }
        if (sourceConditionGroup != null) {
            return true;
        }
        return false;
    }

}
