/* BasedefichesLib_Explore
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import java.util.*;
import net.mapeadores.basedefiches.extraction.*;


/**
 * Cette classe permet la sélection de mots-clés d'une base de fiches répondant à certains critères.
 * Ces critères sont définis par des instances de MotcleFilterWrapper ou de MotcleSelectOptions.
 *
 * @author Vincent Calame
 */
public class MotcleSelector {

    Basedefiches basedefiches;
    HashMap thesaurusSelectBundleMap = new HashMap();
    ArrayList thesaurusSelectBundleList = new ArrayList();
    int langInteger;

    /**
     * Construit une nouvelle instance sélectionnant des mots-clés sur la base de fiches
     * en argument.
     */
    public MotcleSelector(Basedefiches basedefiches, int langInteger) {
        this.basedefiches = basedefiches;
        this.langInteger = langInteger;
    }

    /**
     * Retourne l'instance de Basedefiches sur laquelle se fait la sélection.
     */
    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public int getLangInteger() {
        return langInteger;
    }

    /**
     * Retourne le nombre de thésaurus qui ont au moins un mot-clé dans la sélection.
     */
    public int getThesaurusCount() {
        return thesaurusSelectBundleList.size();
    }

    /**
     * Retourne le i-ème thésaurus de la sélection.
     *<p>for(int i = 0; i < getThesaurusCount();i++) {Thesaurus thesaurus = getThesaurus(i)} permet
     * de balayer l'ensemble des thésaurus concerné.
     */
    public Thesaurus getThesaurus(int i) {
        ThesaurusSelectBundle thesaurusSelectBundle = (ThesaurusSelectBundle) thesaurusSelectBundleList.get(i);
        return thesaurusSelectBundle.thesaurus;
    }

    public boolean isSelected(Motcle motcle) {
        return (getMotcleFilter(motcle) != null);
    }

    /**
     * Retourne l'instance de MotcleFilter associée à l'argument <code>motcle</code>.
     * Retourne <em>null</em> si le mot-clé n'a pas d'instance de MotcleFilter associée, autrement dit$
     * si le mot-clé n'est pas sélectionné par MotcleSelector.
     * <p>Cette méthode a donc une double fonction : indiquer si un mot-clé est sélectionné ou non et préciser
     * le filtre à appliquer sur le mot-clé s'il est sélectionné.
     */
    public MotcleFilter getMotcleFilter(Motcle motcle) {
        ThesaurusSelectBundle thesaurusSelectBundle = (ThesaurusSelectBundle) thesaurusSelectBundleMap.get(motcle.getThesaurus().getSubsetKey());
        if (thesaurusSelectBundle == null) {
            return null;
        }
        return thesaurusSelectBundle.getMotcleFilter(motcle);
    }

    /**
     * n'a de sens que s'il y a une seule option de sélection.
     */
    public int getPMask(Motcle motcle) {
        ThesaurusSelectBundle thesaurusSelectBundle = (ThesaurusSelectBundle) thesaurusSelectBundleMap.get(motcle.getThesaurus().getSubsetKey());
        if (thesaurusSelectBundle == null) {
            return 0;
        }
        return thesaurusSelectBundle.getPMask(motcle);
    }

    /**
     * Ajoute les critères de sélection définis par l'instance de MotcleFilterWrapper en argument.
     */
    public void addMotcleFilterWrapper(MotcleFilterWrapper wrapper) {
        int count = wrapper.getMotcleSelectOptionsCount();
        if (count == 0) {
            add(wrapper.getMotcleFilter(), new MotcleSelectOptions());
        } else {
            for (int i = 0; i < count; i++) {
                add(wrapper.getMotcleFilter(), wrapper.getMotcleSelectOptions(i));
            }
        }
    }

    /**
     * Ajoute le critère de sélection défini par l'instance de MotcleSelectOptions en argument.
     * Une instance par défaut de MotcleFilter est associée automatiquement à ce critère de sélection.
     */
    public void addMotcleSelectOptions(MotcleSelectOptions selectOptions) {
        MotcleFilter motcleFilter = new MotcleFilter();
        motcleFilter.setWithLib(true);
        add(motcleFilter, selectOptions);
    }

    /*
     * ajoute une paire filtre de mot-clé/critère de sélection.
     */
    private void add(MotcleFilter motcleFilter, MotcleSelectOptions motcleSelectOptions) {
        ThesaurusList thesaurusList = SelectionUtils.getThesaurusList(basedefiches, motcleSelectOptions);
        int thesaurusCount = thesaurusList.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = thesaurusList.getThesaurus(i);
            ThesaurusSelectBundle thesaurusSelectBundle = getThesaurusSelectBundle(thesaurus);
            thesaurusSelectBundle.add(motcleSelectOptions, motcleFilter);
        }
    }

    /*
     * Retourne l'instance de ThesaurusSelectBundle associée au thésaurus et la crèe au besoin.
     */
    private ThesaurusSelectBundle getThesaurusSelectBundle(Thesaurus thesaurus) {
        SubsetKey thesaurusKey = thesaurus.getSubsetKey();
        ThesaurusSelectBundle thesaurusSelectBundle = (ThesaurusSelectBundle) thesaurusSelectBundleMap.get(thesaurusKey);
        if (thesaurusSelectBundle == null) {
            thesaurusSelectBundle = new ThesaurusSelectBundle(thesaurus);
            thesaurusSelectBundleMap.put(thesaurusKey, thesaurusSelectBundle);
            thesaurusSelectBundleList.add(thesaurusSelectBundle);
        }
        return thesaurusSelectBundle;
    }

    /*
     *Classe interne conservant les informations sur les mots-clés sélectionnés d'un thésaurus.
     */

    class ThesaurusSelectBundle {

        Thesaurus thesaurus;
        MotcleSelectUnit[] motcleSelectUnitArray = new MotcleSelectUnit[2];
        MotcleFilter[] motcleFilterArray = new MotcleFilter[2];
        int size = 0;

        ThesaurusSelectBundle(Thesaurus thesaurus) {
            this.thesaurus = thesaurus;
        }

        void add(MotcleSelectOptions motcleSelectOptions, MotcleFilter motcleFilter) {
            if (size == motcleSelectUnitArray.length) {
                MotcleSelectUnit[] nvFSU = new MotcleSelectUnit[size * 2];
                System.arraycopy(motcleSelectUnitArray, 0, nvFSU, 0, size);
                motcleSelectUnitArray = nvFSU;
                MotcleFilter[] nvFB = new MotcleFilter[size * 2];
                System.arraycopy(motcleFilterArray, 0, nvFB, 0, size);
                motcleFilterArray = nvFB;
            }
            MotcleSelectUnit motcleSelectUnit = new MotcleSelectUnit(thesaurus, langInteger);
            motcleSelectUnit.setMotcleSelectOptions(motcleSelectOptions);
            motcleSelectUnitArray[size] = motcleSelectUnit;
            motcleFilterArray[size] = motcleFilter;
            size++;
        }

        MotcleFilter getMotcleFilter(Motcle motcle) {
            for (int i = 0; i < size; i++) {
                if (motcleSelectUnitArray[i].isSelected(motcle) > 0) {
                    return motcleFilterArray[i];
                }
            }
            return null;
        }

        int getPMask(Motcle motcle) {
            return motcleSelectUnitArray[0].isSelected(motcle);
        }

    }

}
