/* BasedefichesLib_Export - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tableexport;

import java.io.IOException;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.compute.ComputedBasedefichesFormatDef;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.format.Calcul;
import net.mapeadores.util.format.FormatDef;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.primitives.Decimal;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class ColDefUtils {

    private ColDefUtils() {
    }

    public static ColDefList fromArray(ColDef[] colDefArray) {
        return new ArrayColDefList(colDefArray);
    }

    public static void writeColDefList(Appendable appendable, ColDefList colDefList) throws IOException {
        int colDefCount = colDefList.getColDefCount();
        for (int i = 0; i < colDefCount; i++) {
            if (i > 0) {
                appendable.append("\n");
            }
            ColDef colDef = colDefList.getColDef(i);
            short type = colDef.getType();
            if (type == ColDef.EXTENSION_TYPE) {
                appendable.append("%");
            }
            appendable.append(colDef.getColName());
            String lib = colDef.getColLib();
            if (lib.length() > 0) {
                appendable.append(" > ");
                appendable.append(lib);
            }
            appendable.append('\n');
            switch (type) {
                case ColDef.FICHEFORMAT_TYPE:
                    BasedefichesFormatDef ficheFormatDef = colDef.getBasedefichesFormatDef();
                    String[] array = ficheFormatDef.toStringArray();
                    appendable.append(array[0]);
                    appendable.append('\n');
                    String format = array[1];
                    String param = array[2];
                    if (format == null) {
                        if (param != null) {
                            appendable.append("-\n");
                        }
                    } else {
                        appendable.append(format);
                        appendable.append('\n');
                    }
                    if (param != null) {
                        appendable.append(param);
                        appendable.append('\n');
                    }
                    break;
                case ColDef.EXTENSION_TYPE:
                    String[] lines = colDef.getExtensionLineArray();
                    int linesLength = lines.length;
                    for (int j = 0; j < linesLength; j++) {
                        appendable.append(lines[j]);
                        appendable.append('\n');
                    }
                    break;
                default:
                    new SwitchException("Unknown type = " + type);
            }
        }

    }

    public static void writeCell(ColDefTableWriter tableWriter, ColDef colDef, String chaine) {
        if (chaine == null) {
            tableWriter.addNullCell(colDef);
            return;
        }
        BasedefichesFormatDef basedefichesFormatDef = colDef.getBasedefichesFormatDef();
        Calcul calcul = basedefichesFormatDef.getCalcul();
        short castType = basedefichesFormatDef.getCastType();
        switch (castType) {
            case FormatDef.INTEGER_CAST:
                try {
                    Decimal decimal = StringUtils.parseDecimal(chaine);
                    long l = decimal.getPartieEntiere();
                    if (calcul != null) {
                        l = calcul.execute(l);
                    }
                    tableWriter.addIntegerCell(colDef, new Long(l));
                } catch (NumberFormatException nfe) {
                    tableWriter.addNullCell(colDef);
                }
                break;
            case FormatDef.DECIMAL_CAST:
                try {
                    Decimal decimal = StringUtils.parseDecimal(chaine);
                    if (calcul != null) {
                        double d = decimal.toDouble();
                        d = calcul.execute(d);
                        decimal = new Decimal(d);
                    }
                    tableWriter.addDecimalCell(colDef, decimal);
                } catch (NumberFormatException nfe) {
                    tableWriter.addNullCell(colDef);
                }
                break;
            case FormatDef.DATE_CAST:
                try {
                    int dateInteger = DateInteger.parse(chaine);
                    tableWriter.addDateCell(colDef, DateInteger.toDate(dateInteger));
                } catch (DateIntegerException ide) {
                    tableWriter.addNullCell(colDef);
                }
                break;
            default:
                if ((chaine.length() == 0) && (basedefichesFormatDef.isConvertEmptyToNull())) {
                    tableWriter.addNullCell(colDef);
                } else {
                    tableWriter.addStringCell(colDef, chaine);
                }
        }
    }


    private static class ArrayColDefList implements ColDefList {

        private ColDef[] colDefArray;

        ArrayColDefList(ColDef[] colDefArray) {
            this.colDefArray = colDefArray;
        }

        public int getColDefCount() {
            return colDefArray.length;
        }

        public ColDef getColDef(int i) {
            return colDefArray[i];
        }

        public String getParameterValue(String parameterName) {
            return null;
        }

    }

    public static ColDefTestHandler getSubsetTestHandler(Subset subset, ColDefErrorHandler errorHandler) {
        return new SubsetTestHandler(subset, errorHandler);
    }


    private static class SubsetTestHandler implements ColDefTestHandler {

        private Subset subset;
        private ColDefErrorHandler errorHandler;

        SubsetTestHandler(Subset subset, ColDefErrorHandler errorHandler) {
            this.subset = subset;
            this.errorHandler = errorHandler;
        }

        public boolean testBasedefichesFormatDef(int lineNumber, BasedefichesFormatDef basedefichesFormatDef) {
            ComputedBasedefichesFormatDef.ErrorHandler eh2 = new ComputedFicheFormatErrorHandler(lineNumber + 1);
            ComputedBasedefichesFormatDef cffd = ComputedBasedefichesFormatDef.compute(subset, basedefichesFormatDef, eh2);
            return (cffd != null);
        }


        private class ComputedFicheFormatErrorHandler implements ComputedBasedefichesFormatDef.ErrorHandler {

            int sourceLineNumber;

            ComputedFicheFormatErrorHandler(int sourceLineNumber) {
                this.sourceLineNumber = sourceLineNumber;
            }

            public void unknownSubfieldKey(SubfieldKey subfieldKey) {
                errorHandler.formatSourceError(sourceLineNumber, "unknownSubfieldKey", subfieldKey);
            }

            public void illegalSubfieldKey(SubfieldKey subfieldKey) {
                errorHandler.formatSourceError(sourceLineNumber, "illegalSubfieldKey", subfieldKey);
            }

            public void wrongFormatPattern(String formatPattern) {
                errorHandler.formatSourceError(sourceLineNumber + 1, "wrongFormatPattern", formatPattern);
            }

            public void unknownFieldKey(FieldKey fieldKey) {
                errorHandler.formatSourceError(sourceLineNumber, "unknownFieldKey", fieldKey);
            }

            public void unknownSubsetKey(SubsetKey subsetKey) {
                errorHandler.formatSourceError(sourceLineNumber, "unknownSubsetKey", subsetKey);
            }

            public void notASlaveCorpusKey(SubsetKey subsetKey) {
                errorHandler.formatSourceError(sourceLineNumber, "notASlaveCorpusKey", subsetKey);
            }

            public void illegalFormatSourceKey(String keyString) {
                errorHandler.formatSourceError(sourceLineNumber, "illegalFormatSourceKey", keyString);
            }

        }

    }

    public static ColDef newBasedefichesFormatDefInstance(String colName, String colLib, BasedefichesFormatDef ficheFormatDef) {
        InternalColDef defaultColDef = new InternalColDef(ColDef.FICHEFORMAT_TYPE, colName);
        defaultColDef.setColLib(colLib);
        defaultColDef.setFicheFormatDef(ficheFormatDef);
        return defaultColDef;
    }

    public static ColDef newExtensionInstance(String colName, String colLib, String[] lines) {
        InternalColDef defaultColDef = new InternalColDef(ColDef.EXTENSION_TYPE, colName);
        defaultColDef.setColLib(colLib);
        defaultColDef.setCustomLineArray(lines);
        return defaultColDef;
    }


    private static class InternalColDef implements ColDef {

        private BasedefichesFormatDef ficheFormatDef;
        private String colName;
        private String colLib = "";
        private String[] customLineArray;
        private short type;

        private InternalColDef(short type, String colName) {
            this.type = type;
            this.colName = colName;
        }

        public String getColName() {
            return colName;
        }

        public String getColLib() {
            return colLib;
        }

        public short getType() {
            return type;
        }

        public BasedefichesFormatDef getBasedefichesFormatDef() {
            return ficheFormatDef;
        }

        public String[] getExtensionLineArray() {
            return customLineArray;
        }

        private void setCustomLineArray(String[] customLineArray) {
            this.customLineArray = customLineArray;
        }

        private void setFicheFormatDef(BasedefichesFormatDef ficheFormatDef) {
            this.ficheFormatDef = ficheFormatDef;
        }

        private void setColLib(String colLib) {
            if (colLib == null) {
                this.colLib = "";
            } else {
                this.colLib = colLib;
            }
        }

    }

}
