/* BasedefichesLib_Impl - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl.croisements;

import net.mapeadores.basedefiches.croisement.CroisementEditor;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.croisement.CroisementList;


/**
 *
 * @author Vincent Calame
 */
public class ImplCroisementItem implements CroisementItem {

    private int[] ticketArray;
    private int[] poidsArray;

    public ImplCroisementItem() {
        this.ticketArray = new int[0];
        this.poidsArray = new int[0];
    }

    public ImplCroisementItem(int[] ticketArray, int[] poidsArray) {
        this.ticketArray = ticketArray;
        this.poidsArray = poidsArray;
    }

    public int getCount() {
        return ticketArray.length;
    }

    public int getPoids(int i) {
        return poidsArray[i];
    }

    public int getTicket(int i) {
        return ticketArray[i];
    }

    public int indexOfTicket(int ticket) {
        return indexOf(ticketArray, ticket);
    }

    public UpdateResult clear() {
        UpdateResult updateResult = new UpdateResult(ticketArray);
        ticketArray = new int[0];
        poidsArray = new int[0];
        return updateResult;
    }

    public UpdateResult update(CroisementList croisementList, int avoidTicket, short updateType) {
        if (updateType == CroisementEditor.CLEAR_EXISTING) {
            return replace(croisementList, avoidTicket);
        } else {
            return append(croisementList, avoidTicket, updateType);
        }
    }

    public UpdateResult append(CroisementList croisementList, int forbiddenTicket, short updateType) {
        int length = ticketArray.length;
        int newLength = croisementList.getSubsetItemCount();
        UpdateResult updateResult = new UpdateResult(length, newLength);
        for (int i = 0; i < newLength; i++) {
            int newTicket = croisementList.getTicket(i);
            int newPoids = croisementList.getPoids(i);
            int idx = indexOf(ticketArray, newTicket);
            if (idx == -1) {
                updateResult.addAdded(newTicket, newPoids);
            } else {
                int currentPoids = poidsArray[idx];
                boolean replace = false;
                if (updateType == CroisementEditor.APPEND_AND_POIDS_REPLACE) {
                    if (newPoids != currentPoids) {
                        replace = true;
                    }
                } else if (updateType == CroisementEditor.APPEND_AND_POIDS_MAX) {
                    if (newPoids > currentPoids) {
                        replace = true;
                    }
                }
                if (replace) {
                    poidsArray[idx] = newPoids;
                    updateResult.addUpdated(newTicket, newPoids);
                }
            }
        }
        int addedCount = updateResult.getAddedCount();
        if (addedCount > 0) {
            enlarge(addedCount);
            for (int i = 0; i < addedCount; i++) {
                ticketArray[i + length] = updateResult.getAddedTicket(i);
                poidsArray[i + length] = updateResult.getAddedPoids(i);
            }
        }
        return updateResult;
    }

    private UpdateResult replace(CroisementList croisementList, int forbiddenTicket) {
        int length = ticketArray.length;
        int newLength = croisementList.getSubsetItemCount();
        UpdateResult updateResult = new UpdateResult(length, newLength);
        for (int i = 0; i < length; i++) {
            int currentTicket = ticketArray[i];
            int idx = indexOf(croisementList, currentTicket);
            if (idx == -1) {
                updateResult.addRemoved(currentTicket);
            } else {
                int currentPoids = poidsArray[i];
                int newPoids = croisementList.getPoids(idx);
                if (currentPoids != newPoids) {
                    updateResult.addUpdated(currentTicket, newPoids);
                }
            }
        }
        int[] newTicketArray = new int[newLength];
        int[] newPoidsArray = new int[newLength];
        int p = 0;
        for (int i = 0; i < newLength; i++) {
            int newTicket = croisementList.getTicket(i);
            if (newTicket == forbiddenTicket) {
                continue;
            }
            int newPoids = croisementList.getPoids(i);
            newTicketArray[p] = newTicket;
            newPoidsArray[p] = newPoids;
            p++;
            if (indexOf(ticketArray, newTicket) == -1) {
                updateResult.addAdded(newTicket, newPoids);
            }
        }
        if (p < newLength) {
            int[] tempo = new int[newLength - 1];
            System.arraycopy(newTicketArray, 0, tempo, 0, newLength - 1);
            newTicketArray = tempo;
            System.arraycopy(newPoidsArray, 0, tempo, 0, newLength - 1);
            newPoidsArray = tempo;
        }
        ticketArray = newTicketArray;
        poidsArray = newPoidsArray;
        return updateResult;
    }

    private int indexOf(CroisementList croisementList, int ticket) {
        int size = croisementList.getSubsetItemCount();
        for (int i = 0; i < size; i++) {
            if (croisementList.getTicket(i) == ticket) {
                return i;
            }
        }
        return -1;
    }

    public void removeTicket(int ticket) {
        int index = indexOf(ticketArray, ticket);
        if (index == -1) {
            throw new IllegalArgumentException("ticket not found : " + ticket);
        }
        removeIndex(index);
    }

    public void updateTicket(int ticket, int poids) {
        int index = indexOf(ticketArray, ticket);
        if (index == -1) {
            throw new IllegalArgumentException("ticket not found : " + ticket);
        }
        poidsArray[index] = poids;
    }

    public void addTicket(int ticket, int poids) {
        int index = indexOf(ticketArray, ticket);
        if (index != -1) {
            throw new IllegalArgumentException("ticket not found : " + ticket);
        }
        int length = ticketArray.length;
        enlarge(1);
        ticketArray[length] = ticket;
        poidsArray[length] = poids;
    }

    protected void removeIndex(int index) {
        ticketArray = removeIndex(ticketArray, index);
        poidsArray = removeIndex(poidsArray, index);
    }

    private int[] removeIndex(int[] array, int index) {
        int length = array.length - 1;
        int[] result = new int[length];
        if (index > 0) {
            System.arraycopy(array, 0, result, 0, index);
        }
        if (index < length) {
            System.arraycopy(array, index + 1, result, index, length - index);
        }
        return result;
    }

    private int indexOf(int[] array, int val) {
        int length = array.length;
        for (int i = 0; i < length; i++) {
            if (array[i] == val) {
                return i;
            }
        }
        return -1;
    }

    private void enlarge(int size) {
        ticketArray = enlarge(ticketArray, size);
        poidsArray = enlarge(poidsArray, size);
    }

    private int[] enlarge(int[] array, int size) {
        int length = array.length;
        int[] result = new int[length + size];
        System.arraycopy(array, 0, result, 0, length);
        return result;
    }


    public static class UpdateResult {

        private int removeCount = 0;
        private int updateCount = 0;
        private int addedCount = 0;
        private int[] removeArray;
        private int[] updateTicketArray;
        private int[] updatePoidsArray;
        private int[] addedTicketArray;
        private int[] addedPoidsArray;

        UpdateResult(int[] removeArray) {
            this.removeArray = removeArray;
            this.removeCount = removeArray.length;
        }

        UpdateResult(int maxlength, int maxAddLength) {
            removeArray = new int[maxlength];
            updateTicketArray = new int[maxlength];
            updatePoidsArray = new int[maxlength];
            addedTicketArray = new int[maxAddLength];
            addedPoidsArray = new int[maxAddLength];
        }

        public int getRemoveCount() {
            return removeCount;
        }

        public int getUpdateCount() {
            return updateCount;
        }

        public int getAddedCount() {
            return addedCount;
        }

        public int getRemoveTicket(int i) {
            return removeArray[i];
        }

        public int getUpdateTicket(int i) {
            return updateTicketArray[i];
        }

        public int getUpdatePoids(int i) {
            return updatePoidsArray[i];
        }

        public int getAddedTicket(int i) {
            return addedTicketArray[i];
        }

        public int getAddedPoids(int i) {
            return addedPoidsArray[i];
        }

        void addRemoved(int ticket) {
            removeArray[removeCount] = ticket;
            removeCount++;
        }

        void addUpdated(int ticket, int poids) {
            updateTicketArray[updateCount] = ticket;
            updatePoidsArray[updateCount] = poids;
            updateCount++;
        }

        void addAdded(int ticket, int poids) {
            addedTicketArray[addedCount] = ticket;
            addedPoidsArray[addedCount] = poids;
            addedCount++;
        }

    }

}
