/* BdfServer - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine;

import java.util.List;
import java.util.ArrayList;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class BalayageLog {

    public static final short UNRESOLVED_PATH = 1;
    public static final short WRONG_PATTERN = 2;
    public static final short NO_OUTPUT = 3;
    public static final short EXTRACTION_NOT_FOUND = 4;
    public static final short EXTRACTION_EXCEPTION = 5;
    public static final short OUTPUTPATH_EXCEPTION = 6;
    public static final short TRANSFORMER_NOT_FOUND = 7;
    public static final short TRANSFORMER_EXCEPTION = 8;
    public static final short NO_XSLTPATH = 9;
    public static final short WRONG_PATH = 10;
    public static final short TRANSFORMATION_EXCEPTION = 11;
    public static final short IO_EXCEPTION = 12;
    public static final short COMMANDCHECK_EXCEPTION = 13;
    public static final short COMMANDIO_EXCEPTION = 14;
    public static final short COMMANDINTERRUPTED_EXCEPTION = 15;
    public static final String EXTRACTION_ATTRIBUTE = "extraction";
    private List<LogItem> logList = new ArrayList<LogItem>();
    private long runStartTime;
    private PrintWriter fileWriter;
    private boolean hasError = false;

    public BalayageLog(String balayageName, File logFile) {
        try {
            fileWriter = new PrintWriter(new OutputStreamWriter(new FileOutputStream(logFile), "UTF-8"));
        } catch (IOException ioe) {
            fileWriter = null;
        }
    }

    public boolean hasError() {
        return hasError;
    }

    public void addBalayageUnitRun(int balayageUnit) {
        LogItem logItem = new BalayageUnitRun(balayageUnit);
        logList.add(logItem);
        printAll(logItem);
    }

    public void addBalayageUnitWarning(int balayageUnitIndex, short type, String value, String message) {
        LogItem logItem = new BalayageUnitWarning(balayageUnitIndex, type, value, message);
        logList.add(logItem);
        hasError = true;
        printAll(logItem);
    }

    public void addOutputWarning(int balayageUnitIndex, int outputIndex, String localisation, short type, String value) {
        LogItem logItem = new OutputWarning(balayageUnitIndex, outputIndex, localisation, type, value);
        logList.add(logItem);
        hasError = true;
        printAll(logItem);
    }

    public void addOutputException(int balayageUnitIndex, int outputIndex, short type, String value, String message) {
        LogItem logItem = new OutputException(balayageUnitIndex, outputIndex, type, value, message);
        logList.add(logItem);
        hasError = true;
        printAll(logItem);
    }

    public void addCommandException(short type, String message) {
        LogItem logItem = new CommandException(type, message);
        logList.add(logItem);
        hasError = true;
        printAll(logItem);
    }

    public void addCommandRun(String scriptName) {
        LogItem logItem = new CommandRun(scriptName);
        logList.add(logItem);
        printAll(logItem);
    }

    public void addFileGeneration(int balayageUnitIndex, int outputIndex, File f, long time) {
        LogItem logItem = new FileGeneration(balayageUnitIndex, outputIndex, f, time);
        printAll(logItem);
    }

    public void addSiteMapGeneration(String path) {
        LogItem logItem = new SiteMapGeneration(path);
        printAll(logItem);
    }

    public void addSiteMapException(short type, String message) {
        LogItem logItem = new SiteMapException(type, message);
        logList.add(logItem);
        hasError = true;
        printAll(logItem);
    }

    public void runStarted() {
        runStartTime = System.currentTimeMillis();
    }

    public void runEnded() {
        LogItem logItem = new RunEnded();
        logList.add(logItem);
        printAll(logItem);
    }

    private void printAll(LogItem logItem) {
        if (fileWriter != null) {
            logItem.print(fileWriter);
        }
    }

    public void close() {
        if (fileWriter != null) {
            fileWriter.close();
        }
    }

    public void printLog(PrintWriter pw) {
        for (LogItem logItem : logList) {
            logItem.print(pw);
        }
    }


    private abstract class LogItem {

        public abstract void print(PrintWriter pw);

    }


    private class RunEnded extends LogItem {

        long time;

        RunEnded() {
            time = System.currentTimeMillis();
        }

        public void print(PrintWriter pw) {
            pw.println("");
            pw.print("===== Generation time  : ");
            printTime(pw, time - runStartTime);
            pw.println("");
        }

    }


    private class BalayageUnitRun extends LogItem {

        int balayageUnitIndex;
        long time;
        int freeMemory;

        BalayageUnitRun(int balayageUnitIndex) {
            this.balayageUnitIndex = balayageUnitIndex;
            time = System.currentTimeMillis();
            freeMemory = (int) Math.floor(((double) Runtime.getRuntime().freeMemory()) / (1024 * 1024));
        }

        public void print(PrintWriter pw) {
            pw.println("");
            pw.print("+------- ");
            pw.print("balayageUnit: ");
            pw.print(balayageUnitIndex + 1);
            pw.print(" (+ ");
            printTime(pw, time - runStartTime);
            pw.println(") / mem = " + freeMemory);
        }

    }


    private class CommandRun extends LogItem {

        String scriptName;

        CommandRun(String scriptName) {
            this.scriptName = scriptName;
        }

        public void print(PrintWriter pw) {
            pw.println("");
            pw.print("+------- ");
            pw.print("command: ");
            pw.println(scriptName);
        }

    }


    private class BalayageUnitWarning extends LogItem {

        int balayageUnitIndex;
        String message;
        short type;
        String value;

        BalayageUnitWarning(int balayageUnitIndex, short type, String value, String message) {
            this.balayageUnitIndex = balayageUnitIndex;
            this.message = message;
            this.type = type;
            this.value = value;
        }

        public void print(PrintWriter pw) {
            pw.println("");
            pw.print("[! ");
            pw.print(typeToString(type));
            pw.print(" !] ");
            pw.print("balayageUnit: ");
            pw.print(balayageUnitIndex + 1);
            if (value != null) {
                pw.print(" value: ");
                pw.print(value);
            }
            if (message != null) {
                pw.print(" message: ");
                pw.print(message);
            }
            pw.println("");

        }

    }


    private class OutputWarning extends LogItem {

        int balayageUnitIndex;
        int outputIndex;
        String localisation;
        short type;
        String value;

        OutputWarning(int balayageUnitIndex, int outputIndex, String localisation, short type, String value) {
            this.balayageUnitIndex = balayageUnitIndex;
            this.outputIndex = outputIndex;
            this.localisation = localisation;
            this.type = type;
            this.value = value;
        }

        public void print(PrintWriter pw) {
            pw.println("");
            pw.print("[! ");
            pw.print(typeToString(type));
            pw.print(" !] ");
            pw.print("balayageUnit: ");
            pw.print(balayageUnitIndex + 1);
            pw.print(" output: ");
            pw.print(outputIndex + 1);
            pw.print(" attribute: ");
            pw.print(localisation);
            pw.print(" value: ");
            pw.println(value);
        }

    }


    private class OutputException extends LogItem {

        int balayageUnitIndex;
        int outputIndex;
        String message;
        short type;
        String value;

        OutputException(int balayageUnitIndex, int outputIndex, short type, String value, String message) {
            this.balayageUnitIndex = balayageUnitIndex;
            this.outputIndex = outputIndex;
            this.message = message;
            this.type = type;
            this.value = value;
        }

        public void print(PrintWriter pw) {
            pw.println("");
            pw.print("[! ");
            pw.print(typeToString(type));
            pw.print(" !] ");
            pw.print("balayageUnit: ");
            pw.print(balayageUnitIndex + 1);
            pw.print(" output: ");
            pw.print(outputIndex + 1);
            if (value != null) {
                pw.print(" value: ");
                pw.print(value);
            }
            if (message != null) {
                pw.print(" message: ");
                pw.print(message);
            }
            pw.println("");

        }

    }


    private class CommandException extends LogItem {

        short type;
        String message;

        CommandException(short type, String message) {
            this.type = type;
            this.message = message;
        }

        public void print(PrintWriter pw) {
            pw.println("");
            pw.print("[! ");
            pw.print(typeToString(type));
            pw.print(" !] ");
            pw.print(message);
            pw.println("");

        }

    }


    private class SiteMapException extends LogItem {

        short type;
        String message;

        SiteMapException(short type, String message) {
            this.type = type;
            this.message = message;
        }

        public void print(PrintWriter pw) {
            pw.println("");
            pw.print("[! ");
            pw.print(typeToString(type));
            pw.print(" !] ");
            pw.print(message);
            pw.println("");

        }

    }


    private class SiteMapGeneration extends LogItem {

        String path;

        SiteMapGeneration(String path) {
            this.path = path;
        }

        public void print(PrintWriter pw) {
            pw.println("");
            pw.print("+------- ");
            pw.print("sitemap: ");
            pw.println(path);
        }

    }


    private class FileGeneration extends LogItem {

        int balayageUnitIndex;
        int outputIndex;
        String path;
        long time;
        int freeMemory;

        FileGeneration(int balayageUnitIndex, int outputIndex, File f, long time) {
            this.balayageUnitIndex = balayageUnitIndex;
            this.outputIndex = outputIndex;
            this.time = time;
            path = f.getAbsolutePath();
            freeMemory = (int) Math.floor(((double) Runtime.getRuntime().freeMemory()) / (1024 * 1024));
        }

        public void print(PrintWriter pw) {
            pw.print("out: ");
            pw.print(balayageUnitIndex + 1);
            pw.print(".");
            pw.print(outputIndex + 1);
            pw.print(" path: ");
            pw.print(path);
            pw.print(" (");
            printTime(pw, time);
            pw.println(") / mem = " + freeMemory);
        }

    }

    static String typeToString(short type) {
        switch (type) {
            case UNRESOLVED_PATH:
                return "unresolvedPath";
            case WRONG_PATTERN:
                return "wrongPattern";
            case NO_OUTPUT:
                return "noOutput";
            case EXTRACTION_NOT_FOUND:
                return "extractionNotFound";
            case EXTRACTION_EXCEPTION:
                return "extractionException";
            case OUTPUTPATH_EXCEPTION:
                return "outputPathException";
            case TRANSFORMER_NOT_FOUND:
                return "transformerNotFound";
            case TRANSFORMER_EXCEPTION:
                return "transformerException";
            case TRANSFORMATION_EXCEPTION:
                return "transformerExceptionDuringTransformation";
            case IO_EXCEPTION:
                return "ioException";
            case NO_XSLTPATH:
                return "noXsltPath";
            case WRONG_PATH:
                return "wrongPath";
            case COMMANDCHECK_EXCEPTION:
                return "commandCheck";
            case COMMANDIO_EXCEPTION:
                return "commandIOException";
            case COMMANDINTERRUPTED_EXCEPTION:
                return "commandInterruptedException";
            default:
                return "";
        }
    }

    static void printTime(PrintWriter pw, long time) {
        float f = ((float) time) / 1000;
        pw.print(String.valueOf(f));
        pw.print("s");
    }

}
