/* BdfServer - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine.runners;

import net.mapeadores.basedefiches.CorpusList;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.ThesaurusList;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.extraction.Extraction;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;
import net.mapeadores.basedefiches.selection.MotcleSelector;
import net.mapeadores.basedefiches.selection.SelectionUtils;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.xml.extraction.ExtractionXMLWriter;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.balayages.Balayage;
import fr.exemole.bdfserver.balayages.BalayageMode;
import fr.exemole.bdfserver.balayages.BalayageUnit;
import fr.exemole.bdfserver.balayages.engine.BalayageLog;
import fr.exemole.bdfserver.balayages.engine.BalayageParameters;
import fr.exemole.bdfserver.balayages.engine.EngineUtils;
import fr.exemole.bdfserver.managers.BalayageManager;
import net.mapeadores.util.text.ValueResolver;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.exceptions.NestedSAXException;
import net.mapeadores.util.primitives.DateInteger;
import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class TransformBalayageUnitRunner extends AbstractBalayageUnitRunner {

    private TransformOutputRunner[] outputRunnerArray;
    private Extraction extraction;
    private boolean testCurrentDate = false;
    private boolean withTransformation;

    public TransformBalayageUnitRunner(BalayageParameters balayageParameters, int balayageIndex, BalayageUnit balayageUnit) {
        super(balayageParameters, balayageIndex, balayageUnit);
        BdfServer bdfServer = balayageParameters.getBdfServer();
        Balayage balayage = balayageParameters.getBalayage();
        this.withTransformation = balayage.isWithTransformation();
        List<TransformOutputRunner> outputList = new ArrayList<TransformOutputRunner>();
        int outputCount = balayageUnit.getOutputCount();
        int p = 0;
        for (int i = 0; i < outputCount; i++) {
            TransformOutputRunner outputRunner = new TransformOutputRunner(balayageParameters, p, balayageUnit.getOutput(i), balayageIndex);
            if (outputRunner.isValid()) {
                outputList.add(outputRunner);
                p++;
            }
        }
        outputRunnerArray = outputList.toArray(new TransformOutputRunner[outputList.size()]);
        BalayageManager balayageManager = bdfServer.getBalayageManager();
        int type = balayageUnit.getType();
        String extractionPath = balayageUnit.getExtractionPath();
        if (extractionPath != null) {
            try {
                extraction = balayageManager.getExtraction(balayage, extractionPath, localisationLists, ficheEligibility);
                if (extraction == null) {
                    balayageLog.addBalayageUnitWarning(balayageIndex, BalayageLog.EXTRACTION_NOT_FOUND, extractionPath, null);
                }
            } catch (NestedIOException ube) {
                balayageLog.addBalayageUnitWarning(balayageIndex, BalayageLog.EXTRACTION_EXCEPTION, extractionPath, ube.getMessage());
            } catch (NestedSAXException ube) {
                balayageLog.addBalayageUnitWarning(balayageIndex, BalayageLog.EXTRACTION_EXCEPTION, extractionPath, ube.getMessage());
            }
        }
        if (extraction == null) {
            extraction = balayageManager.getDefaultExtraction(balayageUnit.getType(), localisationLists, ficheEligibility);
        }
        if (type == FICHE_TYPE) {
            BalayageMode balayageMode = balayageParameters.getBalayageMode();
            if (balayageMode != null) {
                if (balayageMode.isWithCurrentDateLimit()) {
                    testCurrentDate = true;
                }
            }
        }
    }

    protected boolean isEmpty() {
        return (outputRunnerArray.length == 0);
    }

    protected void run(short balayageUnitType) {
        switch (balayageUnitType) {
            case CORPUS_TYPE:
                runCorpusBalayage();
                break;
            case FICHE_TYPE:
                runFicheBalayage();
                break;
            case MOTCLE_TYPE:
                runMotcleBalayage();
                break;
            case THESAURUS_TYPE:
                runThesaurusBalayage();
                break;
            case UNIQUE_TYPE:
                runUniqueBalayage();
                break;
        }
    }

    @Override
    public void dispose() {
        super.dispose();
        for (int i = 0; i < outputRunnerArray.length; i++) {
            outputRunnerArray[i].dispose();
            outputRunnerArray[i] = null;
        }
        outputRunnerArray = null;
        extraction = null;
    }

    private void runCorpusBalayage() {
        String extractionString = null;
        CorpusList corpusList = SelectionUtils.getCorpusList(basedefiches, balayageUnit.getFicheSelectOptions());
        int corpusCount = corpusList.getCorpusCount();
        for (int j = 0; j < corpusCount; j++) {
            long startTime = System.currentTimeMillis();
            Corpus corpus = corpusList.getCorpus(j);
            extractionString = ExtractionXMLWriter.toString(extraction, corpus, !withTransformation);
            ValueResolver patternValueSet = EngineUtils.getValueResolver(corpus);
            for (int i = 0; i < outputRunnerArray.length; i++) {
                if (i > 0) {
                    startTime = System.currentTimeMillis();
                }
                TransformOutputRunner outputRunner = outputRunnerArray[i];
                outputRunner.write(corpus, patternValueSet, extractionString, startTime);
            }
        }
        extractionString = null;
    }

    private void runFicheBalayage() {
        String extractionString = null;
        FicheSelector ficheSelector = new FicheSelector(basedefiches, localisationLists.getWorkingLangInteger(0));
        FicheSelectOptions ficheSelectOptions = balayageUnit.getFicheSelectOptions();
        boolean globalSelect = balayageUnit.isGlobalSelect();
        if (ficheSelectOptions == null) {
            ficheSelector.addFicheSelectOptions(new FicheSelectOptions());
            globalSelect = true;
        } else {
            ficheSelector.addFicheSelectOptions(ficheSelectOptions);
        }
        if (globalSelect) {
            ficheSelector.setFicheEligibility(ficheEligibility);
        }
        AbregeFicheSortedList abregeFicheSortedList = SelectionUtils.getAbregeFicheSortedList(ficheSelector, AbregeFicheSortedList.IDCORPUS_SORTTYPE);
        for (int k = 0; k < abregeFicheSortedList.getCorpusCount(); k++) {
            Corpus corpus = abregeFicheSortedList.getCorpus(k);
            SubsetItemList abregeFicheList = abregeFicheSortedList.getCorpusAbregeFicheList(k);
            int currentDate = DateInteger.current();
            int ficheCount = abregeFicheList.getSubsetItemCount();
            for (int j = 0; j < ficheCount; j++) {
                long startTime = System.currentTimeMillis();
                AbregeFiche abrege = (AbregeFiche) abregeFicheList.getSubsetItem(j);
                if (testCurrentDate) {
                    if (abrege.getModificationDate() != currentDate) {
                        continue;
                    }
                }
                extractionString = ExtractionXMLWriter.toString(extraction, abrege, !withTransformation);
                ValueResolver patternValueSet = EngineUtils.getValueResolver(abrege);
                for (int i = 0; i < outputRunnerArray.length; i++) {
                    if (i > 0) {
                        startTime = System.currentTimeMillis();
                    }
                    TransformOutputRunner outputRunner = outputRunnerArray[i];
                    outputRunner.write(abrege, patternValueSet, extractionString, startTime);
                }
            }
        }
        extractionString = null;
    }

    private void runUniqueBalayage() {
        long startTime = System.currentTimeMillis();
        String extractionString = ExtractionXMLWriter.toString(extraction, null, !withTransformation);
        ValueResolver patternValueSet = EngineUtils.getValueResolver(null);
        for (int i = 0; i < outputRunnerArray.length; i++) {
            if (i > 0) {
                startTime = System.currentTimeMillis();
            }
            TransformOutputRunner outputRunner = outputRunnerArray[i];
            outputRunner.write(null, patternValueSet, extractionString, startTime);
        }
        extractionString = null;
    }

    private void runThesaurusBalayage() {
        String extractionString = null;
        ThesaurusList thesaurusList = SelectionUtils.getThesaurusList(basedefiches, balayageUnit.getMotcleSelectOptions());
        int thesaurusCount = thesaurusList.getThesaurusCount();
        for (int j = 0; j < thesaurusCount; j++) {
            long startTime = System.currentTimeMillis();
            Thesaurus thesaurus = thesaurusList.getThesaurus(j);
            extractionString = ExtractionXMLWriter.toString(extraction, thesaurus, !withTransformation);
            ValueResolver patternValueSet = EngineUtils.getValueResolver(thesaurus);
            for (int i = 0; i < outputRunnerArray.length; i++) {
                if (i > 0) {
                    startTime = System.currentTimeMillis();
                }
                TransformOutputRunner outputRunner = outputRunnerArray[i];
                outputRunner.write(thesaurus, patternValueSet, extractionString, startTime);
            }
        }
        extractionString = null;
    }

    private void runMotcleBalayage() {
        MotcleSelectOptions motcleSelectOptions = balayageUnit.getMotcleSelectOptions();
        MotcleSelector motcleSelector = new MotcleSelector(basedefiches, localisationLists.getWorkingLangInteger(0));
        if (motcleSelectOptions != null) {
            motcleSelector.addMotcleSelectOptions(motcleSelectOptions);
        } else {
            motcleSelector.addMotcleSelectOptions(new MotcleSelectOptions());
        }
        for (int i = 0; i < motcleSelector.getThesaurusCount(); i++) {
            Thesaurus thesaurus = (Thesaurus) motcleSelector.getThesaurus(i);
            for (int j = 0; j < thesaurus.getTicketLength(); j++) {
                Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(j);
                if (motcle != null) {
                    if (motcleSelector.isSelected(motcle)) {
                        writeMotcle(motcle);
                    }
                }
            }
        }
    }

    private void writeMotcle(Motcle motcle) {
        long startTime = System.currentTimeMillis();
        String extractionString = ExtractionXMLWriter.toString(extraction, motcle, !withTransformation);
        ValueResolver patternValueSet = EngineUtils.getValueResolver(motcle);
        for (int i = 0; i < outputRunnerArray.length; i++) {
            if (i > 0) {
                startTime = System.currentTimeMillis();
            }
            TransformOutputRunner outputRunner = outputRunnerArray[i];
            outputRunner.write(motcle, patternValueSet, extractionString, startTime);
        }
        extractionString = null;
    }

}
