/* BdfServer
 *
 * Copyright (C) 2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages.engine.runners;

import fr.exemole.bdfserver.balayages.*;
import fr.exemole.bdfserver.*;
import java.io.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.io.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.langinteger.*;
import fr.exemole.bdfserver.balayages.engine.*;


/**
 *
 * @author Vincent Calame
 */
public class TransformOutputRunner extends AbstractOutputRunner {

    private short langOption = LANG_OPTION_UNKNOWN;
    private String[] langArray;
    private TransformManager transformManager;
    private boolean withTransformation;
    private FileGenerationListener fileGenerationListener;

    public TransformOutputRunner(BalayageParameters balayageParameters, int outputIndex, Output output, int balayageUnitIndex) {
        super(balayageParameters, outputIndex, output, balayageUnitIndex);
        Balayage balayage = balayageParameters.getBalayage();
        BdfServer bdfServer = balayageParameters.getBdfServer();
        this.withTransformation = balayage.isWithTransformation();
        this.fileGenerationListener = balayageParameters.getFileGenerationListener();
        initLangArray();
        langOption = output.getLangOption();
        if (langOption == LANG_OPTION_UNKNOWN) {
            langOption = balayage.getDefaultLangOption();
        }
        if (balayage.isWithTransformation()) {
            InternalTransformerBuilderErrorHandler transformerBuilderErrorHandler = new InternalTransformerBuilderErrorHandler();
            TransformManagerBuilder transformerBuilder = new TransformManagerBuilder(bdfServer, balayage, transformerBuilderErrorHandler);
            transformManager = transformerBuilder.build(output);
            if (transformManager != null) {
                File balayageRootDirectory = balayageParameters.getBalayageRootDirectory();
                String absolutePath = balayageRootDirectory.getAbsolutePath();
                if (!absolutePath.endsWith(File.separator)) {
                    absolutePath = absolutePath + File.separator;
                }
                transformManager.setBalayageRoot(absolutePath);
            }
        }
    }


    private class InternalTransformerBuilderErrorHandler implements TransformManagerBuilder.ErrorHandler {

        public void transformerError(String xsltPath, TransformerErrorListener errorListener) {
            isValid = false;
            balayageLog.addOutputException(balayageUnitIndex, outputIndex, BalayageLog.TRANSFORMER_EXCEPTION, xsltPath, errorListener.getErrorMessage());
        }

        public void wrongXslPath(String xsltPath) {
            balayageLog.addBalayageUnitWarning(balayageUnitIndex, BalayageLog.TRANSFORMER_NOT_FOUND, xsltPath, null);
            isValid = false;
        }

        public void undefinedXslPath() {
            balayageLog.addOutputException(balayageUnitIndex, outputIndex, BalayageLog.NO_XSLTPATH, null, null);
            isValid = false;
        }

        public void wrongBalayageUnitType() {
        }

    }

    private void initLangArray() {
        LocalisationLists localisationsLists = output.getLocalisationLists();
        if (localisationsLists == null) {
            this.langArray = balayageParameters.getLangArray();
        } else {
            int count = localisationsLists.getWorkingLangCount();
            langArray = new String[count];
            for (int i = 0; i < count; i++) {
                langArray[i] = LangInteger.toString(localisationsLists.getWorkingLangInteger(i));
            }
        }
    }

    public void write(Object extractionObject, ValueResolver valueResolver, String extractionString, long startTime) {
        String fileName = EngineUtils.getFileName(output, valueResolver, output.getBalayageUnit().getType(), withTransformation);
        String path = EngineUtils.getPath(output, valueResolver);
        if (langOption == LANG_OPTION_NONE) {
            write(extractionObject, fileName, path, null, extractionString, startTime);
        } else {
            long time = startTime;
            for (int i = 0; i < langArray.length; i++) {
                time = write(extractionObject, fileName, path, langArray[i], extractionString, time);
            }
        }
    }

    private long write(Object extractionObject, String fileName, String path, String lang, String extractionString, long startTime) {
        File directory = rootDirectory;
        if (langOption == LANG_OPTION_DIR) {
            directory = new File(directory, lang);
        }
        if (path != null) {
            directory = new File(directory, path);
        }
        prepareDirectory(directory);
        File file = new File(directory, fileName);
        boolean done = false;
        if (transformManager != null) {
            transformManager.setCurrentFileName(fileName);
            transformManager.setCurrentLang(lang);
            transformManager.setCurrentPath(path);
            done = writeTransformation(extractionObject, file, extractionString);
        } else {
            done = writeXml(file, extractionString);
        }
        if (done) {
            if (fileGenerationListener != null) {
                fileGenerationListener.generateFile(lang, path, fileName);
            }
            long currentTime = System.currentTimeMillis();
            balayageLog.addFileGeneration(balayageUnitIndex, outputIndex, file, currentTime - startTime);
            return currentTime;
        } else {
            return startTime;
        }
    }

    boolean writeXml(File file, String extractionString) {
        try {
            Writer writer = new OutputStreamWriter(new FileOutputStream(file), "UTF-8");
            writer.write(extractionString);
            writer.close();
            return true;
        } catch (IOException ioe) {
            ioe.printStackTrace();
            String msg = ioe.getMessage();
            String message = ioe.getClass().getName();
            if (msg != null) {
                message = message + " : " + msg;
            }
            balayageLog.addOutputException(balayageUnitIndex, outputIndex, BalayageLog.IO_EXCEPTION, file.getName(), message);
            return false;
        }
    }

    private boolean writeTransformation(Object extractionObject, File file, String extractionString) {
        try {
            StringReader stringReader = new StringReader(extractionString);
            StreamSource streamSource = new StreamSource(stringReader);
            OutputStream outputStream = new FileOutputStream(file);
            StreamResult streamResult = new StreamResult(outputStream);
            Transformer transformer = transformManager.getTransformer(extractionObject);
            transformer.transform(streamSource, streamResult);
            outputStream.close();
            stringReader = null;
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            String msg = e.getMessage();
            String message = e.getClass().getName();
            if (msg != null) {
                message = message + " : " + msg;
            }
            balayageLog.addOutputException(balayageUnitIndex, outputIndex, BalayageLog.TRANSFORMATION_EXCEPTION, file.getName(), message);
            return false;
        }
    }

    public void dispose() {
        super.dispose();
        transformManager = null;
        langArray = null;
    }

}
