/* BdfServer - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.imagesource;

import net.mapeadores.util.request.FileValue;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.managers.*;
import java.awt.image.BufferedImage;
import net.mapeadores.util.logging.*;
import net.mapeadores.util.io.*;
import net.mapeadores.util.exceptions.*;
import java.io.*;
import javax.imageio.IIOException;
import javax.imageio.ImageIO;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.album.*;
import net.mapeadores.basedefiches.album.metadata.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.images.*;


/**
 *
 * @author Vincent Calame
 */
public class ImageSourceConnector implements ImageSourceConstants {

    private final BdfServer bdfServer;
    private final short action;
    private CorpusField corpusField;
    private FileValue fileValue;
    private File imagesourceDirectory;
    private File tmpDirectory;
    private Album album;
    private AlbumEditor albumEditor;
    private String tempFileName;

    public ImageSourceConnector(BdfServer bdfServer, CorpusField corpusField, short action) {
        this.bdfServer = bdfServer;
        this.action = action;
        this.corpusField = corpusField;
        SubsetKey albumKey = (SubsetKey) corpusField.getOption(FieldOptions.ALBUMKEY_OPTION);
        album = (Album) bdfServer.getBasedefiches().getSubset(albumKey);
        albumEditor = bdfServer.getBasedefichesEditor().getAlbumEditor(albumKey);
        File outputDirectory = bdfServer.getConfigurationManager().getOutputDirectory(ConfigurationManager.PRIVATE_OUTPUTDIRECTORY);
        tmpDirectory = new File(outputDirectory, "tmp");
        tmpDirectory.mkdir();
    }

    public boolean needFile() {
        switch (action) {
            case CREAT_ACTION:
                return true;
            default:
                return false;
        }
    }

    public boolean needTempFileName() {
        switch (action) {
            case CANCEL_ACTION:
                return true;
            case CONFIRM_ACTION:
                return true;
            default:
                return false;
        }
    }

    public void setTempFileName(String tempFileName) {
        this.tempFileName = tempFileName;
    }

    public void setFile(FileValue fileValue) {
        this.fileValue = fileValue;
    }

    public ActionResult doAction() {
        switch (action) {
            case CREAT_ACTION:
                return doCreatAction();
            case CONFIRM_ACTION:
                return doConfirmAction();
            case CANCEL_ACTION:
                return doCancelAction();
            default:
                throw new SwitchException("action = " + action);
        }
    }

    private ActionResult doCreatAction() {
        long length = fileValue.length();
        if (length < 2) {
            return error("pioche_err_imagesource_emptyfile");
        }
        String extension = getExtension(fileValue.getName());
        if (extension == null) {
            return error("pioche_err_imagesource_noimagefile");
        }
        String tempFileName = null;
        if (extension.equals("pdf")) {
            File localFile = getTempFile("png");
            tempFileName = localFile.getName();
            try {
                InputStream inputStream = fileValue.getInputStream();
                ImagesUtils.convertPdf(inputStream, 1, "png", localFile);
                inputStream.close();
            } catch (IOException ioe) {
                return error("pioche_err_imagesource_ioexception", ioe.getMessage());
            }

        } else if (convertToPng(extension)) {
            File localFile = getTempFile("png");
            tempFileName = localFile.getName();
            try {
                InputStream inputStream = fileValue.getInputStream();
                ImagesUtils.convert(inputStream, "png", localFile);
                inputStream.close();
            } catch (IOException ioe) {
                return error("pioche_err_imagesource_ioexception", ioe.getMessage());
            }

        } else {
            BufferedImage srcImg = null;
            try {
                InputStream inputStream = fileValue.getInputStream();
                srcImg = ImageIO.read(inputStream);
                inputStream.close();
            } catch (IOException ioe) {
                if (ioe instanceof IIOException) {
                    return error("pioche_err_imagesource_unsupportedimagetype", null);
                } else {
                    return error("pioche_err_imagesource_ioexception", ioe.getMessage());
                }
            }
            File localFile = getTempFile(extension);
            tempFileName = localFile.getName();
            try {
                OutputStream outputStream = new FileOutputStream(localFile);
                InputStream inputStream = fileValue.getInputStream();
                IOUtils.copy(inputStream, outputStream);
                inputStream.close();
                outputStream.close();
            } catch (IOException ioe) {
                return error("pioche_err_imagesource_ioexception", ioe.getMessage());
            }
        }
        CommandMessage commandMessage = CommandMessage.done("pioche_done_imagesource_creat", null);
        DefaultActionResult actionResult = success();
        /*String tempFileSrc = "";
        try {
        tempFileSrc = bdfServer.getConfigurationManager().toPrivateURL(tempFileName).toString();
        } catch(MalformedURLException mue) {
        return error("pioche_err_imagesource_ioexception", mue.getMessage());
        }*/
        String tempFileSrc = "prv/tmp/" + tempFileName;
        actionResult.setTempFile(tempFileName, tempFileSrc);
        actionResult.setCommandMessage(commandMessage);
        return actionResult;
    }

    private ActionResult doConfirmAction() {
        File f = getTempFile();
        if (!f.exists()) {
            return error();
        }
        String extension = getExtension(tempFileName);
        short type = AlbumUtils.imageTypeToShort(extension);
        AlbumImage albumImage = null;
        synchronized (bdfServer) {
            try {
                albumImage = albumEditor.createAlbumImage(type, null);
            } catch (ExistingNameException ene) {
            }
        }
        try {
            FileInputStream fileInputStream = new FileInputStream(f);
            albumEditor.updateAlbumImage(albumImage, fileInputStream);
            fileInputStream.close();
        } catch (IOException ioe) {
            return error("pioche_err_imagesource_ioexception", ioe.getMessage());
        }

        DefaultActionResult actionResult = success();
        String baseUrl = "images/" + album.getSubsetKey().getSubsetName() + "/";
        AlbumDim vignetteDim = album.getAlbumMetadata().getAlbumDimForUsage(AlbumConstants.VIGNETTE_USAGE);
        if (vignetteDim != null) {
            baseUrl = baseUrl + vignetteDim.getName() + "/";
        }
        actionResult.setAlbumImage(albumImage, baseUrl + AlbumUtils.toFileName(albumImage));
        return actionResult;
    }

    private ActionResult doCancelAction() {
        File f = getTempFile();
        if (!f.exists()) {
            return error();
        }
        DefaultActionResult actionResult = success();
        f.delete();
        return actionResult;
    }

    private DefaultActionResult success() {
        DefaultActionResult defaultActionResult = DefaultActionResult.successInstance(action, corpusField);
        return defaultActionResult;
    }

    private DefaultActionResult error() {
        return DefaultActionResult.errorInstance(action, corpusField);
    }

    private DefaultActionResult error(String key) {
        return DefaultActionResult.errorInstance(action, corpusField, key);
    }

    private DefaultActionResult error(String key, String value) {
        return DefaultActionResult.errorInstance(action, corpusField, key, value);
    }

    private String getExtension(String name) {
        int idx = name.lastIndexOf(".");
        if (idx == -1) {
            return null;
        }
        String extension = name.substring(idx + 1);
        if (extension.equalsIgnoreCase("png")) {
            return "png";
        }
        if (extension.equalsIgnoreCase("jpg")) {
            return "jpg";
        }
        if (extension.equalsIgnoreCase("jpeg")) {
            return "jpg";
        }
        if (extension.equalsIgnoreCase("bmp")) {
            return "bmp";
        }
        if (extension.equalsIgnoreCase("gif")) {
            return "gif";
        }
        if (extension.equalsIgnoreCase("pdf")) {
            return "pdf";
        }
        return null;
    }

    private boolean convertToPng(String extension) {
        if (extension.equals("gif")) {
            return true;
        }
        if (extension.equals("bmp")) {
            return true;
        }
        return false;
    }

    private File getTempFile() {
        return new File(tmpDirectory, tempFileName);
    }

    private File getTempFile(String extension) {
        int fileNumber = getNewFileNumber();
        String tempName = "tmp-" + String.valueOf(fileNumber) + "." + extension;
        return new File(tmpDirectory, tempName);
    }

    private int getNewFileNumber() {
        int max = 0;
        synchronized (bdfServer) {
            File[] files = tmpDirectory.listFiles();
            int length = files.length;
            for (int i = 0; i < length; i++) {
                File f = files[i];
                String name = f.getName();
                if (name.startsWith("tmp-")) {
                    int idx = name.indexOf(".");
                    try {
                        int number = Integer.parseInt(name.substring(4, idx));
                        max = Math.max(max, number);
                    } catch (NumberFormatException nfe) {
                    }
                }
            }
        }
        return max + 1;
    }

}
