/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.configuration.ConfigurationPath;
import fr.exemole.bdfserver.managers.configuration.ConfigurationPathException;
import fr.exemole.bdfserver.managers.configuration.ConfigurationUtils;
import fr.exemole.bdfserver.managers.configuration.LinkPathGroup;
import fr.exemole.bdfserver.managers.configuration.PageSubstitution;
import java.io.File;
import java.io.FileNotFoundException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.exceptions.SwitchException;


/**
 *
 * @author Vincent Calame
 */
public class ConfigurationManager implements BeanableManager {

    public final static String INSERT_URL = "url_insert";
    public final static String PUBLIC_OUTPUTDIRECTORY = "output._pub";
    public final static String PRIVATE_OUTPUTDIRECTORY = "output._prv";
    public final static String LOGS_OUTPUTDIRECTORY = "output.logs";
    public final static String BALAYAGEINCLUDE_OUTPUTDIRECTORY = "output.balayages/_inc_";
    public final static String WWW_ROOT = "root.www";
    public final static String ACTIONMENU_MAINLINKPATHGROUP = "main.actionmenu";
    private BdfServer bdfServer;
    private LangConfig langConfig = new LangConfig();
    private Map<String, ConfigurationPath> configurationPathMap = new HashMap<String, ConfigurationPath>();
    private Map<String, PageSubstitution> pageSubstitutionMap = new LinkedHashMap<String, PageSubstitution>();
    private Map<String, LinkPathGroup> linkPathGroupMap = new LinkedHashMap<String, LinkPathGroup>();
    public final static short PUBLIC_OUTPUT = 1;
    public final static short PRIVATE_OUTPUT = 2;

    public ConfigurationManager(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public File getOutputDirectory(String outputDirectoryName) {
        if (!outputDirectoryName.startsWith("output.")) {
            throw new IllegalArgumentException("Wront outputDirectoryName : " + outputDirectoryName);
        }
        File result = null;
        ConfigurationPath configurationPath = configurationPathMap.get(outputDirectoryName);
        if (configurationPath != null) {
            result = configurationPath.getSystemDirectory();
        }
        if (result == null) {
            String dirPath = outputDirectoryName.substring("output.".length());
            result = bdfServer.getOutputStorage().getOutputFile(dirPath);
            result.mkdirs();
        }
        return result;
    }

    public String getSystemDirectoryString(String name) {
        ConfigurationPath configurationPath = configurationPathMap.get(name);
        if (configurationPath == null) {
            return "";
        }
        return configurationPath.getSystemDirectoryString();
    }

    public ConfigurationPath getConfigurationPath(String name) {
        ConfigurationPath current = configurationPathMap.get(name);
        if (current == null) {
            return null;
        }
        return current.copy();
    }

    public List<ConfigurationPath> getConfigurationPathList() {
        List<ConfigurationPath> list = new ArrayList<ConfigurationPath>();
        addInList(list, INSERT_URL);
        addInList(list, WWW_ROOT);
        addInList(list, PUBLIC_OUTPUTDIRECTORY);
        addInList(list, PRIVATE_OUTPUTDIRECTORY);
        return list;
    }

    public String checkPath(short outputType, String relativePath, String reversePath) {
        URL racine = null;
        String subdir = null;
        switch (outputType) {
            case PUBLIC_OUTPUT:
                racine = getPublicURL(PUBLIC_OUTPUTDIRECTORY);
                subdir = "pub/";
                break;
            case PRIVATE_OUTPUT:
                racine = getPublicURL(PRIVATE_OUTPUTDIRECTORY);
                subdir = "prv/";
                break;
            default:
                throw new SwitchException("Unknown outputType = " + outputType);
        }
        if (racine != null) {
            return racine.toString() + relativePath;
        }
        return reversePath + subdir + relativePath;
    }

    private void addInList(List<ConfigurationPath> list, String name) {
        ConfigurationPath configurationPath = getConfigurationPath(name);
        if (configurationPath != null) {
            list.add(configurationPath);
        }
    }

    public boolean setSystemDirectory(String rename, String systemDirectoryString) throws ConfigurationPathException {
        return setConfigurationPath(rename, null, systemDirectoryString);
    }

    public boolean setPublicURL(String name, String publicURLString) throws ConfigurationPathException {
        return setConfigurationPath(name, publicURLString, null);
    }

    public boolean setConfigurationPath(String name, String publicURLString, String systemDirectoryString) throws ConfigurationPathException {
        URL publicURL = null;
        File systemDirectory = null;
        boolean malformedPublicURL = false;
        boolean systemDirectoryNotFound = false;
        try {
            publicURL = ConfigurationUtils.toURL(publicURLString);
        } catch (MalformedURLException mue) {
            malformedPublicURL = true;
        }
        try {
            systemDirectory = ConfigurationUtils.toDirectory(systemDirectoryString);
        } catch (FileNotFoundException fnfe) {
            systemDirectoryNotFound = true;
        }
        if ((malformedPublicURL) || (systemDirectoryNotFound)) {
            throw new ConfigurationPathException(malformedPublicURL, systemDirectoryNotFound);
        }
        name = checkCompatibility(name);
        if ((publicURL == null) && (systemDirectory == null)) {
            return removeConfigurationPath(name);
        }
        ConfigurationPath configurationPath = new ConfigurationPath(name, publicURL, systemDirectory);
        ConfigurationPath current = configurationPathMap.get(name);
        if (configurationPath.isSame(current)) {
            return false;
        }
        configurationPathMap.put(name, configurationPath);
        return true;
    }

    public boolean setConfigurationPath(ConfigurationPath configurationPath) throws ConfigurationPathException {
        return setConfigurationPath(configurationPath.getName(), configurationPath.getPublicURLString(), configurationPath.getSystemDirectoryString());
    }

    public File getBalayageRootDirectory(String rootName) {
        String key = "root." + rootName;
        String balayageRootPath = "";
        if (key.equals(WWW_ROOT)) {
            balayageRootPath = getSystemDirectoryString(WWW_ROOT);
        }
        if (!balayageRootPath.isEmpty()) {
            return new File(balayageRootPath);
        }
        File result = bdfServer.getOutputStorage().getOutputFile("balayages/" + rootName);
        result.mkdirs();
        return result;
    }

    public LangConfig getLangConfig() {
        return langConfig;
    }

    /**
     *
     *@throws IllegalArgumentException si langConfig est nul
     */
    public void setLangConfig(LangConfig langConfig) {
        if (langConfig == null) {
            throw new IllegalArgumentException("langConfigBean is null");
        }
        this.langConfig = langConfig;
    }

    public PageSubstitution getPageSubstitution(String pageName) {
        return pageSubstitutionMap.get(pageName);
    }

    public void setPageSubstitution(String originalName, String substitutionName) {
        if (originalName == null) {
            pageSubstitutionMap.remove(originalName);
        } else {
            pageSubstitutionMap.put(originalName, new PageSubstitution(originalName, substitutionName));
        }
    }

    public List<PageSubstitution> getPageSubstitutionList() {
        List<PageSubstitution> result = new ArrayList<PageSubstitution>();
        result.addAll(pageSubstitutionMap.values());
        return result;
    }

    public void setPageSubstitutionList(List<PageSubstitution> pageSubstitutionList) {
        pageSubstitutionMap.clear();
        if (pageSubstitutionList != null) {
            for (PageSubstitution pageSubstitution : pageSubstitutionList) {
                pageSubstitutionMap.put(pageSubstitution.getOriginalName(), pageSubstitution);
            }
        }
    }

    public List<LinkPathGroup> getLinkPathGroupList() {
        List<LinkPathGroup> result = new ArrayList<LinkPathGroup>();
        result.addAll(linkPathGroupMap.values());
        return result;
    }

    public LinkPathGroup getLinkPathGroup(String groupName) {
        return linkPathGroupMap.get(groupName);
    }

    public void setLinkPathGroup(LinkPathGroup linkPathGroup) {
        linkPathGroupMap.put(linkPathGroup.getName(), linkPathGroup);
    }

    public void appendLinkPathGroup(LinkPathGroup linkPathGroup) {
        String groupName = linkPathGroup.getName();
        LinkPathGroup current = linkPathGroupMap.get(groupName);
        if (current == null) {
            linkPathGroupMap.put(groupName, linkPathGroup);
        } else {
            current.append(linkPathGroup);
        }
    }

    public boolean removeLinkPathGroup(String groupName) {
        if (!linkPathGroupMap.containsKey(groupName)) {
            return false;
        }
        linkPathGroupMap.remove(groupName);
        return true;
    }

    private boolean removeConfigurationPath(String name) {
        if (!configurationPathMap.containsKey(name)) {
            return false;
        }
        configurationPathMap.remove(name);
        return true;
    }

    private String checkCompatibility(String name) {
        if (name.equals("public")) {
            return PUBLIC_OUTPUTDIRECTORY;
        } else if (name.equals("private")) {
            return PRIVATE_OUTPUTDIRECTORY;
        } else if (name.equals("logs")) {
            return LOGS_OUTPUTDIRECTORY;
        } else if (name.startsWith("root_")) {
            return "root." + name.substring(5);
        } else {
            return name;
        }
    }

    private URL getPublicURL(String name) {
        ConfigurationPath configurationPath = configurationPathMap.get(name);
        if (configurationPath == null) {
            return null;
        }
        return configurationPath.getPublicURL();
    }

}
