/* BdfServer - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.storage.BdfStorage;
import fr.exemole.bdfserver.ui.AlterUI;
import fr.exemole.bdfserver.ui.ComponentUIList;
import fr.exemole.bdfserver.ui.SubsetKeyGroupTemplates;
import fr.exemole.bdfserver.ui.SubsetKeyTree;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import java.util.LinkedHashMap;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.exceptions.SwitchException;


/**
 * Cette classe gère les questions d'interface (taille des formulaires, quels thésaurus
 * doivent apparaître dans le formulaire de saisie, à quel moment utilise-t-on des listes
 * déroulantes, etc.). Les éléments d'interface s'appuie sur des <em>beans</em>.
 *
 * @author Vincent Calame
 */
public class UIManager implements BeanableManager {

    private Map<SubsetKey, CorpusUI> corpusUIMap = new LinkedHashMap<SubsetKey, CorpusUI>();
    private BdfServer bdfServer;
    private SubsetKeyGroupTemplates subsetKeyGroupTemplates = new SubsetKeyGroupTemplates();
    private SubsetKeyTree corpusSubsetKeyTree = new SubsetKeyTree(subsetKeyGroupTemplates, SubsetKeyTree.CORPUS_GROUP);
    private SubsetKeyTree thesaurusSubsetKeyTree = new SubsetKeyTree(subsetKeyGroupTemplates, SubsetKeyTree.THESAURUS_GROUP);
    private SubsetKeyTree sphereSubsetKeyTree = new SubsetKeyTree(subsetKeyGroupTemplates, SubsetKeyTree.SPHERE_GROUP);

    private UIManager(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public SubsetKeyTree getCorpusSubsetKeyTree() {
        return corpusSubsetKeyTree;
    }

    public SubsetKeyTree getThesaurusSubsetKeyTree() {
        return thesaurusSubsetKeyTree;
    }

    public SubsetKeyTree getSphereSubsetKeyTree() {
        return sphereSubsetKeyTree;
    }

    public SubsetKeyTree getSubsetKeyTree(short type) {
        switch (type) {
            case SubsetKeyTree.CORPUS_GROUP:
                return getCorpusSubsetKeyTree();
            case SubsetKeyTree.THESAURUS_GROUP:
                return getThesaurusSubsetKeyTree();
            case SubsetKeyTree.SPHERE_GROUP:
                return getSphereSubsetKeyTree();
            default:
                throw new SwitchException("wrong type = " + type);
        }
    }

    public SubsetKeyGroupTemplates getSubsetKeyGroupTemplates() {
        return subsetKeyGroupTemplates;
    }

    public CorpusUI getCorpusUI(Corpus corpus) {
        return (CorpusUI) corpusUIMap.get(corpus.getSubsetKey());
    }

    public void corpusCreated(Corpus corpus) {
        CorpusUI corpusUI = new CorpusUI(corpus);
        corpusUIMap.put(corpus.getSubsetKey(), corpusUI);
        checkMasterSubsetUI(corpus.getMasterSubset());
        corpusSubsetKeyTree.addSubsetKey(corpus.getSubsetKey(), null);
    }

    public void corpusRemoved(SubsetKey corpusKey, Subset masterSubset) {
        corpusUIMap.remove(corpusKey);
        corpusSubsetKeyTree.removeSubsetKey(corpusKey);
        IncludeKey includeKey = IncludeKey.createLiageInstance(corpusKey);
        for (CorpusUI corpusUI : corpusUIMap.values()) {
            corpusUI.removeAllIncludeUI(includeKey);
        }
        checkMasterSubsetUI(masterSubset);
    }

    public void sphereCreated(Sphere sphere) {
        sphereSubsetKeyTree.addSubsetKey(sphere.getSubsetKey(), null);
    }

    public void sphereRemoved(SubsetKey sphereKey) {
        sphereSubsetKeyTree.removeSubsetKey(sphereKey);
    }

    public void thesaurusCreated(Thesaurus thesaurus) {
        thesaurusSubsetKeyTree.addSubsetKey(thesaurus.getSubsetKey(), null);
    }

    public void thesaurusRemoved(SubsetKey thesaurusKey) {
        thesaurusSubsetKeyTree.removeSubsetKey(thesaurusKey);
        IncludeKey includeKey = IncludeKey.createIndexationInstance(thesaurusKey);
        for (CorpusUI corpusUI : corpusUIMap.values()) {
            corpusUI.removeAllIncludeUI(includeKey);
        }
    }

    public void corpusFieldCreated(Corpus corpus, CorpusField corpusField) {
        CorpusUI corpusUI = internalGetCorpusUI(corpus);
        corpusUI.checkCorpus();
    }

    public void corpusFieldRemoved(Corpus corpus, FieldKey fieldKey) {
        CorpusUI corpusUI = internalGetCorpusUI(corpus);
        corpusUI.removeAllFieldUI(fieldKey);
    }

    public AlterUI getAlterUI(CorpusUI corpusUI, String uiName) {
        if ((uiName == null) || (uiName.length() == 0)) {
            return null;
        }
        return bdfServer.getBdfStorage().getAlterUI(corpusUI, uiName);
    }

    public ComponentUIList getComponentUIList(CorpusUI corpusUI, String uiName) {
        AlterUI alterUI = bdfServer.getBdfStorage().getAlterUI(corpusUI, uiName);
        return getComponentUIList(corpusUI, alterUI);
    }

    public ComponentUIList getComponentUIList(CorpusUI corpusUI, AlterUI alterUI) {
        if (alterUI == null) {
            return corpusUI.toComponentUIList();
        }
        switch (alterUI.getType()) {
            case AlterUI.COMPONENTUI_FILTER:
                return corpusUI.toComponentUIList(alterUI.getComponentUIFilter());
            case AlterUI.COMPONENTUI_LISTE:
                return alterUI.getComponentUIList();
            default:
                throw new SwitchException("Unknown type : " + alterUI.getType());
        }
    }

    private CorpusUI internalGetCorpusUI(Corpus corpus) {
        return corpusUIMap.get(corpus.getSubsetKey());
    }

    private void checkMasterSubsetUI(Subset masterSubset) {
        if (masterSubset == null) {
            return;
        }
        if (masterSubset instanceof Corpus) {
            CorpusUI masterCorpusUI = getCorpusUI((Corpus) masterSubset);
            masterCorpusUI.checkRattachement();
        }
    }

    /**
     * À appeler impérativement à la fin d'une initialisation afin de garantir la cohérence par rapport à la base de fiches
     */
    private void endInit() {
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        corpusSubsetKeyTree.checkBasedefiches(basedefiches);
        thesaurusSubsetKeyTree.checkBasedefiches(basedefiches);
        sphereSubsetKeyTree.checkBasedefiches(basedefiches);
    }

    public static UIManager newInstance(BdfServer bdfServer, BdfStorage bdfStorage) {
        UIManager uiManager = new UIManager(bdfServer);
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            CorpusUI corpusUI = new CorpusUI(corpus);
            uiManager.corpusUIMap.put(corpus.getSubsetKey(), corpusUI);
        }
        bdfStorage.loadManager(uiManager);
        uiManager.endInit();
        return uiManager;
    }

}
