/* BdfServer - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.policies;

import java.util.AbstractList;
import java.util.List;
import java.util.RandomAccess;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;


/**
 * Politique d'édition d'un thésaurus. L'éventail des politiques disponibles dépend du type de thésaurus.
 * Un thésaurus avec idalpha ne peut pas avoir de politique d'ajout. Un thésaurus multilingue ne peut avoir
 * que AJOUT_AUCUN et AJOUT_TRANSFERT. Un thésaurus babélien peut avoir les quatres politiques.
 *
 * @author Vincent Calame
 */
public class ThesaurusPolicy {

    public static final short AJOUT_AUCUN = 0;
    public static final short AJOUT_AUTORISE = 1;
    public static final short AJOUT_TRANSFERT = 2;
    public static final short AJOUT_VERIFICATION = 3;
    private SubsetKey thesaurusKey;
    private SubsetKey transferThesaurusKey;
    private VerifList verifThesaurusList;
    private boolean editable = true;
    private short ajoutPolicy;
    private short thesaurusType;

    public ThesaurusPolicy(SubsetKey thesaurusKey, short thesaurusType) {
        this.thesaurusKey = thesaurusKey;
        this.thesaurusType = thesaurusType;
    }

    public SubsetKey getThesaurusKey() {
        return thesaurusKey;
    }

    /**
     * Retourne <i>false</i> si le thésaurus n'est éditable en aucune manière (Thésaurus provenant d'une source extérieure par exemple).
     * Même l'administrateur générale ne peut l'éditer.
     */
    public boolean isEditable() {
        return editable;
    }

    public void setEditable(boolean editable) {
        this.editable = editable;
        if (testIllegalState(editable, ajoutPolicy)) {
            setAjoutPolicy(AJOUT_AUCUN, null);
        }
    }

    /**
     * Retourne la politique d'ajout suggérée : AJOUT_AUCUN, AJOUT_AUTORISE, AJOUT_TRANSFERT,AJOUT_VERIFICATION.
     * Cette politique n'est pas obligatoire. Elle sera en générale suivie par les applications permettant à n'importe quel utilisateur d'ajouter des éléments.
     * Elle sera ignorée par les outils d'administration des thésaurus.
     */
    public short getAjoutPolicy() {
        return ajoutPolicy;
    }

    /**
     *
     *L'argument <code>policyParameter</code> dépend du type d'ajout. Ce doit être la clé thésaurus de transfert ou la liste des clés de thésaurus de vérification.
     *Dans ce dernier cas,  la méthode vérifie que ce sont bien des clés de thésaurus et supprimer les doublons éventuels.
     *Il y a envoi d'exception si la liste est vide, si elle contient des valeurs nulles ou autres que des clés de thésaurus.
     *
     *@throws IllegalStateArgument si le thésaurus n'est pas éditable et la politique interdite à appliquer.
     *@throws IllegalArgumentException si l'ajout n'est pas autorisé pour le thésaurus en question.
     *@throws ClassCastException si policyParameter n'est pas du bon type.
     */
    public boolean setAjoutPolicy(short ajoutPolicy, Object policyParameter) {
        if (testIllegalState(editable, ajoutPolicy)) {
            throw new IllegalStateException("thesaurus cannot be editable and allow adding");
        }
        if (testIllegalArgument(thesaurusType, ajoutPolicy)) {
            throw new IllegalArgumentException("ajoutPolicy cannot be applied to this thesaurus due to the thesaurus type");
        }
        boolean done = (this.ajoutPolicy != ajoutPolicy);
        this.ajoutPolicy = ajoutPolicy;

        if (ajoutPolicy == AJOUT_TRANSFERT) {
            if (policyParameter == null) {
                throw new IllegalArgumentException("policyParameter is null");
            }
            SubsetKey newKey = (SubsetKey) policyParameter;
            if ((transferThesaurusKey == null) || (!(transferThesaurusKey.equals(newKey)))) {
                transferThesaurusKey = newKey;
                done = true;
            }
        } else {
            transferThesaurusKey = null;
        }
        if (ajoutPolicy == AJOUT_VERIFICATION) {
            if (policyParameter == null) {
                throw new IllegalArgumentException("policyParameter is null");
            }
            VerifList newList = getNewList((List<SubsetKey>) policyParameter);
            if (newList.size() == 0) {
                throw new IllegalArgumentException();
            }
            if ((verifThesaurusList == null) || (!newList.isEqual(verifThesaurusList))) {
                verifThesaurusList = newList;
                done = true;
            }
        } else {
            verifThesaurusList = null;
        }
        return done;
    }

    private VerifList getNewList(List<SubsetKey> argumentList) {
        VerifList newList = new VerifList();
        for (SubsetKey thesaurusKey : argumentList) {
            if (!thesaurusKey.isThesaurusSubset()) {
                throw new IllegalArgumentException();
            }
            newList.addThesaurusKey(thesaurusKey);
        }
        return newList;
    }

    /**
     * Dans le cas d'une politique de transfert (AJOUT_TRANSFERT), retourne la clé du thésaurus où doit se faire le transfert.
     * Retourne <i>null</i> si la politiquetype d'ajout n'est pas le transfert.
     */
    public SubsetKey getTransferThesaurusKey() {
        return transferThesaurusKey;
    }

    /**
     * Dans le cas d'une politique de vérification, retourne la liste des nom de thésaurus dans lesquels doivent se faire la vérification.
     * Retourne <i>null</> si le type d'ajout n'est pas la vérification.
     * Cette liste n'est pas modifiable.
     */
    public List<SubsetKey> getVerifThesaurusKeyList() {
        return verifThesaurusList;
    }

    /**
     * Indique si le type d'ajout correspond à une politique « laxiste ».
     * Une politique est laxiste si elle permet très facilement l'ajout ou la suppression de mots-clés.
     * Équivalent de
     */
    public static boolean isLaxPolicy(short ajoutType) {
        return ((ajoutType == AJOUT_AUTORISE) || (ajoutType == AJOUT_VERIFICATION));
    }

    public static boolean availablePolicyForMultiThesaurus(short ajoutPolicy) {
        return (ajoutPolicy == AJOUT_AUCUN) || (ajoutPolicy == AJOUT_TRANSFERT);
    }

    private static boolean testIllegalState(boolean editable, short type) {
        return ((!editable) && ((type != AJOUT_TRANSFERT) && (type != AJOUT_AUCUN)));
    }

    private static boolean testIllegalArgument(short thesaurusType, short type) {
        switch (thesaurusType) {
            case ThesaurusMetadata.IDALPHA_TYPE:
                return (type != AJOUT_AUCUN);
            case ThesaurusMetadata.MULTI_TYPE:
                return ((type != AJOUT_AUCUN) && (type != AJOUT_TRANSFERT));
            case ThesaurusMetadata.BABELIEN_TYPE:
                return false;
            default:
                throw new IllegalStateException("thesaurus.getThesaurusMetadata().getThesaurusType() is unknown");
        }
    }


    private class VerifList extends AbstractList<SubsetKey> implements RandomAccess {

        int size = 0;
        SubsetKey[] thesaurusKeyArray = new SubsetKey[2];

        public int size() {
            return size;
        }

        public SubsetKey get(int index) {
            if ((index < 0) || (index >= size)) {
                throw new IndexOutOfBoundsException();
            }
            return thesaurusKeyArray[index];
        }

        private void addThesaurusKey(SubsetKey thsKey) {
            if (thsKey.equals(thesaurusKey)) {
                return;
            }
            for (int i = 0; i < size; i++) {
                if (thesaurusKeyArray[i].equals(thsKey)) {
                    return;
                }
            }
            if (size == thesaurusKeyArray.length) {
                SubsetKey[] newArray = new SubsetKey[size * 2];
                System.arraycopy(thesaurusKeyArray, 0, newArray, 0, size);
                thesaurusKeyArray = newArray;
            }
            thesaurusKeyArray[size] = thsKey;
            size++;
        }

        public boolean isEqual(VerifList verifList) {
            if (this.size != verifList.size) {
                return false;
            }
            for (int i = 0; i < size; i++) {
                if (!this.thesaurusKeyArray[i].equals(verifList.thesaurusKeyArray[i])) {
                    return false;
                }
            }
            return true;
        }

    }

    /**
     *
     *@throws IllegalArgumentException si la valeur est incorrecte
     */
    public static String ajoutPolicyToString(short ajoutPolicy) {
        switch (ajoutPolicy) {
            case AJOUT_AUCUN:
                return "none";
            case AJOUT_AUTORISE:
                return "allow";
            case AJOUT_TRANSFERT:
                return "transfer";
            case AJOUT_VERIFICATION:
                return "verif";
            default:
                throw new IllegalArgumentException("wrong value");
        }
    }

    /**
     *
     *@throws IllegalArgumentException si la valeur est incorrecte
     */
    public static short ajoutPolicyToShort(String s) {
        if (s.equals("none")) {
            return AJOUT_AUCUN;
        }
        if (s.equals("allow")) {
            return AJOUT_AUTORISE;
        }
        if (s.equals("transfer")) {
            return AJOUT_TRANSFERT;
        }
        if (s.equals("verif")) {
            return AJOUT_VERIFICATION;
        }
        throw new IllegalArgumentException("wrong value");

    }

}
