/* BaseDeFiches - Article Management System
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.managers.transformation;

import java.io.Serializable;
import java.text.*;


/**
 *
 * @author Vincent Calame
 */
public final class StreamTransformerKey implements Serializable {

    private TransformationKey transformationKey;
    private String transformerName;
    private String extension;
    private int hashCode;

    /**
     *
     *@throws IllegalArgumentException si transformationKey ou extension sont nul
     */
    public StreamTransformerKey(TransformationKey transformationKey, String extension, String transformerName) {
        if (transformationKey == null) {
            throw new IllegalArgumentException("transformationKey is null");
        }
        if (extension == null) {
            throw new IllegalArgumentException("extension is null");
        }
        if (extension.length() == 0) {
            throw new IllegalArgumentException("extension is empty");
        }
        if (extension.indexOf('/') != -1) {
            throw new IllegalArgumentException("extension contains a '/'");
        }
        this.transformationKey = transformationKey;
        this.transformerName = transformerName;
        this.extension = extension;
        initHashCode();
    }

    private void initHashCode() {
        hashCode = extension.hashCode() + transformationKey.hashCode();
        if (transformerName != null) {
            hashCode = hashCode + transformerName.hashCode();
        }
    }

    public String getTransformerName() {
        return transformerName;
    }

    public TransformationKey getTransformationKey() {
        return transformationKey;
    }

    public String getExtension() {
        return extension;
    }

    public int hashCode() {
        return hashCode;
    }

    public static StreamTransformerKey getDefaultStreamTransformerKey(TransformationKey transformationKey, String extension) {
        return new StreamTransformerKey(transformationKey, extension, null);
    }

    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj.hashCode() != this.hashCode) {
            return false;
        }
        if (!(obj instanceof StreamTransformerKey)) {
            return false;
        }
        StreamTransformerKey other = (StreamTransformerKey) obj;
        if (!other.extension.equals(this.extension)) {
            return false;
        }
        if (!other.transformationKey.equals(this.transformationKey)) {
            return false;
        }
        if (this.transformerName == null) {
            return (other.transformerName == null);
        }
        if (other.transformerName == null) {
            return false;
        }
        return (other.transformerName.equals(this.transformerName));
    }

    public String toString() {
        if (transformerName == null) {
            return transformationKey.toString() + "/" + extension + "/";
        }
        return transformationKey.toString() + "/" + extension + "/" + transformerName;
    }

    public boolean isDefaultTransformerKey() {
        return (transformerName == null);
    }

    public StreamTransformerKey toDefaultTransformerKey() {
        if (transformerName == null) {
            return this;
        }
        return new StreamTransformerKey(transformationKey, extension, null);
    }

    public static StreamTransformerKey parse(String s) throws ParseException {
        if (s == null) {
            throw new ParseException("s is null", 0);
        }
        int idx1 = s.indexOf('/');
        if (idx1 == -1) {
            throw new ParseException("no '/' character", s.length() - 1);
        }
        int idx2 = s.indexOf('/', idx1 + 1);
        if (idx2 == -1) {
            throw new ParseException("no second '/' character", s.length() - 1);
        }
        TransformationKey transformationKey = TransformationKey.parse(s.substring(0, idx1));
        String extension = s.substring(idx1 + 1, idx2);
        if (extension.length() == 0) {
            throw new ParseException("extension is empty", idx1);
        }
        String transformerName = s.substring(idx2 + 1);
        if (transformerName.length() == 0) {
            transformerName = null;
        }
        return new StreamTransformerKey(transformationKey, extension, transformerName);
    }

    public static String toStringKey(TransformationKey transformationKey, String extension) {
        return transformationKey.toString() + "/" + extension;
    }

    public static String toStringKey(StreamTransformerKey streamTransformerKey) {
        return toStringKey(streamTransformerKey.getTransformationKey(), streamTransformerKey.getExtension());
    }

}
