/* BdfServer - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.transformation;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.Manager;
import fr.exemole.bdfserver.storage.TransformationStorage;
import fr.exemole.bdfserver.xsltfactory.DefaultXSLTFactory;
import fr.exemole.bdfserver.xsltfactory.XSLTFactoryConstants;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;
import net.mapeadores.util.exceptions.NestedTransformerException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.io.StreamTransformer;
import net.mapeadores.util.io.TransformerErrorListener;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.LocalisationInfo;


/**
 *
 * @author Vincent Calame
 */
public class TransformationManager implements Manager {

    private static Map<Object, String[]> availableMap = new HashMap<Object, String[]>();

    static {
        String[] array = {"odt"};
        availableMap.put(TransformationKey.COMPILATION_TRANSFORMATIONKEY, array);
        availableMap.put("corpus", array);
    }

    private BdfServer bdfServer;
    private TransformationStorage transformationStorage;
    private Map<TransformationKey, TransformationInfo> transformationInfoMap = new HashMap<TransformationKey, TransformationInfo>();
    private Map<Object, Object> transformerMap = new HashMap<Object, Object>();

    public TransformationManager(BdfServer bdfServer, TransformationStorage transformationStorage) {
        this.bdfServer = bdfServer;
        this.transformationStorage = transformationStorage;
        update();
    }

    public void clearDefaultTransformer() {
        for (Iterator it = transformerMap.keySet().iterator(); it.hasNext();) {
            Object obj = it.next();
            if (obj instanceof XslTransformerKey) {
                XslTransformerKey key = (XslTransformerKey) obj;
                if (key.isDefaultTransformerKey()) {
                    it.remove();
                }
            } else if (obj instanceof StreamTransformerKey) {
                StreamTransformerKey key = (StreamTransformerKey) obj;
                if (key.isDefaultTransformerKey()) {
                    it.remove();
                }
            }
        }
    }

    public void update() {
        transformerMap.clear();
        transformationInfoMap.clear();
        List<TransformationStorage.TransformationStorageInfo> list = transformationStorage.getTransformationStorageInfoList(bdfServer);
        for (TransformationStorage.TransformationStorageInfo info : list) {
            update(info);
        }
    }

    public void saveTransformer(Object transformerKey, InputStream inputStream, boolean pattern) throws IOException {
        transformationStorage.saveTransformer(transformerKey, inputStream, pattern);
        update();
    }

    private void update(TransformationStorage.TransformationStorageInfo info) {
        TransformerFactory factory = bdfServer.newTransformationFactory(null);
        TransformationKey transformationKey = info.getTransformationKey();
        TransformationInfo.Builder builder = TransformationInfo.getBuilder(transformationKey);
        checkDefaultStreamTransformerAvailable(builder);
        List<TransformationStorage.XslTransformerStorageInfo> xslList = info.getXslTransformerStorageInfoList();
        for (TransformationStorage.XslTransformerStorageInfo xslInfo : xslList) {
            XslTransformerInfo info2 = xslInfo.getXslTransformerInfo();
            Source source = xslInfo.getSource();
            if (source != null) {
                TransformerErrorListener errorListener = new TransformerErrorListener();
                factory.setErrorListener(errorListener);
                try {
                    Templates templates = factory.newTemplates(source);
                    transformerMap.put(info2.getXslTransformerKey(), templates);
                } catch (TransformerConfigurationException tce) {
                    errorListener.checkFatalTransformerException(tce);
                }
                if (errorListener.hasError()) {
                    info2.setError(XslTransformerInfo.XSL_ERROR, errorListener.getErrorMessage());
                }
            }
            builder.addXslTransformerInfo(info2);

        }
        List<TransformationStorage.StreamTransformerStorageInfo> streamList = info.getStreamTransformerStorageInfoList();
        for (TransformationStorage.StreamTransformerStorageInfo streamInfo : streamList) {
            StreamTransformerInfo info2 = streamInfo.getStreamTransformerInfo();
            StreamTransformer streamTransformer = streamInfo.getStreamTransformer();
            if (streamTransformer != null) {
                transformerMap.put(info2.getStreamTransformerKey(), streamTransformer);
            }
            builder.addStreamTransformerInfo(info2);
        }
        transformationInfoMap.put(transformationKey, builder.toTransformationInfo());
    }

    public TransformationInfo getTransformationInfo(TransformationKey transformationKey) {
        TransformationInfo transformationInfo = transformationInfoMap.get(transformationKey);
        if (transformationInfo == null) {
            transformationInfo = TransformationInfo.getEmptyInstance(transformationKey, getDefaultStreamTransformerAvailableArray(transformationKey));
            transformationInfoMap.put(transformationKey, transformationInfo);
        }
        return transformationInfo;
    }

    public StreamTransformer getStreamTransformer(StreamTransformerKey streamTransformerKey) {
        StreamTransformer streamTransformer = (StreamTransformer) transformerMap.get(streamTransformerKey);
        if (streamTransformer != null) {
            return streamTransformer;
        }
        if (!streamTransformerKey.isDefaultTransformerKey()) {
            streamTransformerKey = streamTransformerKey.toDefaultTransformerKey();
            StreamTransformer defaultStreamTransformer = (StreamTransformer) transformerMap.get(streamTransformerKey);
            if (defaultStreamTransformer != null) {
                return defaultStreamTransformer;
            }
        }
        if (isDefaultStreamTransformerAvailable(streamTransformerKey)) {
            streamTransformer = createDefaultStreamTransformer(streamTransformerKey.getTransformationKey(), streamTransformerKey.getExtension());
            transformerMap.put(streamTransformerKey, streamTransformer);
            return streamTransformer;
        }
        TransformationInfo transformationInfo = getTransformationInfo(streamTransformerKey.getTransformationKey());
        int index = transformationInfo.indexOfExtension(streamTransformerKey.getExtension());
        if (index == -1) {
            return null;
        } else {
            streamTransformerKey = transformationInfo.getExtensionList(index).getStreamTransformerKey(0);
            return (StreamTransformer) transformerMap.get(streamTransformerKey);
        }
    }

    public Transformer getXslTransformer(XslTransformerKey xslTransformerKey) {
        Templates xslTemplate = (Templates) transformerMap.get(xslTransformerKey);
        if (xslTemplate != null) {
            return bdfServer.newTransformer(xslTemplate);
        }
        if (!xslTransformerKey.isDefaultTransformerKey()) {
            return getXslTransformer(xslTransformerKey.toDefaultTransformerKey());
        }
        xslTemplate = createDefaultXslTransformer(xslTransformerKey.getTransformationKey());
        return bdfServer.newTransformer(xslTemplate);
    }

    public static void checkLocalisationParameters(Transformer transformer, LocalisationInfo localisationInfo) {
        transformer.setParameter("WORKINGLANG", LangInteger.toString(localisationInfo.getWorkingLangInteger()));
        transformer.setParameter("FORMATLOCALE", LangInteger.toString(localisationInfo.getFormatLocaleInteger()));
    }

    private Templates createDefaultXslTransformer(TransformationKey transformationKey) {
        String xslt = DefaultXSLTFactory.getDefaultStyleSheet(bdfServer, transformationKey, XSLTFactoryConstants.XSL_FORMAT);
        if (xslt == null) {
            throw new IllegalArgumentException("Corpus defined in sourceKey is not valid (corpus = " + transformationKey.toCorpusKey().toString() + ")");
        }
        Source source = new StreamSource(new StringReader(xslt));
        try {
            TransformerFactory factory = bdfServer.newTransformationFactory(null);
            Templates xslTemplate = factory.newTemplates(source);
            transformerMap.put(XslTransformerKey.getDefaultXslTransformerKey(transformationKey), xslTemplate);
            return xslTemplate;
        } catch (TransformerConfigurationException tce) {
            throw new NestedTransformerException(tce);
        }
    }

    private StreamTransformer createDefaultStreamTransformer(TransformationKey transformationKey, String extension) {
        if (extension.equals("odt")) {
            return DefaultXSLTFactory.getDefaultStreamTransformer(bdfServer, transformationKey, XSLTFactoryConstants.ODT_FORMAT);
        } else {
            throw new SwitchException("extension = " + extension);
        }
    }

    private static boolean isDefaultStreamTransformerAvailable(StreamTransformerKey streamTransformerKey) {
        String[] array = getDefaultStreamTransformerAvailableArray(streamTransformerKey.getTransformationKey());
        if (array == null) {
            return false;
        }
        String extension = streamTransformerKey.getExtension();
        for (int i = 0; i < array.length; i++) {
            if (array[i].equals(extension)) {
                return true;
            }
        }
        return false;
    }

    private static void checkDefaultStreamTransformerAvailable(TransformationInfo.Builder builder) {
        String[] array = getDefaultStreamTransformerAvailableArray(builder.getTransformationKey());
        if (array == null) {
            return;
        }
        for (int i = 0; i < array.length; i++) {
            builder.addDefaultTransformerAvailable(array[i]);
        }
    }

    private static String[] getDefaultStreamTransformerAvailableArray(TransformationKey transformationKey) {
        if (transformationKey.isCorpusTransformationKey()) {
            return availableMap.get("corpus");
        }
        return availableMap.get(transformationKey);
    }

}
