/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.xml;

import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.managers.LangConfig;
import fr.exemole.bdfserver.managers.configuration.ConfigurationPathException;
import fr.exemole.bdfserver.managers.configuration.LinkPathGroup;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.xml.DOMReader;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class ConfigurationManagerDOMReader extends ManagerDOMReader {

    private ConfigurationManager currentConfigurationManager;

    public ConfigurationManagerDOMReader() {
    }

    public void readManager(Object manager, Element element_xml) {
        readConfigurationManager((ConfigurationManager) manager, element_xml);
    }

    public void readConfigurationManager(ConfigurationManager configurationManager, Element element_xml) {
        this.currentConfigurationManager = configurationManager;
        DOMReader.readChildren(element_xml, new RootHandler());
        this.currentConfigurationManager = null;
    }


    private class RootHandler implements ElementHandler {

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("path-list")) {
                DOMReader.readChildren(element, new PathListHandler());
            } else if (tagName.equals("lang-config")) {
                LangConfigBuilder langConfigBuilder = new LangConfigBuilder();
                DOMReader.readChildren(element, langConfigBuilder);
                langConfigBuilder.flush();
                currentConfigurationManager.setLangConfig(langConfigBuilder.getLangConfig());
            } else if (tagName.equals("page-list")) {
                DOMReader.readChildren(element, new PageListHandler());
            } else if (tagName.equals("linkpath-group")) {
                String name = element.getAttribute("name");
                if (name.length() == 0) {
                    emptyAttributeError(tagName, "name");
                } else {
                    LinkPathGroupHandler linkPathGroupHandler = new LinkPathGroupHandler(name);
                    DOMReader.readChildren(element, linkPathGroupHandler);
                    linkPathGroupHandler.flush();
                }
            } else {
                unknownTagWarning(tagName);
            }
        }

    }


    private class PathListHandler implements ElementHandler {

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("path")) {
                String name = element.getAttribute("name");
                if (name.length() == 0) {
                    emptyAttributeError(tagName, "name");
                    return;
                }
                String publicURL = element.getAttribute("public");
                String systemFile = element.getAttribute("system");
                try {
                    currentConfigurationManager.setConfigurationPath(name, publicURL, systemFile);
                } catch (ConfigurationPathException cpe) {
                    if (cpe.isMalformedPublicURL()) {
                        malformedURLError(tagName, "public", publicURL);
                    }
                    if (cpe.isSystemDirectoryNotFound()) {
                        fileNotFoundError(tagName, "system", systemFile);
                    }
                }
            } else {
                unknownTagWarning(tagName);
            }
        }

    }


    private class PageListHandler implements ElementHandler {

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("page")) {
                String original = element.getAttribute("original");
                if (original.length() == 0) {
                    emptyAttributeError(tagName, "original");
                    return;
                }
                String substitution = element.getAttribute("substitution");
                if (substitution.length() == 0) {
                    emptyAttributeError(tagName, "substitution");
                    return;
                }
                currentConfigurationManager.setPageSubstitution(original, substitution);
            } else {
                unknownTagWarning(tagName);
            }
        }

    }


    private class LinkPathGroupHandler implements ElementHandler {

        private LinkPathGroup linkPathGroup;

        private LinkPathGroupHandler(String name) {
            linkPathGroup = new LinkPathGroup(name);
        }

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("linkpath")) {
                String value = element.getAttribute("value");
                if (value.length() == 0) {
                    emptyAttributeError(tagName, "value");
                    return;
                }
                linkPathGroup.addLinkPath(value);
            } else {
                unknownTagWarning(tagName);
            }
        }

        public void flush() {
            currentConfigurationManager.appendLinkPathGroup(linkPathGroup);
        }

    }


    private class LangConfigBuilder implements ElementHandler {

        LangConfig langConfig;
        List<String> workingLangList = new ArrayList<String>();
        List<String> supplementaryList = new ArrayList<String>();

        LangConfigBuilder() {
            langConfig = new LangConfig();
            langConfig.setAllLanguages(false);
        }

        public LangConfig getLangConfig() {
            return langConfig;
        }

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("all-languages")) {
                langConfig.setAllLanguages(true);
            } else if (tagName.equals("with-original-name")) {
                langConfig.setWithOriginalName(true);
            } else if (tagName.equals("working-lang")) {
                workingLangList.add(element.getAttribute("lang"));
            } else if (tagName.equals("supplementary-lang")) {
                supplementaryList.add(element.getAttribute("lang"));
            } else if (tagName.equals("without-nom-avant")) {
                langConfig.setWithoutNomAvant(true);
            } else {
                unknownTagWarning(tagName);
            }
        }

        void flush() {
            langConfig.setWorkingLangIntegerArray(toStringArray(workingLangList));
            langConfig.setSupplementaryLangIntegerArray(toStringArray(supplementaryList));
        }

        String[] toStringArray(List<String> list) {
            int size = list.size();
            String[] result = new String[size];
            for (int i = 0; i < size; i++) {
                result[i] = list.get(i);
            }
            return result;
        }

    }

}
