/* BdfServer - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;

import net.mapeadores.basedefiches.corpus.fiche.*;
import fr.exemole.bdfserver.request.parsers.blockparsers.BlockParser;
import fr.exemole.bdfserver.request.TextParser;
import fr.exemole.bdfserver.request.TypoOptions;
import net.mapeadores.util.text.StringUtils;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class FicheBlockParser {

    public static final short MODE_FORMULAIRE = 1;
    public static final short MODE_EXPORT = 2;
    private TextParser textParser;
    private ParagraphParser paragraphParser;
    private BlockParser tableParser;
    private BlockParser codeParser;
    private BlockParser ulParser;
    private BlockParser insertParser;
    private BlockParser divParser;
    private ParagraphState paragraphState = new ParagraphState();

    public FicheBlockParser(TypoOptions typoOptions) {
        textParser = new TextParser();
        textParser.setQuoteOptions(typoOptions.getQuoteOptions());
        paragraphParser = new ParagraphParser(textParser);
        tableParser = BlockParser.getTableParser(textParser);
        codeParser = BlockParser.getCodeParser(textParser);
        ulParser = BlockParser.getUlParser(textParser);
        insertParser = BlockParser.getInsertParser(textParser);
        divParser = BlockParser.getDivParser(textParser, typoOptions);
    }

    public void parseFicheBlockList(String string, MutableFicheBlockList ficheBlockList) {
        if (string.length() == 0) {
            return;
        }
        reinit();
        String[] lines = StringUtils.getLineTokens(string, StringUtils.NOTCLEAN);
        for (int k = 0; k < lines.length; k++) {
            k = parseLine(lines, k, ficheBlockList);
        }
    }

    public void reinit() {
        paragraphState.reinit();
    }

    public int parseLine(String[] lineArray, int lineIndex, MutableFicheBlockList ficheBlockList) {
        String line = StringUtils.cleanString(lineArray[lineIndex]);
        LineInfo lineInfo = LineInfo.parse(line);
        if (lineInfo.isAbsolutlyEmpty()) {
            paragraphState.increaseWhiteLineCount();
            return lineIndex;
        }
        BlockParser blockParser = getBlockParser(lineInfo.getLine());
        if (blockParser != null) {
            blockParser.parse(lineArray, lineIndex);
            ficheBlockList.addFicheBlock(blockParser.getFicheBlock());
            int newLineIndex = blockParser.getEndIndex();
            paragraphState.reinit();
            return newLineIndex;
        }
        ParagraphBlock prg = parseLine(lineInfo);
        if (prg instanceof H) {
            paragraphState.checkH();
        } else {
            paragraphState.flushWhiteLines(ficheBlockList);
        }
        ficheBlockList.addFicheBlock(prg);
        return lineIndex;
    }

    private BlockParser getBlockParser(String line) {
        if (ulParser.isStartLine(line)) {
            return ulParser;
        }
        if (codeParser.isStartLine(line)) {
            return codeParser;
        }
        if (tableParser.isStartLine(line)) {
            return tableParser;
        }
        if (insertParser.isStartLine(line)) {
            return insertParser;
        }
        if (divParser.isStartLine(line)) {
            return divParser;
        }
        return null;
    }

    private ParagraphBlock parseLine(LineInfo lineInfo) {
        String line = lineInfo.getLine();
        if (line.length() == 0) {
            P p = new P(P.TYPE_NORMAL);
            p.setClassName(lineInfo.getClassName());
            p.setHtmlId(lineInfo.getHtmlId());
            return p;
        }
        char carac = line.charAt(0);
        switch (carac) {
            case '#':
                return parseH(lineInfo);
            case '>':
                lineInfo.setLine(line.substring(1).trim());
                return parseCitation(lineInfo);
            case '<':
                return parseSource(lineInfo, P.TYPE_NORMAL);
            default:
                P p = new P(P.TYPE_NORMAL);
                paragraphParser.parseParagraphBlock(p, lineInfo);
                return p;
        }

    }

    private H parseH(LineInfo lineInfo) {
        String line = lineInfo.getLine();
        int level = 1;
        for (int i = 1; i < line.length(); i++) {
            char carac = line.charAt(i);
            if (carac == '#') {
                level++;
            } else {
                break;
            }
        }
        H h = new H(level);
        lineInfo.setLine(line.substring(level).trim());
        paragraphParser.parseParagraphBlock(h, lineInfo);
        return h;
    }

    private P parseCitation(LineInfo lineInfo) {
        String line = lineInfo.getLine();
        if ((line.length() > 0) && (line.charAt(0) == '<')) {
            return parseSource(lineInfo, P.TYPE_CITATION);
        }
        P p = new P(P.TYPE_CITATION);
        paragraphParser.parseParagraphBlock(p, lineInfo);
        return p;
    }

    private P parseSource(LineInfo lineInfo, short type) {
        String line = lineInfo.getLine();
        int idx = -1;
        boolean previousQuestion = false;
        boolean previousRemarque = false;
        for (int i = 1; i < line.length(); i++) {
            char carac = line.charAt(i);
            if (carac == ':') {
                idx = i;
                break;
            } else {
                previousQuestion = false;
                previousRemarque = false;
                if (carac == '\\') {
                    i++;
                } else if (carac == '?') {
                    previousQuestion = true;
                } else if (carac == '!') {
                    previousRemarque = true;
                }
            }
        }
        int sourceIdx = idx;
        if ((sourceIdx != -1) && (type == P.TYPE_NORMAL)) {
            if (previousQuestion) {
                type = P.TYPE_QUESTION;
                sourceIdx--;
            } else if (previousRemarque) {
                type = P.TYPE_REMARQUE;
                sourceIdx--;
            }
        }
        P p = new P(type);
        if (sourceIdx != -1) {
            StringBuilder buf = new StringBuilder();
            try {
                for (int i = 1; i < sourceIdx; i++) {
                    if ((i == 1) || (i == (sourceIdx - 1))) {
                        if (line.charAt(i) == ' ') {
                            continue;
                        }
                    }
                    i = textParser.parseOutsideChar(line, i, buf);
                }
            } catch (IOException ioe) {
            }
            p.setSource(buf.toString());
            line = line.substring(idx + 1).trim();
        }
        lineInfo.setLine(line);
        paragraphParser.parseParagraphBlock(p, lineInfo);
        return p;
    }


    private class ParagraphState {

        boolean first = true;
        boolean previousH = false;
        int whiteLineCount = 0;

        private ParagraphState() {
            reinit();
        }

        private void reinit() {
            first = true;
            previousH = false;
            whiteLineCount = 0;
        }

        public boolean isFirst() {
            return first;
        }

        public void increaseWhiteLineCount() {
            if (!first) {
                whiteLineCount++;
            }
        }

        public void checkH() {
            previousH = true;
            whiteLineCount = 0;
            first = false;
        }

        public void flushWhiteLines(MutableFicheBlockList ficheBlockList) {
            if (previousH) {
                whiteLineCount--;
            }
            previousH = false;
            if (whiteLineCount > 1) {
                for (int i = 1; i < whiteLineCount; i++) {
                    ficheBlockList.addFicheBlock(new P(P.TYPE_NORMAL));
                }
            }
            whiteLineCount = 0;
            first = false;
        }

    }

}
