/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.l10n.LocaleBundle;
import fr.exemole.bdfserver.request.TypoOptions;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesEditor;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.CorpusEditor;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.MutableFicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Personne;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.request.FileValue;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class SaisieEngine {

    public final static int ALLFICHE_BITVALUE = 1;
    public final static int INDEXATION_BITVALUE = 2;
    public final static int LIAGE_BITVALUE = 4;
    public final static int ASSOCIATION_BITVALUE = 8;
    public final static int ALL_MASK = 15;
    public final static int PARTIALFICHE_BITVALUE = 16;
    private Basedefiches basedefiches;
    private BasedefichesEditor basedefichesEditor;
    private RequestMap requestMap;
    private AbregeFiche currentAbregeFiche;
    private LocalisationInfo localisationInfo;
    private int saisieMask;
    private String currentSuffix;
    private int currentSuffixLength = 0;
    private Set<String> currentParamNameSet;
    private char indexationSeparateur = ';';
    private char liageSeparateur = ';';
    private char itemListSeparateur = ';';
    private IndexationParser indexationParser;
    private boolean ficheFieldAppend = false;
    private TypoOptions typoOptions;
    private RedacteurKey userRedacteurKey;

    public SaisieEngine(BdfServer bdfServer, BdfUser bdfUser, RequestMap requestMap, int saisieMask) {
        init(bdfServer, bdfUser.getLocalisationInfo(), requestMap, saisieMask);
        this.typoOptions = bdfUser.getTypoOptions();
        this.userRedacteurKey = bdfUser.getRedacteurKey();
    }

    public SaisieEngine(BdfServer bdfServer, LocalisationInfo localisationInfo, RequestMap requestMap, int saisieMask) {
        init(bdfServer, localisationInfo, requestMap, saisieMask);
        this.typoOptions = TypoOptions.getTypoOptions(localisationInfo.getWorkingLangInteger());
    }

    private void init(BdfServer bdfServer, LocalisationInfo localisationInfo, RequestMap requestMap, int saisieMask) {
        this.basedefiches = bdfServer.getBasedefiches();
        this.basedefichesEditor = bdfServer.getBasedefichesEditor();
        this.localisationInfo = localisationInfo;
        this.requestMap = requestMap;
        this.saisieMask = saisieMask;
        this.indexationParser = new IndexationParser(bdfServer, localisationInfo.getWorkingLangInteger());
    }

    public void setFicheFieldAppendMode(boolean ficheFieldAppend) {
        this.ficheFieldAppend = ficheFieldAppend;
    }

    /**
     * Séparateur des mots-clés. Par défaut :  ';'
     */
    public void setIndexationSeparateur(char indexationSeparateur) {
        this.indexationSeparateur = indexationSeparateur;
    }

    /**
     * Séparateur des numéros de fiches. Par défaut : « \n\r\t;,| »
     */
    public void setLiageSeparateur(char liageSeparateur) {
        this.liageSeparateur = liageSeparateur;
    }

    /**
     * Séparateur des champs « en ligne » (Rédacteurs, Information). Par défaut :  ';'
     */
    public void setItemListSeparateur(char itemListSeparateur) {
        this.itemListSeparateur = itemListSeparateur;
    }

    public void updateFiche(AbregeFiche abregeFiche) {
        updateFiche(abregeFiche, null);
    }

    public void updateFiche(AbregeFiche abregeFiche, String paramSuffix) {
        if (paramSuffix == null) {
            currentSuffix = "";
        } else {
            currentSuffix = paramSuffix + "_";
        }
        currentSuffixLength = currentSuffix.length();
        this.currentAbregeFiche = abregeFiche;
        if (currentSuffixLength == 0) {
            currentParamNameSet = requestMap.getParameterNameSet();
        } else {
            currentParamNameSet = new HashSet<String>();
            for (String paramName : requestMap.getParameterNameSet()) {
                if (paramName.startsWith(currentSuffix)) {
                    currentParamNameSet.add(paramName.substring(currentSuffixLength));
                }
            }
        }
        if ((saisieMask & INDEXATION_BITVALUE) != 0) {
            editIndexation();
        }
        if ((saisieMask & LIAGE_BITVALUE) != 0) {
            editLiage();
        }
        if ((saisieMask & ASSOCIATION_BITVALUE) != 0) {
            editAssociation();
        }
        if (((saisieMask & ALLFICHE_BITVALUE) != 0) || ((saisieMask & PARTIALFICHE_BITVALUE) != 0)) {
            boolean all = ((saisieMask & ALLFICHE_BITVALUE) != 0);
            editFiche(all);
        }
    }

    private String getParameter(String paramName) {
        return requestMap.getParameter(currentSuffix + paramName);
    }

    private String[] getParameterValues(String paramName) {
        return requestMap.getParameterValues(currentSuffix + paramName);
    }

    private FileValue[] getFileValues(String paramName) {
        return requestMap.getFileValues(currentSuffix + paramName);
    }

    private void editIndexation() {
        indexationParser.clear();
        for (String paramName : currentParamNameSet) {
            if (paramName.startsWith("thesaurus_")) {
                String chaine = getConcatedParameterValues(paramName, indexationSeparateur);
                String[] tokens = StringUtils.getCleanedTokens(chaine, indexationSeparateur);
                indexationParser.parse(paramName, tokens);
            }
        }
        indexationParser.save(currentAbregeFiche);
    }

    private void editAssociation() {
        AssociationParser associationParser = new AssociationParser(currentAbregeFiche);
        String associationString = IncludeKey.ASSOCIATION.toString();
        for (String paramName : currentParamNameSet) {
            if (paramName.startsWith(associationString)) {
                FileValue[] fileValues = getFileValues(paramName);
                if (fileValues != null) {
                    associationParser.add(fileValues);
                }
            }
        }
        associationParser.save(basedefichesEditor);
    }

    private void editLiage() {
        LiageParser liageParser = new LiageParser(currentAbregeFiche);
        String liageString = IncludeKey.LIAGE.toString();
        for (String paramName : currentParamNameSet) {
            if ((paramName.equals(liageString)) || (paramName.startsWith("corpus_"))) {
                String[] idcorpusArray = getParameterValues(paramName);
                int length = idcorpusArray.length;
                for (int i = 0; i < length; i++) {
                    String[] tokens = StringUtils.getCleanedTokens(idcorpusArray[i], liageSeparateur);
                    liageParser.parse(paramName, tokens);
                }
            }
        }
        liageParser.save(basedefichesEditor.getCroisementEditor());
    }

    private void editFiche(boolean all) {
        FicheBuffer ficheBuffer = new FicheBuffer(currentAbregeFiche, all);
        ficheBuffer.iterate();
        CorpusEditor corpusEditor = basedefichesEditor.getCorpusEditor(currentAbregeFiche.getCorpus().getSubsetKey());
        corpusEditor.saveFiche(currentAbregeFiche, ficheBuffer.toFiche());
        corpusEditor.setDate(currentAbregeFiche, DateInteger.current(), true);
    }

    private String getConcatedParameterValues(String paramName, char delimiter) {
        String[] paramValues = getParameterValues(paramName);
        int length = paramValues.length;
        if (length == 1) {
            return paramValues[0];
        }
        StringBuilder buf = new StringBuilder();
        buf.append(paramValues[0]);
        for (int i = 1; i < length; i++) {
            if (delimiter != 0) {
                buf.append(delimiter);
            }
            buf.append(paramValues[i]);
        }
        return buf.toString();
    }


    private class FicheBuffer {

        private CorpusMetadata corpusMetadata;
        private FicheParser ficheParser;
        private Fiche fiche;
        private Map<FieldKey, FieldBuffer> fieldBufferMap = new HashMap<FieldKey, FieldBuffer>();
        private boolean redacteursDone = false;
        private boolean all;

        FicheBuffer(AbregeFiche abregeFiche, boolean all) {
            corpusMetadata = abregeFiche.getCorpus().getCorpusMetadata();
            if (all) {
                fiche = new Fiche();
            } else {
                fiche = abregeFiche.getCorpus().getFiche(abregeFiche);
            }
            this.all = all;
            LocaleBundle localeBundle = LocaleBundle.getLocaleBundle(localisationInfo.getFormatLocale());
            SubsetKey defaultSphereKey = null;
            if (userRedacteurKey != null) {
                defaultSphereKey = userRedacteurKey.getSphereKey();
            }
            ficheParser = new FicheParser(basedefiches, fiche, defaultSphereKey, localeBundle, typoOptions);
            ficheParser.setFieldAppend(ficheFieldAppend);
        }

        public Fiche toFiche() {
            if ((!redacteursDone) && (all)) {
                if (userRedacteurKey != null) {
                    fiche.setRedacteurs(new MutableFicheItemList(new Personne(userRedacteurKey.toString())));
                }
            }
            return fiche;
        }

        public void iterate() {
            for (String paramName : currentParamNameSet) {
                if (testSubfield(paramName)) {
                    continue;
                }
                CorpusField corpusField = getCorpusField(paramName);
                if (corpusField == null) {
                    continue;
                }
                String s = concatParameterValues(corpusField, paramName);
                parseCorpusField(corpusField, s);
            }
            if (fieldBufferMap.size() > 0) {
                for (FieldBuffer fieldBuffer : fieldBufferMap.values()) {
                    fieldBuffer.flush();
                }
            }
        }

        private String concatParameterValues(CorpusField corpusField, String paramName) {
            char delimiter = 0;
            FieldKey fieldKey = corpusField.getFieldKey();
            if ((fieldKey.isInformationFieldKey()) || (fieldKey.equals(FieldKey.REDACTEURS))) {
                delimiter = itemListSeparateur;
            } else if ((fieldKey.isAnnexeFieldKey()) || (fieldKey.equals(FieldKey.TEXTE))) {
                delimiter = '\n';
            }
            return getConcatedParameterValues(paramName, delimiter);
        }

        private CorpusField getCorpusField(String paramName) {
            try {
                FieldKey fieldKey = FieldKey.parse(paramName);
                CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
                return corpusField;
            } catch (java.text.ParseException pe) {
                return null;
            }
        }

        private void parseCorpusField(CorpusField corpusField, String value) {
            FieldKey fieldKey = corpusField.getFieldKey();
            if (fieldKey.isSpecialFieldKey()) {
                if (fieldKey.equals(FieldKey.TITRE)) {
                    ficheParser.parseTitre(value);
                } else if (fieldKey.equals(FieldKey.SOUSTITRE)) {
                    ficheParser.parseSoustitre(value);
                } else if (fieldKey.equals(FieldKey.LANG)) {
                    ficheParser.parseLang(value);
                } else if (fieldKey.equals(FieldKey.REDACTEURS)) {
                    redacteursDone = true;
                    String[] tokens = StringUtils.getCleanedTokens(value, itemListSeparateur);
                    ficheParser.parseRedacteurs(tokens);
                } else if (fieldKey.equals(FieldKey.TEXTE)) {
                    ficheParser.parseTexte(value);
                } else if (fieldKey.equals(FieldKey.IDCORPUS)) {
                } else {
                    throw new SwitchException("fieldKey = " + fieldKey.toString());
                }
            } else if (fieldKey.isProprieteFieldKey()) {
                ficheParser.parsePropriete(corpusField, cleanProprieteToken(value));
            } else if (fieldKey.isInformationFieldKey()) {
                String[] tokens = StringUtils.getCleanedTokens(value, itemListSeparateur);
                ficheParser.parseInformation(corpusField, tokens);
            } else if (fieldKey.isAnnexeFieldKey()) {
                ficheParser.parseAnnexe(corpusField, value);
            }
        }

        private String cleanProprieteToken(String s) {
            int length = s.length();
            int p = length;
            for (int i = (length - 1); i >= 0; i--) {
                char carac = s.charAt(i);
                if ((carac == itemListSeparateur) || (Character.isWhitespace(carac))) {
                    p--;
                } else {
                    break;
                }
            }
            if (p == 0) {
                return "";
            } else if (p == length) {
                return s;
            } else {
                return s.substring(0, p);
            }
        }

        private boolean testSubfield(String paramName) {
            try {
                SubfieldKey subfieldKey = SubfieldKey.parse(paramName);
                parseSubfield(subfieldKey);
                return true;
            } catch (java.text.ParseException pe) {
                return false;
            }

        }

        private void parseSubfield(SubfieldKey subfieldKey) {
            FieldKey fieldKey = subfieldKey.getFieldKey();
            CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
            if (corpusField == null) {
                return;
            }
            if (!SubfieldKey.isLegalSubfield(subfieldKey.getSubtype(), corpusField.getFicheItemType())) {
                return;
            }
            FieldBuffer fieldBuffer = fieldBufferMap.get(fieldKey);
            if (fieldBuffer == null) {
                fieldBuffer = new FieldBuffer(corpusField);
                fieldBufferMap.put(fieldKey, fieldBuffer);
            }
            fieldBuffer.add(subfieldKey);
        }


        class FieldBuffer {

            CorpusField corpusField;
            List<SubfieldKey> subfieldKeyList = new ArrayList<SubfieldKey>();

            FieldBuffer(CorpusField corpusField) {
                this.corpusField = corpusField;
            }

            void add(SubfieldKey subfieldKey) {
                subfieldKeyList.add(subfieldKey);
            }

            void flush() {
                switch (corpusField.getFicheItemType()) {
                    case CorpusField.MONTANT_FIELD:
                        flushMontant();
                        break;
                    case CorpusField.PERSONNE_FIELD:
                        flushPersonne();
                        break;
                    case CorpusField.GEOPOINT_FIELD:
                        flushGeopoint();
                        break;
                    case CorpusField.IMAGE_FIELD:
                        flushImage();
                        break;
                }
            }

            private void flushMontant() {
                String num = "";
                String cur = "";
                for (SubfieldKey subfieldKey : subfieldKeyList) {
                    String value = getParameter(subfieldKey.toString());
                    short type = subfieldKey.getSubtype();
                    switch (type) {
                        case SubfieldKey.CUR_SUBTYPE:
                            cur = value;
                            break;
                        case SubfieldKey.VALUE_SUBTYPE:
                            num = value;
                            break;
                    }
                }
                ficheParser.parseMontantPropriete(corpusField, num, cur);
            }

            private void flushGeopoint() {
                String latitude = "";
                String longitude = "";
                for (SubfieldKey subfieldKey : subfieldKeyList) {
                    String value = getParameter(subfieldKey.toString());
                    short type = subfieldKey.getSubtype();
                    switch (type) {
                        case SubfieldKey.LATITUDE_SUBTYPE:
                            latitude = value;
                            break;
                        case SubfieldKey.LONGITUDE_SUBTYPE:
                            longitude = value;
                            break;
                        default:
                            continue;
                    }
                }
                ficheParser.parseGeopointPropriete(corpusField, latitude, longitude);
            }

            private void flushPersonne() {
                String nom = "";
                String prenom = "";
                String original = "";
                String nomAvant = "";
                for (SubfieldKey subfieldKey : subfieldKeyList) {
                    String value = getParameter(subfieldKey.toString());
                    short type = subfieldKey.getSubtype();
                    switch (type) {
                        case SubfieldKey.NOM_SUBTYPE:
                            nom = value;
                            break;
                        case SubfieldKey.PRENOM_SUBTYPE:
                            prenom = value;
                            break;
                        case SubfieldKey.ORIGINAL_SUBTYPE:
                            original = value;
                            break;
                        case SubfieldKey.NOMAVANT_SUBTYPE:
                            nomAvant = value;
                            break;
                        default:
                            continue;
                    }
                }
                ficheParser.parsePersonnePropriete(corpusField, nom, prenom, original, nomAvant);
            }

            private void flushImage() {
                String src = "";
                String alt = "";
                String title = "";
                for (SubfieldKey subfieldKey : subfieldKeyList) {
                    String value = getParameter(subfieldKey.toString());
                    short type = subfieldKey.getSubtype();
                    switch (type) {
                        case SubfieldKey.SRC_SUBTYPE:
                            src = value;
                            break;
                        case SubfieldKey.ALT_SUBTYPE:
                            alt = value;
                            break;
                        case SubfieldKey.TITLE_SUBTYPE:
                            title = value;
                            break;
                        default:
                            continue;
                    }
                }
                ficheParser.parseImagePropriete(corpusField, src, alt, title);
            }

        }

    }

}
