/* BdfServer - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers.blockparsers;

import net.mapeadores.basedefiches.corpus.fiche.*;
import fr.exemole.bdfserver.request.parsers.*;
import fr.exemole.bdfserver.request.*;


/**
 *
 * @author Vincent Calame
 */
public class CodeParser extends ZoneBlockParser {

    private LineParser xmlLineParser = new XmlLineParser();
    private LineParser scriptLineParser = new ScriptLineParser();
    private LineParser defaultLineParser = new DefaultLineParser();

    CodeParser(TextParser textParser) {
        super(textParser);
    }

    public boolean isStartLine(String line) {
        return line.startsWith("+++");
    }

    public void parse(String[] lineArray, int startIndex) {
        setLineArray(lineArray);
        LineInfo lineInfo = getLineInfo(startIndex);
        String line = lineInfo.getLine();
        short type = Code.SANS_TYPE;
        if (line.length() > 3) {
            type = Code.typeToShort(line.charAt(3));
        }
        parse(startIndex + 1, type, lineInfo);
    }

    void parse(int beginIndex, short type, LineInfo deblineInfo) {
        LineParser lineParser = null;
        switch (type) {
            case Code.TYPE_XML:
                lineParser = xmlLineParser;
                break;
            case Code.TYPE_SCRIPT:
                lineParser = scriptLineParser;
                break;
            default:
                lineParser = defaultLineParser;
        }
        lineParser.reinit();
        Code code = new Code(lineParser.getType());
        addIdentifiersInfo(code, deblineInfo);
        beginIndex = parseZoneBlockElements(code, beginIndex);
        int indent = 0;
        int stop = getLineArrayLength() - 1;
        int clearLineCount = -99999;
        for (int i = beginIndex; i <= stop; i++) {
            LineInfo lineInfo = getLineInfo(i);
            String line = lineInfo.getLine();
            if (line.startsWith("+++")) {
                stop = i;
                break;
            }
            if (line.length() > 0) {
                if (clearLineCount > 0) {
                    for (int j = 0; j < clearLineCount; j++) {
                        code.addLn(new Ln(""));
                    }
                }
                clearLineCount = 0;
                lineInfo.setLine(testeDebut(line));
                code.addLn(lineParser.parse(lineInfo));
            } else {
                clearLineCount++;
            }
        }
        setParseResult(stop, code);
    }

    private String testeDebut(String s) {
        if (s.charAt(0) == '\\') {
            if (s.length() > 1) {
                char carac2 = s.charAt(1);
                if ((carac2 == '+') || (carac2 == '\\')) {
                    s = s.substring(1);
                } else if ((carac2 == '[') || (carac2 == '\\')) {
                    s = s.substring(1);
                }
            }
        }
        return s;
    }


    private abstract class LineParser {

        protected int indent = 0;
        protected boolean onComment = false;
        protected LineInfo lineInfo;

        public abstract short getType();

        public void reinit() {
            indent = 0;
            onComment = false;
        }

        public int getIndent() {
            return indent;
        }

        abstract Ln parse(LineInfo lineInfo);

        protected Ln createLn(String line, int indentation) {
            Ln ln = new Ln(line);
            ln.setClassName(lineInfo.getClassName());
            ln.setHtmlId(lineInfo.getHtmlId());
            ln.setIndentation(indentation);
            return ln;
        }

        protected void setLineInfo(LineInfo lineInfo) {
            this.lineInfo = lineInfo;
        }

    }


    private class DefaultLineParser extends LineParser {

        public short getType() {
            return Code.SANS_TYPE;
        }

        Ln parse(LineInfo lineInfo) {
            setLineInfo(lineInfo);
            String line = lineInfo.getLine();
            int locid = 0;
            for (int i = 0; i < line.length(); i++) {
                char carac = line.charAt(i);
                if (carac == '_') {
                    locid++;
                } else if (carac == '\\') {
                    if ((i < (line.length() - 1)) && (line.charAt(i + 1) == '_')) {
                        line = line.substring(locid + 1);
                    } else {
                        line = line.substring(locid);
                    }
                    break;
                } else {
                    if (locid > 0) {
                        line = line.substring(locid);
                    }
                    break;
                }
            }
            return createLn(line, locid);
        }

    }


    private class XmlLineParser extends LineParser {

        public short getType() {
            return Code.TYPE_XML;
        }

        Ln parse(LineInfo lineInfo) {
            setLineInfo(lineInfo);
            String line = lineInfo.getLine();
            int ajout = 0;
            int length = line.length();
            for (int j = 0; j < length - 1; j++) {
                char carac = line.charAt(j);
                char suivant = line.charAt(j + 1);
                if (!onComment) {
                    if (carac == '<') {
                        if (suivant == '/') {
                            ajout--;
                        } else if (suivant == '?') {
                        } else if (suivant == '!') {
                            onComment = true;
                        } else {
                            ajout++;
                        }
                    } else if (carac == '/') {
                        if (suivant == '>') {
                            ajout--;
                        }
                    }
                } else {
                    if (carac == '-') {
                        if (j < (length - 2)) {
                            if ((suivant == '-') && (line.charAt(j + 2) == '>')) {
                                onComment = false;
                            }
                        }
                    }
                }
            }
            if (ajout < 0) {
                indent = indent + ajout;
                if (indent < 0) {
                    indent = 0;
                }
            }
            Ln ln = createLn(line, indent);
            if (ajout > 0) {
                indent = indent + ajout;
            }
            return ln;
        }

    }


    private class ScriptLineParser extends LineParser {

        public short getType() {
            return Code.TYPE_SCRIPT;
        }

        Ln parse(LineInfo lineInfo) {
            setLineInfo(lineInfo);
            String line = lineInfo.getLine();
            boolean debut = true;
            int indentChange = 0;
            for (int j = 0; j < line.length(); j++) {
                char carac = line.charAt(j);
                if (carac == '{') {
                    indentChange++;
                    debut = false;
                } else if (carac == '}') {
                    if (debut) {
                        indent--;
                    } else {
                        indentChange--;
                    }
                } else if (carac != ' ') {
                    debut = false;
                }
            }
            Ln ln = createLn(line, Math.max(indent, 0));
            indent = indent + indentChange;
            if (indent < 0) {
                indent = 0;
            }
            return ln;
        }

    }

}
