/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.corpus;

import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.ComponentUIFilter;
import fr.exemole.bdfserver.ui.ComponentUIList;
import fr.exemole.bdfserver.ui.DefaultComponentUIList;
import fr.exemole.bdfserver.ui.options.InputOption;
import fr.exemole.bdfserver.ui.options.LimitOption;
import fr.exemole.bdfserver.ui.options.PiocheSelectOption;
import fr.exemole.bdfserver.ui.options.ValueOption;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusUtils;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.IncludeKey;
import net.mapeadores.basedefiches.include.FilteredIncludeKey;
import net.mapeadores.util.collections.multilist.MultiList;
import net.mapeadores.util.collections.multilist.MultiListItem;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.primitives.BitmaskUtils;


/**
 *
 * @author Vincent Calame
 */
public class CorpusUI {

    final static int MULTILIST_COUNT = 7;
    public final static int ALL_LIST = -1;
    public final static int FIELD_AND_INCLUDE_LIST = 0;
    public final static int FIELDUI_LIST = 1;
    public final static int INCLUDEUI_LIST = 2;
    public final static int INDEXATION_INCLUDEUI_LIST = 3;
    public final static int LIAGE_INCLUDEUI_LIST = 4;
    public final static int COMMENT_LIST = 5;
    public final static int DUPLICATECOMMENTUI_LIST = 6;
    private Map<String, ComponentUI> componentUIMap = new HashMap<String, ComponentUI>();
    private MultiList multiList = new MultiList(MULTILIST_COUNT);
    private Set<SubsetKey> rattachementSet = Collections.emptySet();
    private Corpus corpus;
    private InternalComponentUIEditor internalComponentUIEditor = new InternalComponentUIEditor();

    public CorpusUI(Corpus corpus) {
        this.corpus = corpus;
        checkCorpus();
    }

    /**
     * Vérifie qu'il existe bien une information d'interface pour toutes les méta-données du corpus et
     * les crèe au besoin.
     */
    public void checkCorpus() {
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        checkField(corpusMetadata, FieldKey.TITRE);
        checkField(corpusMetadata, FieldKey.SOUSTITRE);
        checkField(corpusMetadata, FieldKey.REDACTEURS);
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            checkCorpusField(corpusMetadata.getProprieteField(i));
        }
        checkField(corpusMetadata, FieldKey.TEXTE);
        int annexeCount = corpusMetadata.getAnnexeFieldCount();
        for (int i = 0; i < annexeCount; i++) {
            checkCorpusField(corpusMetadata.getAnnexeField(i));
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            checkCorpusField(corpusMetadata.getInformationField(i));
        }
        checkField(corpusMetadata, FieldKey.LANG);
        IncludeUI liageUI = (IncludeUI) componentUIMap.get(IncludeKey.LIAGE.toString());
        if (liageUI == null) {
            liageUI = new DefaultIncludeUI(IncludeKey.LIAGE);
            new ComponentUIItem(liageUI);
            componentUIMap.put(IncludeKey.LIAGE.toString(), liageUI);
        }
        checkRattachement();
    }

    public void checkRattachement() {
        String rattachementKey = IncludeKey.RATTACHEMENT.toString();
        rattachementSet = CorpusUtils.getRattachementSet(corpus);
        IncludeUI rattachementUI = (IncludeUI) componentUIMap.get(rattachementKey);
        if (rattachementSet.size() == 0) {
            if (rattachementUI != null) {
                removeComponentUI(rattachementKey);
            }
            return;
        }
        if (rattachementUI == null) {
            rattachementUI = new DefaultIncludeUI(IncludeKey.RATTACHEMENT);
            new ComponentUIItem(rattachementUI);
            componentUIMap.put(rattachementKey, rattachementUI);
        }
        List<String> keyToRemoveList = new ArrayList<String>();
        for (String key : componentUIMap.keySet()) {
            try {
                FilteredIncludeKey filteredIncludeKey = FilteredIncludeKey.parse(key);
                SubsetKey subsetKey = filteredIncludeKey.getIncludeKey().getSubsetKey();
                if (rattachementSet.contains(subsetKey)) {
                    keyToRemoveList.add(key);
                }

            } catch (java.text.ParseException pe) {
            }
        }
        for (String key : keyToRemoveList) {
            removeComponentUI(key);
        }
    }

    private void checkField(CorpusMetadata corpusMetadata, FieldKey fieldKey) {
        CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
        if (corpusField != null) {
            checkCorpusField(corpusField);
        }
    }

    private void checkCorpusField(CorpusField corpusField) {
        FieldUI fieldUI = (FieldUI) componentUIMap.get(corpusField.getFieldKey().toString());
        if (fieldUI == null) {
            fieldUI = new DefaultFieldUI(corpusField);
            new ComponentUIItem(fieldUI);
            componentUIMap.put(corpusField.getFieldKey().toString(), fieldUI);
        }
    }

    public boolean ownsToRattachement(SubsetKey subsetKey) {
        return (rattachementSet.contains(subsetKey));
    }

    public boolean isAssociationActive() {
        return componentUIMap.containsKey(IncludeKey.ASSOCIATION.toString());
    }

    public void setAssociationActive(boolean active) {
        String key = IncludeKey.ASSOCIATION.toString();
        if (active) {
            if (!isAssociationActive()) {
                requireIncludeUI(new FilteredIncludeKey(IncludeKey.ASSOCIATION));
            }
        } else {
            removeComponentUI(key);
        }
    }

    private CommentUI requireComment(String name) {
        String key = "comment_" + name;
        CommentUI comment = (CommentUI) componentUIMap.get(key);
        if (comment == null) {
            comment = new DefaultCommentUI(name);
            new ComponentUIItem(comment, false);
            componentUIMap.put(key, comment);
        }
        return comment;
    }

    public CommentUI getCommentUI(String name) {
        String key = "comment_" + name;
        return (CommentUI) componentUIMap.get(key);
    }

    public boolean removeComment(String name) {
        return removeComponentUI("comment_" + name);
    }

    public IncludeUI getIncludeUI(FilteredIncludeKey filteredIncludeKey) {
        return (IncludeUI) componentUIMap.get(filteredIncludeKey.toString());
    }

    public boolean removeFilteredInclude(FilteredIncludeKey filteredIncludeKey) {
        return removeComponentUI(filteredIncludeKey.toString());
    }

    private IncludeUI requireIncludeUI(FilteredIncludeKey filteredIncludeKey) {
        IncludeUI includeUI = getIncludeUI(filteredIncludeKey);
        if (includeUI != null) {
            return includeUI;
        }
        IncludeKey includeKey = filteredIncludeKey.getIncludeKey();
        SubsetKey subsetKey = includeKey.getSubsetKey();
        if ((subsetKey != null) && (ownsToRattachement(subsetKey))) {
            return null;
        }
        if (includeKey.equals(IncludeKey.RATTACHEMENT)) {
            return null;
        }
        if (includeKey.isChronoIncludeKey()) {
            throw new IllegalArgumentException("no chrono include key allowed =" + filteredIncludeKey.toString());
        }
        includeUI = new DefaultIncludeUI(filteredIncludeKey);
        componentUIMap.put(filteredIncludeKey.toString(), includeUI);
        new ComponentUIItem(includeUI);
        return includeUI;
    }

    public FieldUI getFieldUI(FieldKey fieldKey) {
        return (FieldUI) componentUIMap.get(fieldKey.toString());
    }

    /**
     *
     *@throws IllegalArgumentExceptino si fieldKey correspond à un champ obligatoire.
     */
    public boolean removeAllFieldUI(FieldKey fieldKey) {
        if (fieldKey.isMandatoryFieldKey()) {
            throw new IllegalArgumentException("fieldKey.isMandatoryFieldKey()");
        }
        return removeComponentUI(fieldKey.toString());
    }

    /**
     * Supprime s'il existe
     */
    public boolean removeAllIncludeUI(IncludeKey includeKey) {
        String racine = includeKey.toString();
        List<String> keyList = new LinkedList<String>(componentUIMap.keySet());
        boolean done = false;
        for (String key : keyList) {
            if (key.startsWith(racine)) {
                if (removeComponentUI(key)) {
                    done = true;
                }
            }
        }
        return done;
    }

    private boolean removeComponentUI(String key) {
        ComponentUI componentUI = (ComponentUI) componentUIMap.get(key);
        if (componentUI == null) {
            return false;
        }
        MultiListItem multiListItem = getComponentUIItem(componentUI);
        multiList.remove(multiListItem);
        componentUIMap.remove(key);
        if (componentUI instanceof CommentUI) {
            List duplicateList = multiList.getItemSecondaryList(DUPLICATECOMMENTUI_LIST);
            for (int i = duplicateList.size() - 1; i >= 0; i--) {
                MultiListItem multi = (MultiListItem) duplicateList.get(i);
                if (multi.getWrappedObject().equals(componentUI)) {
                    multiList.remove(multi);
                }
            }
        }
        return true;
    }

    public SubsetKey getCorpusSubsetKey() {
        return corpus.getSubsetKey();
    }

    /**
     * Retourne une élément d'éléments de l'interface ordonnés suivant leur ordre d'apparation.
     * Les éléments de cette liste dépendent de l'argument idList.
     *<ul>
     *<li>ALL_LIST : tous les éléments de l'interface
     *<li>FIELDUI_LIST : uniquement les élements correspondant aux champs
     *<li>INDEXATION_INCLUDEUI_LIST : uniquement les élements correspondants à des thésaurus
     *<li>FIELD_AND_INCLUDE_LIST : uniquement les éléments de champs et des inclusion.
     *<li>COMMENT_LIST : uniquement les éléménts de commentaire.
     *</ul>
     */
    public List getOrderedComponentUIList(int idList) {
        if (idList == ALL_LIST) {
            return multiList.getWrappedObjectPrimaryList();
        }
        if ((idList < 0) || (idList >= MULTILIST_COUNT)) {
            throw new IllegalArgumentException();
        }
        return multiList.getWrappedObjectSecondaryList(idList);
    }

    public String[] getPositionArray() {
        List primaryList = multiList.getWrappedObjectPrimaryList();
        int size = primaryList.size();
        String[] result = new String[size];
        for (int i = 0; i < size; i++) {
            Object obj = primaryList.get(i);
            result[i] = ((ComponentUI) obj).getKeyString();
        }
        return result;
    }

    private void parsePositionArray(String[] positionArray) {
        multiList.removeAllInSecondaryList(DUPLICATECOMMENTUI_LIST);
        Set<String> duplicateSet = new HashSet<String>();
        for (int i = 0; i < positionArray.length; i++) {
            String token = positionArray[i];
            ComponentUI componentUI = componentUIMap.get(token);
            if (componentUI != null) {
                if (!duplicateSet.contains(token)) {
                    MultiListItem multiListItem = getComponentUIItem(componentUI);
                    setPosition(multiListItem, i);
                    duplicateSet.add(token);
                } else {
                    if (componentUI instanceof CommentUI) {
                        ComponentUIItem item = new ComponentUIItem((CommentUI) componentUI, true);
                        setPosition(item, i);
                    }
                }
            }
        }
    }

    private void setPosition(MultiListItem multiListItem, int position) {
        if ((position < 0) || (position >= multiList.getPrimarySize())) {
            position = multiList.getPrimarySize() - 1;
        }
        multiList.translate(multiListItem, position);
    }

    private MultiListItem getComponentUIItem(ComponentUI componentUI) {
        if (componentUI instanceof CommentUI) {
            return getComponentUIItem(componentUI, COMMENT_LIST);
        }
        if (componentUI instanceof FieldUI) {
            return getComponentUIItem(componentUI, FIELDUI_LIST);
        }
        if (componentUI instanceof IncludeUI) {
            return getComponentUIItem(componentUI, INCLUDEUI_LIST);
        }
        throw new SwitchException("class = " + componentUI.getClass().getName());
    }

    private MultiListItem getComponentUIItem(ComponentUI componentUI, int listIndex) {
        List list = multiList.getItemSecondaryList(listIndex);
        int size = list.size();
        for (int i = 0; i < size; i++) {
            MultiListItem multi = (MultiListItem) list.get(i);
            if (multi.getWrappedObject().equals(componentUI)) {
                return multi;
            }
        }
        return null;
    }

    private int getIndex(ComponentUI componentUI) {
        List primaryList = multiList.getWrappedObjectPrimaryList();
        int size = primaryList.size();
        int index = -1;
        for (int i = 0; i < size; i++) {
            if (componentUI.equals(primaryList.get(i))) {
                index = i;
                break;
            }
        }
        return index;
    }


    private class ComponentUIItem extends MultiListItem {

        ComponentUI componentUI;

        ComponentUIItem(IncludeUI includeUI) {
            super(multiList, getInitialMask(includeUI));
            this.componentUI = includeUI;
        }

        ComponentUIItem(FieldUI fieldUI) {
            super(multiList, BitmaskUtils.getBitValue(FIELDUI_LIST) | BitmaskUtils.getBitValue(FIELD_AND_INCLUDE_LIST));
            this.componentUI = fieldUI;
        }

        ComponentUIItem(CommentUI commentUI, boolean duplicate) {
            super(multiList, BitmaskUtils.getBitValue(duplicate ? DUPLICATECOMMENTUI_LIST : COMMENT_LIST));
            this.componentUI = commentUI;
        }

        public Object getWrappedObject() {
            return componentUI;
        }

    }

    static int getInitialMask(IncludeUI includeUI) {
        int bitmask = BitmaskUtils.getBitValue(FIELD_AND_INCLUDE_LIST) | BitmaskUtils.getBitValue(INCLUDEUI_LIST);
        IncludeKey includeKey = includeUI.getFiteredIncludeKey().getIncludeKey();
        if (includeKey.isIndexationIncludeKey()) {
            bitmask = bitmask | BitmaskUtils.getBitValue(INDEXATION_INCLUDEUI_LIST);
        } else if (includeKey.isLiageIncludeKey()) {
            bitmask = bitmask | BitmaskUtils.getBitValue(LIAGE_INCLUDEUI_LIST);
        }
        return bitmask;
    }

    public ComponentUIEditor getComponentUIEditor() {
        return internalComponentUIEditor;
    }

    public ComponentUIList toComponentUIList() {
        List liste = multiList.getWrappedObjectPrimaryList();
        int ln = liste.size();
        ComponentUI[] array = new ComponentUI[ln];
        for (int i = 0; i < ln; i++) {
            array[i] = (ComponentUI) liste.get(i);
        }
        return new DefaultComponentUIList(array);
    }

    public ComponentUIList toComponentUIList(ComponentUIFilter componentUIFilter) {
        if (componentUIFilter == null) {
            return toComponentUIList();
        }
        List liste = multiList.getWrappedObjectPrimaryList();
        int ln = liste.size();
        ComponentUI[] array = new ComponentUI[ln];
        int p = 0;
        for (int i = 0; i < ln; i++) {
            ComponentUI componentUI = (ComponentUI) liste.get(i);
            componentUI = componentUIFilter.filter(componentUI);
            if (componentUI != null) {
                array[p] = componentUI;
                p++;
            }
        }
        if (p != ln) {
            ComponentUI[] temp = new ComponentUI[p];
            System.arraycopy(array, 0, temp, 0, p);
            array = temp;
        }
        return new DefaultComponentUIList(array);
    }


    private class InternalComponentUIEditor implements ComponentUIEditor {

        public FieldUI getFieldUI(FieldKey fieldKey) {
            return CorpusUI.this.getFieldUI(fieldKey);
        }

        public IncludeUI getIncludeUI(FilteredIncludeKey filteredIncludeKey, boolean creation) {
            if (creation) {
                return CorpusUI.this.requireIncludeUI(filteredIncludeKey);
            } else {
                return CorpusUI.this.getIncludeUI(filteredIncludeKey);
            }
        }

        public boolean setInputOption(ComponentUI componentUI, InputOption inputOption) {
            if (componentUI instanceof DefaultFieldUI) {
                ((DefaultFieldUI) componentUI).internalUpdateInputOption(inputOption);
                return true;
            } else if (componentUI instanceof DefaultIncludeUI) {
                ((DefaultIncludeUI) componentUI).internalUpdateInputOption(inputOption);
                return true;
            }
            return false;
        }

        public boolean setLimitOption(ComponentUI componentUI, LimitOption limitOption) {
            if (componentUI instanceof DefaultIncludeUI) {
                ((DefaultIncludeUI) componentUI).internalUpdateLimitOption(limitOption);
                return true;
            }
            return false;
        }

        public boolean setPiocheSelectOption(ComponentUI componentUI, PiocheSelectOption piocheSelectOption) {
            if (componentUI instanceof DefaultIncludeUI) {
                ((DefaultIncludeUI) componentUI).internalUpdatePiocheSelectOption(piocheSelectOption);
                return true;
            }
            return false;
        }

        public boolean setValueOption(ComponentUI componentUI, ValueOption valueOption) {
            if (componentUI instanceof DefaultFieldUI) {
                ((DefaultFieldUI) componentUI).internalUpdateValueOption(valueOption);
                return true;
            } else if (componentUI instanceof DefaultIncludeUI) {
                ((DefaultIncludeUI) componentUI).internalUpdateValueOption(valueOption);
                return true;
            }
            return false;
        }

        public boolean setLibelleString(ComponentUI componentUI, int langInteger, String nvlibelle) {
            if (componentUI instanceof DefaultIncludeUI) {
                return ((DefaultIncludeUI) componentUI).internalSetLibelleString(langInteger, nvlibelle);
            } else if (componentUI instanceof DefaultCommentUI) {
                return ((DefaultCommentUI) componentUI).internalSetLibelleString(langInteger, nvlibelle);
            }
            return false;
        }

        public boolean setDefaultLibelleUsage(ComponentUI componentUI, boolean b) {
            if (componentUI instanceof DefaultIncludeUI) {
                return ((DefaultIncludeUI) componentUI).internalSetDefaultLibelleUsage(b);
            }
            return false;
        }

        public CommentUI getCommentUI(String name, boolean creation) {
            if (creation) {
                return CorpusUI.this.requireComment(name);
            } else {
                return CorpusUI.this.getCommentUI(name);
            }
        }

        public boolean setLocation(ComponentUI componentUI, int newLocation) {
            if (componentUI instanceof DefaultCommentUI) {
                return ((DefaultCommentUI) componentUI).internalSetLocation(newLocation);
            }
            return false;
        }

        public boolean setPositionArray(String[] positionArray) {
            parsePositionArray(positionArray);
            return true;
        }

    }

}
