/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.xml;

import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.ui.SubsetKeyGroupTemplates;
import fr.exemole.bdfserver.ui.SubsetKeyTree;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.xml.TagUtils;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class TreesDOMReader {

    private ErrorHandler errorHandler;
    private SubsetKeyTree currentSubsetKeyTree;
    private UIManager currentUiManager;

    public TreesDOMReader(ErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
    }

    public void readTrees(UIManager uiManager, Element element_xml) {
        this.currentUiManager = uiManager;
        NodeList liste = element_xml.getChildNodes();
        int length = liste.getLength();
        for (int i = 0; i < length; i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("metadata")) {
                    readMetadata(element);
                }
            }
        }
        for (int i = 0; i < length; i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("metadata")) {
                    continue;
                } else if (tagName.equals("tree")) {
                    String typeString = element.getAttribute("type");
                    if (typeString.length() == 0) {
                        errorHandler.emptyAttributeError("tree", "type");
                        continue;
                    }
                    try {
                        short type = SubsetKeyTree.toShortType(typeString);
                        readTree(uiManager.getSubsetKeyTree(type), element);
                    } catch (IllegalArgumentException iae) {
                        errorHandler.attributeSyntaxError("tree", "type", typeString);
                        continue;
                    }
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
        this.currentUiManager = null;
    }

    private void readMetadata(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("group-def")) {
                    readGroupDef(element);
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }

    private void readGroupDef(Element element_xml) {
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            errorHandler.emptyAttributeError("group-def", "name");
            return;
        }
        SubsetKeyGroupTemplates skgt = currentUiManager.getSubsetKeyGroupTemplates();
        skgt.createTemplate(name);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("lib")) {
                    try {
                        Libelle lib = TagUtils.getLibelle(element);
                        skgt.setLibelleString(name, lib.getLangInteger(), lib.getLibelleString());
                    } catch (LangIntegerException ile) {
                        errorHandler.attributeSyntaxError("lib", "xml:lang", element.getAttribute("xml:lang"));
                    }
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }

    private void readTree(SubsetKeyTree subsetKeyTree, Element element_xml) {
        this.currentSubsetKeyTree = subsetKeyTree;
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("group")) {
                    readGroup(element);
                } else if (tagName.equals("subset")) {
                    readSubset(element, null);
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
        this.currentSubsetKeyTree = null;
    }

    private void readGroup(Element element_xml) {
        String groupName = element_xml.getAttribute("name");
        if (groupName.length() == 0) {
            groupName = null;
        } else {
            if (!currentSubsetKeyTree.getSubsetKeyGroupTemplates().containsGroupTemplate(groupName)) {
                errorHandler.unknwonGroupName(groupName);
                return;
            }
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("subset")) {
                    readSubset(element, groupName);
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }

    private void readSubset(Element element_xml, String groupName) {
        String subsetKeyString = XMLUtils.getData(element_xml);
        if (subsetKeyString.length() == 0) {
            errorHandler.emptyTagError("subset");
            return;
        }
        try {
            SubsetKey subsetKey = SubsetKey.parse(subsetKeyString);
            if (!currentSubsetKeyTree.isLegalSubsetKey(subsetKey)) {
                errorHandler.wrongSubsetKeyError("subset", subsetKey);
                return;
            }
            currentSubsetKeyTree.addSubsetKey(subsetKey, groupName);
        } catch (java.text.ParseException pe) {
            errorHandler.textSyntaxError("subset", subsetKeyString);
        }
    }


    public static interface ErrorHandler {

        public void unknownTagWarning(String tagName);

        public void emptyAttributeError(String tagName, String attributeName);

        public void attributeSyntaxError(String tagName, String type, String attributeValue);

        public void emptyTagError(String tagName);

        public void textSyntaxError(String tagName, String text);

        public void wrongSubsetKeyError(String tagName, SubsetKey subsetKey);

        public void unknwonGroupName(String groupName);

    }

}
