/* BdfServer_DirectoryStorage - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.beans.BdfUserBean;
import fr.exemole.bdfserver.beans.ManagerBean;
import fr.exemole.bdfserver.managers.BeanableManager;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.managers.ManagerBeanStorage;
import fr.exemole.bdfserver.managers.PasswordManager;
import fr.exemole.bdfserver.managers.PermissionManager;
import fr.exemole.bdfserver.managers.PolicyManager;
import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.managers.xml.ConfigurationManagerDOMReader;
import fr.exemole.bdfserver.managers.xml.ConfigurationManagerXMLWriter;
import fr.exemole.bdfserver.managers.xml.ManagerDOMReader;
import fr.exemole.bdfserver.managers.xml.ManagerXMLWriter;
import fr.exemole.bdfserver.managers.xml.PasswordManagerDOMReader;
import fr.exemole.bdfserver.managers.xml.PasswordManagerXMLWriter;
import fr.exemole.bdfserver.managers.xml.PermissionManagerDOMReader;
import fr.exemole.bdfserver.managers.xml.PermissionManagerXMLWriter;
import fr.exemole.bdfserver.managers.xml.PolicyManagerDOMReader;
import fr.exemole.bdfserver.managers.xml.PolicyManagerXMLWriter;
import fr.exemole.bdfserver.managers.xml.SqlExportManagerDOMReader;
import fr.exemole.bdfserver.managers.xml.SqlExportManagerXMLWriter;
import fr.exemole.bdfserver.roles.RoleMap;
import fr.exemole.bdfserver.sqlexport.SqlExportManager;
import fr.exemole.bdfserver.storage.BdfStorage;
import fr.exemole.bdfserver.storage.BdfStorageException;
import fr.exemole.bdfserver.storage.directory.bdf.utils.LogUiErrorHandler;
import fr.exemole.bdfserver.storage.directory.bdf.utils.ManagerErrorHandler;
import fr.exemole.bdfserver.storage.directory.bdf.utils.TreesErrorHandler;
import fr.exemole.bdfserver.ui.AlterUI;
import fr.exemole.bdfserver.ui.DefaultAlterUI;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.xml.ComponentUiFilterDOMReader;
import fr.exemole.bdfserver.ui.xml.CorpusUiDOMReader;
import fr.exemole.bdfserver.ui.xml.DefaultUIFilterBuilder;
import fr.exemole.bdfserver.ui.xml.TreesDOMReader;
import fr.exemole.bdfserver.ui.xml.UiErrorHandler;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserBeanStorage;
import fr.exemole.bdfserver.users.xml.UserDOMReader;
import java.io.File;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesEditor;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.logging.MutableProcessLog;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryBdfStorage implements BdfStorage {

    private BdfDirectories bdfDirectories;
    private DirectoryBasedefiches directoryBasedefiches;
    private DirectoryRoleMap roleMap;
    private MutableProcessLog storageProcessLog;
    private BdfStorageException currentBdfStorageException;
    private InternalExceptionHandler exceptionHandler = new InternalExceptionHandler();

    public DirectoryBdfStorage(File bdfStorageDir, File cacheDir, File backupDir, MutableProcessLog storageProcessLog) {
        this.storageProcessLog = storageProcessLog;
        bdfDirectories = new BdfDirectories(bdfStorageDir, cacheDir, backupDir);
        directoryBasedefiches = new DirectoryBasedefiches(bdfDirectories);
        directoryBasedefiches.init(storageProcessLog, exceptionHandler);
        roleMap = new DirectoryRoleMap(bdfDirectories);
        roleMap.init(storageProcessLog);
    }

    private void checkException() {
        if (currentBdfStorageException != null) {
            throw currentBdfStorageException;
        }
    }

    public RoleMap getRoleMap() {
        return roleMap;
    }

    public void saveBdfUser(BdfUser bdfUser) {
        checkException();
        BdfStorageUtils.saveUser(bdfDirectories, bdfUser);
    }

    public boolean hasBdfUserPrefs(RedacteurKey redacteurKey) {
        StorageFile storageFile = BdfStorageUtils.getUserStorageFile(bdfDirectories, redacteurKey);
        return storageFile.exists();
    }

    public void loadBdfUser(BdfUser bdfUser) {
        StorageFile storageFile = BdfStorageUtils.getUserStorageFile(bdfDirectories, bdfUser.getRedacteurKey());
        Document doc = storageFile.readDocument();
        if (doc == null) {
            return;
        }
        String tagName = doc.getDocumentElement().getTagName();
        if (tagName.equals("bdf-user-bean")) {
            BdfUserBean bdfUserBean = (BdfUserBean) storageFile.decodeBean(BdfUserBean.class);
            if (bdfUserBean != null) {
                BdfUserBeanStorage.fillBdfUser(bdfUser, bdfUserBean);
            }
        } else {
            UserDOMReader domReader = new UserDOMReader();
            domReader.fillUser(bdfUser, doc.getDocumentElement());
        }

    }

    public void loadManager(Object manager) {
        if (manager instanceof PermissionManager) {
            loadManager(manager, "permissionmanager", "permission-manager-bean", new PermissionManagerDOMReader());
        } else if (manager instanceof PasswordManager) {
            loadManager(manager, "passwordmanager", "password-manager-bean", new PasswordManagerDOMReader());
        } else if (manager instanceof SqlExportManager) {
            loadManager(manager, "sqlexportmanager", "sql-export-manager-bean", new SqlExportManagerDOMReader());
        } else if (manager instanceof PolicyManager) {
            loadManager(manager, "policymanager", "policy-manager-bean", new PolicyManagerDOMReader());
        } else if (manager instanceof ConfigurationManager) {
            loadManager(manager, "configurationmanager", "configuration-manager-bean", new ConfigurationManagerDOMReader());
        } else if (manager instanceof UIManager) {
            loadUiManager((UIManager) manager);
        } else if (manager instanceof BeanableManager) {
            loadBeanableManager((BeanableManager) manager);
        }
    }

    private void loadManager(Object manager, String managerName, String beanName, ManagerDOMReader managerDOMReader) {
        StorageFile storageFile = BdfStorageUtils.getConfStorageFile(bdfDirectories, managerName);
        Document doc = storageFile.readDocument();
        if (doc == null) {
            return;
        }
        storageProcessLog.setCurrentURI(storageFile.toProcessURI());
        String tagName = doc.getDocumentElement().getTagName();
        if (tagName.equals(beanName)) {
            BeanableManager beanableManager = (BeanableManager) manager;
            ManagerBean managerBean = (ManagerBean) storageFile.decodeBean(ManagerBeanStorage.getBeanClass(beanableManager));
            if (managerBean != null) {
                ManagerBeanStorage.fillManager(beanableManager, managerBean);
            }
        } else {
            ManagerErrorHandler errorHandler = new ManagerErrorHandler(storageProcessLog);
            managerDOMReader.setErrorHandler(errorHandler);
            managerDOMReader.readManager(manager, doc.getDocumentElement());
        }
    }

    private void loadUiManager(UIManager uiManager) {
        boolean done = false;
        StorageFile treesStorageFile = BdfStorageUtils.getUiStorageFile(bdfDirectories, "trees");
        Document doc = treesStorageFile.readDocument();
        if (doc != null) {
            done = true;
            storageProcessLog.setCurrentURI(treesStorageFile.toProcessURI());
            TreesDOMReader treesDOMReader = new TreesDOMReader(new TreesErrorHandler(storageProcessLog));
            treesDOMReader.readTrees(uiManager, doc.getDocumentElement());
        }
        LogUiErrorHandler uiErrorHandler = new LogUiErrorHandler(storageProcessLog);
        Basedefiches basedefiches = uiManager.getBdfServer().getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            StorageFile corpusUiStorageFile = BdfStorageUtils.getUiStorageFile(bdfDirectories, corpus.getSubsetKey().toString());
            Document doccorpus = corpusUiStorageFile.readDocument();
            if (doccorpus != null) {
                done = true;
                CorpusUI corpusUI = uiManager.getCorpusUI(corpus);
                storageProcessLog.setCurrentURI(corpusUiStorageFile.toProcessURI());
                CorpusUiDOMReader corpusUiDOMReader = new CorpusUiDOMReader(corpusUI.getComponentUIEditor(), uiErrorHandler);
                corpusUiDOMReader.readCorpusUI(doccorpus.getDocumentElement());
            }
        }
        if (!done) {
            loadBeanableManager(uiManager);
            uiManager.getBdfServer().setSaveAllConf(true);
        }
    }

    public AlterUI getAlterUI(CorpusUI corpusUI, String alterName) {
        StorageFile alterUiStorageFile = BdfStorageUtils.getAlterUiStorageFile(bdfDirectories, corpusUI.getCorpusSubsetKey(), alterName);
        if (!alterUiStorageFile.exists()) {
            return null;
        }
        Document doc = alterUiStorageFile.readDocument();
        if (doc == null) {
            return null;
        }
        Element documentElement = doc.getDocumentElement();
        String tagName = documentElement.getTagName();
        if (tagName.equals("ui-filter")) {
            DefaultUIFilterBuilder builder = new DefaultUIFilterBuilder(corpusUI);
            ComponentUiFilterDOMReader reader = new ComponentUiFilterDOMReader(builder, new NullUiErrorHandler());
            reader.readComponentUiFilter(documentElement);
            DefaultAlterUI defaultAlterUI = new DefaultAlterUI(builder.toComponentUIFilter());
            String responseFactory = documentElement.getAttribute("response-factory");
            if (responseFactory.length() > 0) {
                defaultAlterUI.setResponseFactory(responseFactory);
            }
            return defaultAlterUI;
        } else {
            return null;
        }
    }

    private void loadBeanableManager(BeanableManager beanableManager) {
        String fileName = ManagerBeanStorage.getBeanFileName(beanableManager);
        StorageFile storageFile = BdfStorageUtils.getConfStorageFile(bdfDirectories, fileName);
        ManagerBean managerBean = (ManagerBean) storageFile.decodeBean(ManagerBeanStorage.getBeanClass(beanableManager));
        if (managerBean != null) {
            ManagerBeanStorage.fillManager(beanableManager, managerBean);
        }
    }

    public void saveManager(Object manager) {
        checkException();
        if (manager instanceof PermissionManager) {
            saveManager(manager, "permissionmanager", new PermissionManagerXMLWriter());
        } else if (manager instanceof PasswordManager) {
            saveManager(manager, "passwordmanager", new PasswordManagerXMLWriter());
        } else if (manager instanceof SqlExportManager) {
            saveManager(manager, "sqlexportmanager", new SqlExportManagerXMLWriter());
        } else if (manager instanceof PolicyManager) {
            saveManager(manager, "policymanager", new PolicyManagerXMLWriter());
        } else if (manager instanceof ConfigurationManager) {
            saveManager(manager, "configurationmanager", new ConfigurationManagerXMLWriter());
        } else if (manager instanceof UIManager) {
            BdfStorageUtils.saveUIManager(bdfDirectories, (UIManager) manager);
        } else if (manager instanceof BeanableManager) {
            saveManagerBean(ManagerBeanStorage.toBean((BeanableManager) manager));
        }
    }

    private void saveManager(Object manager, String managerName, ManagerXMLWriter managerXMLWriter) {
        BdfStorageUtils.saveManager(bdfDirectories, manager, managerName, managerXMLWriter);
    }

    private void saveManagerBean(ManagerBean managerBean) {
        String fileName = ManagerBeanStorage.getBeanFileName(managerBean);
        StorageFile storageFile = BdfStorageUtils.getConfStorageFile(bdfDirectories, fileName);
        storageFile.encodeBean(managerBean);
    }

    public Basedefiches getBasedefiches() {
        return directoryBasedefiches;
    }

    public BasedefichesEditor getBasedefichesEditor() {
        checkException();
        return directoryBasedefiches.getInternalBasedefichesEditor();
    }


    private class InternalExceptionHandler implements ExceptionHandler {

        public void handleBdfStorageException(BdfStorageException bdfStorageException) {
            currentBdfStorageException = bdfStorageException;
        }

    }


    private static class NullUiErrorHandler implements UiErrorHandler {

        public void unknownTagWarning(String tagName) {
        }

        public void emptyAttributeError(String tagName, String attributeName) {
        }

        public void attributeSyntaxError(String tagName, String type, String attributeValue) {
        }

        public void emptyTagError(String tagName) {
        }

        public void textSyntaxError(String tagName, String text) {
        }

        public void unknownKeyError(String tagName, Object key) {
        }

    }

}
