/* BdfServer_DirectoryStorage - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.storage.BdfStorageException;
import java.util.HashSet;
import java.util.Timer;
import java.util.TimerTask;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.association.DocumentSubset;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.impl.SubsetDataListener;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;


/**
 *
 * @author Vincent Calame
 */
public class DirectorySubsetDataListener implements SubsetDataListener {

    private HashSet<Object> hashSet = new HashSet<Object>();
    private Timer timer = new Timer();
    private boolean isSchedulded = false;
    private BdfDirectories bdfDirectories;
    private Basedefiches basedefiches;
    private ExceptionHandler exceptionHandler;

    DirectorySubsetDataListener(BdfDirectories bdfDirectories, Basedefiches basedefiches, ExceptionHandler exceptionHandler) {
        this.bdfDirectories = bdfDirectories;
        this.basedefiches = basedefiches;
        this.exceptionHandler = exceptionHandler;
    }

    public synchronized void subsetChanged(SubsetKey subsetKey) {
        hashSet.add(subsetKey);
        initTask();

    }

    public synchronized void subsetCoupleChanged(SubsetCouple subsetCouple) {
        hashSet.add(subsetCouple);
        initTask();
    }

    public synchronized void chronoChanged(SubsetKey subsetKey) {
        hashSet.add("chrono+" + subsetKey.toString());
        initTask();
    }

    public synchronized void basedefichesMetadataChanged() {
        hashSet.add("metadata");
        initTask();
    }

    private void initTask() {
        if (!isSchedulded) {
            isSchedulded = true;
            timer.schedule(new SaveTask(this), 10000);
        }
    }

    private synchronized void clear() {
        hashSet.clear();
        isSchedulded = false;
    }


    static class SaveTask extends TimerTask {

        final DirectorySubsetDataListener directorySubsetDataListener;

        private SaveTask(DirectorySubsetDataListener directorySubsetDataListener) {
            this.directorySubsetDataListener = directorySubsetDataListener;
        }

        public void run() {
            BdfDirectories bdfDirectories = directorySubsetDataListener.bdfDirectories;
            Basedefiches basedefiches = directorySubsetDataListener.basedefiches;
            synchronized (directorySubsetDataListener) {
                try {
                    for (Object obj : directorySubsetDataListener.hashSet) {
                        if (obj instanceof SubsetKey) {
                            SubsetKey subsetKey = (SubsetKey) obj;
                            if (subsetKey.isThesaurusSubset()) {
                                Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(subsetKey);
                                if (thesaurus != null) {
                                    BdfStorageUtils.saveThesaurus(bdfDirectories, thesaurus);
                                } else {
                                    BdfStorageUtils.deleteSubset(bdfDirectories, subsetKey);
                                }
                            } else if (subsetKey.isSphereSubset()) {
                                Sphere sphere = (Sphere) basedefiches.getSubset(subsetKey);
                                if (sphere != null) {
                                    BdfStorageUtils.saveSphere(bdfDirectories, sphere);
                                } else {
                                    BdfStorageUtils.deleteSubset(bdfDirectories, subsetKey);
                                }
                            } else if (subsetKey.isCorpusSubset()) {
                                Corpus corpus = (Corpus) basedefiches.getSubset(subsetKey);
                                if (corpus != null) {
                                    BdfStorageUtils.saveCorpusMetadata(bdfDirectories, corpus);
                                } else {
                                    BdfStorageUtils.deleteSubset(bdfDirectories, subsetKey);
                                    StorageFile directoryStorageFile = BdfStorageUtils.getCorpusDirectory(bdfDirectories, subsetKey);
                                    directoryStorageFile.delete();
                                }
                            } else if (subsetKey.isAlbumSubset()) {
                                Album album = (Album) basedefiches.getSubset(subsetKey);
                                if (album != null) {
                                    BdfStorageUtils.saveAlbum(bdfDirectories, album);
                                } else {
                                    BdfStorageUtils.deleteSubset(bdfDirectories, subsetKey);
                                }
                            } else if (subsetKey.equals(SubsetKey.DOCUMENT_SUBSET)) {
                                DocumentSubset documentSubset = (DocumentSubset) basedefiches.getSubset(subsetKey);
                                BdfStorageUtils.saveDocumentSubset(bdfDirectories, documentSubset);
                            }
                        } else if (obj instanceof SubsetCouple) {
                            SubsetCouple couple = (SubsetCouple) obj;
                            Croisement croisement = basedefiches.getCroisement(couple.getSubsetKey1(), couple.getSubsetKey2());
                            if (croisement != null) {
                                BdfStorageUtils.saveCroisement(bdfDirectories, croisement);
                            } else {
                                BdfStorageUtils.deleteSubsetCouple(bdfDirectories, couple);
                            }
                        } else if (obj instanceof String) {
                            String s = (String) obj;
                            if (s.equals("metadata")) {
                                BdfStorageUtils.saveBasedefichesMetadata(bdfDirectories, basedefiches.getBasedefichesMetadata());
                            } else if (s.startsWith("chrono+")) {
                                try {
                                    SubsetKey subsetKey = SubsetKey.parse(s.substring(7));
                                    if (subsetKey.isCorpusSubset()) {
                                        Corpus corpus = (Corpus) basedefiches.getSubset(subsetKey);
                                        if (corpus != null) {
                                            BdfStorageUtils.saveChrono(bdfDirectories, corpus);
                                        } else {
                                            BdfStorageUtils.deleteChrono(bdfDirectories, subsetKey);
                                        }
                                    }
                                } catch (java.text.ParseException pe) {
                                }
                            }
                        }
                    }
                } catch (RuntimeException exception) {
                    ExceptionHandler exceptionHandler = directorySubsetDataListener.exceptionHandler;
                    if (exceptionHandler != null) {
                        if (exception instanceof BdfStorageException) {
                            exceptionHandler.handleBdfStorageException((BdfStorageException) exception);
                        } else {
                            exceptionHandler.handleBdfStorageException(new BdfStorageException(exception));
                        }
                    }
                } finally {
                    directorySubsetDataListener.clear();
                }
            }
        }

    }

}
