/* BdfServer_DirectoryStorage - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.association.AssociationUtils;
import fr.exemole.bdfserver.storage.BdfStorageException;
import net.mapeadores.util.io.FileUtils;
import net.mapeadores.util.io.IOUtils;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class DocumentFile {

    private int idsubset;
    private File documentDirectory;

    private DocumentFile(File documentDirectory, int idsubset) {
        this.documentDirectory = documentDirectory;
        this.idsubset = idsubset;
    }

    public InputStream getInputStream(String extension) {
        File f = getFile(extension);
        if (!f.exists()) {
            return null;
        }
        try {
            return new FileInputStream(f);
        } catch (IOException ioe) {
            throw new BdfStorageException("file : " + f.getAbsolutePath(), ioe);
        }
    }

    public boolean linkTo(String extension, File destination, boolean symbolicLink) throws IOException {
        File f = getFile(extension);
        if (!f.exists()) {
            return false;
        }
        StringBuffer command = new StringBuffer(128);
        command.append("ln ");
        if (symbolicLink) {
            command.append("-s ");
        }
        command.append(f.getCanonicalPath());
        command.append(' ');
        command.append(destination.getCanonicalPath());
        Runtime.getRuntime().exec(command.toString());
        return true;
    }

    public long saveInputStream(String extension, InputStream is) {
        if (!AssociationUtils.validateExtension(extension)) {
            throw new IllegalArgumentException("wrong extension value = " + extension);
        }
        File f = getFile(extension);
        if (!documentDirectory.exists()) {
            documentDirectory.mkdirs();
        }
        try {
            OutputStream output = new FileOutputStream(f);
            int result = IOUtils.copy(is, output);
            is.close();
            output.close();
            return (long) result;

        } catch (IOException ioe) {
            throw new BdfStorageException("file : " + f.getAbsolutePath(), ioe);
        }
    }

    public boolean delete() {
        try {
            if (documentDirectory.exists()) {
                FileUtils.forceDelete(documentDirectory);
                return true;
            } else {
                return false;
            }
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public boolean delete(String extension) {
        File f = getFile(extension);
        if (!f.exists()) {
            return false;
        }
        return f.delete();
    }

    public static DocumentFile getDocumentFile(BdfDirectories bdfDirectories, int idsubset) {
        File documentDirectory = new File(bdfDirectories.getBaseDirectory(), getPath(idsubset));
        DocumentFile documentFile = new DocumentFile(documentDirectory, idsubset);
        return documentFile;
    }

    public DocumentInfo getDocumentInfo() {
        File[] files = documentDirectory.listFiles();
        DocumentInfo documentInfo = null;
        String prefix = String.valueOf(idsubset) + ".";
        int prefixLength = prefix.length();
        for (int i = 0; i < files.length; i++) {
            File f = files[i];
            if (f.isDirectory()) {
                continue;
            }
            String name = f.getName();
            if (name.length() <= prefixLength) {
                continue;
            }
            if (!name.startsWith(prefix)) {
                continue;
            }
            long filelength = f.length();
            String extension = name.substring(prefixLength);
            try {
                if (documentInfo == null) {
                    documentInfo = new DocumentInfo(idsubset, extension, filelength);
                } else {
                    documentInfo.add(extension, filelength);
                }
            } catch (IllegalArgumentException iae) {
            }
        }
        return documentInfo;
    }

    /**
     * Retourne le chemin à partir du numéro
     */
    public static String getPath(int idsubset) {
        StringBuffer buf = new StringBuffer();
        buf.append("basedefiches");
        buf.append(File.separatorChar);
        buf.append(SubsetKey.categoryToString(SubsetKey.CATEGORY_DOCUMENT));
        buf.append(File.separatorChar);
        buf.append("d.");
        if (idsubset < 1000) {
            buf.append("000");
        } else if (idsubset < 10000) {
            buf.append("00");
            buf.append(idsubset / 1000);
        } else if (idsubset < 100000) {
            buf.append("0");
            buf.append(idsubset / 1000);
        }
        buf.append(File.separatorChar);
        buf.append(idsubset);
        return buf.toString();
    }

    public static List<DocumentFile> getDocumentFileList(BdfDirectories bdfDirectories) {
        List<DocumentFile> list = new ArrayList<DocumentFile>();
        File documentsDirectory = new File(bdfDirectories.getBaseDirectory(), "basedefiches" + File.separator + SubsetKey.categoryToString(SubsetKey.CATEGORY_DOCUMENT));
        if ((!documentsDirectory.exists()) || (!documentsDirectory.isDirectory())) {
            return list;
        }
        File[] directories = documentsDirectory.listFiles();
        for (int i = 0; i < directories.length; i++) {
            File directory = directories[i];
            if (!directory.isDirectory()) {
                continue;
            }
            String name = directory.getName();
            if ((name.length() < 2) || (!name.startsWith("d."))) {
                continue;
            }
            try {
                int directoryNumber = Integer.parseInt(name.substring(2));
                addDocumentFile(list, directory, directoryNumber);
            } catch (NumberFormatException nfe) {
            }
        }
        return list;
    }

    private static void addDocumentFile(List<DocumentFile> list, File intervalDirectory, int directoryNumber) {
        File[] directories = intervalDirectory.listFiles();
        for (int i = 0; i < directories.length; i++) {
            File directory = directories[i];
            if (!directory.isDirectory()) {
                continue;
            }
            String name = directory.getName();
            String errorMessage = null;
            try {
                int idsubset = Integer.parseInt(name);
                if (idsubset < 1) {
                    errorMessage = "idsubset < 1 (directoryNumber = " + directoryNumber;
                } else if (((idsubset < 1000) && (directoryNumber != 0)) || (idsubset / 1000 != directoryNumber)) {
                    errorMessage = "Wrong range : idsubset = " + idsubset + " / directoryNumber = " + directoryNumber;
                } else {
                    list.add(new DocumentFile(directory, idsubset));
                }
            } catch (NumberFormatException nfe) {
            }
            if (errorMessage != null) {
                System.out.println(errorMessage);
            }
        }
    }

    private File getFile(String extension) {
        return new File(documentDirectory, String.valueOf(idsubset) + "." + extension);
    }


    public static class DocumentInfo {

        private int idsubset;
        private List<String> extensionList = new ArrayList<String>();
        private List<Long> lengthList = new ArrayList<Long>();

        /**
         *
         *@throws IllegalArgumentException si filelength < 0 ou si l'extension est incorrecte.
         */
        public DocumentInfo(int idsubset, String extension, long filelength) {
            if (idsubset < 1) {
                throw new IllegalArgumentException("id < 1");
            }
            this.idsubset = idsubset;
            add(extension, filelength);
        }

        public int getId() {
            return idsubset;
        }

        public String[] getExtensionArray() {
            return extensionList.toArray(new String[extensionList.size()]);
        }

        public long[] getLengthArray() {
            int size = lengthList.size();
            long[] result = new long[size];
            for (int i = 0; i < size; i++) {
                result[i] = lengthList.get(i);
            }
            return result;
        }

        /**
         *
         *@throws IllegalArgumentException si filelength < 0 ou si l'extension est incorrecte.
         */
        public void add(String extension, long filelength) {
            if (filelength < 0) {
                throw new IllegalArgumentException("filelength < 0");
            }
            if (!AssociationUtils.validateExtension(extension)) {
                throw new IllegalArgumentException("invalid extension");
            }
            extension = extension.toLowerCase();
            boolean done = false;
            for (int i = 0; i < extensionList.size(); i++) {
                String ext2 = extensionList.get(i);
                if (ext2.equals(extension)) {
                    lengthList.set(i, filelength);
                    done = true;
                    break;
                }
            }
            if (!done) {
                extensionList.add(extension);
                lengthList.add(filelength);
            }
        }

    }

}
