/* BdfServer_DirectoryStorage - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.storage.BdfStorageException;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.io.FileUtils;
import net.mapeadores.util.logging.ProcessURI;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;
import org.w3c.dom.Document;


/**
 * Désigne un fichier destiné à stocker des informations de la base.
 *
 * @author Vincent Calame
 */
public class StorageFile {

    public final static String ENCODING = "UTF-8";
    String path;
    File file;
    File backupFile;

    public StorageFile(BdfDirectories bdfDirectories, String path) {
        this.path = path;
        file = new File(bdfDirectories.getBaseDirectory(), path);
        if (bdfDirectories.isWithBackup()) {
            backupFile = new File(bdfDirectories.getBackupDirectory(), path);
        }
    }

    public boolean exists() {
        if (file.exists()) {
            return true;
        }
        if (backupExist()) {
            return true;
        }
        return false;
    }

    public boolean backupExist() {
        return (backupFile != null) && (backupFile.exists());
    }

    public File getFile() {
        return file;
    }

    public ProcessURI toProcessURI() {
        return ProcessURI.fromFile(file);
    }

    public File getBackupFile() {
        return backupFile;
    }

    public void delete() {
        backup();
        try {
            if (file.exists()) {
                FileUtils.forceDelete(file);
            }
        } catch (IOException ioe) {
            throw new BdfStorageException(file, ioe);
        }
        /*try {
        if (backupExist()) FileUtils.forceDelete(backupFile);
        } catch(IOException ioe) {
        throw new BdfStorageException(backupFile,ioe);
        }*/
    }

    public Object decodeBean(Class c) {
        Object result = null;
        try {
            if (!file.exists()) {
                boolean restored = restore();
                if (!restored) {
                    return null;
                }
            }
            result = decode(c, file);
        } catch (Exception ioe) {
            if (!backupExist()) {
                throw new BdfStorageException(file, ioe);
            }
            try {
                result = decode(c, backupFile);
            } catch (Exception ioe2) {
                throw new BdfStorageException(backupFile, ioe2);
            }
            restore();
        }
        return result;
    }

    /*private Object decode(File f) throws IOException {
    XMLDecoder d = new XMLDecoder(new BufferedInputStream(new FileInputStream(f)));
    Object result = d.readObject();
    d.close();
    return result;
    }*/
    /*public void encodeBean(Object obj) {
    try {
    backup();
    XMLEncoder e = new XMLEncoder(new BufferedOutputStream( new FileOutputStream(file)));
    e.writeObject(obj);
    e.close();
    } catch(IOException ioe) {
    throw new BdfStorageException(ioe);
    }
    }*/
    private Object decode(Class c, File f) throws Exception {
        Reader reader = new InputStreamReader(new FileInputStream(f), ENCODING);
        return Unmarshaller.unmarshal(c, reader);
    }

    public void encodeBean(Object obj) {
        backup();
        testPath();
        try {
            Writer writer = new OutputStreamWriter(new FileOutputStream(file), ENCODING);
            Marshaller.marshal(obj, writer);
        } catch (Exception ioe) {
            throw new BdfStorageException(file, ioe);
        }
    }

    public Document readDocument() {
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder docbuilder = null;
        Document document = null;
        try {
            docbuilder = documentBuilderFactory.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            throw new BdfStorageException(pce);
        }
        try {
            InputStream is = new FileInputStream(file);
            document = docbuilder.parse(is);
            is.close();
        } catch (Exception e) {
            if (!backupExist()) {
                if (file.exists()) {
                    throw new BdfStorageException(file, e);
                } else {
                    return null;
                }
            }
            try {
                InputStream is = new FileInputStream(backupFile);
                document = docbuilder.parse(is);
                is.close();
            } catch (Exception e2) {
                throw new BdfStorageException(backupFile, e);
            }
            restore();
        }
        return document;
    }

    public boolean restore() {
        if (backupExist()) {
            try {
                FileUtils.copyFile(backupFile, file);
            } catch (IOException ioe) {
                throw new BdfStorageException(file, ioe);
            }
            return true;
        }
        return false;
    }

    public BufferedWriter getXMLWriter() throws IOException {
        testPath();
        backup();
        OutputStreamWriter writer = new OutputStreamWriter(new FileOutputStream(file), "UTF-8");
        BufferedWriter bufWriter = new BufferedWriter(writer);
        bufWriter.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        return bufWriter;
    }

    private void testPath() {
        file.getParentFile().mkdirs();
    }

    private void backup() {
        if ((file.exists()) && (backupFile != null) && (!file.isDirectory())) {
            try {
                FileUtils.copyFile(file, backupFile);
            } catch (IOException ioe) {
                throw new BdfStorageException(backupFile, ioe);
            }
        }
    }

}
