/* BdfServer_DirectoryStorage - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.transformationstorage;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerKey;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.managers.transformation.XslTransformerKey;
import fr.exemole.bdfserver.storage.TransformationStorage;
import fr.exemole.bdfserver.storage.TransformationStorage.TransformationStorageInfo;
import fr.exemole.bdfserver.storage.directory.transformationstorage.stream.DirectoryStreamTransformerStorageInfo;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.io.IOUtils;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryTransformationStorage implements TransformationStorage {

    private File transformationDirectory;

    public DirectoryTransformationStorage(File transformationDirectory) {
        if ((transformationDirectory.exists()) && (!transformationDirectory.isDirectory())) {
            throw new IllegalArgumentException("transformationDirectory is not a directory : " + transformationDirectory.getPath());
        }
        this.transformationDirectory = transformationDirectory;
    }

    public List<TransformationStorageInfo> getTransformationStorageInfoList(BdfServer bdfServer) {
        if (!transformationDirectory.exists()) {
            return Collections.emptyList();
        }
        if (!transformationDirectory.isDirectory()) {
            return Collections.emptyList();
        }
        List<TransformationStorageInfo> result = new ArrayList<TransformationStorageInfo>();
        File[] list = transformationDirectory.listFiles();
        for (int i = 0; i < list.length; i++) {
            File file = list[i];
            if (file.isDirectory()) {
                String name = file.getName();
                try {
                    TransformationKey transformationKey = TransformationKey.parse(name);
                    InternalTransformationStorageInfo transformationStorageInfo = new InternalTransformationStorageInfo(transformationKey);
                    transformationStorageInfo.scan(file, bdfServer);
                    result.add(transformationStorageInfo);
                } catch (java.text.ParseException pe) {
                } catch (IOException ioe) {
                    throw new NestedIOException(ioe);
                }
            }
        }
        return result;
    }

    public void saveTransformer(Object transformerKey, InputStream inputStream, boolean pattern) throws IOException {
        if (transformerKey instanceof XslTransformerKey) {
            saveXslTransformer((XslTransformerKey) transformerKey, inputStream, pattern);
        } else if (transformerKey instanceof StreamTransformerKey) {
            saveStreamTransformer((StreamTransformerKey) transformerKey, inputStream, pattern);
        }
    }

    private void saveXslTransformer(XslTransformerKey xslTransformerKey, InputStream inputStream, boolean pattern) throws IOException {
        File dir = checkTransformationDirectory(xslTransformerKey.getTransformationKey());
        String transformerName = xslTransformerKey.getTransformerName();
        if (transformerName == null) {
            transformerName = "_default";
        }
        if (pattern) {
            transformerName = transformerName + ".txt";
        } else {
            transformerName = transformerName + ".xsl";
        }
        File destination = new File(dir, transformerName);
        if (destination.exists()) {
            destination.delete();
        }
        OutputStream outputStream = new FileOutputStream(destination);
        IOUtils.copy(inputStream, outputStream);
        outputStream.close();
    }

    private void saveStreamTransformer(StreamTransformerKey streamTransformerKey, InputStream inputStream, boolean pattern) throws IOException {
        File dir = checkTransformationDirectory(streamTransformerKey.getTransformationKey());
        String extension = streamTransformerKey.getExtension();
        String transformerName = streamTransformerKey.getTransformerName();
        if (transformerName == null) {
            transformerName = "_default";
        }
        File streamDir = new File(dir, transformerName + "." + extension);
        checkDirectory(streamDir);
        String fileName = null;
        if (extension.equals("odt")) {
            fileName = "content";
            if (pattern) {
                fileName = fileName + ".txt";
            } else {
                fileName = fileName + ".xsl";
            }
        }
        File destination = new File(streamDir, fileName);
        if (destination.exists()) {
            destination.delete();
        }
        OutputStream outputStream = new FileOutputStream(destination);
        IOUtils.copy(inputStream, outputStream);
        outputStream.close();
        if (extension.equals("odt")) {
            File cssFile = new File(streamDir, "styles.css");
            File stylesFile = new File(streamDir, "styles.xml");
            if ((!cssFile.exists()) && (!stylesFile.exists())) {
                OutputStream outputStream2 = new FileOutputStream(cssFile);
                IOUtils.copy(getDefaultOdCss(streamTransformerKey.getTransformationKey()), outputStream2);
                outputStream2.close();
            }
        }
    }

    private static InputStream getDefaultOdCss(TransformationKey transformationKey) {
        return DirectoryTransformationStorage.class.getResourceAsStream("_default_od.css");
    }

    private File checkTransformationDirectory(TransformationKey transformationKey) {
        String name = transformationKey.toString();
        File dir = new File(transformationDirectory, name);
        checkDirectory(dir);
        return dir;
    }

    private void checkDirectory(File dir) {
        if (!dir.exists()) {
            dir.mkdirs();
        } else if (!dir.isDirectory()) {
            dir.delete();
            dir.mkdirs();
        }
    }


    private class InternalTransformationStorageInfo implements TransformationStorageInfo {

        private TransformationKey transformationKey;
        private SortedMap<String, FileXslTransformerStorageInfo> xslMap = new TreeMap<String, FileXslTransformerStorageInfo>();
        private SortedMap<String, DirectoryStreamTransformerStorageInfo> streamMap = new TreeMap<String, DirectoryStreamTransformerStorageInfo>();

        InternalTransformationStorageInfo(TransformationKey transformationKey) {
            this.transformationKey = transformationKey;
        }

        public TransformationKey getTransformationKey() {
            return transformationKey;
        }

        public List<XslTransformerStorageInfo> getXslTransformerStorageInfoList() {
            List<XslTransformerStorageInfo> list = new ArrayList<XslTransformerStorageInfo>();
            list.addAll(xslMap.values());
            return list;
        }

        public List<StreamTransformerStorageInfo> getStreamTransformerStorageInfoList() {
            List<StreamTransformerStorageInfo> list = new ArrayList<StreamTransformerStorageInfo>();
            list.addAll(streamMap.values());
            return list;
        }

        private void scan(File directory, BdfServer bdfServer) throws IOException {
            File[] list = directory.listFiles();
            for (int i = 0; i < list.length; i++) {
                File file = list[i];
                if (file.isDirectory()) {
                    scanStreamTransformerDirectory(file, bdfServer);
                } else {
                    scanXslTransformerFile(file);
                }
            }
            for (FileXslTransformerStorageInfo info : xslMap.values()) {
                info.process(bdfServer);
            }
        }

        private void scanXslTransformerFile(File file) {
            String name = file.getName();
            int idx = name.lastIndexOf('.');
            if (idx < 1) {
                return;
            }
            String extension = name.substring(idx + 1);
            name = name.substring(0, idx);
            if (extension.equals("txt")) {
                FileXslTransformerStorageInfo bundle = getXslTransformerStorageInfo(name);
                bundle.addTxtFile(file);
            } else if (extension.equals("xsl")) {
                FileXslTransformerStorageInfo bundle = getXslTransformerStorageInfo(name);
                bundle.addXslFile(file);
            }

        }

        private void scanStreamTransformerDirectory(File directory, BdfServer bdfServer) throws IOException {
            String name = directory.getName();
            int idx = name.lastIndexOf('.');
            if (idx < 1) {
                return;
            }
            String extension = name.substring(idx + 1);
            String transformerName = name.substring(0, idx);
            StreamTransformerKey streamTransformerKey = new StreamTransformerKey(transformationKey, extension, transformerName);
            DirectoryStreamTransformerStorageInfo internal = DirectoryStreamTransformerStorageInfo.newInstance(streamTransformerKey, directory, bdfServer);
            streamMap.put(name, internal);
        }

        private FileXslTransformerStorageInfo getXslTransformerStorageInfo(String name) {
            FileXslTransformerStorageInfo bundle = (FileXslTransformerStorageInfo) xslMap.get(name);
            if (bundle == null) {
                XslTransformerKey xslTransformerKey = new XslTransformerKey(transformationKey, name);
                bundle = new FileXslTransformerStorageInfo(xslTransformerKey);
                xslMap.put(name, bundle);
            }
            return bundle;
        }

    }

}
