/* BdfServer_Interaction - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.extension;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.storage.BdfExtensionStorage;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import net.mapeadores.util.exceptions.ImplementationException;


/**
 *
 * @author Vincent Calame
 */
public class BdfExtensionManager {

    private final Map<String, BdfExtensionInfo> factoryMap = new HashMap<String, BdfExtensionInfo>();
    private File extensionPropertiesDirectory;
    private boolean validDirectory = false;

    public BdfExtensionManager(File extensionPropertiesDirectory) {
        this.extensionPropertiesDirectory = extensionPropertiesDirectory;
        reinit();
    }

    public BdfExtension getBdfExtension(BdfContext bdfContext, String extensionName) {
        if (!validDirectory) {
            return null;
        }
        BdfExtensionInfo info = factoryMap.get(extensionName);
        if (info == null) {
            BdfExtensionFactory factory = getBdfExtensionFactory(extensionName);
            if (factory == null) {
                return null;
            }
            BdfExtensionStorage bdfExtensionStorage = bdfContext.getBdfServer().getBdfExtensionStorage();
            info = new BdfExtensionInfo(factory, extensionName, bdfExtensionStorage.getConfigDirectory(extensionName), bdfExtensionStorage.getDataDirectory(extensionName));
            factoryMap.put(extensionName, info);
        }
        return info.getBdfExtension(bdfContext);
    }

    public void reinit() {
        factoryMap.clear();
        if ((extensionPropertiesDirectory.exists()) && (extensionPropertiesDirectory.isDirectory())) {
            this.validDirectory = true;
        } else {
            this.validDirectory = false;
        }
    }

    private BdfExtensionFactory getBdfExtensionFactory(String extensionName) {
        File propertiesFile = new File(extensionPropertiesDirectory, extensionName + ".properties");
        if (!propertiesFile.exists()) {
            return null;
        }
        if (propertiesFile.isDirectory()) {
            return null;
        }
        Properties properties = new Properties();
        try {
            properties.load(new FileInputStream(propertiesFile));
        } catch (IOException ioe) {
            throw new BdfExtensionFactoryException(ioe);
        }
        String className = properties.getProperty("factory");
        if (className != null) {
            className = className.trim();
        }
        if ((className == null) || (className.length() == 0)) {
            throw new BdfExtensionFactoryException("Missing factory parameter in " + propertiesFile.getPath());
        }
        try {
            Class extensionClass = Class.forName(className);
            return (BdfExtensionFactory) extensionClass.newInstance();
        } catch (ClassNotFoundException cnfe) {
            throw new BdfExtensionFactoryException(cnfe);
        } catch (InstantiationException ie) {
            throw new ImplementationException(ie);
        } catch (IllegalAccessException ia) {
            throw new ImplementationException(ia);
        }
    }


    private static class BdfExtensionInfo implements BdfExtensionContext {

        private BdfExtensionFactory bdfExtensionFactory;
        private String registrationName;
        private File configDirectory;
        private File dataDirectory;

        private BdfExtensionInfo(BdfExtensionFactory bdfExtensionFactory, String registrationName, File configDirectory, File dataDirectory) {
            this.bdfExtensionFactory = bdfExtensionFactory;
            this.registrationName = registrationName;
            this.configDirectory = configDirectory;
            this.dataDirectory = dataDirectory;
        }

        public String getRegistrationName() {
            return registrationName;
        }

        public File getConfigDirectory() {
            return configDirectory;
        }

        public File getDataDirectory() {
            return dataDirectory;
        }

        private BdfExtension getBdfExtension(BdfContext bdfContext) {
            return bdfExtensionFactory.newInstance(bdfContext, this);
        }

    }

}
