/* UtilLib - Copyright (c) 2005-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.collections;


/**
 * Abstract base class for {@link IndexAndIntKeyMap}.
 *
 * @author  Vincent Calame
 */
public class AbstractIndexAndIntKeyMap implements IndexAndIntKeyMap {

    Capsule[] capsules;
    IntHashMap table = new IntHashMap();
    int enabledIndexCount = 0;
    int size = 0;

    /** Sole constructor. */
    protected AbstractIndexAndIntKeyMap() {
        capsules = new Capsule[32];
    }

    /**
     * Ajoute le lien entre une clé éntière et une valeur. Si l'index était inactif (isEnabledIndex(getIndexByKey(key)) == false), il est réactivé.
     */
    protected synchronized int putFromKey(int key, Object value) {
        Capsule capsule = (Capsule) table.get(key);
        if (capsule != null) {
            capsule.value = value;
            if (!capsule.enabled) {
                enabledIndexCount++;
                capsule.enabled = true;
            }
            return capsule.index;
        } else {
            if (size == capsules.length) {
                Capsule[] tempcaps = new Capsule[size * 2];
                for (int k = 0; k < size; k++) {
                    tempcaps[k] = capsules[k];
                }
                capsules = tempcaps;
            }
            capsule = new Capsule(key, size, value);
            capsules[size] = capsule;
            table.put(key, capsule);
            size++;
            enabledIndexCount++;
            return size - 1;
        }
    }

    /*
     * Retourne la capsule correspondant à l'index
     */
    private Capsule getCapsuleByIndex(int index) {
        if ((index < 0) || (index >= size)) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
        return capsules[index];
    }

    /**
     * Modifie la valeur à un index.
     */
    protected synchronized void setFromIndex(int index, Object value) {
        Capsule capsule = getCapsuleByIndex(index);
        capsule.value = value;
    }

    /**
     * Supprime la clé entière (autrement dit, désactive l'index correspondant.
     * Résultat similaire à <code>disable(getIndexByKey(key))</code>.
     */
    protected synchronized boolean removeFromKey(int key) {
        Capsule capsule = (Capsule) table.get(key);
        if ((capsule != null) && (capsule.enabled)) {
            capsule.enabled = false;
            capsule.value = null;
            enabledIndexCount--;
            return true;
        } else {
            return false;
        }
    }

    /**
     * Désactive un index.
     */
    protected synchronized void disable(int index) {
        Capsule capsule = getCapsuleByIndex(index);
        if (capsule.enabled) {
            capsule.enabled = false;
            capsule.value = null;
            enabledIndexCount--;
        }
    }

    /**
     * Retourne la clé entière correspondant à l'index. Envoi l'exception DisabledIndexException si l'index a été déactivé.
     */
    public int getKeyByIndex(int index) throws DisabledIndexException {
        Capsule capsule = getCapsuleByIndex(index);
        if (!capsule.enabled) {
            throw new DisabledIndexException(index);
        }
        return capsule.key;
    }

    /**
     * Retourne la valeur à l'index <code>index</code>.
     * Retourne <em>null</em> si l'index n'existe pas ou a été désactivé.
     */
    public Object getValueByIndex(int index) {
        Capsule capsule = getCapsuleByIndex(index);
        return capsule.value;
    }

    /**
     * Retourne la valeur correspondant à la clé entière <code>key</code>. Retourne <em>null</em> s'il n'y a pas de valeur pour cette clé.
     **/
    public Object getValueByKey(int key) {
        Capsule capsule = (Capsule) table.get(key);
        if (capsule == null) {
            return null;
        }
        return capsule.value;
    }

    /**
     * Retourne l'index de la clé entière. Retourne -1 si la clé n'existe pas, c'est à dire si elle n'a jamais été entrée ou si elle a été désactivée antérieurment.
     */
    public int getIndexByKey(int key) {
        Capsule capsule = (Capsule) table.get(key);
        if (capsule == null) {
            return -1;
        }
        if (!capsule.enabled) {
            return -1;
        }
        return capsule.index;
    }

    /**
     * Équivalent de getIntKeyList().size().
     **/
    public int length() {
        return size;
    }

    public int getEnabledIndexCount() {
        return enabledIndexCount;
    }


    /*
     * Capsule qui conserver la correspondance entre clé entière, index et objets.
     */
    private class Capsule {

        int key;
        int index;
        Object value;
        boolean enabled;
        /* enabled doit avoir la même valeur que (value != null) */

        Capsule(int key, int index, Object value) {
            this.key = key;
            this.index = index;
            this.value = value;
            enabled = true;
        }

    }

}
