/* UtilLib - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.text.TextConstants;
import java.text.Collator;


/**
 *
 * @author Vincent Calame
 */
public class Conditions implements ConditionsConstants {

    private Conditions() {
    }

    public static short modeToShort(String s, short defaultMode) {
        if (s == null) {
            return defaultMode;
        }
        if (s.equals("and")) {
            return Conditions.MODE_AND;
        } else if (s.equals("or")) {
            return Conditions.MODE_OR;
        }
        return defaultMode;
    }

    public static String modeToString(short mode) {
        switch (mode) {
            case MODE_AND:
                return "and";
            case MODE_OR:
                return "or";
            default:
                throw new IllegalArgumentException();
        }
    }

    public static short getModeInverse(short mode) {
        if (mode == MODE_AND) {
            return MODE_OR;
        }
        return MODE_AND;
    }

    public static boolean accept(ContentCondition contentCondition, String chaine, Collator collator) {
        short typeCondition = contentCondition.getConditionType();
        String contenu = contentCondition.getText();
        if (typeCondition == EST_VIDE) {
            return (chaine == null) || (chaine.length() == 0);
        }
        if (typeCondition == EST_NON_VIDE) {
            return (chaine != null) && (chaine.length() > 0);
        }
        int taille1 = contenu.length();
        int taille2 = chaine.length();
        if (isExcludingPartConditionType(typeCondition)) {
            if (taille1 > taille2) {
                return true;
            }
            if (typeCondition == NE_CORRESPOND_PAS_A) {
                return !(compare(chaine, contenu, collator) == 0);
            }
            if (typeCondition == NE_COMMENCE_PAS_PAR) {
                return !(compare(chaine.substring(0, taille1), contenu, collator) == 0);
            }
            if (typeCondition == NE_FINIT_PAS_PAR) {
                return !(compare(chaine.substring(taille2 - taille1), contenu, collator) == 0);
            }
            for (int i = 0; i < (taille2 - taille1 + 1); i++) {
                if (compare(chaine.substring(i, i + taille1), contenu, collator) == 0) {
                    return false;
                }
            }
            return true;
        } else {
            if (taille1 > taille2) {
                return false;
            }
            if (typeCondition == CORRESPOND_A) {
                return (compare(chaine, contenu, collator) == 0);
            }
            if (typeCondition == COMMENCE_PAR) {
                return (compare(chaine.substring(0, taille1), contenu, collator) == 0);
            }
            if (typeCondition == FINIT_PAR) {
                return (compare(chaine.substring(taille2 - taille1), contenu, collator) == 0);
            }
            for (int i = 0; i < (taille2 - taille1 + 1); i++) {
                if (compare(chaine.substring(i, i + taille1), contenu, collator) == 0) {
                    return true;
                }
            }
            return false;
        }
    }

    private static int compare(String s1, String s2, Collator collator) {
        if (collator != null) {
            return collator.compare(s1, s2);
        } else {
            return s1.compareTo(s2);
        }
    }

    /**
     * Si la chaine ne contient aucun des caractères suivants : !*;
     * alors les espaces de la chaine sont traités comme des séparateurs
     */
    public static ConditionGroup parseSimpleConditionGroup(String conditions) {
        int length = conditions.length();
        for (int i = 0; i < length; i++) {
            char carac = conditions.charAt(i);
            switch (carac) {
                case '*':
                case '!':
                case ';':
                    return parseConditionGroup(conditions, MODE_AND);
            }
        }
        ConditionGroup conditionGroup = new ConditionGroup(MODE_AND);
        String[] tokens = StringUtils.getCleanedTokens(conditions, ' ');
        int tokenCount = tokens.length;
        for (int i = 0; i < tokenCount; i++) {
            String token = tokens[i];
            ContentConditionBean nv = new ContentConditionBean();
            nv.setConditionType(CONTIENT);
            nv.setText(token);
            conditionGroup.addContentCondition(nv);
        }
        return conditionGroup;
    }

    /**
     * Reconstruit une instance de ConditionGroup à partir d'une chaîne.
     * Le point-virgule est le séparateur des différentes conditions.
     */
    public static ConditionGroup parseConditionGroup(String conditions, short mode) {
        ConditionGroup conditionGroup = new ConditionGroup(mode);
        int length = conditions.length();
        boolean previousEscaped = false;
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < length; i++) {
            char carac = conditions.charAt(i);
            if (previousEscaped) {
                if (carac != ';') {
                    buf.append('\\');
                }
                buf.append(carac);
                previousEscaped = false;
            } else {
                if (carac == ';') {
                    if (buf.length() > 0) {
                        String condition = buf.toString();
                        ContentConditionBean bean = parseContentCondition(condition);
                        if (bean != null) {
                            conditionGroup.addContentCondition(parseContentCondition(condition));
                        }
                        buf = new StringBuilder();
                    }

                } else if (carac == '\\') {
                    previousEscaped = true;
                } else {
                    buf.append(carac);
                }
            }
        }
        if (previousEscaped) {
            buf.append('\\');
        }
        String condition = buf.toString();
        ContentConditionBean bean = parseContentCondition(condition);
        if (bean != null) {
            conditionGroup.addContentCondition(bean);
        }
        return conditionGroup;
    }

    /**
     * Reconstruit une instance de ContentConditionBean à partir d'une chaîne.
     */
    public static ContentConditionBean parseContentCondition(String condition) {
        ContentConditionBean contentConditionBean = new ContentConditionBean();
        if (condition == null) {
            return null;
        }
        condition = condition.trim();
        if (condition.length() == 0) {
            return null;
        }
        boolean non = (condition.charAt(0) == '!');
        if ((non) && (condition.length() == 1)) {
            contentConditionBean.setConditionType(EST_VIDE);
            return contentConditionBean;
        }
        if (non) {
            condition = condition.substring(1);
        }
        String text = "";
        short conditionType = 0;
        boolean commence = false;
        if ((condition.charAt(0) == '*') || (condition.charAt(0) == '%')) {
            commence = true;
            text = condition.substring(1);
        } else if (condition.startsWith("..")) {
            commence = true;
            text = condition.substring(2);
        } else {
            text = condition;
        }
        text = StringUtils.cleanString(text);
        int taille = text.length();
        if (text.length() == 0) {
            if (non) {
                conditionType = EST_VIDE;
            } else {
                conditionType = EST_NON_VIDE;
            }
        } else {
            boolean finit = false;
            if ((text.charAt(taille - 1) == '*') || (text.charAt(taille - 1) == '%')) {
                text = text.substring(0, taille - 1);
                finit = true;
            } else if (text.endsWith("..")) {
                text = text.substring(0, taille - 2);
                finit = true;
            }
            if (commence) {
                if (finit) {
                    if (non) {
                        conditionType = NE_CONTIENT_PAS;
                    } else {
                        conditionType = CONTIENT;
                    }
                } else {
                    if (non) {
                        conditionType = NE_FINIT_PAS_PAR;
                    } else {
                        conditionType = FINIT_PAR;
                    }
                }
            } else {
                if (finit) {
                    if (non) {
                        conditionType = NE_COMMENCE_PAS_PAR;
                    } else {
                        conditionType = COMMENCE_PAR;
                    }
                } else {
                    if (non) {
                        conditionType = NE_CORRESPOND_PAS_A;
                    } else {
                        conditionType = CORRESPOND_A;
                    }
                }
            }
        }
        contentConditionBean.setConditionType(conditionType);
        contentConditionBean.setText(text);
        return contentConditionBean;
    }

    public static ContentConditionBean toOpposite(ContentConditionBean contentConditionBean) {
        ContentConditionBean nv = new ContentConditionBean();
        nv.setConditionType(getOppositeConditionType(contentConditionBean.getConditionType()));
        nv.setText(contentConditionBean.getText());
        return nv;
    }

    public static short getOppositeConditionType(short conditionType) {
        switch (conditionType) {
            case COMMENCE_PAR:
                return NE_COMMENCE_PAS_PAR;
            case FINIT_PAR:
                return NE_FINIT_PAS_PAR;
            case CONTIENT:
                return NE_CONTIENT_PAS;
            case CORRESPOND_A:
                return NE_CORRESPOND_PAS_A;
            case EST_VIDE:
                return EST_NON_VIDE;
            case NE_COMMENCE_PAS_PAR:
                return COMMENCE_PAR;
            case NE_FINIT_PAS_PAR:
                return FINIT_PAR;
            case NE_CONTIENT_PAS:
                return CONTIENT;
            case NE_CORRESPOND_PAS_A:
                return CORRESPOND_A;
            case EST_NON_VIDE:
                return EST_VIDE;
            default:
                return -1;
        }
    }

    public static String toSQL(ContentConditionBean contentConditionBean) {
        StringBuilder buf = new StringBuilder();
        switch (contentConditionBean.getConditionType()) {
            case EST_VIDE:
                buf.append(" IS NULL");
                break;
            case EST_NON_VIDE:
                buf.append(" IS NOT NULL");
                break;
            case COMMENCE_PAR:
                buf.append(" LIKE '");
                preparation(buf, contentConditionBean);
                buf.append("%'");
                break;
            case FINIT_PAR:
                buf.append(" LIKE '%");
                preparation(buf, contentConditionBean);
                buf.append("'");
                break;
            case CONTIENT:
                buf.append(" LIKE '%");
                preparation(buf, contentConditionBean);
                buf.append("%'");
                break;
            case CORRESPOND_A:
                buf.append(" LIKE '");
                preparation(buf, contentConditionBean);
                buf.append("'");
                break;
            case NE_COMMENCE_PAS_PAR:
                buf.append(" NOT LIKE '%");
                preparation(buf, contentConditionBean);
                buf.append("%'");
                break;
            case NE_FINIT_PAS_PAR:
                buf.append(" NOT LIKE '%");
                preparation(buf, contentConditionBean);
                buf.append("'");
                break;
            case NE_CONTIENT_PAS:
                buf.append(" NOT LIKE '%");
                preparation(buf, contentConditionBean);
                buf.append("%'");
                break;
            case NE_CORRESPOND_PAS_A:
                buf.append(" NOT LIKE '");
                preparation(buf, contentConditionBean);
                buf.append("'");
                break;
        }
        return buf.toString();
    }

    private static void preparation(StringBuilder buf, ContentConditionBean contentConditionBean) {
        String text = contentConditionBean.getText();
        for (int i = 0; i < text.length(); i++) {
            char carac = text.charAt(i);
            if (carac == '\'') {
                buf.append("''");
            } else {
                buf.append(carac);
            }
        }
    }

    public static boolean isExcludingPartConditionType(short conditionType) {
        return (conditionType == NE_COMMENCE_PAS_PAR) || (conditionType == NE_FINIT_PAS_PAR) || (conditionType == NE_CONTIENT_PAS) || (conditionType == NE_CORRESPOND_PAS_A);
    }

    public static boolean isIncludingPartConditionType(short conditionType) {
        return (conditionType == COMMENCE_PAR) || (conditionType == FINIT_PAR) || (conditionType == CONTIENT) || (conditionType == CORRESPOND_A);
    }

    /**
     *
     */
    public static String conditionGroupToString(ConditionGroup conditionGroup) {
        ConditionGroupBean conditionGroupBean = conditionGroup.toConditionGroupBean();
        StringBuilder buf = new StringBuilder();
        ContentConditionBean[] array = conditionGroupBean.getContentConditionBeanArray();
        if ((array == null) || (array.length == 0)) {
            return "";
        }
        for (int i = 0; i < array.length; i++) {
            if (i > 0) {
                buf.append(';');
            }
            buf.append(contentConditionToString(array[i]));
        }
        return buf.toString();
    }

    /**
     * Exprime la condition sous forme d'une chaîne.
     */
    public static String contentConditionToString(ContentCondition contentCondition) {
        String text = contentCondition.getText();
        switch (contentCondition.getConditionType()) {
            case COMMENCE_PAR:
                return text + "*";
            case FINIT_PAR:
                return "*" + text;
            case CONTIENT:
                return "*" + text + "*";
            case CORRESPOND_A:
                return text;
            case EST_VIDE:
                return "!*";
            case NE_COMMENCE_PAS_PAR:
                return "!" + text + "*";
            case NE_FINIT_PAS_PAR:
                return "!*" + text;
            case NE_CONTIENT_PAS:
                return "!*" + text + "*";
            case NE_CORRESPOND_PAS_A:
                return "!" + text;
            case EST_NON_VIDE:
                return "*";
            default:
                return "";
        }
    }

    public static String conditionTypeToString(short conditionType) {
        switch (conditionType) {
            case COMMENCE_PAR:
                return ".*";
            case FINIT_PAR:
                return "*.";
            case CONTIENT:
                return "*.*";
            case CORRESPOND_A:
                return "=";
            case EST_VIDE:
                return "!*";
            case NE_COMMENCE_PAS_PAR:
                return "!.*";
            case NE_FINIT_PAS_PAR:
                return "!*.";
            case NE_CONTIENT_PAS:
                return "!*.*";
            case NE_CORRESPOND_PAS_A:
                return "!=";
            case EST_NON_VIDE:
                return "*";
            default:
                return "";
        }
    }

    public static short conditionToShort(String chainecondition) {
        if (chainecondition.equals(".*")) {
            return COMMENCE_PAR;
        } else if (chainecondition.equals("*.")) {
            return FINIT_PAR;
        } else if (chainecondition.equals("*.*")) {
            return CONTIENT;
        } else if (chainecondition.equals("=")) {
            return CORRESPOND_A;
        } else if (chainecondition.equals("!*")) {
            return EST_VIDE;
        } else if (chainecondition.equals("!.*")) {
            return NE_COMMENCE_PAS_PAR;
        } else if (chainecondition.equals("!*.")) {
            return NE_FINIT_PAS_PAR;
        } else if (chainecondition.equals("!*.*")) {
            return NE_CONTIENT_PAS;
        } else if (chainecondition.equals("!=")) {
            return NE_CORRESPOND_PAS_A;
        } else if (chainecondition.equals("*")) {
            return EST_NON_VIDE;
        } else {
            return CORRESPOND_A;
        }
    }

    public static short getFamilleTypeCondition(short typecondition) {
        switch (typecondition) {
            case COMMENCE_PAR:
            case FINIT_PAR:
            case CONTIENT:
            case CORRESPOND_A:
                return FAMILLE_INCLUSION;
            case EST_VIDE:
                return FAMILLE_ABSENT;
            case NE_COMMENCE_PAS_PAR:
            case NE_FINIT_PAS_PAR:
            case NE_CONTIENT_PAS:
            case NE_CORRESPOND_PAS_A:
                return FAMILLE_EXCLUSION;
            case EST_NON_VIDE:
                return FAMILLE_PRESENT;
            default:
                return FAMILLE_INCLUSION;
        }
    }

    /**
     *
     *@throws IllegalArgumentException si conditionType est inconnu.
     */
    public static short getState(short conditionType) {
        switch (conditionType) {
            case COMMENCE_PAR:
            case FINIT_PAR:
            case CONTIENT:
            case CORRESPOND_A:
                return P;
            case EST_VIDE:
                return A;
            case NE_COMMENCE_PAS_PAR:
            case NE_FINIT_PAS_PAR:
            case NE_CONTIENT_PAS:
            case NE_CORRESPOND_PAS_A:
                return PA;
            case EST_NON_VIDE:
                return T;
            default:
                throw new IllegalArgumentException("unknown conditionType");
        }
    }

    public static short merge(short mode, short currentState, short addingState) {
        if (currentState == U) {
            return addingState;
        }
        if (currentState == addingState) {
            return currentState;
        }
        if (mode == MODE_AND) {
            return mergeAndMode(currentState, addingState);
        } else {
            return mergeOrMode(currentState, addingState);
        }
    }

    public static short mergeAndMode(short currentState, short addingState) {
        if (currentState == X) {
            return X;
        }
        if (currentState == N) {
            return addingState;
        }
        if (currentState == A) {
            switch (addingState) {
                case T:
                case P:
                    return X;
                case PA:
                    return A;
                default:
                    return currentState;
            }
        }
        if (currentState == T) {
            switch (addingState) {
                case A:
                    return X;
                case P:
                case PA:
                    return P;
                default:
                    return currentState;
            }
        }
        if (currentState == P) {
            switch (addingState) {
                case A:
                    return X;
                case T:
                case PA:
                    return P;
                default:
                    return currentState;
            }
        }
        if (currentState == PA) {
            switch (addingState) {
                case A:
                    return A;
                case T:
                case P:
                    return P;
                default:
                    return currentState;
            }
        }
        throw new IllegalArgumentException();
    }

    public static short mergeOrMode(short currentState, short addingState) {
        if (currentState == X) {
            return addingState;
        }
        if (currentState == N) {
            return N;
        }
        if (currentState == A) {
            switch (addingState) {
                case T:
                    return N;
                case P:
                case PA:
                    return PA;
                default:
                    return currentState;
            }
        }
        if (currentState == T) {
            switch (addingState) {
                case A:
                    return N;
                case P:
                    return T;
                case PA:
                    return N;
                default:
                    return currentState;
            }
        }
        if (currentState == P) {
            switch (addingState) {
                case A:
                    return PA;
                case T:
                    return T;
                case PA:
                    return PA;
                default:
                    return currentState;
            }
        }
        if (currentState == PA) {
            switch (addingState) {
                case A:
                    return PA;
                case T:
                    return N;
                case P:
                    return PA;
                default:
                    return currentState;
            }
        }
        throw new IllegalArgumentException();
    }

    public static boolean isImpossibleState(short state) {
        return (state == X);
    }

    public static boolean isNeutralState(short state) {
        return ((state == U) || (state == N));
    }

    public static boolean isNotEmptyState(short state) {
        return (state == T);
    }

    public static boolean isEmptyState(short state) {
        return ((state == A) || (state == PA));
    }

    public static boolean isPartState(short state) {
        return ((state == P) || (state == PA));
    }

    public static int toTextConstants(short condition) {
        switch (condition) {
            case COMMENCE_PAR:
                return TextConstants.STARTSWITH;
            case FINIT_PAR:
                return TextConstants.ENDSWITH;
            case CONTIENT:
                return TextConstants.CONTAINS;
            case CORRESPOND_A:
                return TextConstants.MATCHES;
            default:
                throw new IllegalArgumentException("wrong condition value");
        }
    }

    public static String typeToString(short type) {
        switch (type) {
            case COMMENCE_PAR:
                return "*.";
            case FINIT_PAR:
                return ".*";
            case CONTIENT:
                return "*.*";
            case CORRESPOND_A:
                return ".";
            case EST_VIDE:
                return "!*";
            case NE_COMMENCE_PAS_PAR:
                return "!*.";
            case NE_FINIT_PAS_PAR:
                return "!.*";
            case NE_CONTIENT_PAS:
                return "!*.*";
            case NE_CORRESPOND_PAS_A:
                return "!.";
            case EST_NON_VIDE:
                return "*";
            default:
                throw new IllegalArgumentException("wrong type value = " + type);
        }
    }

    public static short typeToShort(String type) {
        if (type.equals("*.")) {
            return COMMENCE_PAR;
        } else if (type.equals(".*")) {
            return FINIT_PAR;
        } else if (type.equals("*.*")) {
            return CONTIENT;
        } else if (type.equals(".")) {
            return CORRESPOND_A;
        } else if (type.equals("!*")) {
            return EST_VIDE;
        } else if (type.equals("!*.")) {
            return NE_COMMENCE_PAS_PAR;
        } else if (type.equals("!.*")) {
            return NE_FINIT_PAS_PAR;
        } else if (type.equals("!*.*")) {
            return NE_CONTIENT_PAS;
        } else if (type.equals("!.")) {
            return NE_CORRESPOND_PAS_A;
        } else if (type.equals("*")) {
            return EST_NON_VIDE;
        } else {
            throw new IllegalArgumentException("wrong type value = " + type);
        }
    }

}
