/* UtilLib - Copyright (c) 2005-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conf;

import net.mapeadores.util.primitives.PositiveInteger;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.tableparser.*;
import net.mapeadores.util.collections.ArrayUtils;
import net.mapeadores.util.attr.SimpleAttribute;
import net.mapeadores.util.attr.DefaultSimpleAttribute;
import java.util.Map;
import java.util.LinkedHashMap;
import java.util.Properties;
import java.util.List;
import java.util.ArrayList;
import java.text.MessageFormat;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.GraphicsEnvironment;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.File;


/**
 *
 * @author  Vincent Calame
 */
public class Conf implements Cloneable {

    public final static short OK_RESULT = 1;
    public final static short UNKNOWNKEY_RESULT = -1;
    public final static short BADVALUE_RESULT = -2;
    private Map<String, Object> hashMap = new LinkedHashMap<String, Object>();
    private ConfListener[] confListeners = new ConfListener[0];
    private boolean eventsQueued = false;
    private List queuedEventList = new ArrayList();

    public Conf() {
    }

    protected void addAll(Conf confMap) {
        for (Map.Entry<String, Object> entry : confMap.hashMap.entrySet()) {
            String key = entry.getKey();
            Object value = entry.getValue();
            if (value instanceof Boolean) {
                putBoolean(key, ((Boolean) value).booleanValue());
            } else if (value instanceof File) {
                putDirectory(key, new File(((File) value).getPath()));
            } else if (value instanceof Integer) {
                putInteger(key, ((Integer) value).intValue());
            } else if (value instanceof PositiveInteger) {
                putPositiveInteger(key, ((PositiveInteger) value).intValue());
            } else if (value instanceof String) {
                putString(key, (String) value);
            } else if (value instanceof Font) {
                putFont(key, (Font) value);
            } else if (value instanceof Rectangle) {
                putRectangle(key, new Rectangle((Rectangle) value));
            } else if (value instanceof LangInteger) {
                putLangInteger(key, ((LangInteger) value).intValue());
            } else if (value instanceof MessageFormat) {
                putMessageFormat(key, (MessageFormat) ((MessageFormat) value).clone());
            } else if (value instanceof FileBundle) {
                putFile(key, ((FileBundle) value).getFile());
            } else if (value instanceof String[]) {
                String[] val = (String[]) value;
                String[] copy = new String[val.length];
                System.arraycopy(val, 0, copy, 0, val.length);
                putStringArray(key, copy);
            } else if (value instanceof URLBundle) {
                putURL(key, ((URLBundle) value).getURL());
            } else {
                throw new IllegalArgumentException("La valeur de clé " + key + " appartient à un classe (" + value.getClass().getName() + ") inconnue de Conf Map");
            }
        }
    }

    public void addConfListener(ConfListener confListener) {
        confListeners = (ConfListener[]) ArrayUtils.addUnique(confListeners, confListener, new ConfListener[confListeners.length + 1]);
    }

    public void removeConfListener(ConfListener confListener) {
        if (confListeners.length > 0) {
            confListeners = (ConfListener[]) ArrayUtils.removeUnique(confListeners, confListener, new ConfListener[confListeners.length - 1]);
        }
    }

    protected void fireConfChanged(String param) {
        if (eventsQueued) {
            queuedEventList.add(param);
        } else {
            ConfEvent confEvent = new ConfEvent(this, param);
            for (int i = 0; i < confListeners.length; i++) {
                confListeners[i].confChanged(confEvent);
            }
        }
    }

    protected void putBoolean(String name, boolean b) {
        hashMap.put(name, new Boolean(b));
    }

    protected void putDirectory(String name, File f) {
        hashMap.put(name, f);
    }

    protected void putInteger(String name, int i) {
        hashMap.put(name, new Integer(i));
    }

    protected void putPositiveInteger(String name, int i) {
        hashMap.put(name, new PositiveInteger(i));
    }

    protected void putString(String name, String s) {
        hashMap.put(name, s);
    }

    protected void putFont(String name, Font f) {
        hashMap.put(name, f);
    }

    protected void putRectangle(String name, Rectangle rectangle) {
        hashMap.put(name, rectangle);
    }

    protected void putLangInteger(String name, int langInteger) {
        hashMap.put(name, new LangInteger(langInteger));
    }

    protected void putMessageFormat(String name, MessageFormat messageFormat) {
        hashMap.put(name, messageFormat);
    }

    protected void putFile(String name, File f) {
        hashMap.put(name, new FileBundle(f));
    }

    protected void putStringArray(String name, String[] stringArray) {
        hashMap.put(name, stringArray);
    }

    protected void putURL(String name, URL url) {
        hashMap.put(name, new URLBundle(url));
    }

    public boolean getBoolean(String name) {
        Boolean bool = (Boolean) hashMap.get(name);
        if (bool == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        return bool.booleanValue();
    }

    public File getDirectory(String name) {
        File file = (File) hashMap.get(name);
        if (file == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        return file;
    }

    public int getInteger(String name) {
        Integer itg = (Integer) hashMap.get(name);
        if (itg == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        return itg.intValue();
    }

    public int getPositiveInteger(String name) {
        PositiveInteger itg = (PositiveInteger) hashMap.get(name);
        if (itg == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        return itg.intValue();
    }

    public String getString(String name) {
        String s = (String) hashMap.get(name);
        if (s == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        return s;
    }

    public Font getFont(String name) {
        Font f = (Font) hashMap.get(name);
        if (f == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        return f;
    }

    public Rectangle getRectangle(String name) {
        Rectangle rectangle = (Rectangle) hashMap.get(name);
        if (rectangle == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        return rectangle;
    }

    public int getLangInteger(String name) {
        LangInteger langInteger = (LangInteger) hashMap.get(name);
        if (langInteger == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        return langInteger.intValue();
    }

    public MessageFormat getMessageFormat(String name) {
        MessageFormat mf = (MessageFormat) hashMap.get(name);
        if (mf == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        return (MessageFormat) mf.clone();
    }

    /*
     * Peut avoir une valeur nulle.
     */
    public File getFile(String name) {
        FileBundle fileBundle = (FileBundle) hashMap.get(name);
        if (fileBundle == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        return fileBundle.getFile();
    }

    /**
     * N'est jamais nulle. Ce n'est pas une copie d'une tableau mais le tableau lui-même, la modification directe du tableau n'est pas conseillée.
     */
    public String[] getStringArray(String name) {
        String[] stringArray = (String[]) hashMap.get(name);
        if (stringArray == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        return stringArray;
    }

    /**
     * Peut avoir une valeur nulle.
     */
    public URL getURL(String name) {
        URLBundle urlBundle = (URLBundle) hashMap.get(name);
        if (urlBundle == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        return urlBundle.getURL();
    }

    public void setBoolean(String name, boolean b) {
        Boolean bool = (Boolean) hashMap.get(name);
        if (bool == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        if (bool.booleanValue() != b) {
            hashMap.put(name, new Boolean(b));
            fireConfChanged(name);
        }
    }

    public void setDirectory(String name, File f) {
        File file = (File) hashMap.get(name);
        if (file == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        if (!f.isDirectory()) {
            f = f.getParentFile();
            if (f == null) {
                throw new IllegalArgumentException("File f is not a directory and has no parent file");
            }
        }
        if (!file.equals(f)) {
            hashMap.put(name, f);
            fireConfChanged(name);
        }
    }

    public void setInteger(String name, int i) {
        Integer itg = (Integer) hashMap.get(name);
        if (itg == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        if (itg.intValue() != i) {
            hashMap.put(name, new Integer(i));
            fireConfChanged(name);
        }
    }

    public void setPositiveInteger(String name, int i) {
        if (i <= 0) {
            return;
        }
        PositiveInteger itg = (PositiveInteger) hashMap.get(name);
        if (itg == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        if (itg.intValue() != i) {
            hashMap.put(name, new PositiveInteger(i));
            fireConfChanged(name);
        }
    }

    public void setString(String name, String s) {
        String value = (String) hashMap.get(name);
        if (value == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        if (!value.equals(s)) {
            hashMap.put(name, s);
            fireConfChanged(name);
        }
    }

    public void setFont(String name, Font f) {
        Font value = (Font) hashMap.get(name);
        if (value == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        if (!value.equals(f)) {
            hashMap.put(name, f);
            fireConfChanged(name);
        }
    }

    public void setRectangle(String name, Rectangle rectangle) {
        Rectangle value = (Rectangle) hashMap.get(name);
        if (value == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        if (!value.equals(rectangle)) {
            hashMap.put(name, rectangle);
            fireConfChanged(name);
        }
    }

    public void setLangInteger(String name, int i) {
        LangInteger langInteger = (LangInteger) hashMap.get(name);
        if (langInteger == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        if (langInteger.intValue() != i) {
            hashMap.put(name, new LangInteger(i));
            fireConfChanged(name);
        }
    }

    public void setMessageFormat(String name, MessageFormat messageFormat) {
        MessageFormat mf = (MessageFormat) hashMap.get(name);
        if (mf == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        if (!mf.equals(messageFormat)) {
            hashMap.put(name, messageFormat);
            fireConfChanged(name);
        }
    }

    public void setFile(String name, File f) {
        FileBundle fileBundle = (FileBundle) hashMap.get(name);
        if (fileBundle == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        if ((f != null) && (f.isDirectory())) {
            throw new IllegalArgumentException("File f is a directory");
        }
        if (fileBundle.replaceFile(f)) {
            fireConfChanged(name);
        }
    }

    public void setStringArray(String name, String[] stringArray) {
        String[] currentArray = (String[]) hashMap.get(name);
        if (currentArray == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        boolean dif = (currentArray.length != stringArray.length);
        if (!dif) {
            for (int i = 0; i < currentArray.length; i++) {
                if (!currentArray[i].equals(stringArray[i])) {
                    dif = true;
                    break;
                }
            }
        }
        if (dif) {
            hashMap.put(name, stringArray);
            fireConfChanged(name);
        }
    }

    public void setURL(String name, URL url) {
        URLBundle urlBundle = (URLBundle) hashMap.get(name);
        if (urlBundle == null) {
            throw new IllegalArgumentException("illegal name value : " + name);
        }
        if (urlBundle.replaceURL(url)) {
            fireConfChanged(name);
        }
    }

    public short set(String name, String value) {
        if (value == null) {
            return BADVALUE_RESULT;
        }
        Object currentValue = hashMap.get(name);
        if (currentValue == null) {
            return UNKNOWNKEY_RESULT;
        }
        if (currentValue instanceof Boolean) {
            Boolean b = booleanTest(value);
            if (b == null) {
                return BADVALUE_RESULT;
            }
            setBoolean(name, b.booleanValue());
        } else if (currentValue instanceof File) {
            File f = directoryTest(value);
            if (f == null) {
                return BADVALUE_RESULT;
            }
            if (!f.isDirectory()) {
                f = f.getParentFile();
                if (f == null) {
                    return BADVALUE_RESULT;
                }
            }
            setDirectory(name, f);
        } else if (currentValue instanceof Integer) {
            Integer itg = integerTest(value);
            if (itg == null) {
                return BADVALUE_RESULT;
            }
            setInteger(name, itg.intValue());
        } else if (currentValue instanceof PositiveInteger) {
            PositiveInteger itg = positiveIntegerTest(value);
            if (itg == null) {
                return BADVALUE_RESULT;
            }
            setPositiveInteger(name, itg.intValue());
        } else if (currentValue instanceof String) {
            setString(name, value);
        } else if (currentValue instanceof Font) {
            setFont(name, toFont(value));
        } else if (currentValue instanceof Rectangle) {
            setRectangle(name, toRectangle(value));
        } else if (currentValue instanceof LangInteger) {
            LangInteger itg = langIntegerTest(value);
            if (itg == null) {
                return BADVALUE_RESULT;
            }
            setLangInteger(name, itg.intValue());
        } else if (currentValue instanceof MessageFormat) {
            MessageFormat messageFormat = messageFormatTest(value);
            if (messageFormat == null) {
                return BADVALUE_RESULT;
            }
            setMessageFormat(name, messageFormat);
        } else if (currentValue instanceof FileBundle) {
            FileBundle fileBundle = (FileBundle) currentValue;
            if (value.length() == 0) {
                setFile(name, null);
            } else {
                File f = new File(value);
                if (f.isDirectory()) {
                    return OK_RESULT;
                }
                setFile(name, f);
            }
        } else if (currentValue instanceof String[]) {
            setStringArray(name, toStringArray(value));
        } else if (currentValue instanceof URLBundle) {
            URLBundle urlBundle = (URLBundle) currentValue;
            if (value.length() == 0) {
                setURL(name, null);
            } else {
                try {
                    URL url = new URL(value);
                    setURL(name, url);
                } catch (MalformedURLException mue) {
                    return BADVALUE_RESULT;
                }
            }
        } else {
            throw new IllegalArgumentException("La valeur de clé " + name + " appartient à un classe (" + value.getClass().getName() + ") inconnue de Conf Map");
        }
        return OK_RESULT;
    }

    private Boolean booleanTest(String value) {
        if ((value.equalsIgnoreCase("true")) || (value.equals("1"))) {
            return Boolean.TRUE;
        }
        if ((value.equalsIgnoreCase("false")) || (value.equals("0"))) {
            return Boolean.FALSE;
        }
        return null;
    }

    private File directoryTest(String value) {
        File file = new File(value);
        if (!file.isDirectory()) {
            file = file.getParentFile();
            if (file == null) {
                return null;
            }
        }
        if (!file.exists()) {
            try {
                file.mkdirs();
            } catch (SecurityException ioe) {
                return null;
            }
        }
        return file;
    }

    private Integer integerTest(String value) {
        try {
            int i = Integer.parseInt(value);
            return new Integer(i);
        } catch (NumberFormatException nfe) {
            return null;
        }
    }

    private PositiveInteger positiveIntegerTest(String value) {
        try {
            int i = Integer.parseInt(value);
            if (i <= 0) {
                return null;
            }
            return new PositiveInteger(i);
        } catch (NumberFormatException nfe) {
            return null;
        }
    }

    private LangInteger langIntegerTest(String value) {
        try {
            int i = LangInteger.parse(value);
            return new LangInteger(i);
        } catch (LangIntegerException nfe) {
            return null;
        }
    }

    private MessageFormat messageFormatTest(String value) {
        try {
            return new MessageFormat(value);
        } catch (IllegalArgumentException iae) {
            return null;
        }
    }

    public void saveIntoProperties(Properties properties, String suffixe) {
        for (Map.Entry<String, Object> entry : hashMap.entrySet()) {
            String key = entry.getKey();
            if (!isTransient(key)) {
                if (suffixe != null) {
                    key = suffixe + key;
                }
                properties.put(key, toString(entry.getValue()));
            }
        }
    }

    /**
     * Doit être surclassé par des classes parentes afin d'indiquer qu'un paramètre ne doit pas être enregistré.
     * Par défaut, retourne false;
     */
    public boolean isTransient(String param) {
        return false;
    }

    public String toString(Object value) {
        if (value instanceof Boolean) {
            if (((Boolean) value).booleanValue()) {
                return "true";
            } else {
                return "false";
            }
        }
        if (value instanceof File) {
            return ((File) value).getPath();
        }
        if (value instanceof Font) {
            return toString((Font) value);
        }
        if (value instanceof Rectangle) {
            return toString((Rectangle) value);
        }
        if (value instanceof MessageFormat) {
            return ((MessageFormat) value).toPattern();
        }
        if (value instanceof FileBundle) {
            File file = ((FileBundle) value).getFile();
            if (file == null) {
                return "";
            } else {
                return file.getPath();
            }
        }
        if (value instanceof String[]) {
            return toString((String[]) value);
        }
        return value.toString();
    }

    public String toString(Font font) {
        String fontname = font.getName();
        float fontsize = font.getSize();
        int fontstyle = font.getStyle();
        StringBuffer buf = new StringBuffer();
        buf.append("font-family:");
        if (fontname.indexOf(' ') > -1) {
            buf.append('"');
            buf.append(fontname);
            buf.append('"');
        } else {
            buf.append(fontname);
        }
        buf.append(';');
        buf.append("font-size:");
        buf.append(String.valueOf(fontsize));
        buf.append("pt");
        buf.append(';');
        buf.append("font-style:");
        if ((fontstyle & Font.ITALIC) != 0) {
            buf.append("italic");
        } else {
            buf.append("normal");
        }
        buf.append(';');
        buf.append("font-weight:");
        if ((fontstyle & Font.BOLD) != 0) {
            buf.append("bold");
        } else {
            buf.append("normal");
        }
        buf.append(';');
        return buf.toString();
    }

    public Font toFont(String string) {
        String fontName = null;
        String[] tab = string.split(";");
        int fontStyle = 0;
        int fontSize = 11;
        for (int i = 0; i < tab.length; i++) {
            String token = tab[i].trim();
            if (!token.startsWith("font-")) {
                continue;
            }
            token = token.substring(5);
            int idx = token.indexOf(':');
            if (idx < 0) {
                continue;
            }
            String value = token.substring(idx + 1).trim();
            if (token.startsWith("family")) {
                int idxvirg = value.indexOf(',');
                if (idxvirg != - 1) {
                    value = value.substring(0, idxvirg).trim();
                }
                if (value.startsWith("\"")) {
                    value = value.substring(1);
                }
                if (value.endsWith("\"")) {
                    value = value.substring(0, value.length() - 1);
                }
                fontName = value;
            } else if (token.startsWith("size")) {
                try {
                    if (value.endsWith("pt")) {
                        fontSize = (int) Float.parseFloat(value.substring(0, value.length() - 2));
                    }
                } catch (NumberFormatException nfe) {
                }
                if (fontSize < 4) {
                    fontSize = 4;
                }
            } else if (token.startsWith("style")) {
                if (value.equals("italic")) {
                    fontStyle = fontStyle | Font.ITALIC;
                }
            } else if (token.startsWith("weight")) {
                if (value.equals("bold")) {
                    fontStyle = fontStyle | Font.BOLD;
                }
            }

        }
        if (fontName != null) {
            String[] fontes = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
            boolean existe = false;
            for (int i = 0; i < fontes.length; i++) {
                if (fontes[i].compareToIgnoreCase(fontName) == 0) {
                    fontName = fontes[i];
                    existe = true;
                    break;
                }
            }
            if (!existe) {
                fontName = null;
            }
        }
        if (fontName == null) {
            fontName = "SansSerif";
        }
        return new Font(fontName, fontStyle, fontSize);
    }

    public String toString(Rectangle rectangle) {
        StringBuffer buf = new StringBuffer();
        buf.append(rectangle.x);
        buf.append(',');
        buf.append(rectangle.y);
        buf.append(',');
        buf.append(rectangle.width);
        buf.append(',');
        buf.append(rectangle.height);
        return buf.toString();
    }

    public String toString(String[] stringArray) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < stringArray.length; i++) {
            if (i != 0) {
                buf.append(',');
            }
            boolean quoted = false;
            StringBuffer smallBuf = new StringBuffer();
            String strg = stringArray[i];
            for (int j = 0; j < strg.length(); j++) {
                char carac = strg.charAt(j);
                if (carac == '\"') {
                    smallBuf.append("\\\"");
                    quoted = true;
                } else if (carac == '\\') {
                    smallBuf.append("\\\\");
                    quoted = true;
                } else {
                    smallBuf.append(carac);
                    if ((carac > '~') || (carac < '!') || (carac == ',') || (carac == ';') || (carac == '\'')) {
                        quoted = true;
                    }
                }
            }
            if (quoted) {
                buf.append('\"');
            }
            buf.append(smallBuf.toString());
            if (quoted) {
                buf.append('\"');
            }
        }
        return buf.toString();
    }

    public Rectangle toRectangle(String string) {
        String[] tab = string.split(",");
        if (tab.length != 4) {
            return new Rectangle(60, 60, 420, 420);
        }
        int[] intValues = new int[4];
        for (int i = 0; i < 4; i++) {
            try {
                intValues[i] = Integer.parseInt(tab[i]);
            } catch (NumberFormatException nfe) {
                if (i < 2) {
                    intValues[i] = 60;
                } else {
                    intValues[i] = 420;
                }
            }
        }
        return new Rectangle(intValues[0], intValues[1], intValues[2], intValues[3]);
    }

    public String[] toStringArray(String string) {
        if (string.length() == 0) {
            return new String[0];
        }
        String[][] lignes = TableParser.parse(string, new CsvParameters());
        if (lignes.length == 0) {
            return new String[0];
        }
        if (lignes.length == 1) {
            return lignes[0];
        }
        int total = 0;
        for (int i = 0; i < lignes.length; i++) {
            total = total + lignes[i].length;
        }
        String[] result = new String[total];
        int p = 0;
        for (int i = 0; i < lignes.length; i++) {
            String[] ligne = lignes[i];
            for (int j = 0; j < ligne.length; j++) {
                result[p] = ligne[j];
                p++;
            }
        }
        return result;
    }

    public void queueEvents() {
        eventsQueued = true;
    }

    public void flushEvents() {
        if (eventsQueued) {
            eventsQueued = false;
            if (queuedEventList.size() > 0) {
                ConfEvent confEvent = new ConfEvent(this, queuedEventList);
                queuedEventList.clear();
                for (int i = 0; i < confListeners.length; i++) {
                    confListeners[i].confChanged(confEvent);
                }
            }
        }
    }

    /**
     * Tous les couples clés / valeur sont converties sous la forme d'un tableau de deux chaines.
     */
    public List<SimpleAttribute> convertToSimpleAttributeList(boolean excludeTransient) {
        List<SimpleAttribute> list = new ArrayList<SimpleAttribute>();
        for (Map.Entry<String, Object> entry : hashMap.entrySet()) {
            String key = entry.getKey();
            if ((excludeTransient) && (isTransient(key))) {
                continue;
            }
            list.add(new DefaultSimpleAttribute(key, toString(entry.getValue())));
        }
        return list;
    }


    private class FileBundle {

        File file;

        FileBundle(File f) {
            this.file = f;
        }

        File getFile() {
            return file;
        }

        boolean replaceFile(File f) {
            if (file == null) {
                if (f == null) {
                    return false;
                } else {
                    file = f;
                    return true;
                }
            }
            if (f == null) {
                file = f;
                return true;
            }
            if (f.equals(file)) {
                return false;
            }
            file = f;
            return true;
        }

    }


    private class URLBundle {

        URL url;

        URLBundle(URL url) {
            this.url = url;
        }

        URL getURL() {
            return url;
        }

        boolean replaceURL(URL u) {
            if (url == null) {
                if (u == null) {
                    return false;
                } else {
                    url = u;
                    return true;
                }
            }
            if (u == null) {
                url = u;
                return true;
            }
            if (u.equals(url)) {
                return false;
            }
            url = u;
            return true;
        }

    }

}
