/* UtilLib - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.langinteger;

import net.mapeadores.util.collections.IntHashMap;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.text.collation.map.SortedCollatedKeyMap;
import net.mapeadores.util.text.collation.map.SearchResultUnit;
import net.mapeadores.util.text.collation.map.SearchResultUnitList;
import net.mapeadores.util.common.CommonUtils;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.exceptions.InternalResourceException;
import java.util.Locale;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Collections;
import java.io.InputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.BufferedReader;


/**
 *
 * @author Vincent Calame
 */
public class LocaleItem {

    private static final short LANG_RESOURCE = 1;
    private static final short COUNTRY_RESOURCE = 2;
    static LocaleItem defaultLocaleItem;
    private Locale locale;
    private SortedMap<String, String> languageSortedMap = new TreeMap<String, String>();
    private SortedMap<String, String> countrySortedMap = new TreeMap<String, String>();
    private SortedMap<String, String> unmodifiableLanguageSortedMap = Collections.unmodifiableSortedMap(languageSortedMap);
    private SortedMap<String, String> unmodifiableCountrySortedMap = Collections.unmodifiableSortedMap(countrySortedMap);
    private IntHashMap codeMap = new IntHashMap();
    private SortedCollatedKeyMap collatedKeyMap;
    private HashMap geoDirectionMap = new HashMap();

    /**
     * Appelé quand le fichier existe dans le logiciel
     */
    LocaleItem(String langString) {
        this.locale = new Locale(langString);
        initResource(langString, LANG_RESOURCE);
        initResource(langString, COUNTRY_RESOURCE);
        if (defaultLocaleItem == null) {
            defaultLocaleItem = this;
        }
        checkResourceValues(langString);
        initGeoDirection(langString);
        collate();
    }

    /**
     * Fait appel aux données de Java
     */
    LocaleItem(Locale locale) {
        this.locale = locale;
        initGeoDirection(locale.getLanguage());
        initFromJRE();
        collate();
    }

    public Locale getLocale() {
        return locale;
    }

    public SortedMap<String, String> getLanguageSortedMap() {
        return unmodifiableLanguageSortedMap;
    }

    public SortedMap<String, String> getCountrySortedMap() {
        return unmodifiableCountrySortedMap;
    }

    public String getCardinalDirection(short direction, boolean onlyInitiale) {
        String initiale = CommonUtils.getGeoCode(direction);
        String key = null;
        if (onlyInitiale) {
            key = initiale + "i";
        } else {
            key = initiale + "t";
        }
        String lib = (String) geoDirectionMap.get(key);
        if (lib == null) {
            return initiale;
        }
        return lib;
    }

    public String getLangLibelle(int langInteger, boolean onlyMain) {
        if (langInteger >= 0) {
            return "";
        }
        Variante variante = (Variante) codeMap.get(langInteger);
        if (variante != null) {
            if (onlyMain) {
                return variante.getMainString();
            } else {
                return variante.getSourceString();
            }
        }
        int countryInteger = LangInteger.toCountryInteger(langInteger);
        if (countryInteger == LangInteger.NO_LINGUISTIC_CONTENT) {
            return LangInteger.toString(langInteger) + "?";
        } else {
            int langPart = LangInteger.getLangPart(langInteger);
            variante = (Variante) codeMap.get(langPart);
            String lib = " (" + getCountryLibelle(countryInteger, true) + ")";
            if (variante == null) {
                return LangInteger.toString(langPart) + "?" + lib;
            } else {
                return variante.getMainString() + lib;
            }
        }
    }

    public String getCountryLibelle(int langInteger, boolean onlyMain) {
        int countryInteger = LangInteger.toCountryInteger(langInteger);
        if (countryInteger == LangInteger.NO_LINGUISTIC_CONTENT) {
            return "";
        }
        Variante variante = (Variante) codeMap.get(countryInteger);
        if (variante == null) {
            return LangInteger.getCountry(countryInteger) + "?";
        }
        if (onlyMain) {
            return variante.getMainString();
        } else {
            return variante.getSourceString();
        }
    }

    public Integer getLangInteger(String lib) {
        return (Integer) collatedKeyMap.getValue(lib);
    }

    public int getLangCount() {
        return languageSortedMap.size();
    }

    public int[] getLangIntegerArray() {
        int size = languageSortedMap.size();
        int[] result = new int[size];
        int p = 0;
        for (String code : languageSortedMap.keySet()) {
            try {
                result[p] = LangInteger.parse(code, true);
                p++;
            } catch (LangIntegerException lie) {
                throw new ShouldNotOccurException(lie);
            }
        }
        return result;
    }

    public int[] getLangIntegerArray(String searchString, int type) {
        SearchResultUnitList searchResultUnitList = collatedKeyMap.search(searchString, type, null);
        int size = searchResultUnitList.getSearchResultUnitCount();
        Set<Integer> resultSet = new LinkedHashSet<Integer>();
        for (int i = 0; i < size; i++) {
            SearchResultUnit searchResultUnit = searchResultUnitList.getSearchResultUnit(i);
            Integer itg = (Integer) searchResultUnit.getValue();
            int langInteger = itg.intValue();
            if (LangInteger.getLangPart(langInteger) != LangInteger.NO_LINGUISTIC_CONTENT) {
                resultSet.add(itg);
            }
        }
        size = resultSet.size();
        int[] result = new int[size];
        int p = 0;
        for (Integer itg : resultSet) {
            result[p] = itg.intValue();
            p++;
        }
        return result;
    }

    public int getCountryCount() {
        return countrySortedMap.size();
    }

    public int[] getCountryIntegerArray() {
        int size = countrySortedMap.size();
        int[] result = new int[size];
        int p = 0;
        for (String code : countrySortedMap.keySet()) {
            try {
                result[p] = CountryUtils.parseCountryCode(code);
                p++;
            } catch (LangIntegerException lie) {
                throw new ShouldNotOccurException(lie);
            }
        }
        return result;
    }

    public int[] getCountryIntegerArray(String searchString, int type) {
        SearchResultUnitList resultItemList = collatedKeyMap.search(searchString, type, null);
        int size = resultItemList.getSearchResultUnitCount();
        Set<Integer> resultSet = new LinkedHashSet<Integer>();
        for (int i = 0; i < size; i++) {
            SearchResultUnit entry = resultItemList.getSearchResultUnit(i);
            Integer itg = (Integer) entry.getValue();
            int langInteger = itg.intValue();
            if ((LangInteger.getLangPart(langInteger) == LangInteger.NO_LINGUISTIC_CONTENT) && (langInteger != LangInteger.NO_LINGUISTIC_CONTENT)) {
                resultSet.add(itg);
            }

        }
        size = resultSet.size();
        int[] result = new int[size];
        int p = 0;
        for (Integer itg : resultSet) {
            result[p] = itg.intValue();
            p++;
        }
        return result;
    }

    private void initResource(String l10nString, short type) {
        String prefix = null;
        switch (type) {
            case LANG_RESOURCE:
                prefix = "lang";
                break;
            case COUNTRY_RESOURCE:
                prefix = "country";
                break;
            default:
                throw new SwitchException("type = " + type);
        }
        try {
            InputStream is = getClass().getResourceAsStream("l10n/" + prefix + "_" + l10nString + ".csv");
            if (is == null) {
                return;
            }
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
            String ligne = null;
            while ((ligne = bufReader.readLine()) != null) {
                if (ligne.length() > 0) {
                    int idx = ligne.indexOf(',');
                    if (idx != -1) {
                        String codeString = ligne.substring(0, idx);
                        int codeInteger = 0;
                        switch (type) {
                            case LANG_RESOURCE:
                                codeInteger = LangInteger.parse(codeString, true);
                                break;
                            case COUNTRY_RESOURCE:
                                codeInteger = CountryUtils.parseCountryCode(codeString);
                                break;
                        }
                        putCode(codeInteger, ligne.substring(idx + 1).trim(), type);
                    }
                }
            }
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        } catch (LangIntegerException lie) {
            throw new InternalResourceException(lie);
        }
    }

    private void putCode(int codeInteger, String lib, short type) {
        Variante variante = new Variante(codeInteger, lib);
        String codeString = null;
        Map<String, String> map = null;
        switch (type) {
            case LANG_RESOURCE:
                codeString = LangInteger.toString(codeInteger);
                map = languageSortedMap;
                break;
            case COUNTRY_RESOURCE:
                codeString = LangInteger.getCountry(codeInteger);
                map = countrySortedMap;
                break;
            default:
                throw new SwitchException("type = " + type);
        }
        codeMap.put(codeInteger, variante);
        map.put(codeString, variante.getSourceString());
    }

    private void checkResourceValues(String langString) {
        HashSet<String> countrySet = new HashSet<String>(CountryUtils.getIsoCodeSet());
        for (String code : countrySortedMap.keySet()) {
            boolean done = countrySet.remove(code);
            if (!done) {
                throw new InternalResourceException("code : " + code + " présent dans l10n/country_" + langString + ".csv absent de iso31166.csv et iso3166-complement.csv");
            }
        }
        if (!countrySet.isEmpty()) {
            for (String code : countrySet) {
                System.out.println("code : " + code + " absent de l10n/country_" + langString + ".csv");
                try {
                    int codeInteger = CountryUtils.parseCountryCode(code);
                    putCode(codeInteger, code, COUNTRY_RESOURCE);
                } catch (LangIntegerException lie) {
                    throw new ShouldNotOccurException(lie);
                }
            }
        }
        HashSet<String> langSet = new HashSet<String>(LangUtils.getIsoMixedCodeSet());
        for (String code : languageSortedMap.keySet()) {
            boolean done = langSet.remove(code);
            if (!done) {
                System.out.println("manque le code iso :" + code);
            }
        }
        if (!langSet.isEmpty()) {
            for (String code : langSet) {
                System.out.println("code : " + code + " absent de l10n/lang_" + langString + ".csv");
                try {
                    int codeInteger = LangInteger.parse(code, true);
                    putCode(codeInteger, code, LANG_RESOURCE);
                } catch (LangIntegerException lie) {
                    throw new ShouldNotOccurException(lie);
                }
            }
        }

    }

    private void initFromJRE() {
        for (int codeInteger : defaultLocaleItem.codeMap.keySet()) {
            Variante variante = (Variante) defaultLocaleItem.codeMap.get(codeInteger);
            codeMap.put(codeInteger, variante);
            String codeString = null;
            Map<String, String> map = null;
            if (LangInteger.isCountryInteger(codeInteger)) {
                codeString = LangInteger.getCountry(codeInteger);
                map = countrySortedMap;
            } else {
                codeString = LangInteger.toString(codeInteger);
                map = languageSortedMap;
            }
            map.put(codeString, variante.getSourceString());
        }

        String[] codes = Locale.getISOLanguages();
        for (int i = 0; i < codes.length; i++) {
            String code = codes[i];
            Locale loc = new Locale(code, "");
            String lib = loc.getDisplayLanguage(locale);
            try {
                int langInteger = LangInteger.parse(code);
                putCode(langInteger, lib, LANG_RESOURCE);
            } catch (LangIntegerException lie) {
                throw new InternalResourceException(lie);
            }
        }
        String[] codesCountry = Locale.getISOCountries();
        for (int i = 0; i < codesCountry.length; i++) {
            String code = codesCountry[i].intern();
            Locale loc = new Locale("", code);
            try {
                int countryCode = CountryUtils.parseCountryCode(code);
                putCode(countryCode, loc.getDisplayCountry(locale), COUNTRY_RESOURCE);
            } catch (LangIntegerException lie) {
                throw new InternalResourceException(lie);
            }
        }
    }

    private void collate() {
        collatedKeyMap = new SortedCollatedKeyMap(locale);
        for (Integer itg : codeMap.keySet()) {
            Variante variante = (Variante) codeMap.get(itg.intValue());
            collatedKeyMap.putValue(variante.getMainString(), itg);
            int count = variante.getVarianteCount();
            for (int i = 0; i < count; i++) {
                collatedKeyMap.putValue(variante.getVariante(i), itg);
            }
        }
    }


    private class Variante {

        String sourceString;
        String mainString;
        String[] variantes;

        Variante(int langInteger, String sourceString) {
            this.sourceString = sourceString.trim();
            parse(langInteger);
        }

        public String getSourceString() {
            return sourceString;
        }

        public String getMainString() {
            return mainString;
        }

        public int getVarianteCount() {
            if (variantes == null) {
                return 0;
            }
            return variantes.length;
        }

        public String getVariante(int i) {
            return variantes[i];
        }

        private void parse(int langInteger) {
            String[] tokens = StringUtils.getTokens(sourceString, ';', StringUtils.EMPTY_EXCLUDE);
            int length = tokens.length;
            if (length == 0) {
                throw new InternalResourceException("empty line = " + LangInteger.toString(langInteger));
            }
            mainString = simplify(tokens[0]);
            if (length > 1) {
                variantes = new String[length - 1];
                for (int i = 0; i < (length - 1); i++) {
                    variantes[i] = simplify(tokens[i + 1]);
                }
            }
        }

        private String simplify(String s) {
            int idx = s.indexOf('(');
            if (idx != -1) {
                s = s.substring(0, idx).trim();
            }
            int idx2 = s.indexOf(',');
            if (idx2 != -1) {
                s = s.substring(idx2 + 1).trim() + " " + s.substring(0, idx2).trim();
            }
            return s.intern();
        }

    }

    private void initGeoDirection(String l10nString) {
        try {
            InputStream is = getClass().getResourceAsStream("l10n/geo_" + l10nString + ".csv");
            if (is == null) {
                return;
            }
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
            String ligne = null;
            while ((ligne = bufReader.readLine()) != null) {
                if (ligne.length() > 0) {
                    int idx = ligne.indexOf(',');
                    if (idx != -1) {
                        String key = ligne.substring(0, idx).trim();
                        String lib = ligne.substring(idx + 1).trim();
                        if (lib.length() != 0) {
                            geoDirectionMap.put(key, lib);
                        }
                    }
                }
            }
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        }
    }

}
