/* UtilLib - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.sxio;

import net.mapeadores.util.io.StreamTransformer;
import net.mapeadores.util.io.StreamTransformerErrorHandler;
import net.mapeadores.util.io.IOUtils;
import java.util.Map;
import java.util.Iterator;
import java.io.OutputStream;
import java.io.File;
import java.io.IOException;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;


/**
 *
 * @author Vincent Calame
 */
public class SXStreamTransformer implements StreamTransformer {

    private String extension;
    private String mimeType;
    private String name;
    private File stylesFile;
    private Templates contentTemplate;

    private SXStreamTransformer(String extension, String mimeType) {
        this.extension = extension;
        this.mimeType = mimeType;
    }

    private void initStyles(InputStream stylesInputStream) throws IOException {
        stylesFile = File.createTempFile("styles", ".xml");
        OutputStream outputStream = new FileOutputStream(stylesFile);
        IOUtils.copy(stylesInputStream, outputStream);
        outputStream.close();
    }

    private void initTemplates(Source source) throws TransformerException {
        TransformerFactory tFactory = TransformerFactory.newInstance();
        contentTemplate = tFactory.newTemplates(source);
    }

    public static SXStreamTransformer buildFromDirectory(String extension, File sxDirectory, StreamTransformerErrorHandler streamTransformerErrorHandler) throws IOException {
        String mimeType = SXIO.getMimeType(extension);
        if (mimeType.length() == 0) {
            throw new IllegalArgumentException("wrong extension = " + extension);
        }
        SXStreamTransformer sxStreamTransformer = new SXStreamTransformer(extension, mimeType);
        File stylesFile = new File(sxDirectory, "styles.xml");
        File contentFile = new File(sxDirectory, "content.xsl");
        boolean error = false;
        if (!stylesFile.exists()) {
            streamTransformerErrorHandler.missingResourceError(stylesFile.toURI().toString());
            error = true;
        } else if (stylesFile.isDirectory()) {
            streamTransformerErrorHandler.missingResourceError(stylesFile.toURI().toString());
            error = true;
        } else {
            FileInputStream stylesInputStream = new FileInputStream(stylesFile);
            sxStreamTransformer.initStyles(stylesInputStream);
            stylesInputStream.close();
        }
        if (!contentFile.exists()) {
            streamTransformerErrorHandler.missingResourceError(contentFile.toURI().toString());
            error = true;
        } else if (contentFile.isDirectory()) {
            streamTransformerErrorHandler.missingResourceError(contentFile.toURI().toString());
            error = true;
        } else {
            try {
                sxStreamTransformer.initTemplates(new StreamSource(contentFile));
            } catch (TransformerException te) {
                streamTransformerErrorHandler.wrongResourceError(contentFile.toURI().toString(), te.getMessageAndLocation());
                error = true;
            }
        }
        if (error) {
            return null;
        }
        return sxStreamTransformer;
    }

    public String getName() {
        return name;
    }

    public String getExtension() {
        return extension;
    }

    public String getMimeType() {
        return mimeType;
    }

    public void transform(Source source, OutputStream outputStream, Map transformerParameters) throws TransformerException {
        Transformer transformer = contentTemplate.newTransformer();
        for (Iterator it = transformerParameters.entrySet().iterator(); it.hasNext();) {
            Map.Entry entry = (Map.Entry) it.next();
            String key = (String) entry.getKey();
            transformer.setParameter(key, entry.getValue());
        }
        SXContentSource sxContentSource = new TransformSXContentSource(extension, transformer, source);
        SXStylesSource sxStylesSource = new InputStreamSXStylesSource(stylesFile, "UTF-8");
        try {
            SXIO.write(outputStream, sxContentSource, sxStylesSource);
        } catch (SXIOException sxioe) {
            Throwable exception = sxioe.getCause();
            if (exception != null) {
                if (exception instanceof TransformerException) {
                    throw (TransformerException) exception;
                } else {
                    throw new TransformerException(exception);
                }
            } else {
                throw new TransformerException(sxioe);
            }
        }
    }

}
