/* UtilLib - Copyright (c) 2005-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import net.mapeadores.util.collections.LinkedIntHashMap;


/**
 * Cette classe gère un ensemble de libellés pour des langues données. Les chaînes sont systématiques nettoyées avec
 * StringUtils.cleanString() et une chaîne de longueur nulle est assimiliée à <em>null</em>.
 *
 * @author  Vincent Calame
 */
public class LibelleByLangMap {

    static final String DEFAULT_LIBELLESTRING = "?";
    InternalLib defaultInternalLib;
    private LinkedIntHashMap internalLibMap;
    private String defaultLibelleString = null;

    /**
     * Avec ce constructeur le libellé par défaut est le libellé du premier libellé en vo.
     */
    public LibelleByLangMap() {
    }

    /**
     * Cette méthode fixe un libellé par défaut à utiliser lorsque le libellé n'existe pas.
     * L'argument est nettoyé et ne doit pas être vide.
     *
     *@throws IllegalArgumentException si defaultLibelleString est vide ou null.
     */
    public LibelleByLangMap(String defaultLibelleString) {
        if (defaultLibelleString == null) {
            throw new IllegalArgumentException("defaultLibelleString == null");
        }
        defaultLibelleString = StringUtils.cleanString(defaultLibelleString);
        if (defaultLibelleString.length() == 0) {
            throw new IllegalArgumentException("cleaned defaultLibelleString is empty");
        }
        this.defaultLibelleString = defaultLibelleString;
    }

    public int getLibelleCount() {
        if (defaultInternalLib == null) {
            return 0;
        }
        if (internalLibMap == null) {
            return 1;
        } else {
            return 1 + internalLibMap.size();
        }
    }

    public Libelle getLibelleByIndex(int index) {
        if (defaultInternalLib == null) {
            throw new IndexOutOfBoundsException();
        }
        if (index == 0) {
            return defaultInternalLib;
        }
        if (internalLibMap == null) {
            throw new IndexOutOfBoundsException();
        }
        return (Libelle) internalLibMap.getValueAt(index - 1);
    }

    public String getLibelleString(int langInteger, boolean defaut) {
        Libelle lib = getLibelle(langInteger);
        if (lib == null) {
            if (defaut) {
                if (defaultLibelleString != null) {
                    return defaultLibelleString;
                }
                if (defaultInternalLib == null) {
                    return DEFAULT_LIBELLESTRING;
                } else {
                    return defaultInternalLib.getLibelleString();
                }
            } else {
                return null;
            }
        }
        return lib.getLibelleString();
    }

    /**
     * 
     *Le premier Libellé créé détermine la langue par défaut.
     */
    public Libelle createLibelle(int langInteger, String nvlibelle) {
        nvlibelle = StringUtils.cleanString(nvlibelle);
        if ((nvlibelle == null) || (nvlibelle.length() == 0)) {
            return null;
        }
        InternalLib lib = new InternalLib(langInteger, nvlibelle);
        if ((defaultInternalLib == null) || (defaultInternalLib.getLangInteger() == langInteger)) {
            defaultInternalLib = lib;
            return lib;
        } else {
            if (internalLibMap == null) {
                internalLibMap = new LinkedIntHashMap();
            }
            internalLibMap.put(langInteger, lib);
        }
        return lib;
    }

    public Libelle getLibelle(int langInteger) {
        if (defaultInternalLib == null) {
            return null;
        }
        if (langInteger == defaultInternalLib.getLangInteger()) {
            return defaultInternalLib;
        }
        if (internalLibMap == null) {
            return null;
        }
        return (InternalLib) internalLibMap.get(langInteger);
    }

    public Libelle removeLibelle(int langInteger) {
        if (defaultInternalLib == null) {
            return null;
        }
        if (langInteger == defaultInternalLib.getLangInteger()) {
            InternalLib oldlib = defaultInternalLib;
            if (internalLibMap == null) {
                defaultInternalLib = null;
            } else {
                defaultInternalLib = (InternalLib) internalLibMap.getValueAt(0);
                internalLibMap.remove(defaultInternalLib.getLangInteger());
                if (internalLibMap.size() == 0) {
                    internalLibMap = null;
                }
            }
            return oldlib;
        } else if (internalLibMap != null) {
            InternalLib lib = (InternalLib) internalLibMap.remove(langInteger);
            if (internalLibMap.size() == 0) {
                internalLibMap = null;
            }
            return lib;
        }
        return null;
    }

    public LibelleBean[] toLibelleBeanArray() {
        if (defaultInternalLib == null) {
            return new LibelleBean[0];
        }
        if (internalLibMap == null) {
            LibelleBean[] array = new LibelleBean[1];
            array[0] = new LibelleBean(defaultInternalLib);
            return array;
        }
        int size = internalLibMap.size();
        LibelleBean[] array = new LibelleBean[1 + size];
        array[0] = new LibelleBean(defaultInternalLib);
        for (int i = 0; i < size; i++) {
            array[i + 1] = new LibelleBean((InternalLib) internalLibMap.getValueAt(i));
        }
        return array;
    }

    /**
     * Modifie les valeus du libellé.
     * Si <code>nvlibelle</code> est null ou de longueur nulle après nettoyage des espaces blancs superflus, la méthode est équivalent à <em>removeLibelle(lib.getLangInteger)</em>.
     * Retourne <em>true</em> si un changement a réellement eu lieu, <em>false</em> sinon.
     */
    public boolean setLibelle(Libelle lib, String nvlibelle) {
        nvlibelle = StringUtils.cleanString(nvlibelle);
        if ((nvlibelle == null) || (nvlibelle.length() == 0)) {
            Libelle oldLib = removeLibelle(lib.getLangInteger());
            return (oldLib != null);
        }
        InternalLib intlib = (InternalLib) lib;
        if (intlib.libelle.equals(nvlibelle)) {
            return false;
        }
        intlib.setLibelle(nvlibelle);
        return true;
    }

    /**
     * Crèe le libellé s'il n'existait pas.
     */
    public boolean setOrCreateLibelle(int langInteger, String libelleString, boolean isvo) {
        Libelle libelle = getLibelle(langInteger);
        if (libelle == null) {
            libelle = createLibelle(langInteger, libelleString);
            return true;
        } else {
            return setLibelle(libelle, libelleString);
        }
    }


    private class InternalLib implements Libelle {

        String libelle;
        int langInteger;

        InternalLib(int langInteger, String libelle) {
            this.langInteger = langInteger;
            setLibelle(libelle);
        }

        public String getLibelleString() {
            return libelle;
        }

        /**
         * Le libelle ne doit pas être null.
         */
        public void setLibelle(String libelle) {
            if (libelle == null) {
                throw new IllegalArgumentException("ibelle argument is null");
            }
            this.libelle = libelle;
        }

        public int getLangInteger() {
            return langInteger;
        }

    }

}
