/* UtilLib - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.alphabet;

import net.mapeadores.util.collections.IntHashMap;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.exceptions.InternalResourceException;
import java.util.List;
import java.util.ArrayList;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;


/**
 *
 * @author Vincent Calame
 */
public class Alphabet {

    static final IntHashMap articleArrayMap = new IntHashMap();
    private String sourceString;
    private String articleString;
    private String alphabetString;
    private boolean withEspace;

    public Alphabet() {
    }

    private Alphabet(String sourceString) {
        setSourceString(sourceString);
        setAlphabetString(sourceString);

    }

    public static Alphabet newInstance(String sourceString, int langInteger) {
        Article[] array = getArray(langInteger);
        if (array == null) {
            return new Alphabet(sourceString);
        }
        Alphabet alphabet = new Alphabet();
        alphabet.setSourceString(sourceString);
        String articleString = null;
        boolean withEspace = false;
        String alphabetString = sourceString;
        for (int i = 0; i < array.length; i++) {
            Article currentArticle = array[i];
            if (sourceString.startsWith(currentArticle.getArticleString())) {
                int length = currentArticle.getArticleLength();
                if (sourceString.length() > length) {
                    articleString = currentArticle.getTrimedArticleString();
                    withEspace = currentArticle.isWithEspace();
                    alphabetString = sourceString.substring(length);
                }
                break;
            }
        }
        alphabet.setAlphabetString(alphabetString);
        alphabet.setArticleString(articleString);
        alphabet.setWithEspace(withEspace);
        return alphabet;
    }

    private static Article[] getArray(int langInteger) {
        Object obj = articleArrayMap.get(langInteger);
        if (obj == null) {
            obj = initArray(langInteger);
            if (obj == null) {
                int langPart = LangInteger.getLangPart(langInteger);
                if (langPart != langInteger) {
                    obj = getArray(langPart);
                }
            }
            if (obj == null) {
                obj = Boolean.FALSE;
            }
            articleArrayMap.put(langInteger, obj);
        }
        if (obj instanceof Boolean) {
            return null;
        } else {
            return (Article[]) obj;
        }
    }

    /*
     * Les articles doivent commencer par les plus longs (voir la boucle de newInstance)
     */
    private static Article[] initArray(int langInteger) {
        InputStream inputStream = Alphabet.class.getResourceAsStream("resources/article_" + LangInteger.toString(langInteger) + ".txt");
        if (inputStream == null) {
            return null;
        }
        try {
            List<String> list = new ArrayList<String>();
            BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, "UTF-8"));
            String ligne = null;
            while ((ligne = reader.readLine()) != null) {
                ligne = ligne.trim();
                if ((ligne.length() > 0) && (ligne.charAt(0) != '#')) {
                    ligne = ligne.replace('_', ' ');
                    list.add(ligne);
                }
            }
            reader.close();
            int size = list.size();
            Article[] result = new Article[size];
            for (int i = 0; i < size; i++) {
                result[i] = new Article(list.get(i));
            }
            return result;
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        }
    }

    public boolean isWithArticle() {
        return (!sourceString.equals(alphabetString));
    }

    public String getSourceString() {
        return sourceString;
    }

    public void setSourceString(String sourceString) {
        this.sourceString = sourceString;
    }

    public String getArticleString() {
        return articleString;
    }

    public void setArticleString(String articleString) {
        this.articleString = articleString;
    }

    public String getAlphabetString() {
        return alphabetString;
    }

    public void setAlphabetString(String alphabetString) {
        this.alphabetString = alphabetString;
    }

    public boolean isWithEspace() {
        return withEspace;
    }

    public void setWithEspace(boolean withEspace) {
        this.withEspace = withEspace;
    }


    private static class Article {

        private String articleString;
        private boolean withEspace;
        private String trimedArticleString;

        Article(String articleString) {
            this.articleString = articleString;
            if (articleString.charAt(articleString.length() - 1) == ' ') {
                trimedArticleString = articleString.substring(0, articleString.length() - 1);
                withEspace = true;
            } else {
                withEspace = false;
                trimedArticleString = articleString;
            }
        }

        public String getArticleString() {
            return articleString;
        }

        public boolean isWithEspace() {
            return withEspace;
        }

        public String getTrimedArticleString() {
            return trimedArticleString;
        }

        public int getArticleLength() {
            return articleString.length();
        }

    }

}
