/* UtilLib - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.lexie;

import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.collation.map.CollatedKeyMap;
import net.mapeadores.util.text.collation.map.SortedCollatedKeyMap;
import net.mapeadores.util.text.collation.map.UnmodifiableCollatedKeyMap;
import net.mapeadores.util.localisation.LocalisationLexieFilters;
import net.mapeadores.util.text.SubstringPosition;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.LinkedHashMap;
import java.util.Collections;
import java.util.Locale;


/**
 *
 * @author Vincent Calame
 */
public class LexieMap {

    private SortedCollatedKeyMap collatedKeyMap;
    private UnmodifiableCollatedKeyMap unmodifiableCollatedKeyMap;
    private LexieFilter lexieFilter;
    private Map<String, ParsedLexieSource> lexieSourceMap = new LinkedHashMap<String, ParsedLexieSource>();
    private Map<String, ParsedLexieSource> unmodifLexieSourceMap = Collections.unmodifiableMap(lexieSourceMap);

    public LexieMap(Locale locale) {
        collatedKeyMap = new SortedCollatedKeyMap(locale);
        init(LangInteger.fromLocale(locale));
    }

    public LexieMap(int langInteger) {
        collatedKeyMap = new SortedCollatedKeyMap(langInteger);
        init(langInteger);
    }

    private void init(int langInteger) {
        unmodifiableCollatedKeyMap = new UnmodifiableCollatedKeyMap(collatedKeyMap);
        lexieFilter = LocalisationLexieFilters.getLexieFilter(langInteger);
    }

    public CollatedKeyMap getCollatedKeyMap() {
        return unmodifiableCollatedKeyMap;
    }

    public Map<String, ParsedLexieSource> getParsedLexieSourceMap() {
        return unmodifLexieSourceMap;
    }

    public void setLexieFilter(LexieFilter lexieFilter) {
        this.lexieFilter = lexieFilter;
    }

    public void addLexieSource(LexieSource lexieSource) {
        ImplParsedLexieSource ipls = new ImplParsedLexieSource(lexieSource);
        lexieSourceMap.put(lexieSource.getSourceId(), ipls);

        InternalLexieParserListener internalLexieParserListener = new InternalLexieParserListener(ipls);
        LexieParser.parse(lexieSource.getSourceText(), internalLexieParserListener);
    }


    private class InternalLexieParserListener implements LexieParserListener {

        ImplParsedLexieSource parsedLexieSource;

        InternalLexieParserListener(ImplParsedLexieSource parsedLexieSource) {
            this.parsedLexieSource = parsedLexieSource;
        }

        public void flushLexie(String lexie, int lexieStartIndex) {
            if ((lexieFilter != null) && (!lexieFilter.acceptLexie(lexie))) {
                return;
            }
            String collatedLexie = collatedKeyMap.toCollatedKey(lexie);
            ImplLexieUnit lexieUnit = (ImplLexieUnit) collatedKeyMap.getValueByCollatedKey(collatedLexie);
            if (lexieUnit == null) {
                lexieUnit = new ImplLexieUnit(collatedLexie, collatedKeyMap.size() + 1);
                collatedKeyMap.putValueByCollatedKey(collatedLexie, lexieUnit);
            }
            SubstringPosition substringPosition = new SubstringPosition(lexieStartIndex, lexie.length());
            ImplOccurence implOccurence = new ImplOccurence(lexieUnit, parsedLexieSource, substringPosition);
            parsedLexieSource.addOccurence(implOccurence);
            lexieUnit.addOccurence(implOccurence);
            lexieUnit.checkCanonical(lexie);
        }

        public void checkSpecialChar(char carac) {
        }

    }


    private class ImplParsedLexieSource implements ParsedLexieSource {

        private LexieSource lexieSource;
        List<Occurence> occurenceList = new ArrayList();
        private List<Occurence> unmodifiableOccurenceList = Collections.unmodifiableList(occurenceList);

        ImplParsedLexieSource(LexieSource lexieSource) {
            this.lexieSource = lexieSource;
        }

        public LexieSource getLexieSource() {
            return lexieSource;
        }

        public List<Occurence> getOccurenceList() {
            return unmodifiableOccurenceList;
        }

        void addOccurence(Occurence occurence) {
            occurenceList.add(occurence);
        }

    }


    private class ImplOccurence implements Occurence {

        ImplLexieUnit implLexieUnit;
        SubstringPosition substringPosition;
        ImplParsedLexieSource implParsedLexieSource;
        int occurencePosition;

        ImplOccurence(ImplLexieUnit implLexieUnit, ImplParsedLexieSource implParsedLexieSource, SubstringPosition substringPosition) {
            this.implLexieUnit = implLexieUnit;
            this.substringPosition = substringPosition;
            this.implParsedLexieSource = implParsedLexieSource;
        }

        public TextLexieUnit getLexieUnit() {
            return implLexieUnit;
        }

        public SubstringPosition getSubstringPosition() {
            return substringPosition;
        }

        public ParsedLexieSource getParsedLexieSource() {
            return implParsedLexieSource;
        }

        public int getOccurencePosition() {
            return occurencePosition;
        }

        void setOccurencePosition(int occurencePosition) {
            this.occurencePosition = occurencePosition;
        }

    }


    private class ImplLexieUnit implements TextLexieUnit {

        private String collatedLexie;
        private String canonicalLexie;
        private List<Occurence> occurenceList = new ArrayList<Occurence>();
        private List<Occurence> unmodifiableOccurenceList = Collections.unmodifiableList(occurenceList);
        private String lexieId;

        public ImplLexieUnit(String collatedLexie, int id) {
            this.collatedLexie = collatedLexie;
            this.lexieId = String.valueOf(id);
        }

        public String getCollatedLexie() {
            return collatedLexie;
        }

        public String getCanonicalLexie() {
            return canonicalLexie;
        }

        public List<Occurence> getOccurenceList() {
            return unmodifiableOccurenceList;
        }

        public String getLexieId() {
            return lexieId;
        }

        void addOccurence(ImplOccurence occurence) {
            int occurencePosition = 1;
            if (occurenceList.size() > 0) {
                ImplOccurence last = (ImplOccurence) occurenceList.get(occurenceList.size() - 1);
                if (last.getParsedLexieSource().equals(occurence.getParsedLexieSource())) {
                    occurencePosition = last.getOccurencePosition() + 1;
                }
            }
            occurence.setOccurencePosition(occurencePosition);
            occurenceList.add(occurence);
        }

        void checkCanonical(String lexie) {
            if (canonicalLexie == null) {
                canonicalLexie = lexie;
            } else {
                if (lexie.length() < canonicalLexie.length()) {
                    canonicalLexie = lexie;
                }
            }
        }

    }

}
