/* UtilLib - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.tableparser;

import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.exceptions.NestedClassLoaderException;
import java.util.List;
import java.util.ArrayList;
import java.io.IOException;
import java.io.StringReader;
import java.io.BufferedReader;


/**
 *
 * @author Vincent Calame
 */
public class TableParser {

    final static int WAITING_FROM_ARGUMENT_END_STEP = 3;
    final static int WAITING_FROM_VALUE_STEP = 5;
    final static int VALUE_STEP = 9;
    final static int QUOTED_VALUE_STEP = 10;
    final static int WAITING_FROM_EQUAL_STEP = 6;
    private static CsvParser csvParser = null;

    private TableParser() {
    }

    public static String[][] parse(String s, Parameters parameters) {
        if (parameters instanceof OOCopyParameters) {
            return parseOOCopy(s, (OOCopyParameters) parameters);
        }
        if (parameters instanceof CsvParameters) {
            if (csvParser != null) {
                return csvParser.parse(s, (CsvParameters) parameters);
            }
            SimpleCsvParser scp = new SimpleCsvParser();
            return scp.parse(s, (CsvParameters) parameters);
        }
        if (parameters instanceof MysqlParameters) {
            return MysqlParser.parse(s, (MysqlParameters) parameters);
        }
        throw new IllegalArgumentException("unknown TableParser.Parameters implementations = " + parameters.getClass().getName());
    }

    private static String[][] parseOOCopy(String s, OOCopyParameters parameters) {
        String[] lines = StringUtils.getLineTokens(s, StringUtils.NOTCLEAN);
        int length = lines.length;
        String[][] result = new String[length][];
        for (int i = 0; i < length; i++) {
            String line = lines[i];
            String[] tokens = StringUtils.getTokens(line, '\t', StringUtils.NOTCLEAN);
            result[i] = tokens;
        }
        return result;
    }

    public static boolean isCsvParserInit() {
        return (csvParser != null);
    }

    public static void initCsvParser(String className) {
        try {
            Class csvParserClass = Class.forName(className);
            csvParser = (CsvParser) csvParserClass.newInstance();
        } catch (Exception e) {
            throw new NestedClassLoaderException(e);
        }
    }


    public static interface Parameters {
    }


    private static class SimpleCsvParser implements CsvParser {

        LineParser lineParser = new LineParser();

        public String[][] parse(String s, CsvParameters parameters) {
            List<String[]> ligneList = new ArrayList<String[]>();
            try {
                StringReader stringReader = new StringReader(s);
                BufferedReader bufreader = new BufferedReader(stringReader);
                String line = null;
                while ((line = bufreader.readLine()) != null) {
                    try {
                        String[] array = parseLine(line);
                        ligneList.add(array);
                    } catch (IllegalArgumentException ise) {
                        break;
                    }
                }
            } catch (IOException ioe) {
            }
            return ligneList.toArray(new String[ligneList.size()][]);
        }

        public String[] parseLine(String line) {
            int length = line.length();
            for (int i = 0; i < length; i++) {
                i = lineParser.parseCharAt(line, i);
            }
            return lineParser.result();
        }

    }


    private static class LineParser {

        List<String> resultList = new ArrayList<String>();
        int step = WAITING_FROM_VALUE_STEP;
        StringBuilder buf = new StringBuilder();
        boolean previousWhite = false;

        public int parseCharAt(String line, int index) {
            char carac = line.charAt(index);
            if (Character.isWhitespace(carac)) {
                testWhiteChar(carac);
                return index;
            }
            if (step != QUOTED_VALUE_STEP) {
                if (testEnd(carac)) {
                    return index;
                }
            }
            switch (step) {
                case WAITING_FROM_ARGUMENT_END_STEP:
                case WAITING_FROM_VALUE_STEP:
                    if (carac == '\"') {
                        step = QUOTED_VALUE_STEP;
                    } else {
                        step = VALUE_STEP;
                        index = parseCharAt(carac, line, index);
                    }
                    break;
                case VALUE_STEP:
                    if (previousWhite) {
                        buf.append(' ');
                    }
                    index = parseCharAt(carac, line, index);
                    break;
                case QUOTED_VALUE_STEP:
                    if (carac == '\"') {
                        step = WAITING_FROM_ARGUMENT_END_STEP;
                    } else {
                        index = parseCharAt(carac, line, index);
                    }
                    break;
            }
            previousWhite = false;
            return index;
        }

        int parseCharAt(char carac, String line, int index) {
            if (carac == '\\') {
                if (index < (line.length() - 1)) {
                    char next = line.charAt(index + 1);
                    if (next == 'n') {
                        buf.append('\n');
                    } else {
                        buf.append(next);
                    }
                    index = index + 1;
                }
            } else {
                buf.append(carac);

            }
            return index;
        }

        boolean testEnd(char carac) {
            if (carac == ',') {
                flushResult();
                step = WAITING_FROM_VALUE_STEP;
                return true;
            }
            return false;
        }

        void testWhiteChar(char carac) {
            switch (step) {
                case WAITING_FROM_EQUAL_STEP:
                    break;
                case VALUE_STEP:
                    previousWhite = true;
                    break;
                case QUOTED_VALUE_STEP:
                    buf.append(carac);
                    break;
            }

        }

        private void flushResult() {
            resultList.add(buf.toString());
            buf = new StringBuilder();
        }

        public String[] result() {
            flushResult();
            String[] resultArray = resultList.toArray(new String[resultList.size()]);
            resultList.clear();
            return resultArray;
        }

    }

}
