/* UtilLib - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.xml;

import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.text.insertpattern.api.ErrorHandler;
import net.mapeadores.util.text.insertpattern.api.InsertionBuffer;
import net.mapeadores.util.text.insertpattern.api.InsertionUnit;
import net.mapeadores.util.text.insertpattern.impl.InsertpatternParser;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.xml.ns.NameSpace;
import net.mapeadores.util.xml.ns.NameSpaceUtils;
import java.util.Map;
import java.util.LinkedHashMap;
import java.util.Stack;
import java.io.IOException;
import java.io.Reader;


/**
 *
 * @author Vincent Calame
 */
public class PatternXSLTResolver {

    private CustomResolver customResolver;
    private XSLTWriter rootXsltWriter;
    private XSLTWriter currentXsltWriter;

    public PatternXSLTResolver() {
    }

    public void setAppendable(Appendable appendable, int indentValue) {
        rootXsltWriter = new XSLTWriter();
        rootXsltWriter.setAppendable(appendable);
        rootXsltWriter.setIndentLength(indentValue);
        currentXsltWriter = rootXsltWriter;
    }

    public void setCustomResolver(CustomResolver customResolver) {
        this.customResolver = customResolver;
    }

    public XSLTWriter getCurrentXSLTWriter() {
        return currentXsltWriter;
    }

    public void parse(Reader reader, ErrorHandler errorHandler) throws IOException {
        InsertpatternParser parser = new InsertpatternParser();
        if (errorHandler != null) {
            parser.setErrorHandler(errorHandler);
        }
        XsltHandler handler = new XsltHandler();
        parser.setInsertionHandler(handler);
        try {
            parser.parse(reader);
        } catch (NestedIOException nioe) {
            throw nioe.getIOException();
        }
        handler.resolve();
        rootXsltWriter.closeXslStyleSheet();
    }


    private class XsltHandler extends InsertionBuffer {

        private boolean started = false;
        private boolean onTemplate = false;
        private Stack firstClauseStack = new Stack();
        private Map<String, NameSpace> nameSpaceMap = new LinkedHashMap<String, NameSpace>();

        XsltHandler() {
        }

        private void startXSLT() {
            if (!started) {
                try {
                    rootXsltWriter.startXslStyleSheetOpenTag();
                    for (NameSpace nameSpace : nameSpaceMap.values()) {
                        rootXsltWriter.addNameSpaceAttribute(nameSpace);
                    }
                    rootXsltWriter.endOpenTag();
                } catch (IOException ioe) {
                    throw new NestedIOException(ioe);
                }
            }
            started = true;
        }

        private void checkNameSpace(Instruction instruction) {
            int count = instruction.getLength();
            String firstValue = instruction.getArgument(0).getValue();
            if (firstValue != null) {
                nameSpaceMap.put("", new NameSpace("", firstValue));
            }
            for (int i = 1; i < count; i++) {
                Argument argument = instruction.getArgument(i);
                String key = argument.getKey();
                String value = argument.getValue();
                if (value == null) {
                    addNameSpaceMap(key);
                } else {
                    nameSpaceMap.put(key, new NameSpace(key, value));
                }
            }
        }

        private void addNameSpaceMap(String name) {
            if (!name.startsWith("_")) {
                return;
            }
            name = name.substring(1);
            nameSpaceMap.putAll(NameSpaceUtils.getNameSpaceMap(name));
        }

        private void testStart() {
            if (!started) {
                startXSLT();
            }
        }

        @Override
        public void appendString(String litteralString) {
            if (!started) {
                if (litteralString.trim().length() == 0) {
                    return;
                } else {
                    startXSLT();
                }
            }
            super.appendString(litteralString);
            setIsXslInstruction(true);
        }

        private void setIsXslInstruction(boolean isXslInstruction) {
            InsertionUnit insertionUnit = getLastInsertionUnit();
            if (isXslInstruction) {
                insertionUnit.setCustomObject(Boolean.TRUE);
            } else {
                insertionUnit.setCustomObject(Boolean.FALSE);
                customResolver.testInsertionUnit(insertionUnit);
            }
        }

        @Override
        public void doInsertion(Instruction[] instructionArray) {
            String firstKey = instructionArray[0].getArgument(0).getKey();
            if (!started) {
                boolean isNameSpaceInstruction = firstKey.equals("_Xns");
                if (isNameSpaceInstruction) {
                    checkNameSpace(instructionArray[0]);
                }
                startXSLT();
                if (isNameSpaceInstruction) {
                    return;
                }
            }
            super.doInsertion(instructionArray);
            if (firstKey.startsWith("_X")) {
                setIsXslInstruction(true);
            } else {
                setIsXslInstruction(false);
            }
        }

        @Override
        public void startIf(String clause, boolean unique) {
            testStart();
            super.startIf(clause, unique);
            if (clause.startsWith("_X")) {
                firstClauseStack.add(Boolean.TRUE);
                setIsXslInstruction(true);
            } else {
                firstClauseStack.add(Boolean.FALSE);
                setIsXslInstruction(false);
            }
        }

        @Override
        public void startElseIf(String clause) {
            testStart();
            super.startElseIf(clause);
            boolean current = (((Boolean) firstClauseStack.peek())).booleanValue();
            setIsXslInstruction(current);
        }

        @Override
        public void startElse() {
            testStart();
            super.startElse();
            boolean current = (((Boolean) firstClauseStack.peek())).booleanValue();
            setIsXslInstruction(current);
        }

        @Override
        public void endIf(boolean unique, boolean onElse) {
            testStart();
            super.endIf(unique, onElse);
            boolean current = (((Boolean) firstClauseStack.pop())).booleanValue();
            setIsXslInstruction(current);
        }

        private void flush() throws IOException {
            if (onTemplate) {
                onTemplate = false;
                currentXsltWriter.closeXslTemplate();
            }
        }

        public void resolve() {
            testStart();
            try {
                for (InsertionUnit insertionUnit : getInsertionUnitList()) {
                    resolve(insertionUnit);
                }
                flush();
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }

        private void resolve(InsertionUnit insertionUnit) throws IOException {
            Boolean bool = (Boolean) insertionUnit.getCustomObject();
            if (bool.booleanValue()) {
                resolveXslItem(insertionUnit);
            } else {
                boolean isTemplateInstruction = customResolver.isTemplateInstruction(insertionUnit);
                if (isTemplateInstruction) {
                    flush();
                }
                customResolver.resolveInsertionUnit(insertionUnit);
                if (isTemplateInstruction) {
                    setOnTemplate();
                }
            }
        }

        private void resolveXslItem(InsertionUnit insertionUnit) throws IOException {
            switch (insertionUnit.getType()) {
                case InsertionBuffer.APPEND_STRING:
                    currentXsltWriter.append(insertionUnit.getString());
                    break;
                case InsertionBuffer.DO_INSERTION:
                    resolveInsertion(insertionUnit.getInstructionArray());
                    break;
                case InsertionBuffer.START_IF:
                    resolveStartIf(insertionUnit.getClause(), insertionUnit.isUnique());
                    break;
                case InsertionBuffer.START_ELSEIF:
                    resolveStartElseIf(insertionUnit.getClause());
                    break;
                case InsertionBuffer.START_ELSE:
                    resolveStartElse();
                    break;
                case InsertionBuffer.END_IF:
                    resolveEndIf(insertionUnit.isUnique(), insertionUnit.isOnElse());
                    break;
                default:
                    throw new SwitchException("unknown insertionUnit.getType() = " + insertionUnit.getType());
            }

        }

        private void resolveInsertion(Instruction[] instructionArray) throws IOException {
            String firstKey = instructionArray[0].getArgument(0).getKey();
            if (firstKey.equals("_XT")) {
                startTemplate(instructionArray[0]);
            } else if (firstKey.equals("_XAT")) {
                addApplyTemplates(instructionArray);
            } else if (firstKey.equals("_XCT")) {
                addCallTemplate(instructionArray);
            } else if (firstKey.equals("_XV")) {
                addVariable(instructionArray);
            } else if (firstKey.equals("_XP")) {
                addParam(instructionArray);
            } else if (firstKey.equals("_XVO")) {
                addValue(instructionArray[0]);
            } else if (firstKey.equals("_X_")) {
                addText(instructionArray[0]);
            } else if (firstKey.equals("_X_s")) {
                currentXsltWriter.addSpaceXslText();
            } else if (firstKey.equals("_X_n")) {
                currentXsltWriter.addNewlineXslText();
            } else if (firstKey.equals("_X_t")) {
                currentXsltWriter.addTabXslText();
            } else {
                doMultiInstruction(instructionArray);
            }
        }

        private void resolveStartIf(String clause, boolean unique) throws IOException {
            clause = clause.substring(2).trim();
            if (unique) {
                currentXsltWriter.openXslIf(clause);
            } else {
                currentXsltWriter.openXslChoose();
                currentXsltWriter.openXslWhen(clause);
            }
        }

        private void resolveStartElseIf(String clause) throws IOException {
            currentXsltWriter.closeXslWhen();
            clause = clause.substring(2).trim();
            currentXsltWriter.openXslWhen(clause);
        }

        private void resolveStartElse() throws IOException {
            currentXsltWriter.closeXslWhen();
            currentXsltWriter.openXslOtherwise();

        }

        private void resolveEndIf(boolean unique, boolean onElse) throws IOException {
            if (unique) {
                currentXsltWriter.closeXslIf();
            } else {
                if (onElse) {
                    currentXsltWriter.closeXslOtherwise();
                } else {
                    currentXsltWriter.closeXslWhen();
                }
                currentXsltWriter.closeXslChoose();
            }
        }

        private void doMultiInstruction(Instruction[] instructionArray) throws IOException {
            int length = instructionArray.length;
            for (int i = 0; i < length; i++) {
                Instruction instruction = instructionArray[i];
                String key = instruction.getArgument(0).getKey();
                if (key.equals("_Ximport")) {
                    addImport(instruction);
                } else if (key.equals("_Xinclude")) {
                    addInclude(instruction);
                } else if (key.equals("_Xoutput")) {
                    addOutput(instruction);
                }
            }
        }

        private void startTemplate(Instruction instruction) throws IOException {
            flush();
            Argument argument = instruction.getArgument(0);
            String value = argument.getValue();
            Attributes attributes = new Attributes(instruction);
            if (value == null) {
                currentXsltWriter.openNameXslTemplate(attributes.name, attributes.mode, attributes.priority);
            } else {
                currentXsltWriter.openMatchXslTemplate(value, attributes.mode, attributes.priority);
            }
            setOnTemplate();
        }

        private void setOnTemplate() {
            onTemplate = true;
        }

        private void addValue(Instruction instruction) throws IOException {
            Argument argument = instruction.getArgument(0);
            String value = argument.getValue();
            Attributes attributes = new Attributes(instruction);
            if (attributes.doe) {
                currentXsltWriter.addXslValueOf(value, true);
            } else {
                currentXsltWriter.addXslValueOf(value);
            }
        }

        private void addText(Instruction instruction) throws IOException {
            Argument argument = instruction.getArgument(0);
            String value = argument.getValue();
            Attributes attributes = new Attributes(instruction);
            if (attributes.doe) {
                currentXsltWriter.addXslText(value, true);
            } else {
                currentXsltWriter.addXslText(value);
            }
        }

        private void addVariable(Instruction[] instructionArray) throws IOException {
            Instruction mainInstruction = instructionArray[0];
            String name = mainInstruction.getArgument(0).getValue();
            String select = "";
            int length = mainInstruction.getLength();
            for (int i = 1; i < length; i++) {
                Argument argument = mainInstruction.getArgument(i);
                String key = argument.getKey();
                if (key.equals("select")) {
                    select = argument.getValue();
                }
            }
            currentXsltWriter.addXslVariable(name, select, false);
        }

        private void addParam(Instruction[] instructionArray) throws IOException {
            Instruction mainInstruction = instructionArray[0];
            String name = mainInstruction.getArgument(0).getValue();
            String select = "";
            int length = mainInstruction.getLength();
            for (int i = 1; i < length; i++) {
                Argument argument = mainInstruction.getArgument(i);
                String key = argument.getKey();
                if (key.equals("select")) {
                    select = argument.getValue();
                }
            }
            currentXsltWriter.addXslParam(name, select, false);
        }

        private void addApplyTemplates(Instruction[] instructionArray) throws IOException {
            Instruction mainInstruction = instructionArray[0];
            String select = mainInstruction.getArgument(0).getValue();
            Attributes attributes = new Attributes(mainInstruction);
            int length = instructionArray.length;
            if (length == 1) {
                currentXsltWriter.addXslApplyTemplates(select, attributes.mode);
            } else {
                currentXsltWriter.openXslApplyTemplates(select, attributes.mode);
                currentXsltWriter.closeXslApplyTemplates();
            }
        }

        private void addCallTemplate(Instruction[] instructionArray) throws IOException {
            Instruction mainInstruction = instructionArray[0];
            String name = mainInstruction.getArgument(0).getValue();
            Attributes attributes = new Attributes(mainInstruction);
            int length = instructionArray.length;
            if (length == 1) {
                currentXsltWriter.addXslCallTemplate(name, attributes.mode);
            } else {
                currentXsltWriter.openXslCallTemplate(name, attributes.mode);
                currentXsltWriter.closeXslCallTemplate();
            }
        }

        private void addImport(Instruction instruction) throws IOException {
            String arg = instruction.getArgument(0).getValue();
            if (arg != null) {
                currentXsltWriter.addXslImport(arg);
            }
            int size = instruction.getLength();
            for (int i = 1; i < size; i++) {
                String key = instruction.getArgument(i).getKey();
                currentXsltWriter.addXslImport(key);
            }
        }

        private void addInclude(Instruction instruction) throws IOException {
            String arg = instruction.getArgument(0).getValue();
            if (arg != null) {
                currentXsltWriter.addXslInclude(arg);
            }
            int size = instruction.getLength();
            for (int i = 1; i < size; i++) {
                String key = instruction.getArgument(i).getKey();
                currentXsltWriter.addXslInclude(key);
            }
        }

        private void addOutput(Instruction instruction) throws IOException {
            currentXsltWriter.startOpenTag("xsl:output");
            String val0 = instruction.getArgument(0).getValue();
            if (val0 != null) {
                currentXsltWriter.addAttribute("method", val0);
            }
            int size = instruction.getLength();
            for (int i = 1; i < size; i++) {
                Argument argument = instruction.getArgument(i);
                String key = argument.getKey();
                String val = argument.getValue();
                if (key.equals("doctype")) {
                    if (val != null) {
                        if (val.equals("strict")) {
                            currentXsltWriter.addAttribute("doctype-public", "-//W3C//DTD HTML 4.01//EN");
                            currentXsltWriter.addAttribute("doctype-system", "http://www.w3.org/TR/html4/strict.dtd");
                        }
                    }
                } else {
                    if (isValidOutputAttribute(key)) {
                        currentXsltWriter.addAttribute(key, val);
                    } else {
                        //log warning
                    }

                }
            }
            currentXsltWriter.closeEmptyTag();
        }

    }

    private String getValue(Instruction instruction, String param) {
        int length = instruction.getLength();
        for (int i = 1; i < length; i++) {
            Argument argument = instruction.getArgument(i);
            String key = argument.getKey();
            if (key.equals(param)) {
                return argument.getValue();
            }
        }
        return null;
    }


    static class Attributes {

        String mode = null;
        String name = null;
        String priority = null;
        boolean doe = false;

        Attributes(Instruction instruction) {
            int length = instruction.getLength();
            for (int i = 1; i < length; i++) {
                Argument arg = instruction.getArgument(i);
                String key = arg.getKey();
                String value = arg.getValue();
                if (key.equals("doe")) {
                    doe = true;
                } else if (key.equals("mode")) {
                    mode = value;
                } else if (key.equals("name")) {
                    name = value;
                } else if (key.equals("priority")) {
                    priority = value;
                }
            }
        }

    }

    static private boolean isValidOutputAttribute(String s) {
        if (s.equals("version")) {
            return true;
        } else if (s.equals("encoding")) {
            return true;
        } else if (s.equals("omit-xml-declaration")) {
            return true;
        } else if (s.equals("standalone")) {
            return true;
        } else if (s.equals("doctype-public")) {
            return true;
        } else if (s.equals("doctype-system")) {
            return true;
        } else if (s.equals("cdata-section-elements")) {
            return true;
        } else if (s.equals("indent")) {
            return true;
        } else if (s.equals("media-type")) {
            return true;
        }
        return false;
    }


    static public interface CustomResolver {

        public boolean isTemplateInstruction(InsertionUnit insertionUnit);

        public void testInsertionUnit(InsertionUnit insertionUnit);

        public void resolveInsertionUnit(InsertionUnit insertionUnit) throws IOException;

    }

}
