/* UtilLib_Image - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.images;

import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.imageio.ImageIO;


/**
 *
 * @author Vincent Calame
 */
public class ImageRescale {

    public static void rescale(File srcFile, File destFile, RescaleDim rescaleDim, String format) throws IOException {
        BufferedImage srcImg = ImageIO.read(srcFile);
        BufferedImage destImg = getScaledInstance(srcImg, rescaleDim);
        ImageIO.write(destImg, format, destFile);
    }

    public static void rescale(InputStream inputStream, OutputStream outputStream, RescaleDim rescaleDim, String format) throws IOException {
        BufferedImage srcImg = ImageIO.read(inputStream);
        BufferedImage destImg = getScaledInstance(srcImg, rescaleDim);
        ImageIO.write(destImg, format, outputStream);
    }

    public static void rescale(InputStream inputStream, File destFile, RescaleDim rescaleDim, String format) throws IOException {
        BufferedImage srcImg = ImageIO.read(inputStream);
        BufferedImage destImg = getScaledInstance(srcImg, rescaleDim);
        ImageIO.write(destImg, format, destFile);
    }

    public static BufferedImage getScaledInstance(BufferedImage img, RescaleDim rescaleDim) {
        int width = img.getWidth();
        int height = img.getHeight();
        short type = rescaleDim.getType();
        int targetWidth = rescaleDim.getWidth();
        int targetHeight = rescaleDim.getHeight();
        if (rescaleDim.isOnlyMax()) {
            if (type == RescaleDim.WIDTH_RESCALE) {
                if (width <= targetWidth) {
                    return img;
                }
            } else if (type == RescaleDim.HEIGHT_RESCALE) {
                if (height <= targetHeight) {
                    return img;
                }
            } else if (type == RescaleDim.DIM_RESCALE) {
                if ((width <= targetWidth) && (height <= targetHeight)) {
                    return img;
                }
            }
        }

        float ratio = 0;

        if (type == RescaleDim.WIDTH_RESCALE) {
            if (targetWidth == width) {
                return img;
            }
            ratio = ((float) targetWidth) / width;
            targetHeight = (int) (ratio * height);
        } else if (type == RescaleDim.HEIGHT_RESCALE) {
            if (targetHeight == height) {
                return img;
            }
            ratio = ((float) targetHeight) / height;
            targetWidth = (int) (ratio * width);
        } else if (type == RescaleDim.DIM_RESCALE) {
            float widthRatio = ((float) targetWidth) / width;
            float heightRatio = ((float) targetHeight) / height;
            if (widthRatio < heightRatio) {
                ratio = widthRatio;
                targetHeight = (int) (ratio * height);
            } else {
                ratio = heightRatio;
                targetWidth = (int) (ratio * width);
            }
        }
        if (ratio > 1) {
            return getScaledInstance(img, targetWidth, targetHeight, RenderingHints.VALUE_INTERPOLATION_BICUBIC, false);
        } else {
            return getScaledInstance(img, targetWidth, targetHeight, RenderingHints.VALUE_INTERPOLATION_BILINEAR, true);
        }
    }

    /**
     *
     * Convenience method that returns a scaled instance of the
     * provided {@code BufferedImage}. From <a href="http://today.java.net/pub/a/today/2007/04/03/perils-of-image-getscaledinstance.html">The Perils of Image.getScaledInstance()</a>
     * by Chris Campbell
     *
     * @param img the original image to be scaled
     * @param targetWidth the desired width of the scaled instance,
     *    in pixels
     * @param targetHeight the desired height of the scaled instance,
     *    in pixels
     * @param hint one of the rendering hints that corresponds to
     *    {@code RenderingHints.KEY_INTERPOLATION} (e.g.
     *    {@code RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR},
     *    {@code RenderingHints.VALUE_INTERPOLATION_BILINEAR},
     *    {@code RenderingHints.VALUE_INTERPOLATION_BICUBIC})
     * @param higherQuality if true, this method will use a multi-step
     *    scaling technique that provides higher quality than the usual
     *    one-step technique (only useful in downscaling cases, where
     *    {@code targetWidth} or {@code targetHeight} is
     *    smaller than the original dimensions, and generally only when
     *    the {@code BILINEAR} hint is specified)
     * @return a scaled version of the original {@code BufferedImage}
     */
    public static BufferedImage getScaledInstance(BufferedImage img,
            int targetWidth,
            int targetHeight,
            Object hint,
            boolean higherQuality) {
        int type = (img.getTransparency() == Transparency.OPAQUE)
                ? BufferedImage.TYPE_INT_RGB : BufferedImage.TYPE_INT_ARGB;
        BufferedImage ret = (BufferedImage) img;
        int w, h;
        if (higherQuality) {
            // Use multi-step technique: start with original size, then
            // scale down in multiple passes with drawImage()
            // until the target size is reached
            w = img.getWidth();
            h = img.getHeight();
        } else {
            // Use one-step technique: scale directly from original
            // size to target size with a single drawImage() call
            w = targetWidth;
            h = targetHeight;
        }

        do {
            if (higherQuality && w > targetWidth) {
                w /= 2;
                if (w < targetWidth) {
                    w = targetWidth;
                }
            }

            if (higherQuality && h > targetHeight) {
                h /= 2;
                if (h < targetHeight) {
                    h = targetHeight;
                }
            }

            BufferedImage tmp = new BufferedImage(w, h, type);
            Graphics2D g2 = tmp.createGraphics();
            g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, hint);
            g2.drawImage(ret, 0, 0, w, h, null);
            g2.dispose();

            ret = tmp;
        } while (w != targetWidth || h != targetHeight);

        return ret;
    }

}
